
// *****************************************************************************
//
// source:    AC_SVD_evnt.C: User code for VDS CnA part
//
// type:      source code
//
// created:   03 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_SVD_evnt()
//
// called by: AC_evnt()
//
// hints:     see $HBROOT/ARTE/dev/include/dq/DQ.h for COMP_XYZ and LVL_ABC
//
// changes:
// 03 Nov 1999 (AG)  further developments
// 19 Nov 1999 (AG)  cna -> ac (alca)
// 22 Nov 1999 (AG)  includes in alca
// 10 Dez 1999 (MBr) user code added
//
// ATTENTION: The current Version is running ONLY with HOLMES reco!!!!
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "alca/AC_SVD.hh"

// VDS includes:
#include "Class_HOLMES.h"
#include "Class_GEDE.h"

#include "alca/AC_SVD_ALIGN5.h"
#include "alca/AC_SVD_L4dqCheck.h"

#include "Struct_Track.h"
#include "Struct_Hitb.h"
#include "Struct_Gede.h"

// how often to print:
#define NP_HOLMES 5

// definition for the alignment:
#define NCUT 6 // hit/track >6

#define FITU 1 // align lateral
#define FITZ 1 // align along z
#define FITS 1 // align rotations


// The externals:
extern HOLMES_class *HOLMESp;
extern int FAGRR;

// pointer to the alignment structures:
ALIGN5_class    *ALIGN5p; 
L4dqCheck_class *L4HELIXp; 

//----------------------------------------------------------
// store/retrieve the alignment data:
int storealign(int type, int i, int j, double value ) {
  if (type == ALIGN5_SMAT) {
    rhp_hfilld(AC_ID(COMP_SVD,LVL_TST,101),
	       float(i),float(j),value);
  }
  if (type == ALIGN5_SVEC) {
    rhp_hf1(AC_ID(COMP_SVD,LVL_TST,102),
	    float(i),value);
  }  
  return(1);
}
  
int retrievealign(int type, int i, int j, double *value) {
  if (type == ALIGN5_SMAT) {
    *value = rhp_hxy(AC_ID(COMP_SVD,LVL_TST,101),
		      float(i),float(j));
  }
  if (type == ALIGN5_SVEC) {
    *value = rhp_hxd(AC_ID(COMP_SVD,LVL_TST,102),
		     float(i));    
  }  
  return(1);
}


int storel4helix(int side, int chip, int val) {
 rhp_hfill(AC_ID(COMP_SVD,LVL_TST,201),
	 float(side),float(chip),float(val)); 
 return(1);
}

// ------------------------------------------------------------------


//
// program code
//
int
AC_SVD_evnt()
{
  static int align_initialised = 0;
  //  Gede_t       *gedeP;
  static int fixed_sides[] = {40600,40601,20600,20601};
  
  // -------------- check for holmes: --------------------
  static int err_holmes = NP_HOLMES;
  if (( HOLMESp == NULL ) && (err_holmes>0)) {
    printf("VDS: AVIATION: usevent: NO holmes pointer found !\n");
    if( FAGRR )
      grr_error( GRR_SWARNING,
		 "VDS: AVIATION: usevent: NO holmes pointer found !", 0 );
    -- err_holmes;
  }
 
  // -------------- initialise if first called: --------------------------------
  if (( !align_initialised ) && ( HOLMESp != NULL )) {
    printf("VDS_ALIGN5: Initialising..\n");
    ALIGN5p = new ALIGN5_class(HOLMESp,FITU,FITZ,FITS,fixed_sides,
			       storealign,retrievealign);
    printf("VDS: AVIATION: ALIGN5 structs okay!\n");

    L4HELIXp = new L4dqCheck_class(HOLMESp,storel4helix);
    //L4HELIXp = new L4dqCheck_class(HOLMESp,NULL);
    printf("VDS: AVIATION: L4helix: structs okay!\n");

    // ok, now initialised:
    align_initialised = 1;
  }

  // -------------- Work ONLY if we have the dear HOLMES: ----------------------
  if (HOLMESp != NULL) {
    // -------------- The checks for healthy HELIX chips: ----------------------
    for (ArteTableIterator<HITB> hitb; hitb.valid(); hitb.next() ) {
      if (hitb->gede->cmp != 1) continue; // check for VDS only
      
      L4HELIXp->fAccuHit(hitb->gede->gesl->nsl, hitb->gede->sec,hitb->gede->lay,
			 ((hitb->x - hitb->gede->xfw[0]) / hitb->gede->pit));
    }
    // -------------- The Alignment: ----------------------
    for (int tra=0; tra<HOLMESp->ntracks();++tra) {
      Track_t track=*(HOLMESp->trackP(tra,0.0));

      // track quality checks: (to be improved!!)
       // the least one: number of hits:
       int maxhit = track.nhit;   
        if (maxhit < NCUT) continue; // only >ncut hits/track ! 
       // process align data from tracks:
       ALIGN5p->fGather(tra);
       
    } // of track loop
  } // of if (HOLMESp != NULL)


  return 0;
}
