
// *****************************************************************************
//
// source:    RC_init.C
//
// type:      source code
//
// created:   23 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    RC_init()
//
// called by: acinit_()
//
// changes:
// 24 Nov 1999 (AG) improve print-out
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE icludes
#include "runcfg.h"
#include "grr.h"

// Arte includes
#include "farm/farm.hh"

// externals
extern int FADEB;

// reco flags
extern int recoTARGET;
extern int recoVDS;
extern int recoVDSmodule;
extern int recoHPT;
extern int recoOTR;
extern int recoRICH;
extern int recoTRD;
extern int recoECAL;
extern int recoMUON;

//
// function
//
int
RC_init()
{
  // show up
  cout << "RC_init()" << endl;

  // inits
  int rc = 0;

  // reco flags
  char *rrecomask = getenv( PRCMRECOMASK );
  int mask = 0;
  if( rrecomask ) mask = atoi( rrecomask );
  char txt[80];
  memset( txt, 0, 80 );
  sprintf( txt, "RC_init(): RECO Mask=%d", mask );
  ARTE_error( GRR_SINFO, txt, 0 );

  // print
  //if( FADEB ){
    cout << "RC_init(): Reco in run (PRCMRECOMASK):"    << endl;
    cout << "           Mask=" << mask                  << endl;
    cout << "           SVD= " << faReco( COMP_M_SVD )  << endl;
    cout << "           ITR= " << faReco( COMP_M_ITR )  << endl;
    cout << "           HIPT=" << faReco( COMP_M_HIPT ) << endl;
    cout << "           OTR= " << faReco( COMP_M_OTR )  << endl;
    cout << "           RICH=" << faReco( COMP_M_RICH ) << endl;
    cout << "           TRD= " << faReco( COMP_M_TRD )  << endl;
    cout << "           ECAL=" << faReco( COMP_M_ECAL ) << endl;
    cout << "           MUON=" << faReco( COMP_M_MUON ) << endl;
    //}

  // print initial reco flags
  if( FADEB ){
    cout << "RC_init(): initial RECO flags:" << endl;
    cout << "           TARGET="    << recoTARGET    << endl;
    cout << "           VDS="       << recoVDS       << endl;
    cout << "           VDSmodule=" << recoVDSmodule << endl;
    cout << "           HIPT="      << recoHPT       << endl;
    cout << "           OTR="       << recoOTR       << endl;
    cout << "           RICH="      << recoRICH      << endl;
    cout << "           TRD="       << recoTRD       << endl;
    cout << "           ECAL="      << recoECAL      << endl;
    cout << "           MUON="      << recoMUON      << endl;
  }

  // store inital settings
  static int first = 1;
  static int recoTARGET_init    = 0;
  static int recoVDS_init       = 0;
  static int recoVDSmodule_init = 0;
  static int recoHPT_init       = 0;
  static int recoOTR_init       = 0;
  static int recoRICH_init      = 0;
  static int recoTRD_init       = 0;
  static int recoECAL_init      = 0;
  static int recoMUON_init      = 0;
  if( first ){
    first = 0; 
    recoTARGET_init    = recoTARGET;
    recoVDS_init       = recoVDS;
    recoVDSmodule_init = recoVDSmodule;
    recoHPT_init       = recoHPT;
    recoOTR_init       = recoOTR;
    recoRICH_init      = recoRICH;
    recoTRD_init       = recoTRD;
    recoECAL_init      = recoECAL;
    recoMUON_init      = recoMUON;
  }

  // set flags
  recoTARGET    = recoTARGET_init    & faReco( COMP_M_SVD );
  recoVDS       = recoVDS_init       & faReco( COMP_M_SVD );
  recoVDSmodule = recoVDSmodule_init & faReco( COMP_M_SVD );
  recoHPT       = recoHPT_init       & faReco( COMP_M_HIPT );
  recoOTR       = recoOTR_init       & faReco( COMP_M_OTR );
  recoRICH      = recoRICH_init      & faReco( COMP_M_RICH );
  recoTRD       = recoTRD_init       & faReco( COMP_M_TRD );
  recoECAL      = recoECAL_init      & faReco( COMP_M_ECAL );
  recoMUON      = recoMUON_init      & faReco( COMP_M_MUON );

  // print final reco flags
  if( FADEB ){
    cout << "RC_init(): final RECO flags:" << endl;
    cout << "           TARGET="    << recoTARGET    << endl;
    cout << "           VDS="       << recoVDS       << endl;
    cout << "           VDSmodule=" << recoVDSmodule << endl;
    cout << "           HIPT="      << recoHPT       << endl;
    cout << "           OTR="       << recoOTR       << endl;
    cout << "           RICH="      << recoRICH      << endl;
    cout << "           TRD="       << recoTRD       << endl;
    cout << "           ECAL="      << recoECAL      << endl;
    cout << "           MUON="      << recoMUON      << endl;
  }

  return rc;
}
