// ********************************************************************
//
//  Routine:      acGlobalAlignmentTransformation.C
//  Author:       T. Oest
//
//  Description:  applies the global alignment transformation
//                on GEDE tables
//
//  Modification:
//   -- update gede->xfw[0] for VDS                  27/09/99 T.Oest
//   -- routine reactivated                          07/01/00 T.Oest 
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"
//#include "auto/ACGA.hh"

extern "C"
{
 void rinv_(const int    & n,
            const float  * wMeas,
            const int    & idim,
            const float  * r,
            const int    & ifail);
}

const int indexARTE[3][3] = {{0, 3,  6},
                             {1, 4,  7},
                             {2, 5,  8}};

//
// function
//
void acGlobalAlignmentForGEDE() {

//----------------------------------------------------------------
// loop over ACGA table
//----------------------------------------------------------------
    for( ArteTable<ACGA>::iterator acga = ArteTable<ACGA>::begin();
         acga != ArteTable<ACGA>::end(); ++acga ) {

//----------------------------------------------------------------
// acga rotation matrix:
// e.g. z-rotation:
//      ( 0 1 2 )    ( cos -sin 0 )
//      ( 3 4 5 )  = ( sin  cos 0 )
//      ( 6 7 8 )    (  0    0  1 )
//----------------------------------------------------------------
    float GEDElikeMatrix[3][3];

    for (int i=0; i<3; i++){for (int j=0; j<3; j++){
     GEDElikeMatrix[i][j]  = (*acga)->rot[indexARTE[i][j]];
    };};

 /*
    cout << "  " << endl;
    cout << " acga matrix " << endl;
    cout << GEDElikeMatrix[0][0] << " " 
         << GEDElikeMatrix[0][1] << " " 
         << GEDElikeMatrix[0][2] << endl;
    cout << GEDElikeMatrix[1][0] << " " 
         << GEDElikeMatrix[1][1] << " " 
         << GEDElikeMatrix[1][2] << endl;
    cout << GEDElikeMatrix[2][0] << " "  
         << GEDElikeMatrix[2][1] << " "  
         << GEDElikeMatrix[2][2] << endl;
 */

//----------------------------------------------------------------
// loop over GEDE tables
//----------------------------------------------------------------
    for( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
         gede != ArteTable<GEDE>::end(); ++gede ) {

      if ((*acga)->cmp == (*gede)->cmp){

        float GEDErot[3][3], GEDErotnew[3][3]; 

//----------------------------------------------------------------
// transformation
//     new GEDE rot matrix:   Gnew = G * (acga->rot)**-1
//----------------------------------------------------------------

        for (int i=0; i<3; i++){for (int j=0; j<3; j++){
         GEDErot[i][j]    = (*gede)->rot[indexARTE[i][j]];
        };};



        for (int i=0; i<3; i++){for (int j=0; j<3; j++){

         GEDErotnew[i][j] = 0.0;
         for (int k=0; k<3; k++){
            GEDErotnew[i][j] += GEDErot[i][k] * GEDElikeMatrix[k][j];
	 };
         (*gede)->rot[indexARTE[i][j]] = GEDErotnew[i][j];
        };};

//----------------------------------------------------------------
// transformation
//     new position       :   Rnew = R + Gnew * dr
//----------------------------------------------------------------
       float deltaz, dr[3], drRot[3];

       deltaz = 0.5*((*gede)->zmin+(*gede)->zmax) - (*acga)->z;

       dr[0] = (*acga)->dx + (*acga)->dtx * deltaz;
       dr[1] = (*acga)->dy + (*acga)->dty * deltaz;
       dr[2] = (*acga)->dz;

       for (int i=0; i<3; i++){
        drRot[i] = 0.0;
        for (int j=0; j<3; j++){
        drRot[i] = drRot[i] + GEDErotnew[i][j] * dr[j];
       };};

       (*gede)->xmin = (*gede)->xmin + drRot[0];
       (*gede)->xmax = (*gede)->xmax + drRot[0];
       (*gede)->ymin = (*gede)->ymin + drRot[1];
       (*gede)->ymax = (*gede)->ymax + drRot[1];
       (*gede)->zmin = (*gede)->zmin + drRot[2];
       (*gede)->zmax = (*gede)->zmax + drRot[2];

//----------------------------------------------------------------
// detector specific updates
//----------------------------------------------------------------

       if ((*gede)->cmp == 1){
        (*gede)->sxmn = (*gede)->sxmn + drRot[0];
        (*gede)->sxmx = (*gede)->sxmx + drRot[0];
        (*gede)->symn = (*gede)->symn + drRot[1];
        (*gede)->symx = (*gede)->symx + drRot[1];

	(*gede)->xfw[0] = (*gede)->cos * (*gede)->sxmn
	                 -(*gede)->sin * (*gede)->symn 
                         + (*gede)->xfw[1];
       };

       if ((*gede)->cmp == 2){
        (*gede)->sxmn = (*gede)->sxmn + drRot[0];
        (*gede)->sxmx = (*gede)->sxmx + drRot[0];
        (*gede)->symn = (*gede)->symn + drRot[1];
        (*gede)->symx = (*gede)->symx + drRot[1];
       };


      };
    };
};
};



