// *********************************************************************
//
//  Routine:      acread_
//  Author:       T.Oest
//
//  Description:  general steering routine to apply internal and
//                
//
//  Modification:
//  - acGlobAlChange() switched off (causes a segmentation fault
//    if the database is not defined)                   T.Oest  2/8/99
//  - VDS and global alignment constants read in        T.Oest 16/9/99
//  - keytable code by M. Hissmann added                T.Oest 20/9/99
//  - call to acReadTARGET added                        T.Oest 24/9/99
//  - call of acReadDQ added                            T.Oest 29/10/99
//  - alignment is read in only for DST if reco on or
//    KeyBook was called in the kumac                   T.Oest 02/12/99
//
// ********************************************************************


#include <iostream.h>
#include <stdlib.h>
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "database/keytable.h"

extern int KeyBookWasCalled;

int acReadTARGET(int);
int acReadVDS(int);
int acReadOTR(int);
int acReadITR(int);
int acReadHPT(int);
int acReadTRD(int);
int acReadRICH(int);
int acReadMUON(int);
int acReadGlobal(int);
int acReadDQ(int);

extern int recoVDS;
extern int recoOTR;
extern int recoECAL;
extern int recoMUON;
extern int recoRICH;
extern int recoTRD;
extern int recoHPT;

static int firstCall         = 0;
static int readAlignment     = 0;

//*****************************************************************
// function
//*****************************************************************

extern "C" void acread_() {

//==================================================================
// no action for MC
//==================================================================

  if (ArteTable<MTRA>::size() > 0 ) return ;

//==================================================================
// check for the first call if alignment should be read in
//   - no reading  for MINIs (ArteTable<DEVT>::size() == 0)
//   - reading     if reconstruction is on or KeyBookWasCalled == 1
//==================================================================

  if (firstCall == 0){

   firstCall = 1;

   int recoITR    = 0;

   if ((recoVDS    != 0 || recoOTR  != 0 ||
        recoITR    != 0 || recoECAL != 0 || recoMUON != 0 ||
        recoRICH   != 0 || recoTRD  != 0 || recoHPT  != 0 ||
        KeyBookWasCalled == 1) 
        && ArteTable<DEVT>::size() != 0) readAlignment = 1;

  }

//==================================================================
// 1. check for a new alignment/calibration key (DKEY table)
//      KTstatus will contain status of KeyTable
//               -2  no KUIP keytable initalisation
//               -1  faild to find keytable 
//               >0  new keytable
//==================================================================

  int KTstatus = -2;
  if (KeyBookWasCalled == 1) KTstatus = KeyLoadTables(faEVHDdbmajor());

  if (KTstatus==-1) {
      // just protest loudly ! 
      // the keytable with version could not be found
      // faEVHDdbmajor() faEVHDdbminor()
      KTstatus=0;
     }


//==================================================================
// 2. ARTE steering
//==================================================================

  int flagDQ     = 0;

  if (KTstatus > 0)  flagDQ = acReadDQ(KTstatus);

//==================================================================
// 3. set alignment constants, transforms NEDE to GEDE tables
//  - no action if no reconstruction will be done
//  - alignment sensitive parameters should be taken from 
//    the subdetector database (no off values with respect to NEDE)
//  - constant values (pitch) should be taken from NEDE
//  - return flag: > 0   constants were changed
//                 = 0   no change of constants
//  -              < 0   error, old constants are kept
//==================================================================

  int flagTARGET = 0;
  int flagVDS    = 0;
  int flagITR    = 0;
  int flagOTR    = 0;
  int flagMUON   = 0;
  int flagTRD    = 0;
  int flagRICH   = 0;
  int flagHPT    = 0;
  int flagGLOBAL = 0;
  
  if (KTstatus!=0) {
 
    flagTARGET = acReadTARGET(KTstatus);

  if (readAlignment == 1)  {

     flagVDS    = acReadVDS(KTstatus);
     //flagITR    = acReadITR(KTstatus);
     //flagOTR    = acReadOTR(KTstatus);
     //flagMUON   = acReadMUON(KTstatus);
     //flagTRD    = acReadTRD(KTstatus);
     flagRICH   = acReadRICH(KTstatus);
     //flagHPT    = acReadHPT(KTstatus);
     flagGLOBAL = acReadGlobal(KTstatus);
  };
  };

//==================================================================
// 5. error handling (not yet implemented)
//==================================================================

 return;
}

