
// *****************************************************************************
//
// source:    utstage.C
//
// created:    1 Jun 1999
//
// Author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    utstage_()
//
// called by: arte/ifopen.F
//
// changes:
// 09 Sep 1999 (AG) replace cerr by cout
//
// *****************************************************************************

// includes
#include <iostream>
#include <string.h>

#include "stage.h"

#include "farm/fa.hh"

// externals
extern int  FADEB;
extern char *HBSTAGE;
extern struct {
  char ctape[256];
  char cdisk[256];
  } stage_;

//
// program code
//
extern "C" void
utstage_()
{
  // handle names
  char *ptr = strtok( stage_.ctape, " " ); // find end of tape name
  char tmp[256];                           // temporary tape name
  strcpy( tmp, ptr );                      // copy tape name
  memset( stage_.cdisk, 0, 256 );          // reset disk file name
  memset( stage_.ctape, 0, 256 );          // reset tape file name
  strcpy( stage_.ctape, tmp );             // copy clean tape name
  cout << "STAGE: Start staging of tape file=" << stage_.ctape << endl;

#if ( defined linux ) || ( defined sun ) || ( defined sgi )
  
  // staging implemented for Linux, Sun, SGI
  int rc = l4aStageF_( stage_.ctape, HBSTAGE, FADEB );
  if( rc == L4A_OK ){
#ifdef _OUT_
    cout << "STAGE: Staging finished for tape file=" << stage_.ctape << endl;
#endif
  }
  else if( rc == L4A_FAILURE ){
#ifdef _ERR_
    cout << "STAGE: ERROR: Staging FAILED for tape file=" << stage_.ctape << endl; 
#endif
    return;
  }
  else if( rc == L4A_OSM_FAILURE ){
#ifdef _ERR_
    cout << "STAGE: ERROR: Staging FAILED due to OSM errors for tape file=" << stage_.ctape << endl; 
#endif
    return;
  }
  else if( rc == L4A_NO_ACS_FILE ){
#ifdef _ERR_
    cout << "STAGE: ERROR: No such file on tape file=" << stage_.ctape << endl;
#endif
    return;
  }
  else {
#ifdef _ERR_
    cout << "STAGE: ERROR: While staging tape file=" << stage_.ctape << " rc=" << rc << endl; 
#endif
    return;
  }

  // tape of disk file
  memset( stage_.cdisk, 0, 256 );
  strcat( stage_.cdisk, HBSTAGE );
  strcat( stage_.cdisk, stage_.ctape + 4 ); // omit "/acs"

  cout << "STAGE: Staged  tape file=" << stage_.ctape << endl;
  cout << "STAGE: Created disk file=" << stage_.cdisk << endl;
  
#else
  
  // staging NOT implemented
  cout << "STAGE: Staging NOT implemented for your machine!" << endl; 
  return;
  
#endif
}
