//                              -*- Mode: C++ -*-
//: CmdBase.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdBase.hh"
#ifdef USEKUIP
  extern "C"  {
    int   ku_npar();   // return the number of parameters 
    int   ku_geti();   // read an integer
    double ku_getr();  // read a float
    char* ku_gets();   // read a string
    char* ku_getc();   // read an uppercase string
    char* ku_getf();   // read a filename
    char* ku_getl();   // read a comma separated list
  } 

#endif

/*
CLASS
  CmdBase
  Abstract base class to define an interface for the commands
  issued throu the Kuip interface or throu other ways (to be defined)
  The flag USEKUIP is used to select the KUIP interface. 


*/

///////// CmdBase member function definitions //////////////////

CmdBase::CmdBase(){}
CmdBase::~CmdBase(){}


//--
//  Return the number of parameters
int CmdBase::readPars() const{
  int i=0;
# ifdef USEKUIP
    i = ku_npar();
# endif
  return i;
}

//--
//  Read a float
float CmdBase::readFloat() const{
  float x=0.0;
# ifdef USEKUIP
    x = ku_getr();
# endif
  return x;
}

//--
//  Read an integer
int CmdBase::readInt() const{
  int i=0;
# ifdef USEKUIP
    i = ku_geti();
# endif
  return i;
}

//--
//  Read a string
char* CmdBase::readString() const{
  char *s=0;
# ifdef USEKUIP
    s = ku_gets();
# endif
  return s;
}

//--
//  Read an Uppercase string
char* CmdBase::readUppercaseString() const{
  char *s=0;
# ifdef USEKUIP
    s = ku_getc();
# endif
  return s;
}

//--
//  Read a string
char* CmdBase::readFileName() const{
  char *s=0;
# ifdef USEKUIP
    s = ku_getf();
# endif
  return s;
}

//--
//  Read the next item
char* CmdBase::readItem() const{
  char *s=0;
# ifdef USEKUIP
    s = ku_getl();
# endif
  return s;
}

#ifdef USEKUIP
  //--
  //   Read a command from external
  void CmdBase::readAndExe(){
    read();
    execute();
  }
#endif




