//                              -*- Mode: C++ -*-
//: CmdCareECDB.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/4/98
// Last Update ........
//

#ifndef CMDCAREECDBMIZ_HH_
#define CMDCAREECDBMIZ_HH_

#include "care/CmdBase.hh" 
#include "care/EcalDB.hh"


/*
CLASS
  CmdCareECDBMIZ
  Class to deal with the care/ECDBMIZ kuip command
  Define the name of the MIZZI file containing the calibration DB

*/

class CmdCareECDBMIZ : public CmdBase {
public:
  enum Tbset {TALL, TECBO, TECHS, TPED, TSENS, THOT, TSLOW, TFAST};

  //-- Constructors
  CmdCareECDBMIZ();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareECDBMIZ();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--g
  //   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  //   Read acommand from external
  virtual void read();
# endif
  //--
  //   Dump internal data
  virtual void dump();
  //--
  //   Static access member function
  char* calibfile();

  //  Set DB properties
  void setDB(char *nam, int imajor, int iminor);

  //  Set the main Infos
  //-- 
  void setInfo(int time1, int time2, char* name, int version);

  //  Get functions
  inline int Major() const;
  inline int Minor() const;

  //--
  // Read the database
  int readDB(Tbset whichtable, int access);
  //--
  // Write the database
  int writeDB(Tbset whichtable, int access);

  // get the new info
  inline const EcalDB::TableInfo & Info() const;

  /// build up the ECHF table
  int makeECHF();

private:
  
  int readECBO(EcalDB & mydb, int access);
  int readECHS(EcalDB & mydb, int access);
  int readECHF(EcalDB & mydb, int access);
  int readECPD(EcalDB & mydb, int access);
  int readECSN(EcalDB & mydb, int access);
  int readECHT(EcalDB & mydb, int access);

  int writeECBO(EcalDB & mydb, int access);
  int writeECHS(EcalDB & mydb, int access);
  int writeECHF(EcalDB & mydb, int access);
  int writeECPD(EcalDB & mydb, int access);
  int writeECSN(EcalDB & mydb, int access);
  int writeECHT(EcalDB & mydb, int access);

  ///////////////////// Data Part /////////////////////////////////
  //--
  static  char filename[256];
  enum  {none, dbread, dbwrite} action;
  Tbset tbset;
  int match;
  int major;
  int minor;
  EcalDB::TableInfo info;
  
};

int CmdCareECDBMIZ::Major() const {
    return major;
}

int CmdCareECDBMIZ::Minor() const {
    return minor;
}

// get the new info
const EcalDB::TableInfo & CmdCareECDBMIZ::Info() const{
  return info;
}

#endif  //   CMDCAREECDBMIZ_HH_

