//!!!!!!!!!!!!!!!!!!!!!!
using namespace std;//!!
//!!!!!!!!!!!!!!!!!!!!!!
#include "Class_MGEDE.hh"
#include <string>

  const char *delim = "\n*************";

  //==============================================================================
  // Constructors
  //==============================================================================
  MGede::MGede() {init();}

  MGede::MGede(Gede gede) {
    init();
    vgede.push_back(gede);
    map_insert(gede, vgede.size() - 1);
  }

  MGede::MGede(ArteGedeIt begin, ArteGedeIt end) {

    for(ArteGedeIt it = begin; it != end; ++it) {
      ArtePointer<GEDE> agede = *it;

      // Select VDS Gede's
      if( agede->cmp != 1 ) {
	break;
      }

      // fill gede from ARTE
      Gede gede(agede);
      vgede.push_back(gede);

      map_insert(gede, vgede.size() - 1);
    }
  }

  //==============================================================================
  // round()
  //==============================================================================
  inline double MGede::round(double z) const { 
    return ((z-floor(z))<0.5) ? floor(z) : ceil(z); 
  }

  //==============================================================================
  // z_key()
  //==============================================================================
  inline int MGede::z_key(double z) const { return static_cast<int>(round(z*100.)); }

  //==============================================================================
  // init()
  //==============================================================================
  void MGede::init() {
    vgede.clear();
    index_map.clear();
    z_map.clear();
  }

  //==============================================================================
  // map_insert()
  //==============================================================================
  void MGede::map_insert(const Gede& gede, const int pos) {
    index_map.insert(  Gede_M::value_type(gede.index(), pos));
    z_map.insert(     Gede_zM::value_type(z_key(gede.z()), pos));
  }

  //==============================================================================
  // operator[]
  //==============================================================================
  Gede& MGede::operator[](int index) {
    // for sake of speed, checking if index exists in index_map is canceled!
    // use operator[] with care!

    //if side is not in index_map, the first element is returned!
    //    if(index_map.find(index) == index_map.end())
    //      return vgede[0];
    //    else
      return vgede[index_map[index]];
  }

  //==============================================================================
  // operator[]
  //==============================================================================
  Gede& MGede::operator[](double z) {
    //if side is not in z_map, the first element is returned!
    return vgede[z_map.lower_bound(z_key(z))->second];
  }

  //==============================================================================
  // size()
  //==============================================================================
  int MGede::size() const { return vgede.size(); }

  //==============================================================================
  // push_back()
  //==============================================================================
  void MGede::push_back(const Gede& gede) {
    vgede.push_back(gede);
    map_insert(gede, vgede.size() - 1);
  }

  //==============================================================================
  // push_back()
  //==============================================================================
  void MGede::push_back(const int deadgede) {
    sdeadgede.insert(deadgede);
  }

  //==============================================================================
  // is_dead()
  //==============================================================================
  bool MGede::is_dead(int gede) const {
    if(sdeadgede.find(gede) != sdeadgede.end())
      return true;
    else
      return false;
  }

  //==============================================================================
  // index_order()
  //==============================================================================
  void MGede::index_order(void) {
    cout << "side_order: ";
    for( Gede_MIt it = index_map.begin(); it != index_map.end(); ++it)
      cout << it->first << " ";
    cout << endl;
  }

  //==============================================================================
  // z_order()
  //==============================================================================
  void MGede::z_order(void) {
    cout << "z-order: ";
    for( Gede_zMIt it = z_map.begin(); it != z_map.end(); ++it)
      cout << it->first << " ";
    cout << endl;
  }

  //==============================================================================
  // begin()
  //==============================================================================
  MGede::iterator MGede::begin() {
    iterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = index_map.begin();
    return tmp;
  }

  //==============================================================================
  // end()
  //==============================================================================
  MGede::iterator MGede::end() {
    iterator tmp;
    tmp.begin = vgede.end();
    tmp.iter  = index_map.end();
    return tmp;
  }

  //==============================================================================
  // lower_bound()
  //==============================================================================
  MGede::iterator MGede::lower_bound(int key) {
    iterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = index_map.lower_bound(key);
    return tmp;
  }

  //==============================================================================
  // upper_bound()
  //==============================================================================
  MGede::iterator MGede::upper_bound(int key) {
    iterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = index_map.upper_bound(key);
    return tmp;
  }

  //==============================================================================
  // find()
  //==============================================================================
  MGede::iterator MGede::find(int key) {
    iterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = index_map.find(key);
    return tmp;
  }

  //==============================================================================
  // lower_bound()
  //==============================================================================
  MGede::iterator MGede::lower_bound(unsigned int key) {
    return lower_bound(static_cast<int>(key));
  }

  //==============================================================================
  // upper_bound()
  //==============================================================================
  MGede::iterator MGede::upper_bound(unsigned int key) {
    return upper_bound(static_cast<int>(key));
  }

  //==============================================================================
  // find()
  //==============================================================================
  MGede::iterator MGede::find(unsigned int key) {
    return find(static_cast<int>(key));
  }

  //==============================================================================
  // zbegin()
  //==============================================================================
  MGede::ziterator MGede::zbegin() {
    ziterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = z_map.begin();
    return tmp;
  }

  //==============================================================================
  // zend()
  //==============================================================================
  MGede::ziterator MGede::zend() {
    ziterator tmp;
    tmp.begin = vgede.end();
    tmp.iter  = z_map.end();
    return tmp;
  }

  //==============================================================================
  // lower_bound()
  //==============================================================================
  MGede::ziterator MGede::lower_bound(const double key) {
    ziterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = z_map.lower_bound(z_key(key));
    return tmp;
  }

  //==============================================================================
  // upper_bound()
  //==============================================================================
  MGede::ziterator MGede::upper_bound(const double key) {
    ziterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = z_map.upper_bound(z_key(key));
    return tmp;
  }

  //==============================================================================
  // find()
  //==============================================================================
  MGede::ziterator MGede::find(const double key) {
    ziterator tmp;
    tmp.begin = vgede.begin();
    tmp.iter  = z_map.find(z_key(key));
    return tmp;
  }

  //==============================================================================
  //  
  // class MGedeIterator member functions
  //
  //==============================================================================
  MGedeIterator::MGedeIterator() {}

  //==============================================================================
  // operator++
  //==============================================================================
  MGedeIterator::self& MGedeIterator::operator++() {
    ++iter;
    return *this;
  }

  //==============================================================================
  // operator--
  //==============================================================================
  MGedeIterator::self& MGedeIterator::operator--() {
    --iter;
    return *this;
  }

  //==============================================================================
  // operator+ (maybe dangerous)
  //==============================================================================
  const MGedeIterator::self MGedeIterator::operator+(int n) {
    self tmp=*this;
    for(int i=0; i<n; i++)
      ++tmp;
    return tmp;
  }

  //==============================================================================
  // operator*
  //==============================================================================
  Gede& MGedeIterator::operator*() const {
    return *(begin + iter->second);
  }
  
  //==============================================================================
  // operator->
  //==============================================================================
  Gede* MGedeIterator::operator->() const {
    return &(operator*());
  }

  //==============================================================================
  //
  // class MGedeZIterator member functions
  //
  //==============================================================================
  MGedeZIterator::MGedeZIterator() {}

  //==============================================================================
  // operator++
  //==============================================================================
  MGedeZIterator::self& MGedeZIterator::operator++() {
    ++iter;
    return *this;
  }

  //==============================================================================
  // operator--
  //==============================================================================
  MGedeZIterator::self& MGedeZIterator::operator--() {
    --iter;
    return *this;
  }

  //==============================================================================
  // operator+ (maybe dangerous)
  //==============================================================================
  const MGedeZIterator::self MGedeZIterator::operator+(int n) {
    self tmp=*this;
    for(int i=0; i<n; i++)
      ++tmp;
    return tmp;
  }

  //==============================================================================
  // operator*
  //==============================================================================
  Gede& MGedeZIterator::operator*() const {
    return *(begin + iter->second);
  }

  //==============================================================================
  // operator->
  //==============================================================================
  Gede* MGedeZIterator::operator->() const {
    return &(operator*());
  }

