//!!!!!!!!!!!!!!!!!!!!!!
using namespace std;//!!
//!!!!!!!!!!!!!!!!!!!!!!
#include "arte/arte.hh"
#include "Class_MGEDE.hh"


  //==============================================================================
  // Load Gede from DB or ARTE
  //==============================================================================
  const MGede& LoadGede(void) {
    static bool firsttime = true;
    static int gedeidx[8][4][4]; // cross reference to Arte Table index
    static MGede gedeMap;

    if (ArteTable<MCIN>::booked()) {
      for(ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin(); gede != ArteTable<GEDE>::end(); ++gede) {
	if(gede->cmp == 1) {
	  Gede vdsgede(*gede);
	  gedeMap.push_back(vdsgede);
	}
      }
      return gedeMap;
    }


    if(firsttime) {
      for(int i=0; i<8; i++)
	for(int j=0; j<4; j++)
	  for(int k=0; k<4; k++)
	    gedeidx[i][j][k] = 0;

      // fill artegedeidx array
      for(ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin(); gede !=
	    ArteTable<GEDE>::end(); ++gede) {
	if(gede->cmp != 1) continue;
	
	int sl = atoi( gede->gesl->name+2 );
	gedeidx[sl-1][gede->lay-1 ][gede->sec-1] = (*gede).index();
      }
      
      firsttime = false;
    }
    
    // fill GEDE table with geometry from file
    set<int> dbgedes;
    for ( LedaTableIterator<class VDS_Side> side; side.valid(); side.next() ) {
      int gedeid = gedeidx[atoi(side->slname+2)-1][side->lay-1][side->sec-1];

      //ignore dead sides
      if(side->status == 0) {
	cout << "VDS side (" << side->id << ") is dead, ignoring it" << endl;
	dbgedes.insert(gedeid);
	gedeMap.push_back(gedeid); //store gede-idx of dead planes
	continue;
      }

      if(gedeid == 0) {
	cout << "VDS side (" << side->id << ") not found in GEDE Table (expected GEDE row " 
	     << gedeid << ")" << endl;
      }

      Gede cgede(side, gedeid);
      if(firsttime) {
	gedeMap.push_back(cgede);
      } else {
	if(gedeMap.find(gedeid) == gedeMap.end())
	  gedeMap.push_back(cgede);
	else
	  gedeMap[gedeid] = cgede;
      }
      dbgedes.insert(gedeid);
    }

    // print out missing Gede table rows
    for(ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin(); gede !=
	  ArteTable<GEDE>::end(); ++gede) {
      if(gede->cmp != 1)
	continue;
      if(dbgedes.find((*gede).index()) == dbgedes.end()) {
	cout.setf(ios::right,ios::adjustfield);
	cout << "----> No database entry for GEDE Table row " 
	     << setw(3)  << (*gede).index() 
	     << " (SL="  << gede->gesl->name 
	     << ", sec=" << gede->sec
	     << ", lay=" << gede->lay << ")" << endl;
      }
    }

    return gedeMap;
  }

