// 
//  Authors:  Silvia Masciocchi                Created:  05-Feb-99 ver. 1.01
//            Ivan Kisel                       Modified: 07-Jun-99 ver. 2.01
//            Max-Planck-Institut fuer Physik            01-Jul-99 ver. 2.02 
//            Munchen                                    03-Jul-99 ver. 2.03 
//  Email:    sma@mppmu.mpg.de                           14-Jul-99 ver. 2.04
//            ikisel@mppmu.mpg.de                        30-Jul-99 ver. 2.05
//            HERA-B Collaboration                       01-Sep-99 ver. 2.06
//                                                       19-Sep-99 ver. 3.01
//                                                       07-Oct-99 ver. 3.02
//                                                       07-Oct-99 ver. 3.03
//                                                       12-Oct-99 ver. 3.04
//                                                       25-Oct-99 ver. 3.05
//                                                       05-Nov-99 ver. 3.06
//                                                       18-Nov-99 ver. 3.07
//                                                       01-Dec-99 ver. 3.08
//                                                       11-Dec-99 ver. 3.09
//                                                       17-Dec-99 ver. 3.10
//                                                       12-Jan-00 ver. 3.11
//                                                       23-Jan-00 ver. 3.12
//
// cats.c
// VDS reconstruction program - main part, uses fill_banks.C .
// The algorithm is based on the cellular automaton NIM A 387 (1997) 433-442. 
//
// Version 3.xx does reconstruction of tracks from space points.
//
 
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//#define f2cFortran

#include <arte/Arte.h>
#include <arte/ArteUtil.h>

#include <auto/MTRA.h>
#include <auto/MIMP.h>
#include <auto/HITB.h>
#include <auto/GEDE.h>
#include <auto/GESL.h>
#include <auto/GTAR.h>
#include <auto/RTRA.h>
#include <auto/TOPP.h>
#include <auto/MVER.h>
#include <auto/RVER.h>

#include "cats/hbook.h"
#include "cats/higz.h"

//#include "cats/cfortran.h"
#include "cats/cats.h"

//#define TARGET

#define NCHAMBERS  10
#define NPLANES   200
#define NPROJ       4
#define NTARGETS    8
#define MAXNUMBEROFHITS     20000
#define MAXNUMBEROFPOINTS   20000
#define MAXNUMBEROFSEGMENTS 20000
#define MAXNUMBEROFTRACKS   10000
#define maxcomb             400000
#define MAXNUMBEROFVERTICES 100
#define MAXHITSPERPLANE     100

extern int acglobaltolocal(float *, float *, int);
extern int EventFilter(void);
extern void fill_GEDE_array(void);
extern void fill_banks(int, int *, 
		       float *, float *, float *, float *, float *, 
		       float *, float *, float *, float *, float *, 
		       float *, float *, int *);
extern int  fill_mimp_points(int, float *, float *, float *);
extern void dinv_(int* n, double* da, int* m, int* work, int* ifail);
extern float prob_(float *, int *);
extern void time_start();
extern void time_stop();
#ifdef TARGET
extern void garbage_event(int, int *, float, float, float);
#endif

static int Black =  1, 
    DarkGray     =  2, 
    Blue         =  3, 
    Green        =  4, 
    Cyan         =  5, 
    Red          =  6, 
    Magenta      =  7, 
    Brown        =  8, 
    LightBlue    =  9, 
    LightGreen   = 10, 
    LightCyan    = 11, 
    LightRed     = 12, 
    LightMagenta = 13, 
    Yellow       = 14, 
    LightGray    = 15, 
    White        =  0;

       void fill_cats();
static void Draw_Event();
static void graph_ini();
static void graph_close();
static void KalmanRefit();
#ifdef XXX
static void TrackRefit();
#endif
static void Space_Points();
static void Space_Automaton();
static void Space_Track_Finder();
static void Space_Find_Track_Chi_2(int n, int k);
#ifdef TARGET
static void CatsTargets(int *, float *, float *, float *, float *, int *, int *);
#endif


                 /* distances in cm */


float z_nsl[NCHAMBERS+1];              /* z-pos of the 1st plane      */


static float xpl[1000], ypl[1000];
static int   n_mimps;
static float mimp_x[100], mimp_y[100], mimp_z[100];

static int   nchambers, ichamber[NCHAMBERS+1];/* real number of superlayers  */
                                              /*and numbers in the GESL table*/
static int   nplanes;
//////////////////////////////////////////////////////////////////////////
int   Nsl_Sec[NCHAMBERS+1][5][5];      /* plane from nsl, sect, proj  */
//////////////////////////////////////////////////////////////////////////

static int   ntargets,  itarget[NTARGETS+1];  /* numbers of targets wires    */

static float Z_TARGET;                        /* mean target z-position      */
static float GTAR_x[NTARGETS+1], GTAR_y[NTARGETS+1], GTAR_z[NTARGETS+1];

static int   fh[NPLANES], lh[NPLANES], fhc[NPLANES], lhc[NPLANES];
static int   npl; 

/////////////////////////////////////////
int   npr[NPLANES];
float zpl[NPLANES], coeff[NPLANES];

int   GEDE_cmp[NPLANES],  GEDE_nsl[NPLANES],
  GEDE_sec[NPLANES],  GEDE_lay[NPLANES];
float GEDE_sxmn[NPLANES], GEDE_sxmx[NPLANES];
float GEDE_symn[NPLANES], GEDE_symx[NPLANES];
float GEDE_cos[NPLANES],  GEDE_sin[NPLANES]; 
float GEDE_xfw[NPLANES],  GEDE_nwi_x_pit[NPLANES];

int   cell_f_tmp[201][51], cell_l_tmp[201][51];
int   cell_f[201][51], cell_l[201][51];
/////////////////////////////////////////
static int   ihit[MAXNUMBEROFHITS+1];
static int   ifl[MAXNUMBEROFHITS+1], jfl[MAXNUMBEROFHITS+1];
static int   ihc[MAXNUMBEROFHITS+1], jhc[MAXNUMBEROFHITS+1];

static int   h_t_pointer[MAXNUMBEROFHITS+1]; // hit to track pointer

/////////////////////////////////////////////
float HITB_x[MAXNUMBEROFHITS+1];    /* original value                */
////////////////////////////////////////////

static float X_Hit[MAXNUMBEROFHITS+1];     /* coordinate ???                */
static float Z_Hit[MAXNUMBEROFHITS+1];     /* coordinate ???                */
static int   L_Hit[MAXNUMBEROFHITS+1];     /* layer                         */
static int   M_Hit[MAXNUMBEROFHITS+1];     /* which part of a layer 1,2,3,4 */
static int   IPL_Hit[MAXNUMBEROFHITS+1];   /* plane number of a hit         */
static int   Used_Hit[MAXNUMBEROFHITS+1];  /* 1 - used, 0 - not             */

static int   N_Points;           /* total number of points */
static int   NC_Points;          /* total number of unused points */

static float X_Point[MAXNUMBEROFPOINTS+1];     /* x-coordinate               */
static float Y_Point[MAXNUMBEROFPOINTS+1];     /* y-coordinate               */
static float Z_Point[MAXNUMBEROFPOINTS+1];     /* z-coordinate               */
static int   NSL_Point[MAXNUMBEROFPOINTS+1];   /* layer                      */
static int   SEC_Point[MAXNUMBEROFPOINTS+1];   /* detector number - see M_Hit*/
static int   Hit_Point[MAXNUMBEROFPOINTS+1][5];/* pointers to user hits      */
static int   N_Point[MAXNUMBEROFPOINTS+1];     /* number of used hits        */
static float E_Point[MAXNUMBEROFPOINTS+1];     /* error of reconstruction    */
static int   Good_Point[MAXNUMBEROFPOINTS+1];  /* to remove bad combinations */
static int   IC_Point[MAXNUMBEROFPOINTS+1];    /* index for sorting          */

static int   TrackPoint[MAXNUMBEROFPOINTS+1];  /* free or on a track         */
static int   FirstPoint[MAXNUMBEROFPOINTS+1];  /* first point on a track     */
static int   OnTheTrack[MAXNUMBEROFPOINTS+1];  /* track number               */


static int   N_Segments, Total_N_Segments;

static float X_Start[MAXNUMBEROFSEGMENTS+1],   X_End[MAXNUMBEROFSEGMENTS+1];
static float Y_Start[MAXNUMBEROFSEGMENTS+1],   Y_End[MAXNUMBEROFSEGMENTS+1];
static float Z_Start[MAXNUMBEROFSEGMENTS+1],   Z_End[MAXNUMBEROFSEGMENTS+1];

static int   FirstSegment[MAXNUMBEROFHITS+1], LastSegment[MAXNUMBEROFHITS+1];
static int   TrackHit[MAXNUMBEROFHITS+1]; // free or on a track at the depth level


/*
The first and the last segments starting at a hit and ending at
another hit on the next plane. Index is the number of the hit.
*/

/* Arrays describing segments. Index is the number of the segment.  */


static int   StartOfSegment[MAXNUMBEROFSEGMENTS+1], 
      EndOfSegment[MAXNUMBEROFSEGMENTS+1];


/* Starting and ending hits of a segment. */

static int   First_Segment[MAXNUMBEROFSEGMENTS+1], 
      Last_Segment[MAXNUMBEROFSEGMENTS+1];

/*
The first and the last segments starting at the ending hit of a
segment and making a small [ < arccos(COSPHIMAX) ] angle with the segment.
I: I would prefer FirstNeighbor & LastNeighbor instead.
*/

static int   HasNeighbour[MAXNUMBEROFSEGMENTS+1];
static int   LevelOfSegment[MAXNUMBEROFSEGMENTS+1];
static int   CorrectIndex[MAXNUMBEROFSEGMENTS+1];

static int   StartOnPlane[NCHAMBERS+1], EndOnPlane[NCHAMBERS+1];
/*
The first and the last hit on a plane. Index is the number of the plane.
(!) StartOnPlane>=EndOnPlane.
*/

static int   SetNo;

static int   PlaneSet[10][10]={{8,1,2,3,4,5,6,7,8},{4,1,2,4,5},{4,1,3,4,5}};
//static int   CoPlaneSet[10][10]={{0},{1,3},{1,2}};

static int   NumberOfTracks, Level;
static int   LengthOfTrack[MAXNUMBEROFTRACKS+1];
static int   GoodTrack[MAXNUMBEROFTRACKS+1];
static float Chi_2[MAXNUMBEROFTRACKS+1];
static float Qual[MAXNUMBEROFTRACKS+1];
static int   idx_tr[MAXNUMBEROFTRACKS+1];
//static int   n_sector[MAXNUMBEROFTRACKS+1];

static int   CurrentTrackStart[NCHAMBERS+1];
static int   TrackStart[MAXNUMBEROFTRACKS+1][NCHAMBERS+1];

static float x_start_track, y_start_track, z_start_track;
//static float x_end_track,   y_end_track,   z_end_track;

static float X_Start_Track[MAXNUMBEROFTRACKS+1];
static float Y_Start_Track[MAXNUMBEROFTRACKS+1];
static float Z_Start_Track[MAXNUMBEROFTRACKS+1];
static float X_End_Track[MAXNUMBEROFTRACKS+1];
static float Y_End_Track[MAXNUMBEROFTRACKS+1];
static float Z_End_Track[MAXNUMBEROFTRACKS+1];


static int   Call;

static float SHIFT      =  10.0;
static float shift;
////////////////////////////////////////////////////
int   base_plane[5] = {0, 1, 11, 2, 7};
////////////////////////////////////////////////////
static int   shift_coeff[5] = {0, 3, 1, -1, -3};

static int   Ntk, nh_trk[MAXNUMBEROFTRACKS+1];
static float Xp_trk[MAXNUMBEROFTRACKS+1], X0_trk[MAXNUMBEROFTRACKS+1];
static float Yp_trk[MAXNUMBEROFTRACKS+1], Y0_trk[MAXNUMBEROFTRACKS+1];
static float Chi2_trk[MAXNUMBEROFTRACKS+1], Pinv_trk[MAXNUMBEROFTRACKS+1];
static float Xpf_trk[MAXNUMBEROFTRACKS+1], X0f_trk[MAXNUMBEROFTRACKS+1];
static float Ypf_trk[MAXNUMBEROFTRACKS+1], Y0f_trk[MAXNUMBEROFTRACKS+1];
static float Cf_trk[10*(MAXNUMBEROFTRACKS+1)];
static float Xpe_trk[MAXNUMBEROFTRACKS+1], X0e_trk[MAXNUMBEROFTRACKS+1];
static float Ype_trk[MAXNUMBEROFTRACKS+1], Y0e_trk[MAXNUMBEROFTRACKS+1];
static float Ce_trk[10*(MAXNUMBEROFTRACKS+1)];
static int   hits[MAXNUMBEROFHITS+3*MAXNUMBEROFTRACKS+1];

static int   nhits, nfound, hh[NPLANES];
static int   gfound, gg[NPLANES];


static int   MINLEVEL    = 3;
static int   n_at_least  = 5;
static float TARGETSCALE = 10.0;

static float DOUBLETARGETCUT;// = 2.0*TARGETREGION;
#ifdef TARGET
static float VERTEXSIZE  = 0.0;
static float TARGETSIZE  = 0.0;
static float ZSTATION1   = 0.0;
static float ZSTATION2   = 0.0;
#endif
static float MINDIST;        /* distance for hit searching around a track  */
static float TARGETREGION;   /* half of a target window                    */
static int   ind_pl, ind_ba; /* plane to calculate det. efficiency         */
static int   DRAW;           /* switch on the display                      */

static int EventNumber = 0;
static int back[NPLANES];

static int neve, ntr_pl[26], found_pl[26], found2_pl[26], found3_pl[26];
static int chip, ntr_ch[26][10], found3_ch[26][10]; 


static float x_station[3], y_station[3], z_station[3];
static int   n_station[3];
static float X1, Y1, Z1, X2, Y2, Z2;


/* structure for IOL */
Cats_Iol_t cats_iol = {
  "cats"                  , IOL_NVERBS(Cats_Iol_t,0),
  {"search window"        ,0.0100            ,  " " , IOL_TYPE_FLOAT},
  {"target window"        ,0.3               ,  " " , IOL_TYPE_FLOAT},
  {"det. plane off"       ,0                 ,  " " , IOL_TYPE_INT},
  {"graphics on"          ,0                 ,  " " , IOL_TYPE_FLAG},

};

static int   cats_first = 0;


void cats_param_init()
{
  /* initialize iol parameters of silarte package */
  iol_declare_structure((IOL_GENERIC *) &cats_iol);

}


void  CatsEvent(){
  int   ih, nh, ifh, ilh, ipl, ifpl, ilpl, ich, nch, ic, ihfirst;
  float xt, yt, zt;
#ifdef TARGET  
  int   n_vertices, wire_vertex[MAXNUMBEROFVERTICES+1], 
    ntr_vertex[MAXNUMBEROFVERTICES+1];
  float x_vertex[MAXNUMBEROFVERTICES+1], y_vertex[MAXNUMBEROFVERTICES+1],
    z_vertex[MAXNUMBEROFVERTICES+1], w_vertex[MAXNUMBEROFVERTICES+1];
#endif

  int i,itr, nsl_pl[11],nsl_found, itar;
  int trn,trn_c[11],f1s,f2s,f3s,f3s_c[11];
  int stripn;
  float x,xloc,xx,yy,dx,dy,MinDist,res, TargetMinDist;
  int nsl_hit[3], ih1, ipl1, ih2, ipl2, ihhf, ihhl, ipr, ibpl;
#ifdef XXX
  float x1, x2, s1, c1, s2, c2, det, A, B;
#endif
  float local[5], global[5];
  int   ista;

  MINDIST      = cats_iol.search_window.value;
  TARGETREGION = cats_iol.target_window.value;
  ind_pl       = cats_iol.gede_plane_off.value;
  DRAW         = cats_iol.draw_display.value;

  if (cats_first==0){
    cats_first = 10;
    fill_cats();
    cats_first = 1;
  }

  EventNumber++;
  //if (EventNumber<70) return;

  
  /* check quality of real data */
  if(!EventFilter()) return;
    

  /* recall the target wires for each event */
  ntargets = 0;
  Z_TARGET = 0.0;
  z_station[1] = 0.0; x_station[1] = 0.0; y_station[1] = 0.0; n_station[1] = 0;
  z_station[2] = 0.0; x_station[2] = 0.0; y_station[2] = 0.0; n_station[2] = 0;
  global[3] = 0.; global[4] = 0.; /* for transformation of target wires as space points */

  for(ih=1; ih <=NGTAR(); ih++){
    if (GTAR(ih).wnr!=0){
      xt = GTAR(ih).x;
      yt = GTAR(ih).y;
      zt = GTAR(ih).z;

      if (xt*xt+yt*yt<1.0){
	ntargets++;
        itarget[ntargets] = ih;
	Z_TARGET = Z_TARGET + zt;

	global[0] = xt; global[1] = yt; global[2] = zt;
	acglobaltolocal(local, global, 1);

	GTAR_x[ntargets] = local[0];
	GTAR_y[ntargets] = local[1];
	GTAR_z[ntargets] = local[2];

	if (zt<-3.0) 
	  ista = 1;
	else 
	  ista = 2;

	n_station[ista]++;
	x_station[ista]+= local[0];
	y_station[ista]+= local[1];
	z_station[ista]+= local[2];

      }
    }
    else
      goto no_more_targets;
  }
no_more_targets:;  

  if (ntargets!=0){
    Z_TARGET = Z_TARGET/ntargets;
    if (n_station[1]!=0){
      x_station[1] = x_station[1]/n_station[1];
      y_station[1] = y_station[1]/n_station[1];
      z_station[1] = z_station[1]/n_station[1];
    }
    else
      z_station[1] = -5.0;
    if (n_station[2]!=0){
      x_station[2] = x_station[2]/n_station[2];
      y_station[2] = y_station[2]/n_station[2];
      z_station[2] = z_station[2]/n_station[2];
    }
    else
      z_station[2] = -1.0;
  }
  else{
    printf("CATS : ******                  NO TARGETS                  ******\n");
    ntargets = 1;
    Z_TARGET = -4.7;
    GTAR_x[ntargets] = 0.0;
    GTAR_y[ntargets] = 0.0;
    GTAR_z[ntargets] = -4.7;
    printf("CATS : ****** Dummy target at (0.0, 0.0, -4.7) introduced ******\n");
    z_station[1] = -5.0;
    z_station[2] = -1.0;
}

  shift = SHIFT; 
  
  ind_ba = back[ind_pl];

  if(DRAW) Draw_Event();

  for(ipl=1; ipl <= nplanes; ipl++){ 
    fh[ipl] = 0;
    lh[ipl] =-1;
    fhc[ipl]= 0;
    lhc[ipl]=-1;
    for(ic=0; ic <= 50; ic++){ 
      cell_f[ipl][ic]= 100000;
      cell_l[ipl][ic]=-100000;
      cell_f_tmp[ipl][ic]= 100000;
      cell_l_tmp[ipl][ic]=-100000;
    }
  }


  npl = 0;
  nh = 0;
  for(ich=1; ich <= nchambers; ich++){
    nch = ichamber[ich];
    ifpl = GESL(nch).fpl;
    ilpl = ifpl + GESL(nch).npl;
    for(ipl=ifpl; ipl < ilpl; ipl++){ 
      if (ipl==ind_pl || 
	  ipl==ind_ba) continue; /* hide tested plane and its back*/
#ifdef XXX

      if (ipl==ind_pl) continue; /* hide tested plane and its back*/

      if (ind_pl!=0){
	if ((GEDE_nsl[ipl] == GEDE_nsl[ind_pl])&&
	    (GEDE_sec[ipl] == GEDE_sec[ind_pl]))
	  continue;
      }
#endif

      ifh = GEDE(ipl).hit1;
      ilh = GEDE(ipl).hitl;
      if (ilh-ifh>=MAXHITSPERPLANE)
	printf("CATS : Noise: VDS plane %3d with %d hits excluded\n", ipl, (ilh-ifh));
      if ((ilh>=ifh)&&(ilh-ifh<MAXHITSPERPLANE)){ 
        /*
	  fh[ipl] = nh+1;
	  lh[ipl] = nh+1+(ilh-ifh);
	  fhc[ipl] = nh+1;
	  lhc[ipl] = nh+1+(ilh-ifh);
	  if (ipl > npl) npl = ipl;
	  */
        if (nh+1+(ilh-ifh)>MAXNUMBEROFHITS){
	  printf("CATS : *** MXNUMBEROFHITS reached %d ***\n", nh);
	  return;
	}
        ihfirst = 0;
        for(ih=ifh; ih <= ilh; ih++){ 
          if (HITB(ih).gede!=ipl){
            printf("CATS : HITB problem on plane %3d : hitb->gede %3d, hitb->z %5.2f, hitb->x %5.2f\n",  
                   ipl, HITB(ih).gede, HITB(ih).z, HITB(ih).x);
            continue;
          }

          nh++;
          
          if (ihfirst == 0){
            fh[ipl]  = nh;
            fhc[ipl] = nh;
            ihfirst = 1;
          }
          lh[ipl]  = nh;
          lhc[ipl] = nh;
          if (ipl > npl) npl = ipl;
          
	  ihit[nh] = ih;
	  ihc[nh]  = nh;
	  jhc[nh]  = nh;
	  ifl[nh]  = 0;
	  jfl[nh]  = 0;
	  h_t_pointer[nh] = 0;

	  HITB_x[nh] = HITB(ih).x;
	  X_Hit[nh]  = coeff[ipl]*HITB(ih).x;
	  Z_Hit[nh]  = HITB(ih).z;
	  L_Hit[nh]  = GEDE_nsl[ipl];
	  M_Hit[nh]  = GEDE_sec[ipl];
	  IPL_Hit[nh]= ipl;

	  Used_Hit[nh] = (0==1);
	  TrackHit[nh] = 0;
	  FirstSegment[nh]= 0;
	  LastSegment[nh] =-1;

	  ic = 50.0*((HITB_x[nh]-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	  
	  if ((ic<0)||(ic>50)) continue;
	  if (cell_f[ipl][ic] == 100000){
	    cell_f[ipl][ic]     = nh;
	    cell_f_tmp[ipl][ic] = nh;
	  }
	  cell_l[ipl][ic]     = nh+1;
	  cell_l_tmp[ipl][ic] = nh+1;

	}

	if (cell_f_tmp[ipl][1]<cell_f[ipl][0])
	  cell_f[ipl][0]=cell_f_tmp[ipl][1];
	if (cell_l_tmp[ipl][1]>cell_l[ipl][0])
	  cell_l[ipl][0]=cell_l_tmp[ipl][1];
	
	for(ic=1; ic <= 49; ic++){
	  if (cell_f_tmp[ipl][ic-1]<cell_f[ipl][ic])
	    cell_f[ipl][ic]=cell_f_tmp[ipl][ic-1];
	  if (cell_f_tmp[ipl][ic+1]<cell_f[ipl][ic])
	    cell_f[ipl][ic]=cell_f_tmp[ipl][ic+1];
	  if (cell_l_tmp[ipl][ic+1]>cell_l[ipl][ic])
	    cell_l[ipl][ic]=cell_l_tmp[ipl][ic+1];
	  if (cell_l_tmp[ipl][ic-1]>cell_l[ipl][ic])
	    cell_l[ipl][ic]=cell_l_tmp[ipl][ic-1];
	}
	
	if (cell_f_tmp[ipl][49]<cell_f[ipl][50])
	  cell_f[ipl][50]=cell_f_tmp[ipl][49];
	if (cell_l_tmp[ipl][49]>cell_l[ipl][50])
	  cell_l[ipl][50]=cell_l_tmp[ipl][49];

      }
    }
  }



  //  Space_Points();
  //time_start();
  Space_Track_Finder();
  //time_stop();

  //time_start();
  fill_banks(Ntk, nh_trk, 
	     Xpf_trk, X0f_trk, Ypf_trk, Y0f_trk, Cf_trk,
	     Xpe_trk, X0e_trk, Ype_trk, Y0e_trk, Ce_trk,
	     Chi2_trk, Pinv_trk, hits);
  //time_stop();


  /*******************  EFFICIENCY CODE  ********************/

  if (ind_pl!=0){

    printf("Number of tracks found: %d\n", Ntk);

    xx=0.0; yy=0.0;
    if (Ntk<=100) {
      neve++;
      nh = 0;
      for (itr=0;itr<Ntk;itr++){
        
        stripn=0.;
        for(ih=1; ih <= 10; ih++){ 
          nsl_pl[ih]=0;
        }

	nsl_found = 0;
	ihhf = 0;
	ihhl = 0;
	ipl1 = 0;
	ipl2 = 0;
        for (ih=nh;ih<nh+nh_trk[itr];ih++){
	  if (ihhf == 0) 
	    ihhf = ih;
	  ihhl = ih;
	  if (ipl1 == 0)
	    ipl1 = HITB(hits[ih]).gede;
	  ipl2 = HITB(hits[ih]).gede;

          nsl_pl[GESL(GEDE(HITB(hits[ih]).gede).ogesl).nsl]++;
	  if (GESL(GEDE(HITB(hits[ih]).gede).ogesl).nsl==
	      GESL(GEDE(ind_pl).ogesl).nsl)
	    if (GEDE(HITB(hits[ih]).gede).sec==
		GEDE(ind_pl).sec){
	      nsl_found++;
	      nsl_hit[nsl_found] = ih;
	    }
        }
        nh=nh+nh_trk[itr]+1; /* skip (-1): hits of diff. tracks are */
	                     /* separated by (-1) */
				
	if (nh_trk[itr]==5) continue;

	if ((ind_pl < ipl1)||(ind_pl > ipl2)) continue;
	/* crossing the target wires? */

	TargetMinDist = 10.0*TARGETREGION;
	for (itar=1; itar<=ntargets; itar++){
	  xx=X0_trk[itr]+Xp_trk[itr]*GTAR_z[itar];
	  yy=Y0_trk[itr]+Yp_trk[itr]*GTAR_z[itar];
	  if ( fabs(xx-GTAR_x[itar])+fabs(yy-GTAR_y[itar])<TargetMinDist )
	    TargetMinDist = fabs(xx-GTAR_x[itar])+fabs(yy-GTAR_y[itar]);
	}
	
				  
	dx = 0.0;
	dy = 0.0;
	/* very probably out of the detector active area */
//        if (nsl_found<1) continue;

#ifdef XXX
 	ih1 = hits[nsl_hit[1]];
	ipl1 =  HITB(ih1).gede;

	xx=X0_trk[itr]+Xp_trk[itr]*zpl[ipl1];
	yy=Y0_trk[itr]+Yp_trk[itr]*zpl[ipl1]; 

	x1 = X0_trk[itr]*GEDE_cos[ipl1] - Y0_trk[itr]*GEDE_sin[ipl1];
	x2 = xx*GEDE_cos[ipl1] - yy*GEDE_sin[ipl1];
	
	A = (x2-x1)/zpl[ipl1]; /* slope in proj. ipl1 */
	B = x2-A*zpl[ipl1];
	
 	dx = A*HITB(ih1).z+B-HITB(ih1).x;
	
	ih2 = hits[nsl_hit[2]];
	ipl2 = HITB(ih2).gede;

	x1 = A*HITB(ih2).z+B-dx;
	x2 = HITB(ih2).x;
	
	c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
	c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
	det=c1*s2-c2*s1;
	
	/* track correction at the superlayer */
	xx = (x1*s2-s1*x2)/det; 
	yy =-(x1*c2-c1*x2)/det; 
	zz = HITB(ih2).z; 

	dx = xx - (X0_trk[itr]+Xp_trk[itr]*zz);
	dy = yy - (Y0_trk[itr]+Yp_trk[itr]*zz);
	
	printf("Track correction %f %f\n", dx, dy);
#endif

	nsl_found = 0;
        for(ih=1; ih <= 10; ih++){ 
          //printf("Superlayer %d, nsl_pl=%d \n",ih,nsl_pl[ih]);
          if (nsl_pl[ih]!=0) nsl_found++;
        }
	
	if (Chi2_trk[itr]>0.5) continue;
	if (nsl_found>=3){
          //printf("Track %2d. Enough superlayers, do the rest !!!! \n",itr);
          xx=X0_trk[itr]+Xp_trk[itr]*zpl[ind_pl] + dx;
          yy=Y0_trk[itr]+Yp_trk[itr]*zpl[ind_pl] + dy; 
          if ( xx > GEDE_sxmn[ind_pl]+0.2 &&
               xx < GEDE_sxmx[ind_pl]-0.2 &&
               yy > GEDE_symn[ind_pl]+0.2 &&
               yy < GEDE_symx[ind_pl]-0.2 ) {
	    
	    printf("(%5.2f %5.2f) inside active (%5.2f %5.2f) (%5.2f %5.2f)\n", xx, yy, GEDE_sxmn[ind_pl]+0.2, GEDE_sxmx[ind_pl]-0.2, GEDE_symn[ind_pl]+0.2, GEDE_symx[ind_pl]-0.2);

            x = xx*GEDE_cos[ind_pl] - yy*GEDE_sin[ind_pl]; 

            stripn = (int)((x-(GEDE_xfw[ind_pl]-0.5*GEDE(ind_pl).pit))/GEDE(ind_pl).pit);

            if (stripn<=0 || stripn >= GEDE(ind_pl).nwi[0])
              printf("!!!!!!!!!!!!!!!!!!!!!BAD STRIP NUMBER !!!!!!!!!!!!!\n");
            chip=(stripn/128.)+1;
	    printf("x  %f, xfw %f dx %f  strip %d chip %d\n", x, GEDE_xfw[ind_pl], x - GEDE_xfw[ind_pl], stripn, chip);
            hfill(305,stripn,0.0,1.0);

	    if ((stripn > 0)&&(fabs(stripn*GEDE(ind_pl).pit)>0.1)){

	      if ((nh_trk[itr]>=5)&&(nh_trk[itr]<26)){
		ntr_pl[nh_trk[itr]]++;
		ntr_ch[nh_trk[itr]][chip]++;
		printf("Track %2d : Hit in the acceptance, chip %2d, and track with %2d hits and chi2 %7.3f\n",
		       itr, chip, nh_trk[itr], Chi2_trk[itr]);
		//printf("ntr_pl[%d]=%d , ntr_ch[%d][%d]=%d \n",
		//     nh_trk[itr],ntr_pl[nh_trk[itr]],nh_trk[itr],chip,
	        //     ntr_ch[nh_trk[itr]][chip]);
              } 

	      MinDist = 20.0*MINDIST;
	      res = 1000.0;

	      ifh = GEDE(ind_pl).hit1;
	      ilh = GEDE(ind_pl).hitl;
	      for(ih=ifh; ih <= ilh; ih++){ 
//	      for(ih=1; ih <= NMIMP(); ih++){ 
//	      for(ih=1; ih <= NHITB(); ih++){ 
		//if (MIMP(ih).gede != ind_pl) continue;
		//if (HITB(ih).gede != ind_pl) continue;
                //x = xx*GEDE_cos[ind_pl] - yy*GEDE_sin[ind_pl]; 
                //xx=MIMP(ih).x*GEDE_cos[ind_pl]-MIMP(ih).y*GEDE_sin[ind_pl]; 
		xloc = HITB(ih).x;

                if (fabs(x-xloc)<MinDist){
		  MinDist = fabs(x-xloc);
		  res     = x-xloc;
		}       
	      }
	      
	      if (MinDist< 5.0*MINDIST){
		if (nh_trk[itr]>=8){
		  hfill(230, res, 0.0, 1.0);
		  printf("track with 8 or more hits\n");
		}
		if (nh_trk[itr]>=10){
		  hfill(231, res, 0.0, 1.0);
		  printf("track with 10 or more hits\n");
		}
		if (nh_trk[itr]==5){ 
		  hfill(235, res, 0.0, 1.0);
		  printf("track with 5 hits\n");
		}
		if (nh_trk[itr]==6){
		  hfill(236, res, 0.0, 1.0);
		  printf("track with 6 hits\n");
		}
		if (nh_trk[itr]==7){
		  hfill(237, res, 0.0, 1.0);
		  printf("track with 7 hits\n");
		}
		if (nh_trk[itr]==8){
		  hfill(238, res, 0.0, 1.0);
		  printf("track with 8 hits\n");
		}
		if (nh_trk[itr]==9){
		  hfill(239, res, 0.0, 1.0);
		  printf("track with 9 hits\n");
		}
		if (nh_trk[itr]==10){ 
		  hfill(240, res, 0.0, 1.0);
		  printf("track with 10 hits\n");
		}
	      }
	      
	      if (MinDist< MINDIST){
		if ((nh_trk[itr]>=5)&&(nh_trk[itr]<26)){
		  found_pl[nh_trk[itr]]++;
		  printf("found within 1 sigma, found_pl[%d]=%d \n",
			 nh_trk[itr],found_pl[nh_trk[itr]]);
		}
	      }                   
	      
	      if (MinDist< 2.0*MINDIST){
		if ((nh_trk[itr]>=5)&&(nh_trk[itr]<26))
		  found2_pl[nh_trk[itr]]++;
		printf("found within 2 sigma, found2_pl[%d]=%d \n",
		       nh_trk[itr],found2_pl[nh_trk[itr]]);
	      }
	      
	      if (MinDist< 3.0*MINDIST){
		hfill(800, xx, yy, 1.0);
		hfill(802, (float)nh_trk[itr], 0.0, 1.0);
		hfill(804, TargetMinDist, 0.0, 1.0);
		if ((nh_trk[itr]>=5)&&(nh_trk[itr]<26)){
		  found3_pl[nh_trk[itr]]++;
		  printf("found within 3 sigma, found3_pl[%d]=%d \n",
			 nh_trk[itr],found3_pl[nh_trk[itr]]);
		  /*hfill(300,xx,yy,1.0);  NOT BOOKED!*/
		  hfill(302,stripn,0.0,1.0);
		  /*printf("Strip number: %f,  chip number: %d",stripn,chip);*/
		  found3_ch[nh_trk[itr]][chip]++;
		  printf(" +%d", nh_trk[itr]); /* hit found */
		}
		else{
		  printf(" -%d", nh_trk[itr]); /* hit not found */ 
		}
	      }
	      else{
		/*hfill(301,xx,yy,1.0);  NOT BOOKED!*/
		hfill(303,stripn,0.0,1.0);
		hfill(801, xx, yy, 1.0);
		hfill(803, (float)nh_trk[itr], 0.0, 1.0);
		hfill(805, TargetMinDist, 0.0, 1.0);
		printf("---------------------->\n");
		printf("----------------------> Hit not found\n");
		printf("---------------------->\n");

		if (DRAW){
		  isplci(Red);
		  
		  for(ipr=1; ipr <= 4; ipr++){
		    shift = shift_coeff[ipr]*SHIFT;
		    ibpl = base_plane[ipr];
		    ih1 = hits[ihhf];
		    ipl1 =  HITB(ih1).gede;

		    ih2 = hits[ihhl];
		    ipl2 =  HITB(ih2).gede;

		    printf("shift %f ibpl %d, planes %d - %d\n", shift, ibpl, ipl1, ipl2);

		    xx=X0_trk[itr]+Xp_trk[itr]*HITB(ih1).z;
		    yy=Y0_trk[itr]+Yp_trk[itr]*HITB(ih1).z;
		    
		    xpl[0] = HITB(ih1).z;
		    ypl[0] = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]+shift;

		    printf("draw1 x,y,z %f %f %f \n", HITB(ih1).z, xx, yy);
		    
		    xx=X0_trk[itr]+Xp_trk[itr]*HITB(ih2).z;
		    yy=Y0_trk[itr]+Yp_trk[itr]*HITB(ih2).z;
		    
		    xpl[1] = HITB(ih2).z;
		    ypl[1] = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]+shift;
		    
		    printf("draw2 x,y,z %f %f %f \n", HITB(ih2).z, xx, yy);
		    ipl(2,xpl,ypl);
		    iuwk(1,1);
		  }	
		}



	      }           
	    }
	  }
	}
      }
      
      printf("\n");

    }

    printf("==> ind_pl %d : \n", ind_pl);
    for (ih=5;ih<26;ih++){
      if (ntr_pl[ih]!=0)
        printf(" %d-(%d)-%f/%f/%f \n", ih,ntr_pl[ih],
               ((float)found_pl[ih]/ntr_pl[ih]),
               ((float)found2_pl[ih]/ntr_pl[ih]),
               ((float)found3_pl[ih]/ntr_pl[ih]));
      else
        printf(" %d-(%d)-________ \n", ih,ntr_pl[ih]);
    }
    printf("\n");
    
    trn=0;f1s=0;f2s=0;f3s=0;
    for (i=1;i<=10;i++){
      trn_c[i]=0;f3s_c[i]=0;
    }

    //printf("trn %d, f1s %d,f2s %d, f3s %d\n",trn,f1s,f2s,f3s);
    for (ih=8;ih<26;ih++){
      //printf("nhits %d: ntr_pl %d, found_pl %d, found2_pl %d, found3_pl %d \n",
      //     ih, ntr_pl[ih], found_pl[ih], found2_pl[ih],found3_pl[ih]); 
      trn = trn + ntr_pl[ih];
      f1s = f1s + found_pl[ih];
      f2s = f2s + found2_pl[ih];
      f3s = f3s + found3_pl[ih];
      //printf("nhits %d: trn %d, f1s %d,f2s %d, f3s %d \n",ih,trn,f1s,f2s,f3s);
    }
    for (i=1;i<=10;i++){
      for (ih=8;ih<26;ih++){
	trn_c[i] = trn_c[i] + ntr_ch[ih][i];
	f3s_c[i] = f3s_c[i] + found3_ch[ih][i];
      }
    }
    if (trn!=0){
      printf("Events used: %d \n",neve);
      printf("Tracks with >=8 hits (%d): %f / %f / %f \n",
	     trn,((float)(f1s)/(float)(trn)),
	     ((float)(f2s)/(float)(trn)),((float)(f3s)/(float)(trn)));
      for(ih=1;ih<=10;ih++){
        if(trn_c[ih]!=0){
          printf("   chip %d-(%d), 3 sigma: %f \n",ih, trn_c[ih],
		 ((float)(f3s_c[ih])/(float)(trn_c[ih])));
	}
        else{
          printf("   chip %d- NO ENTRIES! \n",ih);
        }
      }
    }

    // Try using >= 10 hits

    trn=0;f1s=0;f2s=0;f3s=0;
    for (i=1;i<=10;i++){
      trn_c[i]=0;f3s_c[i]=0;
    }

    for (ih=10;ih<26;ih++){
      trn = trn + ntr_pl[ih];
      f1s = f1s + found_pl[ih];
      f2s = f2s + found2_pl[ih];
      f3s = f3s + found3_pl[ih];
    }
    for (i=1;i<=10;i++){
      for (ih=10;ih<26;ih++){
	trn_c[i] = trn_c[i] + ntr_ch[ih][i];
	f3s_c[i] = f3s_c[i] + found3_ch[ih][i];
      }
    }
    if (trn!=0){
      printf("Tracks with >=10 hits (%d): %f / %f / %f \n",
	     trn,((float)(f1s)/(float)(trn)),
	     ((float)(f2s)/(float)(trn)),((float)(f3s)/(float)(trn)));
      for(ih=1;ih<=10;ih++){
        if(trn_c[ih]!=0){
          printf("   chip %d-(%d), 3 sigma: %f \n",ih, trn_c[ih],
		 ((float)(f3s_c[ih])/(float)(trn_c[ih])));
	}
        else{
          printf("   chip %d- NO ENTRIES! \n",ih);
        }
      }
    }

  }

  /***********************************************************/

  
#ifdef TARGET
  CatsTargets(&n_vertices, x_vertex, y_vertex, z_vertex, 
	       w_vertex, ntr_vertex, wire_vertex);

  for (ih=1; ih<=n_vertices; ih++){
    hfill(200, z_vertex[ih]-GTAR(wire_vertex[ih]).z, 0.0, 1.0);
    hfill(202, x_vertex[ih], y_vertex[ih], w_vertex[ih]);
    hfill(203, (float)ntr_vertex[ih], w_vertex[ih], 1.0);
    hfill(204,(float)n_vertices, 0.0, 1.0);
  }

  if (n_vertices==1)
    hfill(209,(float)Ntk, 0.0, 1.0);
  if (n_vertices==0)
    hfill(210,(float)Ntk, 0.0, 1.0);

  for (itr=0;itr< Ntk;itr++){      /* itr => track index */
    if (nh_trk[itr]>=10)
      hfill(720, Xp_trk[itr], Yp_trk[itr], 1.0);
  }
#endif




  return;
  
}

#ifdef TARGET
void CatsTargets(int *n_vertices, 
		 float *x_vertex, float *y_vertex, float *z_vertex, 
		 float *w_vertex, int *ntr_vertex, int *wire_vertex){

  //////////////////////////////////////////////////////////////////////////
  // Input track parameters are used from iternal cats variables.         //
  //                                                                      //
  // Output parameters:                                                   //
  //     n_vertices    - number of primary vertices found;                //
  //     x(y,z)_vertex - position of a vertex;                            //
  //     w_vertex      - weight of a vertex (effective number of tracks); //
  //     ntr_vertex    - number of tracks produced a vertex;              //
  //     wire_vertex   - wire produced a vertex                           //
  // 
  //////////////////////////////////////////////////////////////////////////

  
  float target_scale, target_shift;
  int   i_v, n_v[20], it_itr_v[20][500], itr_ptr[20][500];
  float x_v[10][500], y_v[10][500], w_v[10][500];
  float x_best, y_best, w_best, dist;
  int   ok_vertex, itr_best, it_best, j, n_vertex, iz;
  int   iver, itver[300];
  float dtver[300];
  float xpl[1000], ypl[1000];
  int   itr, it, i, ih, ista, nsta, i_v_max;
  float xx, yy, zz, x_tmp, y_tmp, z_tmp, w_tmp, www;
  int   itr_vertex[MAXNUMBEROFTRACKS+1];
  float w3[4], z3[4], zw3, zw3_tmp, ww3, w3_max, A, B;
  int   itr_cluster[MAXNUMBEROFTRACKS+1];

  /////////// for correction by hand ////////////////////////////

  float gtar_x[10], gtar_y[10], gtar_z[10]; // target parameters
  int   ntar, itar[10];                     // nuber of targets
  float gtar_z_stations[3];                 // positions of target stations  

  ntar = 0;
  for(ih=1; ih <=NGTAR(); ih++){
    if (GTAR(ih).x*GTAR(ih).x+GTAR(ih).y*GTAR(ih).y<1.0){
      ntar++;
      itar[ntar] = ih;            /* remember real target number     */
    }
    gtar_x[ih] = GTAR(ih).x;      /* local copy of targets positions */
    gtar_y[ih] = GTAR(ih).y;      /* for corrections by hand         */
    gtar_z[ih] = GTAR(ih).z;
  }


  // corrections for real data
  /*
  gtar_y[1] = GTAR(1).y - 0.00775;
  gtar_y[2] = GTAR(2).y + 0.10041;
  gtar_x[3] = GTAR(3).x + 0.07038;
  gtar_x[4] = GTAR(4).x + 0.18937;
  gtar_y[5] = GTAR(5).y + 0.03772;
  gtar_y[6] = GTAR(6).y + 0.1112;
  gtar_x[7] = GTAR(7).x + 0.08883;
  gtar_x[8] = GTAR(8).x + 0.21336;          
  */
 ////////////// for correction by hand ///////////////////////////

 if (ZSTATION1 == 0.0) 
   gtar_z_stations[1] = -5.0;          // position of the target station #1
 if (ZSTATION2 == 0.0) 
   gtar_z_stations[2] = -1.0;          // position of the target station #2

 nsta = 2;

  if (VERTEXSIZE == 0.0) 
    VERTEXSIZE = 10.0*MINDIST;
  if (TARGETSIZE == 0.0) 
    TARGETSIZE = 10.0*MINDIST;

  itr_best = 0; //dummy to avoid compiler warning
  
  if(DRAW){
    target_scale = 5.0; /* to scale target wires drawing */
    ismk(3);
    ismksc(0.5);
    
    for (it=1; it<=ntar; it++){
      /* draw the wires */
      isplci(it);
      ispmci(it);
      ih = itar[it];            /* real target number */
      if (GTAR(ih).z< 0.5*(gtar_z_stations[1]+gtar_z_stations[2]))
	target_shift = 0.0;
      else 
	target_shift = 10.0;   /* to separate stations */

      xpl[0] = target_scale*(gtar_x[ih]+target_shift);
      ypl[0] = target_scale*(gtar_y[ih]-0.5*GTAR(ih).dy);
      xpl[1] = target_scale*(gtar_x[ih]+target_shift);
      ypl[1] = target_scale*(gtar_y[ih]+0.5*GTAR(ih).dy);
      ipl(2,xpl,ypl);
      
      xpl[0] = target_scale*(gtar_x[ih]-0.5*GTAR(ih).dx+target_shift);
      ypl[0] = target_scale*(gtar_y[ih]);
      xpl[1] = target_scale*(gtar_x[ih]+0.5*GTAR(ih).dx+target_shift);
      ypl[1] = target_scale*(gtar_y[ih]);
      ipl(2,xpl,ypl);
    }

    for (ista=1; ista<=2; ista++){
      /* draw the wires */
      ispmci(Black);
      if (ista==1)
	target_shift = 0.0;
      else 
	target_shift = 10.0;   /* to separate stations */
          
      for (itr=0;itr< Ntk;itr++){  /* itr => track index */
        xx=X0_trk[itr]+Xp_trk[itr]*gtar_z[ih];
        yy=Y0_trk[itr]+Yp_trk[itr]*gtar_z[ih];
	
        xpl[0] = target_scale*(xx+target_shift);
        ypl[0] = target_scale*yy;
        ipm(1,xpl,ypl);
      }
    }
  }
  
  for (itr=0;itr< Ntk;itr++){
    itr_vertex[itr]=(1==0);   /* not from primaries (default) */
  }
  n_vertex = 0;               /* no vertices initially */
  
		  new_vertex:; /* start search for new vertex */
  
  for (it=1; it<=ntar; it++){
    ih = itar[it];
    n_v[it] = 0;                     /* it  => target index */
    i_v     = 0;
    for (itr=0;itr< Ntk;itr++){      /* itr => track index */
      if ((itr_vertex[itr]!=0)||     /* already assigned to a vertex ??? */
	  (nh_trk[itr]<5)) continue; /* long track -> not ghost */
      xx=X0_trk[itr]+Xp_trk[itr]*gtar_z[ih];
      yy=Y0_trk[itr]+Yp_trk[itr]*gtar_z[ih];
      
      ok_vertex=(1==0); /* check if the track is close to the wire */
      if (GTAR(ih).dx>GTAR(ih).dy){
	if (fabs(yy-gtar_y[ih])<TARGETSIZE) ok_vertex = (1==1);
      }
      else{
	if (fabs(xx-gtar_x[ih])<TARGETSIZE) ok_vertex = (1==1);
      }
      
      if (ok_vertex){ /* assign new track to a wire */
	i_v++;
	n_v[it]++;
	it_itr_v[it][itr] = i_v; /* pointer into a sequence */
	x_v[it][i_v] = xx;
	y_v[it][i_v] = yy;
	itr_ptr[it][i_v] = itr;  /* pointer to a track */
      }
    }  /* tracks */
  } /* targets */
  
  
  ok_vertex=(1==0);
  x_best = 0.0; y_best = 0.0; w_best = 0.0; it_best = 0;
  dist   = 0.0;
  for (it=1; it<=ntar; it++){
    for (i=1;i<=n_v[it];i++){
      w_v[it][i] = 0.0;
      for (j=1;j<=n_v[it];j++){
	/*     use Tukey's bi-weight */
	dist = sqrt((x_v[it][i]-x_v[it][j])*(x_v[it][i]-x_v[it][j])+
		    (y_v[it][i]-y_v[it][j])*(y_v[it][i]-y_v[it][j]));
	if (dist<VERTEXSIZE){
	  w_v[it][i] = w_v[it][i]+
	    (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE))*
	    (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE));
	}
      }
      if (w_v[it][i]>=w_best){
	x_best = x_v[it][i]; /* central track of a cluster */
	y_best = y_v[it][i];
	w_best = w_v[it][i];
	itr_best = itr_ptr[it][i]; /* central track of a track group */
	it_best  = it;          /* target with the biggest cluster of tracks */
	ok_vertex=(1==1);
      }
    }
  }
  
  if (ok_vertex){
    n_vertex++;              /* new primary vertex found */
    x_vertex[n_vertex] = 0.0;
    y_vertex[n_vertex] = 0.0;
    w_vertex[n_vertex] = 0.0;
    ntr_vertex[n_vertex] = 0;
    
    it  = it_best;           /* placed on this wire (among used wires!!!) */
    ih = itar[it];

    for (iz=-100;iz<=100;iz++){   /* step along z */
      zz = gtar_z[ih] + 0.01*iz;
      x_best=X0_trk[itr_best]+Xp_trk[itr_best]*zz;
      y_best=Y0_trk[itr_best]+Yp_trk[itr_best]*zz;

      for (j=1;j<=n_v[it];j++){/* loop over all tracks close to the wire */
	itr = itr_ptr[it][j];
	x_v[it][j] = X0_trk[itr]+Xp_trk[itr]*zz;
	y_v[it][j] = Y0_trk[itr]+Yp_trk[itr]*zz;
      }


      /* recalculate weights */
      for (i=1;i<=n_v[it];i++){
	w_v[it][i] = 0.0;
	for (j=1;j<=n_v[it];j++){
	  dist = sqrt((x_v[it][i]-x_v[it][j])*(x_v[it][i]-x_v[it][j])+
		      (y_v[it][i]-y_v[it][j])*(y_v[it][i]-y_v[it][j]));
	  if (dist<VERTEXSIZE){
	    w_v[it][i] = w_v[it][i]+
	      (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE))*
	      (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE));
	  }
	}
      }

      i_v = 0;                 /* number of used tracks */
      x_tmp = 0.0; y_tmp = 0.0; z_tmp = 0.0; w_tmp = 0.0;

      for (j=1;j<=n_v[it];j++){/* loop over all tracks close to the wire */
	dist = sqrt((x_best-x_v[it][j])*(x_best-x_v[it][j])+
		    (y_best-y_v[it][j])*(y_best-y_v[it][j]));
	if (dist<VERTEXSIZE){      /* cut on a cluster size             */
	  itr = itr_ptr[it][j];
	  www =
	    (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE))*
	    (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE));
	  i_v++;                     /* add new track to a vertex cluster */
	  x_tmp += www*(X0_trk[itr]+Xp_trk[itr]*zz);
	  y_tmp += www*(Y0_trk[itr]+Yp_trk[itr]*zz);
	  w_tmp += www;
	}
      } // tracks
      if (w_tmp>w_vertex[n_vertex]){
	x_vertex[n_vertex] = x_tmp/w_tmp;
	y_vertex[n_vertex] = y_tmp/w_tmp;
	z_vertex[n_vertex] = zz;
	w_vertex[n_vertex] = w_tmp;
	ntr_vertex[n_vertex]=i_v;
	wire_vertex[n_vertex]=itar[it_best];
      }
    }//zz

    if (w_vertex[n_vertex]==0.0){
      printf("CATS : *** Zeros vertex weight ***\n");
    }
    if (ntr_vertex[n_vertex]<4){
      n_vertex--;
      goto no_more_vertices;
    }

    //#ifdef XXX
    i_v = 0;
    for (j=1;j<=n_v[it];j++){/* loop over all tracks for Thomas */
      itr = itr_ptr[it][j];
      x_v[it][j] = X0_trk[itr]+Xp_trk[itr]*z_vertex[n_vertex];
      y_v[it][j] = Y0_trk[itr]+Yp_trk[itr]*z_vertex[n_vertex];
      
      dist = sqrt((x_vertex[n_vertex]-x_v[it][j])*
		  (x_vertex[n_vertex]-x_v[it][j])+
		  (y_vertex[n_vertex]-y_v[it][j])*
		  (y_vertex[n_vertex]-y_v[it][j]));
      if (dist<VERTEXSIZE){      /* cut on a cluster size             */
	itr_cluster[i_v] = itr_ptr[it][j]+1;  // RSEG number
	i_v++;
	/*
	printf("Get track %2d (%7.4f, %7.4f) (%7.4f, %7.4f)\n", 
	       itr_cluster[i_v-1], 
	       X0_trk[itr], Xp_trk[itr], Y0_trk[itr], Yp_trk[itr]);
	*/
      }
    }
    xx = x_vertex[n_vertex];
    yy = y_vertex[n_vertex];
    zz = z_vertex[n_vertex];
    //    printf("call thomas with %2d (%7.4f, %7.4f, %7.4f)\n", i_v, xx, yy, zz);  
//    garbage_event(i_v, itr_cluster, xx, yy, zz);

    if (NRVER()>0){
      printf("Reco vertex %2d (%7.4f, %7.4f, %7.4f)\n", NRVER(), 
	     RVER(NRVER()).xv, RVER(NRVER()).yv, RVER(NRVER()).zv);

      hfill(205, RVER(NRVER()).xv-xx, 0.0, 1.0);
      hfill(206, RVER(NRVER()).nt-(float)i_v, 0.0, 1.0);
      hfill(207, RVER(NRVER()).zv-zz, 0.0, 1.0);

      hfill(208, RVER(NRVER()).zv-GTAR(ih).z, 0.0, 1.0);

    }
    //#endif

    //////////////////////////////////////////////////////////////////////

    ww3 = 0.0;
    zw3_tmp = z_vertex[n_vertex];
    zw3 = zw3_tmp;

    i_v_max = 0;
    w3_max  = 0.0;
    for (iz=1;iz<=3;iz++){   /* step along z */
      zz = zw3 + 0.01*(iz-2);
      x_best=X0_trk[itr_best]+Xp_trk[itr_best]*zz;
      y_best=Y0_trk[itr_best]+Yp_trk[itr_best]*zz;
	
      for (j=1;j<=n_v[it];j++){/* loop over all tracks close to the wire */
	itr = itr_ptr[it][j];
	x_v[it][j] = X0_trk[itr]+Xp_trk[itr]*zz;
	y_v[it][j] = Y0_trk[itr]+Yp_trk[itr]*zz;
      }
      
      
      /* recalculate weights */
      for (i=1;i<=n_v[it];i++){
	w_v[it][i] = 0.0;
	for (j=1;j<=n_v[it];j++){
	  dist = sqrt((x_v[it][i]-x_v[it][j])*(x_v[it][i]-x_v[it][j])+
		      (y_v[it][i]-y_v[it][j])*(y_v[it][i]-y_v[it][j]));
	  if (dist<VERTEXSIZE){
	    w_v[it][i] = w_v[it][i]+
	      (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE))*
	      (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE));
	  }
	}
      }
      
      i_v = 0;                 /* number of used tracks */
      x_tmp = 0.0; y_tmp = 0.0; z_tmp = 0.0; w_tmp = 0.0;
      
      for (j=1;j<=n_v[it];j++){/* loop over all tracks close to the wire */
	dist = sqrt((x_best-x_v[it][j])*(x_best-x_v[it][j])+
		    (y_best-y_v[it][j])*(y_best-y_v[it][j]));
	if (dist<VERTEXSIZE){      /* cut on a cluster size             */
	  itr = itr_ptr[it][j];
	  www =
	    (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE))*
	    (1.0-(dist/VERTEXSIZE)*(dist/VERTEXSIZE));
	  i_v++;                     /* add new track to a vertex cluster */
	  x_tmp += www*(X0_trk[itr]+Xp_trk[itr]*zz);
	  y_tmp += www*(Y0_trk[itr]+Yp_trk[itr]*zz);
	  w_tmp += www;
	}
      } // tracks
      
      if (i_v> i_v_max) 
	i_v_max = i_v;
      if (w_tmp> w3_max) 
	w3_max = w_tmp;
      
      
      z3[iz] = zz;
      w3[iz] = w_tmp;
      
    }//zz
    
    if (i_v_max>=5) {
    
      A = ((w3[2]-w3[3])*(z3[1]-z3[3])-(w3[1]-w3[3])*(z3[2]-z3[3]))/
	((z3[2]*z3[2]-z3[3]*z3[3])*(z3[1]-z3[3])-
	 (z3[1]*z3[1]-z3[3]*z3[3])*(z3[2]-z3[3]));      
      B = ((w3[1]-w3[3])-A*(z3[1]*z3[1]-z3[3]*z3[3]))/(z3[1]-z3[3]);
      zw3 = -B/(2.0*A);
      
//      printf("??? ::: (%5.2f,%5.2f) (%5.2f,%5.2f) (%5.2f,%5.2f) => minimum at z = %7.4f\n", z3[1], w3[1], z3[2], w3[2], z3[3], w3[3], zw3); 
//      hfill(200, zw3-GTAR(ih).z, 0.0, (float)i_v_max);

    }
    //////////////////////////////////////////////////////////////////////


//    printf("Vertex found %d (%5.2f, %5.2f, %7.4f, %5.2f) with %3d tracks on the target %1d\n", n_vertex, x_vertex[n_vertex],  y_vertex[n_vertex], z_vertex[n_vertex],  w_vertex[n_vertex], ntr_vertex[n_vertex], wire_vertex[n_vertex]);
    

    /* mark found tracks */
    zz = z_vertex[n_vertex];
    x_best=X0_trk[itr_best]+Xp_trk[itr_best]*zz;
    y_best=Y0_trk[itr_best]+Yp_trk[itr_best]*zz;
    i_v = 0;
    for (j=1;j<=n_v[it];j++){/* loop over all tracks close to the wire */
      itr = itr_ptr[it][j];
      xx = X0_trk[itr]+Xp_trk[itr]*zz;
      yy = Y0_trk[itr]+Yp_trk[itr]*zz;
      dist = sqrt((x_best-xx)*(x_best-xx)+
		  (y_best-yy)*(y_best-yy));
      if (dist<VERTEXSIZE){      /* cut on a cluster size             */
	i_v++;
	itr_vertex[itr]=n_vertex;
	}
    } // tracks
    //    printf("+++ %d tracks deleted \n", i_v); 
    
  }
  else
    goto no_more_vertices;
  /*
    printf("%3d :: OK %1d/%5.2f ==> Best target %1d z = %5.2f => z = %5.2f\n",
	 n_vertex, ok_vertex, w_vertex[n_vertex],
	 wire_vertex[n_vertex], gtar_z[wire_vertex[n_vertex]], 
	 z_vertex[n_vertex]);
  */
  //  hfill(100, gtar_z[wire_vertex[n_vertex]]-z_vertex[n_vertex], 0.0, 1.0);
  
  if(DRAW){
    
    target_scale = 5.0; /* to scale target wires drawing */
    ispmci(it_best);
    ismksc(w_best);
    ismk(4);
    
    ih = itar[it_best];            /* real target number */
    if (GTAR(ih).z< 0.5*(gtar_z_stations[1]+gtar_z_stations[2]))
      target_shift = 0.0;
    else 
      target_shift = 10.0;   /* to separate stations */
    
    xpl[0] = target_scale*(x_best+target_shift);
    ypl[0] = target_scale*y_best;
    ipm(1,xpl,ypl);
  }
  goto new_vertex;
		  no_more_vertices:;
  

  /* compare with the simulated vertices */

  i_v = 0;
  for (iver=1; iver<=NMVER(); iver++){
    if (MVER(iver).flag==1){ // flag == 1 => primary
      i_v++;
      dtver[i_v] = 1000.0;
      itver[i_v] = 0;
    }
  }

  i_v = 0;
  for (iver=1; iver<=NMVER(); iver++){
    if (MVER(iver).flag==1){ // flag == 1 => primary
      i_v++;
      for (it=1; it<=ntar; it++){
	ih = itar[it];
	if (GTAR(ih).dx>GTAR(ih).dy){
	  if (sqrt((MVER(iver).z-gtar_z[ih])*(MVER(iver).z-gtar_z[ih])+
		   (MVER(iver).y-gtar_y[ih])*(MVER(iver).y-gtar_y[ih]))
	      <dtver[i_v]) {
	    dtver[i_v]= sqrt(
			     (MVER(iver).z-gtar_z[ih])*
			     (MVER(iver).z-gtar_z[ih])+
			     (MVER(iver).y-gtar_y[ih])*
			     (MVER(iver).y-gtar_y[ih]));
	    itver[i_v] = it;
	  }
	}
	else{
	  if (sqrt((MVER(iver).z-gtar_z[ih])*(MVER(iver).z-gtar_z[ih])+
		   (MVER(iver).x-gtar_x[ih])*(MVER(iver).x-gtar_x[ih]))
	      <dtver[i_v]) {
	    dtver[i_v]= sqrt(
			     (MVER(iver).z-gtar_z[ih])*
			     (MVER(iver).z-gtar_z[ih])+
			     (MVER(iver).x-gtar_x[ih])*
			     (MVER(iver).x-gtar_x[ih]));
	    itver[i_v] = it;
	  }
	}
      }
      ih = itar[itver[i_v]];
      hfill(701, MVER(iver).z-gtar_z[ih], 0.0, 1.0);

    }
  }
  
  printf("Total vertices %d:", NMVER());
  i_v = 0;
  for (iver=1; iver<=NMVER(); iver++){
    if (MVER(iver).flag==1){ // flag == 1 => primary
      i_v++;
      if (itver[i_v]!=0){
	
	hfill(700, MVER(iver).x, MVER(iver).y, 1.0);
	hfill(703, (float)itver[i_v], 0.0, 1.0);
	ih = itar[itver[i_v]];
	if (GTAR(ih).dx>GTAR(ih).dy)
	  dist= MVER(iver).y-gtar_y[ih];
	else
	  dist= MVER(iver).x-gtar_x[ih];
	hfill(702, dist, 0.0, 1.0);
	

	if (NRVER()>0){
	  hfill(704, RVER(NRVER()).xv-MVER(iver).x,	0.0, 1.0);
	  hfill(705, RVER(NRVER()).yv-MVER(iver).y,	0.0, 1.0);
	  hfill(706, RVER(NRVER()).zv-MVER(iver).z,	0.0, 1.0);
	}
	if(DRAW){
	  
	  target_scale = 5.0; /* to scale target wires drawing */
	  ispmci(itver[i_v]);
	  ismksc(3.0);
	  ismk(28);
	  
	  ih = itar[itver[i_v]];            /* real target number */
	  if (GTAR(ih).z< 0.5*(gtar_z_stations[1]+gtar_z_stations[2]))
	    target_shift = 0.0;
	  else 
	    target_shift = 10.0;   /* to separate stations */
	
	  xpl[0] = target_scale*(MVER(iver).x+target_shift);
	  ypl[0] = target_scale*MVER(iver).y;
	  ipm(1,xpl,ypl);
	}
      }
    }
  }
  printf("\n");
  
  if(DRAW){
    iuwk(1,1);
  }
  
  (*n_vertices) = n_vertex;
  return;
}
#endif

void Space_Track_Finder(){
  
  int   i, jj, ic, jc, ipr, ibpl, ipl, it, ipc, npc;
  int   k, kc, first, last, level_i, level_j, first_track, First_Track;
  int   ReCorrectIdx[MAXNUMBEROFSEGMENTS+1];
  int   ip, ih;
  int   itr, jtr, itrc, jtrc, jjc;
  int   MinLevel, is, ie, call, ich;
  float MinDist, Ax, Bx, Ay, By, x, xx, yy, qual;
  float x_n, y_n, z_n, x_m, y_m, z_m;
  int   ipr12_first, ipr12_last, ipr34_first, ipr34_last;
  float gMinDist;
  int   il, hits_on_nsl[11], hits_in_pr[5];
  int   start, stop, isec;

  if (DRAW && NMIMP()){
    ismksc(0.5);
    ismk(3);
    ispmci(Green);
    
    for(ipr=1; ipr <= 4; ipr++){ 
      shift = shift_coeff[ipr]*SHIFT;
      ibpl = base_plane[ipr];
      for(ih=1; ih <= NMIMP(); ih++){ 
	if ( GEDE(MIMP(ih).gede).cmp != 1) continue; 
	xpl[0] = MIMP(ih).z;
	ypl[0] = MIMP(ih).x*GEDE_cos[ibpl] - MIMP(ih).y*GEDE_sin[ibpl]+shift;
	ipm(1,xpl,ypl);
      }
    }

    isplci(Green);
    
    for(itr=1; itr <= NMTRA(); itr++){ 
      n_mimps = fill_mimp_points(itr, mimp_x, mimp_y, mimp_z);
      for(ipr=1; ipr <= 4; ipr++){ 
	shift = shift_coeff[ipr]*SHIFT;
	ibpl = base_plane[ipr];
	for(ih=1; ih < n_mimps; ih++){ 
	  xpl[0] = mimp_z[ih];
	  ypl[0] = mimp_x[ih]*GEDE_cos[ibpl] - 
	    mimp_y[ih]*GEDE_sin[ibpl]+shift;
	  xpl[1] = mimp_z[ih+1];
	  ypl[1] = mimp_x[ih+1]*GEDE_cos[ibpl] - 
	    mimp_y[ih+1]*GEDE_sin[ibpl]+shift;
//	  ipl(2,xpl,ypl);
	}
      }
    }
    
    iuwk(1,1);
    
  }




  if (DRAW){
    ismksc(0.5);
    ismk(3);
    ispmci(Red);
    
    for(ipr=1; ipr <= 4; ipr++){ 
      shift = shift_coeff[ipr]*SHIFT;
      for(ipl=1; ipl <= npl; ipl++){ 
	if (npr[ipl] != ipr) continue;
	for (ih=fh[ipl]; ih<=lh[ipl]; ih++){
	  xpl[0] = Z_Hit[ih];
	  ypl[0] = X_Hit[ih]+shift;
	  ipm(1,xpl,ypl);
	}
      }
    }
    
    iuwk(1,1);
    
  }


  Ntk = 0;
  nhits=0;

  //  if (N_Points==MAXNUMBEROFPOINTS) return; // no tracking for very noisy events


  NumberOfTracks = 0;
  TrackStart[NumberOfTracks][1] = 0; /* dummy track */
  TrackStart[NumberOfTracks][2] = 0;

  MinLevel = MINLEVEL;

  for (call=0; call<=2; call++){
    Call = call;
    SetNo = 0;
    //time_start();
    Space_Points();
    //time_stop();
    if (N_Points==MAXNUMBEROFPOINTS) return; // no tracking for very noisy events
    //time_start();
    Space_Automaton();
    //time_stop();
    First_Track = NumberOfTracks+1;

    if ((Call == 0)||(Call == 2))
      DOUBLETARGETCUT = 2.0*TARGETREGION;
    else
      DOUBLETARGETCUT = 2.0*TARGETSCALE*TARGETREGION;

    if (call==0){
      //time_start();
    while (Level>=MinLevel){
      first_track = NumberOfTracks+1;

      
      for (k=1;k<=N_Segments;k++){
	kc = CorrectIndex[k];
	
	if (LevelOfSegment[kc] == Level){
	  
	  CurrentTrackStart[1] = StartOfSegment[kc];
	  x_start_track = X_Start[kc];
	  y_start_track = Y_Start[kc];
	  z_start_track = Z_Start[kc];
	  
	  Space_Find_Track_Chi_2(1, kc); 
	  /*
	  NumberOfTracks++;
	  TrackStart[NumberOfTracks][1] = StartOfSegment[kc];
	  TrackStart[NumberOfTracks][2] = EndOfSegment[kc];
	  LengthOfTrack[NumberOfTracks] = 2;
	  X_Start_Track[NumberOfTracks] = X_Start[kc];
	  Y_Start_Track[NumberOfTracks] = Y_Start[kc];
	  Z_Start_Track[NumberOfTracks] = Z_Start[kc];
	  X_End_Track[NumberOfTracks]   = X_End[kc];
	  Y_End_Track[NumberOfTracks]   = Y_End[kc];
	  Z_End_Track[NumberOfTracks]   = Z_End[kc];
	  
	  Qual[NumberOfTracks] = 0.0;
	  idx_tr[NumberOfTracks]=NumberOfTracks;
	  GoodTrack[NumberOfTracks]=1;
	  */
	}
      }
      
      if (NumberOfTracks == MAXNUMBEROFTRACKS){
	printf("CATS : <<< MAXIMUM NUMBER OF TRACKS EXCEEDED >>>\n");
	goto max_tracks;
      }

     
      for (itr=first_track;itr<=NumberOfTracks;itr++){

	qual = 0.0;   /* not really chi_2 but quality of a track */
                       /* the higher number - the better track    */
	
	x_n = X_Start_Track[itr];
	y_n = Y_Start_Track[itr];
	z_n = Z_Start_Track[itr];
	x_m = X_End_Track[itr];
	y_m = Y_End_Track[itr];
	z_m = Z_End_Track[itr];
	
	Ax = (x_m-x_n)/(z_m-z_n); /* slope in proj. x */
	Bx = x_m-Ax*z_m;
	Ay = (y_m-y_n)/(z_m-z_n); /* slope in proj. y */
	By = y_m-Ay*z_m;
	
	
	nfound = 0;
	for(ipl=1; ipl <= npl; ipl++){ 
	  xx=Bx+Ax*zpl[ipl];
	  if ( xx < GEDE_sxmn[ipl] ||
	       xx > GEDE_sxmx[ipl] ) continue;
	  yy=By+Ay*zpl[ipl]; 
	  if ( yy < GEDE_symn[ipl] ||
	       yy > GEDE_symx[ipl] ) continue;
	  
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl]; 
	  
	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	  
	  if ((ic<0)||(ic>50)) continue;
	  
	  MinDist = MINDIST;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){
	    if (fabs(x-HITB_x[ih])<MinDist) {
	      MinDist    = fabs(x-HITB_x[ih]);
	    }
	  }
	  
	  if (MinDist<MINDIST){
	    nfound++;
	    qual += 1.0-MinDist/MINDIST;
	  }
	}
	
	qual += nfound;

	/* crossing the target wires? */

	MinDist = DOUBLETARGETCUT;
	for (it=1; it<=ntargets; it++){
	  xx=Bx+Ax*GTAR_z[it];
	  yy=By+Ay*GTAR_z[it];
	  if ( fabs(xx-GTAR_x[it])+fabs(yy-GTAR_y[it])<MinDist )
	    MinDist = fabs(xx-GTAR_x[it])+fabs(yy-GTAR_y[it]);
	}
	
	qual += 4.0*(1.0-MinDist/DOUBLETARGETCUT);

	Qual[NumberOfTracks] = qual;


	/* flag used hits */
	for (i=1;i<=LengthOfTrack[itr];i++) {
	  ip = TrackStart[itr][i];
	  for (ih=1;ih<=N_Point[TrackStart[itr][i]];ih++) {
	    jfl[Hit_Point[ip][ih]]=1;  /* flag hit as used */
	  }
	}
      }
      

      npc = 0;
      for (ipc=1;ipc<=NC_Points;ipc++) {
	ip = IC_Point[ipc];
	nfound = 0;
	for (ih=1;ih<=N_Point[ip];ih++) {
	  if (jfl[Hit_Point[ip][ih]]==0) nfound++;;
	}
	if (nfound<2)
	  TrackPoint[ip] = 1;
	else
	  IC_Point[++npc] = ip;
      }
      NC_Points = npc;

      
      if (Level>=MinLevel){
	jc=0; //calculate new set of segments
	for (ic=1; ic<=N_Segments; ic++){ // on previouse set
	  i = CorrectIndex[ic];           // real segment
	  ReCorrectIdx[ic] = 0;           // pointer to the new set
	  if ((TrackPoint[StartOfSegment[i]] == 0)&&
	      (TrackPoint[EndOfSegment[i]] == 0)){
	    jc++;                         // add to the new set
	    ReCorrectIdx[ic] = jc;        // point from old set to the new one
	    CorrectIndex[jc] = i;         // point from new set to real segment  
	    first=First_Segment[i];       // remember first/last
	    last=Last_Segment[i];
	    First_Segment[i]=0;
	    Last_Segment[i]=-1;
	    level_i=0;
	    for (k=first; k<=last; k++) { // check status of neighbours
	      kc=ReCorrectIdx[k];         // if there is corresp. in new set
	      if (kc>0) {                 // found
		if (First_Segment[i]==0) First_Segment[i]=kc;
		Last_Segment[i]=kc;      
		level_i=1;
		LevelOfSegment[CorrectIndex[kc]]=1;
	      }
	    }
	    LevelOfSegment[i]=level_i;
	  }
	}
	N_Segments = jc;
	
	Level = 0;
	for (ic=1; ic<=N_Segments; ic++){
	  i = CorrectIndex[ic];
	  if (LevelOfSegment[i]){
	    first = First_Segment[i];
	    last  = Last_Segment[i];
	    level_i = 0;
	    for (jc=first;jc<=last;jc++){
	      level_j = LevelOfSegment[CorrectIndex[jc]];
	      if (level_j > level_i)
		level_i = level_j;
	    }
	    LevelOfSegment[i] = ++level_i;
	    if (level_i > Level)
	      Level = level_i;
	  }
	}
	
      }
    }/* while (Level>= MinLevel); */
    //time_stop();
    }

    /* start working with single tracks */
    
    first_track = NumberOfTracks+1;
    
    //time_start();
    
    for (i=1; i<=Total_N_Segments; i++){
      if (NumberOfTracks == MAXNUMBEROFTRACKS){
	printf("CATS : <<< MAXIMUM NUMBER OF TRACKS EXCEEDED >>>\n");
	goto max_tracks;
      }
      is = StartOfSegment[i];
      ie = EndOfSegment[i];

      if ((TrackPoint[is]!=0)&&(TrackPoint[ie]!=0)) continue;

      qual = 0.0;   /* not really chi_2 but quality of a track */
                     /* the higher number - the better track    */
      
      x_n = X_Start[i]; y_n = Y_Start[i]; z_n = Z_Start[i];
      x_m = X_End[i];   y_m = Y_End[i];   z_m = Z_End[i];
      
      Ax = (x_m-x_n)/(z_m-z_n); // slope in proj. x
      Bx = x_m-Ax*z_m;
      Ay = (y_m-y_n)/(z_m-z_n); // slope in proj. y
      By = y_m-Ay*z_m;

      nfound = 0;
      for(ipl=1; ipl <= npl; ipl++){ 
	xx=Bx+Ax*zpl[ipl];
	if ( xx < GEDE_sxmn[ipl] ||
	     xx > GEDE_sxmx[ipl] ) continue;
	yy=By+Ay*zpl[ipl]; 
	if ( yy < GEDE_symn[ipl] ||
	     yy > GEDE_symx[ipl] ) continue;
	
	x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl]; 
	
	ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	
	if ((ic<0)||(ic>50)) continue;
	
	MinDist = MINDIST;
	for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){
	  if (fabs(x-HITB_x[ih])<MinDist) {
	    MinDist    = fabs(x-HITB_x[ih]);
	  }
	}
	
	if (MinDist<MINDIST){
	  qual += 1.0-MinDist/MINDIST;
	  nfound++;
	}
      }
      
      if (nfound < n_at_least) continue; //weak track? 
      
      qual += nfound;
      
      /* crossing the target wires? */

      MinDist = DOUBLETARGETCUT;
      for (it=1; it<=ntargets; it++){
	xx=Bx+Ax*GTAR_z[it];
	yy=By+Ay*GTAR_z[it];
	if ( fabs(xx-GTAR_x[it])+fabs(yy-GTAR_y[it])<MinDist )
	  MinDist = fabs(xx-GTAR_x[it])+fabs(yy-GTAR_y[it]);
      }
      
      qual += 4.0*(1.0-MinDist/DOUBLETARGETCUT);

      
      NumberOfTracks++;
      TrackStart[NumberOfTracks][1] = is;
      TrackStart[NumberOfTracks][2] = ie;
      LengthOfTrack[NumberOfTracks] = 2;
      X_Start_Track[NumberOfTracks] = X_Start[i];
      Y_Start_Track[NumberOfTracks] = Y_Start[i];
      Z_Start_Track[NumberOfTracks] = Z_Start[i];
      X_End_Track[NumberOfTracks]   = X_End[i];
      Y_End_Track[NumberOfTracks]   = Y_End[i];
      Z_End_Track[NumberOfTracks]   = Z_End[i];
      
      Qual[NumberOfTracks] = qual;
      idx_tr[NumberOfTracks]=NumberOfTracks;
      GoodTrack[NumberOfTracks]=1;
    }
    
  max_tracks:;

    //time_stop();
    
    //time_start();

    for (itrc=First_Track;itrc<=NumberOfTracks;itrc++){
      itr = idx_tr[itrc];
      /* sort tracks */
      if (GoodTrack[itr]==1){
	jjc=itrc;
	jj =itr ;
	for (jtrc=(itrc+1);jtrc<=NumberOfTracks;jtrc++){
	  jtr = idx_tr[jtrc];
	  if (GoodTrack[jtr]==1){
	    if (Qual[jtr] > Qual[jj]){
	      jjc=jtrc;
	      jj =jtr ;
	    }
	  }
	}
	
	idx_tr[itrc]=jj;
	idx_tr[jjc ]=itr;
	
	/* collect track hits */
	
	itr = jj;
	
	x_n = X_Start_Track[itr];
	y_n = Y_Start_Track[itr];
	z_n = Z_Start_Track[itr];
	x_m = X_End_Track[itr];
	y_m = Y_End_Track[itr];
	z_m = Z_End_Track[itr];
	
	Ax = (x_m-x_n)/(z_m-z_n); /* slope in proj. x */
	Bx = x_m-Ax*z_m;
	Ay = (y_m-y_n)/(z_m-z_n); /* slope in proj. y */
	By = y_m-Ay*z_m;


	nfound = 0;
	gfound = 0;
	ipr12_first = 0;
	ipr12_last  = 0;
	ipr34_first = 0;
	ipr34_last  = 0;

	for(ipr=1; ipr <= 4; ipr++){ 
	  hits_in_pr[ipr] = 0;
	}

	for( il=1; il<=10;il++){
	  hits_on_nsl[il] = 0;
	}

	for(ipl=1; ipl <= npl; ipl++){ 
	  /*	  if (GEDE_sec[ipl]==n_s_co) continue; */
	  
	  xx=Bx+Ax*zpl[ipl];
	  if ( xx < GEDE_sxmn[ipl] ||
	       xx > GEDE_sxmx[ipl] ) continue;
	  yy=By+Ay*zpl[ipl]; 
	  if ( yy < GEDE_symn[ipl] ||
	       yy > GEDE_symx[ipl] ) continue;
	  
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl]; 
	  
	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	  
	  if ((ic<0)||(ic>50)) continue;

	  MinDist = 2.0*MINDIST;
	  gMinDist = 2.0*MINDIST;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){
	    if (fabs(x-HITB_x[ih])<gMinDist) {
	      gMinDist   = fabs(x-HITB_x[ih]);
	      gg[gfound] = ih;
	    }
	    if (ifl[ih]==0)
	      if (fabs(x-HITB_x[ih])<MinDist) {
		MinDist    = fabs(x-HITB_x[ih]);
		hh[nfound] = ih;
	      }
	  }
	  
	  if (gMinDist<2.0*MINDIST){
	    gfound++;
	    //	    hits_in_pr[npr[ipl]]++;
	    //	    hits_on_nsl[GEDE_nsl[ipl]]++;
	    if ((npr[ipl]==1)||(npr[ipl]==2)){
	      if (ipr12_first==0)
		ipr12_first = GEDE_nsl[ipl];
	      ipr12_last = GEDE_nsl[ipl];
	    }
	    else{
	      if (ipr34_first==0)
		ipr34_first = GEDE_nsl[ipl];
	      ipr34_last = GEDE_nsl[ipl];
	    }
	  }
	  
	  if (MinDist<2.0*MINDIST){
	    hits_in_pr[npr[ipl]]++;
	    hits_on_nsl[GEDE_nsl[ipl]]++;
	    nfound++;
	  }
	}

	if (nfound < n_at_least){
	  GoodTrack[itr] = 0;
	  continue; 
	}

	if ((ipr12_last-ipr12_first==0)||(ipr34_last-ipr34_first==0)){
	  GoodTrack[itr] = 0;
	  continue; 
	}

	/* store found hits */
	
	for(i=0; i < nfound; i++) {
	  ih = hh[i];
	  hits[nhits++]=ihit[ih];
	  ifl[ih]=1;  /* flag hit as used */
	}

	hits[nhits++]=-1; /* end of hit list */
	
	/* store found track parameters */
	X0_trk[Ntk]=Bx;
	Xp_trk[Ntk]=Ax;
	Y0_trk[Ntk]=By;
	Yp_trk[Ntk]=Ay;
	nh_trk[Ntk]=nfound;
	
	X1 = X_Start_Track[itr];
	Y1 = Y_Start_Track[itr];
	Z1 = Z_Start_Track[itr];
	X2 = X_End_Track[itr];
	Y2 = Y_End_Track[itr];
	Z2 = Z_End_Track[itr];

//	TrackRefit();
	KalmanRefit();

	/*chi2 confidence level*/

#ifdef TARGET
	qual = Chi2_trk[Ntk];
	nfound = nfound-4;
	hfill(710, prob_(&qual, &nfound), 0.0, 1.0);
	nfound = nfound+4;

#endif

	if (DRAW){
	  isplci(Red);

	  Bx = X0_trk[Ntk];
	  Ax = Xp_trk[Ntk];
	  By = Y0_trk[Ntk];
	  Ay = Yp_trk[Ntk];
	  
	  
	  for(ipr=1; ipr <= 4; ipr++){
	    shift = shift_coeff[ipr]*SHIFT;
	    ibpl = base_plane[ipr];
	    
	    xx=Bx+Ax*Z_Hit[hh[0]];
	    yy=By+Ay*Z_Hit[hh[0]];

	    xpl[0] = Z_Hit[hh[0]];
	    ypl[0] = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]+shift;

	    xx=Bx+Ax*Z_Hit[hh[nfound-1]];
	    yy=By+Ay*Z_Hit[hh[nfound-1]];

	    xpl[1] = Z_Hit[hh[nfound-1]];
	    ypl[1] = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]+shift;

	    ipl(2,xpl,ypl);
	  }	
	}


	if(++Ntk >= MAXNUMBEROFTRACKS)  {
	  printf("CATS : *** Space track *** == > too many tracks %d\n", Ntk); 
	  return;
	}

      }
   
    }
    //time_stop();




    if (DRAW)
      printf("Track candidates: long %3d; short %3d => %3d\n", 
	     first_track - First_Track, NumberOfTracks - first_track, Ntk);


    while ((NumberOfTracks>0)&&(GoodTrack[NumberOfTracks]==0))
      NumberOfTracks--;
    

    for (ich=1; ich <= nchambers; ich++){
      for (isec=1; isec <= 4; isec++){
	for (ipr=1; ipr <= 4; ipr++){
	  
	  ipl =  Nsl_Sec[ich][isec][ipr];
	  if (ipl==0) continue;
	  
	  start = fh[ipl];
	  stop  = lh[ipl];
	  if (stop<start) continue;
	  for (ih=start; ih<=stop; ih++){
	    if (ifl[ih]==1)
	      Used_Hit[ih] = 1;
	    else
	      Used_Hit[ih] = 0;
	  }
	}
      }
    }


    /*
    for (ich=0; ich<=nchambers; ich++) {
      StartOnPlane[ich] = 0;
      EndOnPlane[ich]   =-1;
    }
    
    npc = 0;
    for (ipc=1;ipc<=NC_Points;ipc++) {
      ip = IC_Point[ipc];
      TrackPoint[ip] = 0;
      nfound = 0;
      for (ih=1;ih<=N_Point[ip];ih++) {
	if (ifl[Hit_Point[ip][ih]]==0) nfound++;;
      }
      if (nfound<2)
	TrackPoint[ip] = 1;
      else{
	IC_Point[++npc] = ip;
	ich=NSL_Point[ip];
	if (StartOnPlane[ich] == 0)
	  StartOnPlane[ich] = npc;
	EndOnPlane[ich] = npc;
      }
    }
    NC_Points = npc;
    */
    
  }/*call */

       
  if (DRAW){
    printf(" *** Tracks found ===> %3d\n",Ntk);
    
    iuwk(1,1);
  }

}






void Space_Find_Track_Chi_2(int n, int k){
  
  int   i;
  int   l, lc, lvl;
  int   first, last;
  float chi_2;
  int   start, end, point;
  float k1, k2, x, y;

  lvl=Level-n; n++;

  if (lvl == 0){

    start = CurrentTrackStart[1];
    end   = EndOfSegment[k];
    chi_2 = 0.0;//N_Point[start];
    for (i=2;i<n;i++){
      point=CurrentTrackStart[i];
      k1=(Z_Point[end]-Z_Point[point])/(Z_Point[end]-Z_Point[start]);
      k2=(Z_Point[point]-Z_Point[start])/(Z_Point[end]-Z_Point[start]);
      x=k1*X_Point[start]+k2*X_Point[end];
      y=k1*Y_Point[start]+k2*Y_Point[end];
      
      chi_2 += (1.0-fabs(X_Point[point]-x)/MINDIST)+
	(1.0-fabs(Y_Point[point]-y)/MINDIST);// + N_Point[point];
    }
    //chi_2 += N_Point[CurrentTrackStart[n]];


    if ((TrackStart[NumberOfTracks][1] != CurrentTrackStart[1])||
	(TrackStart[NumberOfTracks][2] != CurrentTrackStart[2])){

      if (NumberOfTracks < MAXNUMBEROFTRACKS)
	NumberOfTracks++;
      idx_tr[NumberOfTracks]=NumberOfTracks;
      GoodTrack[NumberOfTracks] = 1;
    }
    else
      if ((LengthOfTrack[NumberOfTracks] > n)||
	  ((LengthOfTrack[NumberOfTracks] == n)&&
	   (Chi_2[NumberOfTracks] > chi_2)) )
	goto a;   
    
    Chi_2[NumberOfTracks] = chi_2;
    LengthOfTrack[NumberOfTracks] = n;
    X_Start_Track[NumberOfTracks] = x_start_track;
    Y_Start_Track[NumberOfTracks] = y_start_track;
    Z_Start_Track[NumberOfTracks] = z_start_track;
    X_End_Track[NumberOfTracks]   = X_End[k];
    Y_End_Track[NumberOfTracks]   = Y_End[k];
    Z_End_Track[NumberOfTracks]   = Z_End[k];
    for (i=1;i<n;i++)
      TrackStart[NumberOfTracks][i] = CurrentTrackStart[i];
    TrackStart[NumberOfTracks][n]=EndOfSegment[k];
  a:;
  }   
  else {
    first = First_Segment[k];
    last  = Last_Segment[k];
    for (l=first;l<=last;l++){
      lc = CorrectIndex[l];
      if (LevelOfSegment[lc] == lvl){
	CurrentTrackStart[n] = StartOfSegment[lc];
	
	Space_Find_Track_Chi_2(n, lc);
      }
    }
  }
  return;
}













void Space_Automaton(){

  int   i, ii, j, ij, n, m, nc, mc;
  int   index, ih1, ih2, ipl1, ipl2;
  int   start_on_plane, end_on_plane, iplane, eplane;
  float x_n, y_n, z_n, x_m, y_m, z_m;
  int   start, end, first, last;
  int   has_i, level_i, level_j;
  float x, y, x1, x2, z2, c1, s1, c2, s2, det, A, B, dx;
  float A1, A2, B1, B2, x_b[3], x_u[3], y_b[3], y_u[3], targetcut;
  int   it, from_a_target, skip;
#ifdef XXX
  int   ipr, ibpl;
#endif


  if ((Call==0)||(Call==2))
    targetcut = TARGETREGION+0.2;
  else
    targetcut = TARGETSCALE*TARGETREGION;

  skip = 0;

  if (Call==2)
    skip = 1;
  
  index = 0;

  /* create segments in upstream direction pointing from the target      */
  /* to have the last segment on each track connected with the target  */

  for (ii=1; ii<PlaneSet[SetNo][0]-skip; ii++){
    iplane=PlaneSet[SetNo][ii];
    eplane=PlaneSet[SetNo][ii+1+skip];
    start_on_plane = StartOnPlane[iplane];
    end_on_plane   = EndOnPlane[iplane];
    for (mc=StartOnPlane[eplane]; mc<=EndOnPlane[eplane]; mc++){
      m = IC_Point[mc];
      if (TrackPoint[m]==0){
	x_m = X_Point[m];
	y_m = Y_Point[m];
	z_m = Z_Point[m];
	FirstSegment[m] = index+1;

	for (it=1; it<=2; it++){
	  A1 = (x_m-(x_station[it]-targetcut))/(z_m-z_station[it]);
	  B1 = x_m-A1*z_m;
	  x_b[it]= A1*z_nsl[iplane]+B1-3.0*MINDIST;
	  A2 = (x_m-(x_station[it]+targetcut))/(z_m-z_station[it]);
	  B2 = x_m-A2*z_m;
	  x_u[it]= A2*z_nsl[iplane]+B2+3.0*MINDIST;
	  
	  A1 = (y_m-(y_station[it]-targetcut))/(z_m-z_station[it]);
	  B1 = y_m-A1*z_m;
	  y_b[it]= A1*z_nsl[iplane]+B1-3.0*MINDIST;
	  A2 = (y_m-(y_station[it]+targetcut))/(z_m-z_station[it]);
	  B2 = y_m-A2*z_m;
	  y_u[it]= A2*z_nsl[iplane]+B2+3.0*MINDIST;
	}

	for (nc = start_on_plane; nc<=end_on_plane; nc++){
	  n = IC_Point[nc];
	  if ((Call==0) && (SEC_Point[n] != SEC_Point[m])) continue;
	  //if ((SEC_Point[n] != SEC_Point[m])&&(eplane!=8)) continue;
	  if (TrackPoint[n]==0){ /* free */
	    x_n = X_Point[n];
	    y_n = Y_Point[n];
	    z_n = Z_Point[n];

	    
	    from_a_target = 0;
	    if (((x_n>x_b[1])&&(x_n<x_u[1])&&(y_n>y_b[1])&&(y_n<y_u[1]))||
		((x_n>x_b[2])&&(x_n<x_u[2])&&(y_n>y_b[2])&&(y_n<y_u[2])))
	      from_a_target = 1;

	    if (from_a_target){
	    
	      if (index==MAXNUMBEROFSEGMENTS){
		printf("CATS : <<< MAXIMUM NUMBER OF SEGEMENTS EXCEEDED >>>\n");
		goto a;
	      }
	      index++;

	      StartOfSegment[index] = n; /* from the target direction */ 
	      EndOfSegment[index] = m;


	      /* determine position of a segment */

	      ih1 = Hit_Point[n][1];
	      ipl1 =  IPL_Hit[ih1];
	      x1 = x_n*GEDE_cos[ipl1] - y_n*GEDE_sin[ipl1];
	      x2 = x_m*GEDE_cos[ipl1] - y_m*GEDE_sin[ipl1];

	      A = (x2-x1)/(z_m-z_n); // slope in proj. ipl1
	      B = x2-A*z_m;

	      dx = A*Z_Hit[ih1]+B-HITB_x[ih1];;

	      ih2 = Hit_Point[n][2];
	      ipl2 =  IPL_Hit[ih2];
	      x2 = HITB_x[ih2];
	      z2 = Z_Hit[ih2];

	      x1 = A*z2+B-dx;

	      c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
	      c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
	      det=c1*s2-c2*s1;

	      /* space point coordinates of a segment's start */
	      X_Start[index] = (x1*s2-s1*x2)/det;
	      Y_Start[index] =-(x1*c2-c1*x2)/det;
	      Z_Start[index] = z2;


	      ih1 = Hit_Point[m][1];
	      ipl1 =  IPL_Hit[ih1];
	      x1 = x_n*GEDE_cos[ipl1] - y_n*GEDE_sin[ipl1];
	      x2 = x_m*GEDE_cos[ipl1] - y_m*GEDE_sin[ipl1];

	      A = (x2-x1)/(z_m-z_n); // slope in proj. ipl1
	      B = x2-A*z_m;

	      dx = A*Z_Hit[ih1]+B-HITB_x[ih1];;

	      ih2 = Hit_Point[m][2];
	      ipl2 =  IPL_Hit[ih2];
	      x2 = HITB_x[ih2];
	      z2 = Z_Hit[ih2];

	      x1 = A*z2+B-dx;

	      c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
	      c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
	      det=c1*s2-c2*s1;
	      
	      /* space point coordinates of a segment's end */
	      X_End[index] = (x1*s2-s1*x2)/det;
	      Y_End[index] =-(x1*c2-c1*x2)/det;
	      Z_End[index] = z2;


	      has_i = 0;
	      start = FirstSegment[n];
	      end   = LastSegment[n];
	      first =  0;
	      last  = -1;

	      for (j=start; j<=end; j++){
		A = (X_End[index]-X_Start[j])/(Z_End[index]-Z_Start[j]); 
		B = X_End[index]-A*Z_End[index];
		x = A*Z_Start[index]+B;
		A = (Y_End[index]-Y_Start[j])/(Z_End[index]-Z_Start[j]); 
		B = Y_End[index]-A*Z_End[index];
		y = A*Z_Start[index]+B;

		if ((fabs(x-X_Start[index])<MINDIST)&&
		    (fabs(y-Y_Start[index])<MINDIST)){
		  has_i = (1==1);
		  HasNeighbour[j] = (1==1);
		  if (first==0) first = j;
		  last = j;
		}
	      }
	      HasNeighbour[index]  = has_i;
	      First_Segment[index] = first;
	      Last_Segment[index]  = last;
	    }
	  }
	}
	LastSegment[m]  = index;
      }
    }
  }

a:

  Total_N_Segments = index;


  j=0;
  Level = 0;
  for (i=1; i<=index; i++){
    if (HasNeighbour[i]){
      j++;
      CorrectIndex[j] = i;
      HasNeighbour[i] = j;
      if (First_Segment[i]>0)
	First_Segment[i] = HasNeighbour[First_Segment[i]];
      if (Last_Segment[i]>0)
	Last_Segment[i]  = HasNeighbour[Last_Segment[i]];
      first = First_Segment[i];
      last  = Last_Segment[i];
      level_i = 0;
      for (ij=first;ij<=last;ij++){
	level_j = LevelOfSegment[CorrectIndex[ij]];
	if (level_j > level_i)
	  level_i = level_j;
      }
      LevelOfSegment[i] = ++level_i;
      if (level_i > Level)
	Level = level_i;
    }
  }
  N_Segments = j;

#ifdef XXX
  if(DRAW){
    for (i=1;i<=Total_N_Segments;i++){
      isplci(Black);
      for(ipr=1; ipr <= 4; ipr++){
	shift = shift_coeff[ipr]*SHIFT+0.3;
	ibpl = base_plane[ipr];
	
	xpl[0] = Z_Start[i];
	ypl[0] = X_Start[i]*GEDE_cos[ibpl] - 
	  Y_Start[i]*GEDE_sin[ibpl]+shift;
	xpl[1] = Z_End[i];
	ypl[1] = X_End[i]*GEDE_cos[ibpl] - 
	  Y_End[i]*GEDE_sin[ibpl]+shift;
	ipl(2,xpl,ypl);
	
      }
    }
    iuwk(1,1);
  }
#endif



  
}





 

void Space_Points(){
  int   ipr1, ipr2, ipr3, ipr4, ipl1, ipl2, ipl3, ipl4, ipl, ich, isec; 
  int   ic, ih, ip, ip1;
  int   np_tmp4, np_tmp3, np_tmp2;
  int   start1, stop1, start2, stop2, ih1, ih2, ih3, ih4, ipr, ibpl;
  float c1, s1, c2, s2, det, x1, x2, z1, z2, z3, z4, xx, yy, zz;
  float x, MINDIST_SPACE, MinDist, e_tmp;

  MINDIST_SPACE = 1.5*MINDIST;//0.5*MINDIST;

  ih3 = 0; ih4 = 0; /* dummy initialization */

  N_Points = 0;


  for (ich=1; ich <= nchambers; ich++){
    for (isec=1; isec <= 4; isec++){
      /* reconstruct space points from 4 hits */
      
      np_tmp4 = N_Points;

      ipr1=1; 
      ipr2=3;
      ipr3=2;
      ipr4=4;

      ipl1 =  Nsl_Sec[ich][isec][ipr1];
      ipl2 =  Nsl_Sec[ich][isec][ipr2];
      ipl3 =  Nsl_Sec[ich][isec][ipr3];
      ipl4 =  Nsl_Sec[ich][isec][ipr4];
	  
      if ((ipl1==0)||(ipl2==0)||(ipl3==0)||(ipl4==0)) goto tripple;

      c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
      c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
      det=c1*s2-c2*s1;

      start1 = fh[ipl1];
      stop1  = lh[ipl1];
      if (stop1<start1) goto tripple;
      start2 = fh[ipl2];
      stop2  = lh[ipl2];
      if (stop2<start2) goto tripple;

      for (ih1=start1; ih1<=stop1; ih1++){
	if (Used_Hit[ih1]) continue;
	x1 = HITB_x[ih1];
	z1 = Z_Hit[ih1];

	for (ih2 = start2; ih2<=stop2; ih2++){
	  if (Used_Hit[ih2]) continue;
	  x2 = HITB_x[ih2];
	  z2 = Z_Hit[ih2];
	  
	/* space point coordinates with correction */
	  xx= ((x1*z2/z1)*s2-s1*x2)/det;
	  yy=-((x1*z2/z1)*c2-c1*x2)/det;
 	  zz = z2;

	  if ( xx < GEDE_sxmn[ipl2]-0.1 ||
	       xx > GEDE_sxmx[ipl2]+0.1 ) continue;
	  if ( yy < GEDE_symn[ipl2]-0.1 ||
	       yy > GEDE_symx[ipl2]+0.1 ) continue;
	  

	  z3 = 0.0;
	  ipl = ipl3;
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	  if ((ic<0)||(ic>50)) continue;
	    
	  MinDist = MINDIST_SPACE;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	    if (Used_Hit[ih]) continue;
	    if (fabs(x*Z_Hit[ih]/zz-HITB_x[ih])<MinDist){
	      MinDist = fabs(x*Z_Hit[ih]/zz-HITB_x[ih]);
	      z3 = Z_Hit[ih];
	      ih3 = ih;
	    }
	  }
	    
	  if (MinDist==MINDIST_SPACE) continue;

	  e_tmp = MinDist;

	  z4 = 0.0;
	  ipl = ipl4;
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	  if ((ic<0)||(ic>50)) continue;
	    
	  MinDist = MINDIST_SPACE;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	    if (Used_Hit[ih]) continue;
	    if (fabs(x*Z_Hit[ih]/zz-HITB_x[ih])<MinDist){
	      MinDist = fabs(x*Z_Hit[ih]/zz-HITB_x[ih]);
	      z4 = Z_Hit[ih];
	      ih4 = ih;
	    }
	  }
	    
	  if (MinDist==MINDIST_SPACE) continue;
	  
	  e_tmp += MinDist;

	  if (N_Points==MAXNUMBEROFPOINTS){
	    printf("CATS : Max number of space points exceeded %d\n", N_Points);
	    goto finish;
	  }
	  N_Points++;
	  X_Point[N_Points] = xx;
	  Y_Point[N_Points] = yy;
	  Z_Point[N_Points] = zz;
	  NSL_Point[N_Points] = ich;
	  SEC_Point[N_Points] = isec;
	  Hit_Point[N_Points][1] = ih1;
	  Hit_Point[N_Points][2] = ih2;
	  Hit_Point[N_Points][3] = ih3;
	  Hit_Point[N_Points][4] = ih4;
	  Used_Hit[ih1] = (1==1);
	  Used_Hit[ih2] = (1==1);
	  Used_Hit[ih3] = (1==1);
	  Used_Hit[ih4] = (1==1);
	  N_Point[N_Points] = 4;
	  Good_Point[N_Points] = (1==1);
	  E_Point[N_Points] = e_tmp;
	  IC_Point[N_Points] = N_Points;

	}
      }

      
      if (DRAW){ 
	ismksc(1.5);
	ispmci(Red);
	ismk(24);
	
	for(ip=np_tmp4+1; ip <= N_Points; ip++){
	  if (Good_Point[ip])
	    for(ipr=1; ipr <= 4; ipr++){ 
	      
	      ibpl = base_plane[ipr];
	      shift = shift_coeff[ipr]*SHIFT;
	      
	      x = X_Point[ip]*GEDE_cos[ibpl] - Y_Point[ip]*GEDE_sin[ibpl]; 
	      
	      xpl[0] = Z_Point[ip];
	      ypl[0] = x+shift;
	      //ipm(1,xpl,ypl);
	    }
	}    
      }
      
    tripple:;      

      np_tmp3 = N_Points;
      
      /* reconstruct space points from 3 hits */
      
      ipr1=1; 
      ipr2=3;
      ipr3=2;
      ipr4=4;
      
      ipl1 =  Nsl_Sec[ich][isec][ipr1];
      if (ipl1==0) goto second_tripple;
      ipl2 =  Nsl_Sec[ich][isec][ipr2];
      if (ipl2==0) goto second_tripple;
      
      c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
      c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
      det=c1*s2-c2*s1;
      
      start1 = fh[ipl1];
      stop1  = lh[ipl1];
      if (stop1<start1) goto second_tripple;
      start2 = fh[ipl2];
      stop2  = lh[ipl2];
      if (stop2<start2) goto second_tripple;
      
      for (ih1=start1; ih1<=stop1; ih1++){
	if (Used_Hit[ih1]) continue;
	x1 = HITB_x[ih1];
	z1 = Z_Hit[ih1];
	for (ih2 = start2; ih2<=stop2; ih2++){
	if (Used_Hit[ih2]) continue;
	  x2 = HITB_x[ih2];
	  z2 = Z_Hit[ih2];
	  
	/* space point coordinates with correction */

	  xx= (x1*s2*z2/z1-s1*x2)/det;
	  yy=-(x1*c2*z2/z1-c1*x2)/det;
	  zz = z2;
	  
	  if ( xx < GEDE_sxmn[ipl2]-0.1 ||
	       xx > GEDE_sxmx[ipl2]+0.1 ) continue;
	  if ( yy < GEDE_symn[ipl2]-0.1 ||
	       yy > GEDE_symx[ipl2]+0.1 ) continue;
	  

	  ipl3 =  Nsl_Sec[ich][isec][ipr3];
	  if (ipl3==0) goto label_4;

	  z3 = 0.0;
	  ipl = ipl3;
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	  if ((ic<0)||(ic>50)) goto label_4;
	    
	  MinDist = MINDIST_SPACE;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	    if (Used_Hit[ih]) continue;
	    if (fabs(x*Z_Hit[ih]/zz-HITB_x[ih])<MinDist){
	      MinDist = fabs(x*Z_Hit[ih]/zz-HITB_x[ih]);
	      z3 = Z_Hit[ih];
	      ih3 = ih;
	    }
	  }
	    
	  if (MinDist==MINDIST_SPACE) goto label_4;

	  e_tmp = MinDist;
	  
	  if (N_Points==MAXNUMBEROFPOINTS){
	    printf("CATS : Max number of space points exceeded %d\n", N_Points);
	    goto finish;
	  }
	  N_Points++;
	  X_Point[N_Points] = xx;
	  Y_Point[N_Points] = yy;
	  Z_Point[N_Points] = zz;
	  NSL_Point[N_Points] = ich;
	  SEC_Point[N_Points] = isec;
	  Hit_Point[N_Points][1] = ih1;
	  Hit_Point[N_Points][2] = ih2;
	  Hit_Point[N_Points][3] = ih3;
	  Hit_Point[N_Points][4] = 0;
	  N_Point[N_Points] = 3;
	  Good_Point[N_Points] = (1==1);
	  E_Point[N_Points] = e_tmp;
	  IC_Point[N_Points] = N_Points;
	  
	  continue;

	label_4:;

	  ipl4 =  Nsl_Sec[ich][isec][ipr4];
	  if (ipl4==0) continue;
	  	  
	  z4 = 0.0;
	  ipl = ipl4;
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	  if ((ic<0)||(ic>50)) continue;
	    
	  MinDist = MINDIST_SPACE;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	    if (Used_Hit[ih]) continue;
	    if (fabs(x*Z_Hit[ih]/zz-HITB_x[ih])<MinDist){
	      MinDist = fabs(x*Z_Hit[ih]/zz-HITB_x[ih]);
	      z4 = Z_Hit[ih];
	      ih4 = ih;
	    }
	  }
	    
	  if (MinDist==MINDIST_SPACE) continue;
	  
	  e_tmp = MinDist;

	  if (N_Points==MAXNUMBEROFPOINTS){
	    printf("CATS : Max number of space points exceeded %d\n", N_Points);
	    goto finish;
	  }
	  N_Points++;
	  X_Point[N_Points] = xx;
	  Y_Point[N_Points] = yy;
	  Z_Point[N_Points] = zz;
	  NSL_Point[N_Points] = ich;
	  SEC_Point[N_Points] = isec;
	  Hit_Point[N_Points][1] = ih1;
	  Hit_Point[N_Points][2] = ih2;
	  Hit_Point[N_Points][3] = ih4;
	  Hit_Point[N_Points][4] = 0;
	  N_Point[N_Points] = 3;
	  Good_Point[N_Points] = (1==1);
	  E_Point[N_Points] = e_tmp;
	  IC_Point[N_Points] = N_Points;
	}
      }

    second_tripple:;

      ipr1=2; 
      ipr2=4;
      ipr3=1;
      ipr4=3;
      
      ipl1 =  Nsl_Sec[ich][isec][ipr1];
      if (ipl1==0) goto end_tripple;
      ipl2 =  Nsl_Sec[ich][isec][ipr2];
      if (ipl2==0) goto end_tripple;

      c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
      c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
      det=c1*s2-c2*s1;

      start1 = fh[ipl1];
      stop1  = lh[ipl1];
      if (stop1<start1) goto end_tripple;
      start2 = fh[ipl2];
      stop2  = lh[ipl2];
      if (stop2<start2) goto end_tripple;

      for (ih1=start1; ih1<=stop1; ih1++){
	if (Used_Hit[ih1]) continue;
	x1 = HITB_x[ih1];
	z1 = Z_Hit[ih1];
	for (ih2 = start2; ih2<=stop2; ih2++){
	if (Used_Hit[ih2]) continue;
	  x2 = HITB_x[ih2];
	  z2 = Z_Hit[ih2];
	  
	/* space point coordinates with correction */

	  xx= (x1*s2*z2/z1-s1*x2)/det;
	  yy=-(x1*c2*z2/z1-c1*x2)/det;
 	  zz = z2;
	  
	  if ( xx < GEDE_sxmn[ipl2]-0.1 ||
	       xx > GEDE_sxmx[ipl2]+0.1 ) continue;
	  if ( yy < GEDE_symn[ipl2]-0.1 ||
	       yy > GEDE_symx[ipl2]+0.1 ) continue;
	  

	  ipl3 =  Nsl_Sec[ich][isec][ipr3];
	  if (ipl3==0) goto label_3;

	  z3 = 0.0;
	  ipl = ipl3;
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	  if ((ic<0)||(ic>50)) goto label_3;
	    
	  MinDist = MINDIST_SPACE;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	    if (Used_Hit[ih]) continue;
	    if (fabs(x*Z_Hit[ih]/zz-HITB_x[ih])<MinDist){
	      MinDist = fabs(x*Z_Hit[ih]/zz-HITB_x[ih]);
	      z3 = Z_Hit[ih];
	      ih3 = ih;
	    }
	  }
	    
	  if (MinDist==MINDIST_SPACE) goto label_3;

	  e_tmp = MinDist;
	  
	  if (N_Points==MAXNUMBEROFPOINTS){
	    printf("CATS : Max number of space points exceeded %d\n", N_Points);
	    goto finish;
	  }
	  N_Points++;
	  X_Point[N_Points] = xx;
	  Y_Point[N_Points] = yy;
	  Z_Point[N_Points] = zz;
	  NSL_Point[N_Points] = ich;
	  SEC_Point[N_Points] = isec;
	  Hit_Point[N_Points][1] = ih1;
	  Hit_Point[N_Points][2] = ih2;
	  Hit_Point[N_Points][3] = ih3;
	  Hit_Point[N_Points][4] = 0;
	  N_Point[N_Points] = 3;
	  Good_Point[N_Points] = (1==1);
	  E_Point[N_Points] = e_tmp;
	  IC_Point[N_Points] = N_Points;

	  continue;

	label_3:;

	  ipl4 =  Nsl_Sec[ich][isec][ipr4];
	  if (ipl4==0) continue;
	  	  
	  z4 = 0.0;
	  ipl = ipl4;
	  x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	  ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	  if ((ic<0)||(ic>50)) continue;
	    
	  MinDist = MINDIST_SPACE;
	  for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	    if (Used_Hit[ih]) continue;
	    if (fabs(x*Z_Hit[ih]/zz-HITB_x[ih])<MinDist){
	      MinDist = fabs(x*Z_Hit[ih]/zz-HITB_x[ih]);
	      z4 = Z_Hit[ih];
	      ih4 = ih;
	    }
	  }
	    
	  if (MinDist==MINDIST_SPACE) continue;
	  
	  e_tmp = MinDist;

	  if (N_Points==MAXNUMBEROFPOINTS){
	    printf("CATS : Max number of space points exceeded %d\n", N_Points);
	    goto finish;
	  }
	  N_Points++;
	  X_Point[N_Points] = xx;
	  Y_Point[N_Points] = yy;
	  Z_Point[N_Points] = zz;
	  NSL_Point[N_Points] = ich;
	  SEC_Point[N_Points] = isec;
	  Hit_Point[N_Points][1] = ih1;
	  Hit_Point[N_Points][2] = ih2;
	  Hit_Point[N_Points][3] = ih4;
	  Hit_Point[N_Points][4] = 0;
	  N_Point[N_Points] = 3;
	  Good_Point[N_Points] = (1==1);
	  E_Point[N_Points] = e_tmp;
	  IC_Point[N_Points] = N_Points;
	}
      }

    end_tripple:;

      /* label 3-points */
      
      //if ((Call==0)||(ich>3))
	for(ip1=np_tmp3+1; ip1 <= N_Points; ip1++){
	  Used_Hit[Hit_Point[ip1][1]] = (1==1);
	  Used_Hit[Hit_Point[ip1][2]] = (1==1);
	  Used_Hit[Hit_Point[ip1][3]] = (1==1);
	}
      

      if (DRAW){ 
	ismksc(1.5);
	ispmci(Green);
	ismk(24);
	
	for(ip=np_tmp3+1; ip <= N_Points; ip++){
	  if (Good_Point[ip])
	    for(ipr=1; ipr <= 4; ipr++){ 
	      
	      ibpl = base_plane[ipr];
	      shift = shift_coeff[ipr]*SHIFT;
	      
	      x = X_Point[ip]*GEDE_cos[ibpl] - Y_Point[ip]*GEDE_sin[ibpl]; 
	      
	      xpl[0] = Z_Point[ip];
	      ypl[0] = x+shift;
	      //ipm(1,xpl,ypl);
	    }
	}    
      }

      np_tmp2 = N_Points;
      
      /* reconstruct space points from 2 hits */
      /* use only perpendicular projections   */

      for (ipr1=1; ipr1 <= 2; ipr1++){
	ipl1 =  Nsl_Sec[ich][isec][ipr1];
	if (ipl1==0) continue;
	c1=GEDE_cos[ipl1]; s1=-GEDE_sin[ipl1];
	start1 = fh[ipl1];
	stop1  = lh[ipl1];
	if (stop1<start1) continue;

	for (ipr2=3; ipr2 <= 4; ipr2++){
	  ipl2 =  Nsl_Sec[ich][isec][ipr2];
	  if (ipl2==0) continue;

	  c2=GEDE_cos[ipl2]; s2=-GEDE_sin[ipl2];
	  det=c1*s2-c2*s1;

	  start2 = fh[ipl2];
	  stop2  = lh[ipl2];
	  if (stop2<start2) continue;

	  for (ih1=start1; ih1<=stop1; ih1++){
	    if (Used_Hit[ih1]) continue;
	    x1 = HITB_x[ih1];
	    z1 = Z_Hit[ih1];
	    for (ih2 = start2; ih2<=stop2; ih2++){
	    if (Used_Hit[ih2]) continue;
	      x2 = HITB_x[ih2];
	      z2 = Z_Hit[ih2];
	      
	      /* space point coordinates with correction */

	      xx= (x1*s2*z2/z1-s1*x2)/det;
	      yy=-(x1*c2*z2/z1-c1*x2)/det;
 	      zz = z2;

	      if ( xx < GEDE_sxmn[ipl2]-0.1 ||
		   xx > GEDE_sxmx[ipl2]+0.1 ) continue;
	      if ( yy < GEDE_symn[ipl2]-0.1 ||
		   yy > GEDE_symx[ipl2]+0.1 ) continue;



	      if (N_Points==MAXNUMBEROFPOINTS){
		printf("CATS : Max number of space points exceeded %d\n", N_Points);
		goto finish;
	      }
	      N_Points++;
	      X_Point[N_Points] = xx;
	      Y_Point[N_Points] = yy;
	      Z_Point[N_Points] = zz;
	      NSL_Point[N_Points] = ich;
	      SEC_Point[N_Points] = isec;
	      Hit_Point[N_Points][1] = ih1;
	      Hit_Point[N_Points][2] = ih2;
	      Hit_Point[N_Points][3] = 0;
	      Hit_Point[N_Points][4] = 0;
	      N_Point[N_Points] = 2;
	      Good_Point[N_Points] = (1==1);
	      E_Point[N_Points] = 0.0;
	      IC_Point[N_Points] = N_Points;
	    }
	  }
	}
      }


      if (DRAW){ 
	ismksc(1.5);
	ispmci(Blue);
	ismk(24);
	
	for(ip=np_tmp2+1; ip <= N_Points; ip++){
	  for(ipr=1; ipr <= 4; ipr++){ 
	    
	    ibpl = base_plane[ipr];
	    shift = shift_coeff[ipr]*SHIFT;
	    
	    x = X_Point[ip]*GEDE_cos[ibpl] - Y_Point[ip]*GEDE_sin[ibpl]; 
	    
	    xpl[0] = Z_Point[ip];
	    ypl[0] = x+shift;
	    //ipm(1,xpl,ypl);
	  }
	}    
      }
    }
  }
  
finish:;

  if (DRAW){ 
    iuwk(1,1);
  }

  for (ich=0; ich<=nchambers; ich++) {
    StartOnPlane[ich] = 0;
    EndOnPlane[ich]   =-1;
  }

  for (ip=1;ip<=N_Points;ip++){
    TrackPoint[ip] = 0;
    FirstPoint[ip] = 0;
    OnTheTrack[ip] = 0;    
    FirstSegment[ip]=0;
    LastSegment[ip]=-1;
    ich=NSL_Point[ip];
    if (StartOnPlane[ich] == 0)
      StartOnPlane[ich] = ip;
    EndOnPlane[ich] = ip;
  }

  NC_Points = N_Points;

}


void KalmanRefit()
{
/****************************************************************
 *                                                              *
 *     This routine provides a procedure for the fast fit of    *
 *     VDS track parameters, using optimized Kalman filter      *
 *                                                              *
 *     (c) Dmitry Emeliyanov, 1999, DESY HH.                    *
 *                                                              *
 ***************************************************************/

  float zf[NPLANES], ze[NPLANES];
  int   i,j,k,ke,gedeP;
  float Rkfs[4], Rkes[4], Rkfl[4], Rkel[4];
  float Kkfs[4], Kkes[4], Kkfl[4], Kkel[4];
  float Gkfs[4][4], Gkes[4][4], Gkfl[4][4], Gkel[4][4];
  float Vfs,Zfs,Vfl,Zfl,sinaf,cosaf,zeta_kfs,zeta_kfl,D_zf,D_z_2f,uf,sumf,z0f;
  float Ves,Zes,Vel,Zel,sinae,cosae,zeta_kes,zeta_kel,D_ze,D_z_2e,ue,sume,z0e;
  float chi2s,chi2l,pinv,sigma_m2,sigma_m_s2_s,sigma_m_s2_l;
  float txy2,coeff,dz212,dz10f,dz20f,dz10e,dz20e;
  float scale_factor2   = 0.00000001;
  float sigma_m_s_small =  3.0;//0.0003;  /* mult scatt  */
  float sigma_m_s_large = 20.0;//0.0020;  /* mult scatt  */
  float sigma_m         = 13.0;//0.0013;  /* measurement */

  txy2 = Xp_trk[Ntk]*Xp_trk[Ntk]+Yp_trk[Ntk]*Yp_trk[Ntk];
  pinv = 4.0*sqrt(txy2);
  coeff = (1.0+txy2)*(1.0+txy2)*(1.0+txy2)*(1.0+txy2)*(1.0+txy2)*
    (1.0+pinv)*(1.0+pinv)*(1.0+pinv)*(1.0+pinv);
  sigma_m_s2_s=sigma_m_s_small*sigma_m_s_small*coeff;
  sigma_m_s2_l=sigma_m_s_large*sigma_m_s_large*coeff;

  sigma_m2 = sigma_m*sigma_m;

  chi2s = chi2l = 0.0;

// first/last hit

  z0f = HITB(ihit[gg[0]]).z;
  z0e = HITB(ihit[gg[gfound-1]]).z;
  Rkfs[0]=Rkfl[0]=Xp_trk[Ntk]*z0f+X0_trk[Ntk];
  Rkes[0]=Rkel[0]=Xp_trk[Ntk]*z0e+X0_trk[Ntk];
  Rkfs[1]=Rkfl[1]=Yp_trk[Ntk]*z0f+Y0_trk[Ntk];
  Rkes[1]=Rkel[1]=Yp_trk[Ntk]*z0e+Y0_trk[Ntk];
  Rkfs[2]=Rkes[2]=Rkfl[2]=Rkel[2]=Xp_trk[Ntk];
  Rkfs[3]=Rkes[3]=Rkfl[3]=Rkel[3]=Yp_trk[Ntk];

//  for(j=0;j<4;j++) for(i=0;i<=j;i++) Gk[i][j]=0.0;
//  Gk[0][0]=Gk[1][1]=(2.0e-3)*(2.0e-3);
//  Gk[2][2]=Gk[3][3]=(5.e-4)*(5.e-4);

  dz212=(Z2-Z1)*(Z2-Z1); dz10f=(Z1-z0f); dz20f=(Z2-z0f); dz10e=(Z1-z0e); dz20e=(Z2-z0e);
  Gkfs[0][0]=Gkfs[1][1]=sigma_m2*(dz20f*dz20f+dz10f*dz10f)/dz212+2.0*sigma_m_s2_s*dz212;
  Gkfl[0][0]=Gkfl[1][1]=sigma_m2*(dz20f*dz20f+dz10f*dz10f)/dz212+2.0*sigma_m_s2_l*dz212;
  Gkes[0][0]=Gkes[1][1]=sigma_m2*(dz20e*dz20e+dz10e*dz10e)/dz212+2.0*sigma_m_s2_s*dz212;
  Gkel[0][0]=Gkel[1][1]=sigma_m2*(dz20e*dz20e+dz10e*dz10e)/dz212+2.0*sigma_m_s2_l*dz212;
  Gkfs[2][2]=Gkfs[3][3]=Gkes[2][2]=Gkes[3][3]=sigma_m2/dz212+2.0*sigma_m_s2_s;
  Gkfl[2][2]=Gkfl[3][3]=Gkel[2][2]=Gkel[3][3]=sigma_m2/dz212+2.0*sigma_m_s2_l;
  Gkfs[0][2]=Gkfs[1][3]=Gkfl[0][2]=Gkfl[1][3]= sigma_m2*(dz20f+dz10f)/dz212;
  Gkes[0][2]=Gkes[1][3]=Gkel[0][2]=Gkel[1][3]=-sigma_m2*(dz20e+dz10e)/dz212;
  Gkfs[0][1]=Gkfs[0][3]=Gkfs[1][2]=Gkfs[2][3]=Gkes[0][1]=Gkes[0][3]=Gkes[1][2]=Gkes[2][3]=0.0;
  Gkfl[0][1]=Gkfl[0][3]=Gkfl[1][2]=Gkfl[2][3]=Gkel[0][1]=Gkel[0][3]=Gkel[1][2]=Gkel[2][3]=0.0;

  for(k=0; k<gfound; k++) {
      uf    = HITB(ihit[gg[k]]).x;
      zf[k] = HITB(ihit[gg[k]]).z;
      gedeP = HITB(ihit[gg[k]]).gede;
      sinaf=GEDE_sin[gedeP];cosaf=GEDE_cos[gedeP];
      D_zf=zf[k]-z0f;D_z_2f=D_zf*D_zf;

      ke = gfound-1-k; // reverse direction
      ue    = HITB(ihit[gg[ke]]).x;
      ze[k] = HITB(ihit[gg[ke]]).z;
      gedeP = HITB(ihit[gg[ke]]).gede;
      sinae=GEDE_sin[gedeP];cosae=GEDE_cos[gedeP];
      D_ze=ze[k]-z0e;D_z_2e=D_ze*D_ze;

      sumf=sume=0.0;
      for(i=0;i<k;i++){
	sumf+=(zf[k]-zf[i])*(zf[k]-zf[i]);
	sume+=(ze[k]-ze[i])*(ze[k]-ze[i]);
      }
      Vfs=sigma_m2+sumf*sigma_m_s2_s;
      Vfl=sigma_m2+sumf*sigma_m_s2_l;
      Ves=sigma_m2+sume*sigma_m_s2_s;
      Vel=sigma_m2+sume*sigma_m_s2_l;

      Zfs=Vfs+cosaf*cosaf*(Gkfs[0][0]+2.*Gkfs[0][2]*D_zf+Gkfs[2][2]*D_z_2f)+
	sinaf*sinaf*(Gkfs[1][1]+2.*Gkfs[1][3]*D_zf+Gkfs[3][3]*D_z_2f)-
	2.*sinaf*cosaf*(Gkfs[0][1]+(Gkfs[0][3]+Gkfs[1][2])*D_zf+Gkfs[2][3]*D_z_2f);
      Zfl=Vfl+cosaf*cosaf*(Gkfl[0][0]+2.*Gkfl[0][2]*D_zf+Gkfl[2][2]*D_z_2f)+
	sinaf*sinaf*(Gkfl[1][1]+2.*Gkfl[1][3]*D_zf+Gkfl[3][3]*D_z_2f)-
	2.*sinaf*cosaf*(Gkfl[0][1]+(Gkfl[0][3]+Gkfl[1][2])*D_zf+Gkfl[2][3]*D_z_2f);
      Zes=Ves+cosae*cosae*(Gkes[0][0]+2.*Gkes[0][2]*D_ze+Gkes[2][2]*D_z_2e)+
	sinae*sinae*(Gkes[1][1]+2.*Gkes[1][3]*D_ze+Gkes[3][3]*D_z_2e)-
	2.*sinae*cosae*(Gkes[0][1]+(Gkes[0][3]+Gkes[1][2])*D_ze+Gkes[2][3]*D_z_2e);
      Zel=Vel+cosae*cosae*(Gkel[0][0]+2.*Gkel[0][2]*D_ze+Gkel[2][2]*D_z_2e)+
	sinae*sinae*(Gkel[1][1]+2.*Gkel[1][3]*D_ze+Gkel[3][3]*D_z_2e)-
	2.*sinae*cosae*(Gkel[0][1]+(Gkel[0][3]+Gkel[1][2])*D_ze+Gkel[2][3]*D_z_2e);

      zeta_kfs=(uf-cosaf*(Rkfs[0]+Rkfs[2]*D_zf)+sinaf*(Rkfs[1]+Rkfs[3]*D_zf));
      zeta_kfl=(uf-cosaf*(Rkfl[0]+Rkfl[2]*D_zf)+sinaf*(Rkfl[1]+Rkfl[3]*D_zf));
      zeta_kes=(ue-cosae*(Rkes[0]+Rkes[2]*D_ze)+sinae*(Rkes[1]+Rkes[3]*D_ze));
      zeta_kel=(ue-cosae*(Rkel[0]+Rkel[2]*D_ze)+sinae*(Rkel[1]+Rkel[3]*D_ze));

      chi2s += zeta_kfs*zeta_kfs/Zfs+zeta_kes*zeta_kes/Zes;
      chi2l += zeta_kfl*zeta_kfl/Zfl+zeta_kel*zeta_kel/Zel;

      Kkfs[0]=cosaf*(Gkfs[0][0]+Gkfs[0][2]*D_zf)-sinaf*(Gkfs[0][1]+Gkfs[0][3]*D_zf);
      Kkfs[1]=cosaf*(Gkfs[0][1]+Gkfs[1][2]*D_zf)-sinaf*(Gkfs[1][1]+Gkfs[1][3]*D_zf);
      Kkfs[2]=cosaf*(Gkfs[0][2]+Gkfs[2][2]*D_zf)-sinaf*(Gkfs[1][2]+Gkfs[2][3]*D_zf);
      Kkfs[3]=cosaf*(Gkfs[0][3]+Gkfs[2][3]*D_zf)-sinaf*(Gkfs[1][3]+Gkfs[3][3]*D_zf);

      Kkfl[0]=cosaf*(Gkfl[0][0]+Gkfl[0][2]*D_zf)-sinaf*(Gkfl[0][1]+Gkfl[0][3]*D_zf);
      Kkfl[1]=cosaf*(Gkfl[0][1]+Gkfl[1][2]*D_zf)-sinaf*(Gkfl[1][1]+Gkfl[1][3]*D_zf);
      Kkfl[2]=cosaf*(Gkfl[0][2]+Gkfl[2][2]*D_zf)-sinaf*(Gkfl[1][2]+Gkfl[2][3]*D_zf);
      Kkfl[3]=cosaf*(Gkfl[0][3]+Gkfl[2][3]*D_zf)-sinaf*(Gkfl[1][3]+Gkfl[3][3]*D_zf);

      Kkes[0]=cosae*(Gkes[0][0]+Gkes[0][2]*D_ze)-sinae*(Gkes[0][1]+Gkes[0][3]*D_ze);
      Kkes[1]=cosae*(Gkes[0][1]+Gkes[1][2]*D_ze)-sinae*(Gkes[1][1]+Gkes[1][3]*D_ze);
      Kkes[2]=cosae*(Gkes[0][2]+Gkes[2][2]*D_ze)-sinae*(Gkes[1][2]+Gkes[2][3]*D_ze);
      Kkes[3]=cosae*(Gkes[0][3]+Gkes[2][3]*D_ze)-sinae*(Gkes[1][3]+Gkes[3][3]*D_ze);

      Kkel[0]=cosae*(Gkel[0][0]+Gkel[0][2]*D_ze)-sinae*(Gkel[0][1]+Gkel[0][3]*D_ze);
      Kkel[1]=cosae*(Gkel[0][1]+Gkel[1][2]*D_ze)-sinae*(Gkel[1][1]+Gkel[1][3]*D_ze);
      Kkel[2]=cosae*(Gkel[0][2]+Gkel[2][2]*D_ze)-sinae*(Gkel[1][2]+Gkel[2][3]*D_ze);
      Kkel[3]=cosae*(Gkel[0][3]+Gkel[2][3]*D_ze)-sinae*(Gkel[1][3]+Gkel[3][3]*D_ze);

      zeta_kfs=zeta_kfs/Zfs;
      zeta_kfl=zeta_kfl/Zfl;
      zeta_kes=zeta_kes/Zes;
      zeta_kel=zeta_kel/Zel;
      for(j=0;j<4;j++){
	Rkfs[j]+=Kkfs[j]*zeta_kfs;
	Rkfl[j]+=Kkfl[j]*zeta_kfl;
	Rkes[j]+=Kkes[j]*zeta_kes;
	Rkel[j]+=Kkel[j]*zeta_kel;
	for(i=0;i<=j;i++){
	  Gkfs[i][j]-=Kkfs[i]*Kkfs[j]/Zfs;
	  Gkfl[i][j]-=Kkfl[i]*Kkfl[j]/Zfl;
	  Gkes[i][j]-=Kkes[i]*Kkes[j]/Zes;
	  Gkel[i][j]-=Kkel[i]*Kkel[j]/Zel;
	}
      }

  }


  if (((Rkfs[2]-Xp_trk[Ntk])*(Rkfs[2]-Xp_trk[Ntk])/(Gkfs[2][2]*scale_factor2)<1.0)&&
      ((Rkes[2]-Xp_trk[Ntk])*(Rkes[2]-Xp_trk[Ntk])/(Gkes[2][2]*scale_factor2)<1.0)&&
      ((Rkfs[3]-Yp_trk[Ntk])*(Rkfs[3]-Yp_trk[Ntk])/(Gkfs[3][3]*scale_factor2)<1.0)&&
      ((Rkes[3]-Yp_trk[Ntk])*(Rkes[3]-Yp_trk[Ntk])/(Gkes[3][3]*scale_factor2)<1.0)){
    X0f_trk[Ntk]=Rkfs[0]-Rkfs[2]*z0f;
    Y0f_trk[Ntk]=Rkfs[1]-Rkfs[3]*z0f;
    Xpf_trk[Ntk]=Rkfs[2];
    Ypf_trk[Ntk]=Rkfs[3];
    
    Cf_trk[10*Ntk+0] = Gkfs[0][0]*scale_factor2;
    Cf_trk[10*Ntk+1] = Gkfs[0][1]*scale_factor2;
    Cf_trk[10*Ntk+2] = Gkfs[1][1]*scale_factor2;
    Cf_trk[10*Ntk+3] = Gkfs[0][2]*scale_factor2;
    Cf_trk[10*Ntk+4] = Gkfs[1][2]*scale_factor2;
    Cf_trk[10*Ntk+5] = Gkfs[2][2]*scale_factor2;
    Cf_trk[10*Ntk+6] = Gkfs[0][3]*scale_factor2;
    Cf_trk[10*Ntk+7] = Gkfs[1][3]*scale_factor2;
    Cf_trk[10*Ntk+8] = Gkfs[2][3]*scale_factor2;
    Cf_trk[10*Ntk+9] = Gkfs[3][3]*scale_factor2;
    
    X0e_trk[Ntk]=Rkes[0]-Rkes[2]*z0e;
    Y0e_trk[Ntk]=Rkes[1]-Rkes[3]*z0e;
    Xpe_trk[Ntk]=Rkes[2];
    Ype_trk[Ntk]=Rkes[3];
    
    Ce_trk[10*Ntk+0] = Gkes[0][0]*scale_factor2;
    Ce_trk[10*Ntk+1] = Gkes[0][1]*scale_factor2;
    Ce_trk[10*Ntk+2] = Gkes[1][1]*scale_factor2;
    Ce_trk[10*Ntk+3] = Gkes[0][2]*scale_factor2;
    Ce_trk[10*Ntk+4] = Gkes[1][2]*scale_factor2;
    Ce_trk[10*Ntk+5] = Gkes[2][2]*scale_factor2;
    Ce_trk[10*Ntk+6] = Gkes[0][3]*scale_factor2;
    Ce_trk[10*Ntk+7] = Gkes[1][3]*scale_factor2;
    Ce_trk[10*Ntk+8] = Gkes[2][3]*scale_factor2;
    Ce_trk[10*Ntk+9] = Gkes[3][3]*scale_factor2;
    
    Chi2_trk[Ntk] = chi2s/2.0;
  }    
  else{
    X0f_trk[Ntk]=Rkfl[0]-Rkfl[2]*z0f;
    Y0f_trk[Ntk]=Rkfl[1]-Rkfl[3]*z0f;
    Xpf_trk[Ntk]=Rkfl[2];
    Ypf_trk[Ntk]=Rkfl[3];
    
    Cf_trk[10*Ntk+0] = Gkfl[0][0]*scale_factor2;
    Cf_trk[10*Ntk+1] = Gkfl[0][1]*scale_factor2;
    Cf_trk[10*Ntk+2] = Gkfl[1][1]*scale_factor2;
    Cf_trk[10*Ntk+3] = Gkfl[0][2]*scale_factor2;
    Cf_trk[10*Ntk+4] = Gkfl[1][2]*scale_factor2;
    Cf_trk[10*Ntk+5] = Gkfl[2][2]*scale_factor2;
    Cf_trk[10*Ntk+6] = Gkfl[0][3]*scale_factor2;
    Cf_trk[10*Ntk+7] = Gkfl[1][3]*scale_factor2;
    Cf_trk[10*Ntk+8] = Gkfl[2][3]*scale_factor2;
    Cf_trk[10*Ntk+9] = Gkfl[3][3]*scale_factor2;
    
    X0e_trk[Ntk]=Rkel[0]-Rkel[2]*z0e;
    Y0e_trk[Ntk]=Rkel[1]-Rkel[3]*z0e;
    Xpe_trk[Ntk]=Rkel[2];
    Ype_trk[Ntk]=Rkel[3];
    
    Ce_trk[10*Ntk+0] = Gkel[0][0]*scale_factor2;
    Ce_trk[10*Ntk+1] = Gkel[0][1]*scale_factor2;
    Ce_trk[10*Ntk+2] = Gkel[1][1]*scale_factor2;
    Ce_trk[10*Ntk+3] = Gkel[0][2]*scale_factor2;
    Ce_trk[10*Ntk+4] = Gkel[1][2]*scale_factor2;
    Ce_trk[10*Ntk+5] = Gkel[2][2]*scale_factor2;
    Ce_trk[10*Ntk+6] = Gkel[0][3]*scale_factor2;
    Ce_trk[10*Ntk+7] = Gkel[1][3]*scale_factor2;
    Ce_trk[10*Ntk+8] = Gkel[2][3]*scale_factor2;
    Ce_trk[10*Ntk+9] = Gkel[3][3]*scale_factor2;
    
    Chi2_trk[Ntk] = chi2l/2.0;
  }

  Pinv_trk[Ntk] = pinv;

  return;   
}
  







#ifdef XXX
void TrackRefit()
{/*============================================================
   refit a track (copy of holmes' refit to be compatible)
   ============================================================
   */
  int    i, j, ih, n, gedeP;
  double u, x, y, tx, ty, chi2;
  double da[16], db[4];
  double scale;  /* = MINDIST*MINDIST; */
  float  v[4];     /* projector for track vectors on local coordinate */
  float  vv[16];   /* projector on v */
  int    work[4];
  int    ifail;
  float  singlehitres = 0.0010;     /* single hit resolution           */
  float  mscatthitres = 0.0200;     /* multiple scattering resolution  */
  float  estimateres;              /* estimated resolution            */

  /*----------------------------------------------- */

  estimateres = singlehitres + mscatthitres*
    sqrt(Xp_trk[Ntk]*Xp_trk[Ntk]+Yp_trk[Ntk]*Yp_trk[Ntk])/0.25;

  scale = estimateres*estimateres;

  // fill weight matrix and constraint vector

  for(i=0; i<16; i++) da[i] = 0;
  for(i=0; i<4;  i++) db[i] = 0;

  chi2 = 0;
  for(ih=0; ih<gfound; ih++) {
    u     = HITB(ihit[gg[ih]]).x;
    gedeP = HITB(ihit[gg[ih]]).gede;
    v[0]  =  GEDE_cos[gedeP];
    v[1]  = -GEDE_sin[gedeP];
    v[2]  =  GEDE_cos[gedeP]*zpl[gedeP];
    v[3]  = -GEDE_sin[gedeP]*zpl[gedeP];
    for(i=0; i<4; i++) {
      for(j=0; j<4; j++) {
        vv[4*i+j] = v[i]*v[j];
      }
    }
    for(j=0; j<16; j++) da[j] += (double)(vv[j]);
    for(j=0; j<4;  j++) db[j] += (double)(v[j]) * u;
    chi2 += u*u;
  }

  // get the covariance matrix

  n = 4;
  dinv_(&n, da, &n, work, &ifail);
  if(ifail) {
     printf("CATS: track refit failed\n");
     return;
  }


  // determine the fit parameters

  x = y = tx = ty = 0.;
  for(i=0; i<4; i++) {
    x  += da[4*0+i]*db[i];
    y  += da[4*1+i]*db[i];
    tx += da[4*2+i]*db[i];
    ty += da[4*3+i]*db[i];
  }

  // fill the covariance matrix

  n=0;
  for(i=0; i<4; i++) {
    for(j=0; j<=i; j++) {
      Cf_trk[10*Ntk+n] = Ce_trk[10*Ntk+n] = scale*da[4*i+j];
      n++;
    }
  }

  // calculate the chi2

  chi2 = (chi2 - db[0]*x - db[1]*y - db[2]*tx - db[3]*ty)/scale;
  if(chi2 < 0){
    chi2 = 0.0;
  }


  Chi2_trk[Ntk] = chi2;

  // finally fill track parameters

  //store found track parameters

  X0f_trk[Ntk]=X0e_trk[Ntk]=x;
  Y0f_trk[Ntk]=Y0e_trk[Ntk]=y;
  Xpf_trk[Ntk]=Xpe_trk[Ntk]=tx;
  Ypf_trk[Ntk]=Ype_trk[Ntk]=ty;


}
#endif



void fill_cats(){
  
  int   ih, ipl, ipr, is, i;
  float local[5], global[5];

  if (cats_first==10)
    printf("CATS version 3.12 started\n");
  else
    printf("CATS version 3.12 filled\n");

  ntargets = 0;
  Z_TARGET = 0.0;
  global[3] = 0.; global[4] = 0.; /* for transformation of target wires as space points */
  for(ih=1; ih <=NGTAR(); ih++){
    if (GTAR(ih).wnr!=0){
      ntargets++;
      itarget[ntargets] = ih;
      Z_TARGET = Z_TARGET + GTAR(ih).z;
      if ((DRAW)&&(cats_first==10)){
	printf("Target %1d wire %1d (dx %5.2f - dy %5.2f), global (%5.2f, %5.2f, %5.2f) ",
	       ntargets, GTAR(ih).wnr, GTAR(ih).dx, GTAR(ih).dy, 
	       GTAR(ih).x, GTAR(ih).y, GTAR(ih).z);
	global[0] = GTAR(ih).x; global[1] = GTAR(ih).y; global[2] = GTAR(ih).z;
	acglobaltolocal(local, global, 1);
	printf("local (%5.2f, %5.2f, %5.2f) coordinates\n",
	       local[0], local[1], local[2]);
      }
    }
  }
  if (ntargets!=0) Z_TARGET = Z_TARGET/ntargets;
  
  nchambers = 0;
  for(ih=1; ih <= 20; ih++){
    if (GESL(ih).cmp==1){
      nchambers++;     
      ichamber[nchambers] = ih;
      if ((DRAW)&&(cats_first==10)){
      printf("%d (%d) cmp= %d, nsl= %d, lay= %d, npl= %d, fpl= %d z= %f\n", 
	     nchambers, ih, GESL(ih).cmp, GESL(ih).nsl, GESL(ih).lay, 
	     GESL(ih).npl, GESL(ih).fpl, GESL(ih).z1);
      }
      z_nsl[nchambers] = GEDE(GESL(ih).fpl).zfw[0];
      //      printf("Z of SL %1d = %5.2f\n", nchambers, z_nsl[nchambers]);
      for(is=1; is <= 4; is++){
	for(ipr=1; ipr <= 4; ipr++){
	  Nsl_Sec[nchambers][is][ipr] = 0;
	}
      }
      nplanes= GESL(ih).fpl+GESL(ih).npl-1;
    }
    if (GESL(ih).cmp==0) goto no_more_chambers;
  }
no_more_chambers:;


  for(ipl=1; ipl <= nplanes; ipl++){ 

    if ( GEDE(ipl).cmp != 1 ) 
      printf("CATS : ******* WRONG IINITIALIZATION *******\n");

    /*  moved to fill_GEDE_array */

    GEDE_cmp[ipl] = GEDE(ipl).cmp;
    GEDE_nsl[ipl] = GESL(GEDE(ipl).ogesl).nsl;
    GEDE_sec[ipl] = GEDE(ipl).sec;
    GEDE_lay[ipl] = GEDE(ipl).lay;
    GEDE_nwi_x_pit[ipl] = GEDE(ipl).nwi[0]*GEDE(ipl).pit;

    /* Determine the "back" of each plane */

    back[ipl] = 0;
    for (ih=1; ih <= nplanes; ih++){
      if ( GEDE(ih).sec == GEDE(ipl).sec &&
	   GESL(GEDE(ih).ogesl).nsl == GESL(GEDE(ipl).ogesl).nsl ) {
	if ((GEDE(ih).lay == 1 && GEDE(ipl).lay ==2) ||
	    (GEDE(ih).lay == 2 && GEDE(ipl).lay ==1) ||
	    (GEDE(ih).lay == 3 && GEDE(ipl).lay ==4) ||
	    (GEDE(ih).lay == 4 && GEDE(ipl).lay ==3) ){
	  back[ipl] = ih;
	  break;
	}
      }
    }

    //printf("Plane  %3d  has as friend  %3d \n", ipl, back[ipl]);

  }
  back[0] = 0;

  fill_GEDE_array();


  if (ind_pl!=0){
    /* Inizialitation for the efficiency part */
    neve = 0;
    for(ih=0; ih < 26; ih++){ 
      ntr_pl[ih] = 0;
      found_pl[ih] = 0;
      found2_pl[ih] = 0;
      found3_pl[ih] = 0;
      for(i=1;i<=10;i++){
	found3_ch[ih][i] = 0;
	ntr_ch[ih][i] = 0;
      }
    }
  }

  if ((ind_pl!=0)&&(cats_first==10)){

    hbook1(230,"Residuals, tracks with more than 7 hits",200,-0.2, 0.2);
    hbook1(231,"Residuals, tracks with more than 9 hits",200,-0.2, 0.2);
    hbook1(235,"Residuals, tracks with 5 hits only",100,-0.2, 0.2);
    hbook1(236,"Residuals, tracks with 6 hits only",100,-0.2, 0.2);
    hbook1(237,"Residuals, tracks with 7 hits only",100,-0.2, 0.2);
    hbook1(238,"Residuals, tracks with 8 hits only",100,-0.2, 0.2);
    hbook1(239,"Residuals, tracks with 9 hits only",100,-0.2, 0.2);
    hbook1(240,"Residuals, tracks with 10 hits only",100,-0.2, 0.2);
    hbook1(302,"Strip number - found",1664,-128.5,1535.5);
    hbook1(303,"Strip number - not found",1664,-128.5,1535.5);
    hbook1(305,"Strip number",13,-128.5,1535.5);
    hbook2(800,"Effective detector point", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(801,"Non effective detector point", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook1(802,"Effective track length", 20, 0., 20.);
    hbook1(803,"Non effective track length", 20, 0., 20.);
    hbook1(804,"Effective target dist", 100, 0., 1.);
    hbook1(805,"Non effective target dist", 100, 0., 1.);
    
  }
    

#ifdef TARGET
  if (cats_first==10){
    hbook1(200,"Vertex z - target z",400,-1.0, 1.0);
    hbook1(201,"Target z - MVER z",200,-1.0, 1.0);
    hbook2(202,"Targets (x,y)", 200,-1.0, 1.0, 200,-1.0, 1.0);
    hbook2(203,"Effective number of primary tracks",20, 0.0, 20.0, 20, 0.0, 20.0);
    hbook1(204,"Number of vertices", 10, 0.0, 10.0);
    hbook1(205,"Thomas x correction", 100, -1.0, 1.0);
    hbook1(206,"Thomas nt correction", 100, -10.0, 10.0);
    hbook1(207,"Thomas z correction", 100, -1.0, 1.0);
    hbook1(208,"(Thomas) Vertex z - target z",400,-1.0, 1.0);
    hbook1(209,"Number of tracks/ev WITH vertex", 30, 0.0, 30.0);
    hbook1(210,"Number of tracks/ev NO vertex", 30, 0.0, 30.0);
    
    hbook1(400,"Hits per good track",20, 0.0, 20.0);
    hbook1(401,"SL of good tracks",10, 0.0, 10.0);
    hbook1(402,"Space points of good tracks",10, 0.0, 10.0);
    hbook1(403,"Strongest space point on good tracks",10, 0.0, 10.0);
    hbook1(404,"Perpendicual projections of good tracks",10, 0.0, 10.0);
    hbook2(405,"Slopes of good tracks", 100, 0.0, 0.1, 100, 0.0, 0.1);
    hbook1(406,"Quality of good tracks", 100, -40.0, 40.0);
    hbook1(407,"Distance for good track", 100, 0.0, 1.0);
    
    hbook1(411,"Hits per MIMP",10, 0.0, 10.0);
    
    hbook1(430,"Hits per ghost track",20, 0.0, 20.0);
    hbook1(431,"SL of ghost tracks",10, 0.0, 10.0);
    hbook1(432,"Space points of ghost tracks",10, 0.0, 10.0);
    hbook1(433,"Strongest space point on ghost tracks",10, 0.0, 10.0);
    hbook1(434,"Perpendicual projections of ghost tracks",10, 0.0, 10.0);
    hbook2(435,"Slopes of ghost tracks", 100, 0.0, 0.1, 100, 0.0, 0.1);
    hbook1(436,"Quality of ghost tracks", 100, -40.0, 40.0);
    hbook1(437,"Distance for ghost track", 100, 0.0, 1.0);
    
    hbook2(500,"All tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(501,"1sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(502,"2sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(503,"3sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(504,"4sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(505,"5sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(506,"6sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(507,"7sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(508,"8sl tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(509,"Reg tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    hbook2(510,"Acc tracks at the SL1", 100, -10.0, 10.0, 100, -10.0, 10.0);
    
    hbook1(600,"Not rec with N nsl", 10, 0.0, 10.0);
    hbook1(601,"Not rec with N ndet", 10, 0.0, 10.0);
    hbook1(602,"Not rec with N mimps", 10, 0.0, 10.0);
    hbook1(603,"Not rec with vertex z pos and more than 2 SP", 210, -10.0, 200.0);
    hbook1(604,"Not rec with first nsl and more than 2 SP", 10, 0.0, 10.0);
    hbook1(605,"Not rec with last nsl and more than 2 SP", 10, 0.0, 10.0);
    hbook1(606,"Not rec with dist nsl and more than 2 SP", 10, 0.0, 10.0);
    hbook1(607,"Not rec with momentum and more than 2 SP", 100, 0.0, 10.0);
    hbook1(608,"Not rec with dist from vert 0 and more than 2 SP", 100, 0.0, 10.0);
    hbook1(609,"Not reco with N hitb", 10, 0.0, 10.0);
    hbook1(610,"Not rec with N space points", 10, 0.0, 10.0);
    hbook1(611,"Not rec with N hitb and more than 2 SP", 10, 0.0, 10.0);
    hbook1(612,"Not rec with diff sec and more than 2 SP", 10, 0.0, 10.0);
    hbook1(613,"Not rec with tot res HITB and more than 2 SP", 100, 0.0, 0.1);
    hbook1(614,"Not rec with loc res MIMP and more than 2 SP", 100, 0.0, 0.1);
    hbook1(615,"Not rec with tot res MIMP and more than 2 SP", 100, 0.0, 0.1);
    hbook1(616,"Not rec with qual and more than 2 SP", 100, 0.0, 30.0);
    
    hbook1(617,"Ghost with quality factor", 100, 0.0, 30.0);
    hbook1(618,"Not reco with quality factor", 100, 0.0, 30.0);
    hbook1(619,"Reco with quality factor", 100, 0.0, 30.0);
    
    hbook1(620,"Ghost with n hits", 100, 0.0, 20.0);
    hbook1(621,"Not rec with n hits", 100, 0.0, 20.0);
    hbook1(622,"Rec with n hits", 100, 0.0, 20.0);
    hbook1(623,"Not rec with momentum", 100, 0.0, 10.0);
    hbook1(624,"Rec with momentum", 100, 0.0, 10.0);
    
    hbook1(625,"Ghost first dist", 100, 0.0, 10.0);
    hbook1(626,"Not rec first dist", 100, 0.0, 10.0);
    hbook1(627,"Rec first dist", 100, 0.0, 10.0);
    
    hbook1(628,"Ghost last dist", 100, 0.0, 10.0);
    hbook1(629,"Not rec last dist", 100, 0.0, 10.0);
    hbook1(630,"Rec last dist", 100, 0.0, 10.0);
    
    hbook2(700,"Simu vertex (x, y) distribution",400,-1.0, 1.0, 400,-1.0, 1.0);
    hbook1(701,"MVER.z - target.z",200,-0.1, 0.1);
    hbook1(702,"MVER.x(y) - targets.x(y)", 200,-0.1, 0.1);
    hbook1(703,"Wire interacted",100, 0.0, 10.0);
    hbook1(704,"Reco.x - MVER.x",200,-0.1, 0.1);
    hbook1(705,"Reco.y - MVER.y",200,-0.1, 0.1);
    hbook1(706,"Reco.z - MVER.z",200,-0.1, 0.1);
    
    hbook1(710, "[h]^2! confidence level" , 50, 0., 1.);
    hbook1(711, "simu [h]^2! " , 50, 0., 5.);
    hbook1(712, "reco [h]^2! " , 50, 0., 5.);
    hbook2(713, "simu [h]^2! on p", 20, 0.0, 20.0, 50, 0.0, 5.0);
    hbook1(714, "simu [h]^2! confidence level" , 50, 0., 1.);
    
    hbook2(720, "ty/tx", 100, -0.1, 0.1, 100, -0.1, 0.1);
  }  
#endif

  if ((DRAW)&&(cats_first==10)){
    graph_ini();
  }


}






void stop_cats(){

  if(DRAW){
    graph_close();
  }
}


void Draw_Event(){
  
  int   iproj, i, ih, ipl;
  
  
  /* test data only in a projection corresponding to this plane slope */
  
  
  for(iproj=1; iproj <= 4; iproj++){ 
    
    shift = shift_coeff[iproj]*SHIFT;
       
    isfaci(White);
    isplci(Red);
    igbox(-15.0,220.0,-10.0+shift,10.0+shift);
       
       /* draw the targets - only their centers - how to project them?.. */

    ispmci(Red);
    ismk(3);
    ismksc(0.5);
    
    for (i=1; i<=ntargets; i++){
      xpl[0] = GTAR(itarget[i]).z;
      ypl[0] = GTAR(itarget[i]).x+shift;
      ipm(1,xpl,ypl);
    }
    
    /* draw layers */
    
    ispmci(Black);
    ismk(3);
    ismksc(0.5);
    

    isplci(Black);
    for (i=1; i<=nchambers; i++){
      ih = ichamber[i];
      for (ipl=GESL(ih).fpl; ipl<GESL(ih).fpl+GESL(ih).npl; ipl++){

        if (GEDE(ipl).ogesl!=0){

          if (npr[ipl]==npr[base_plane[iproj]]){
            xpl[0] = GEDE(ipl).zmin;
            ypl[0] = coeff[ipl]*GEDE(ipl).xfw[0]+shift;
            xpl[1] = GEDE(ipl).zmin;
            ypl[1] = coeff[ipl]*(GEDE(ipl).xfw[0]+
                                 GEDE(ipl).nwi[0]*GEDE(ipl).pit)+shift;
	    if (ipl==ind_pl) 
	      isplci(Magenta);
	    else
	      if (ipl==ind_ba) 
		isplci(Yellow);
	      else
		isplci(Black);
	  ipl(2,xpl,ypl);
          }

        }
      }
    }

  }
  iuwk(1,1);
  
  
}



/***************** Graphics ******************/



/******************************************/
/************ graph_ini() ******************/
/******************************************/



void graph_ini(){
//  int nxpaw=NXPAW;
//  hlimit_(&nxpaw); 

  iginit(0);
  igsse(0, 8);


  iscr(1, Black       ,0.0000, 0.0000, 0.0000);
  iscr(1, DarkGray    ,0.5000, 0.5400, 0.5300);
  iscr(1, Blue        ,0.0000, 0.0000, 1.0000);
  iscr(1, Green       ,0.1333, 0.5451, 0.1333);
  iscr(1, Cyan        ,0.0000, 0.8078, 0.8196);
  iscr(1, Red         ,1.0000, 0.0000, 0.0000);
  iscr(1, Magenta     ,1.0000, 0.0000, 1.0000);
  iscr(1, Brown       ,0.5000, 0.1647, 0.1647);
  iscr(1, LightBlue   ,0.6784, 0.8471, 0.9020);
  iscr(1, LightGreen  ,0.5961, 0.9843, 0.5961);
  iscr(1, LightCyan   ,0.4980, 1.0000, 0.8314);
  iscr(1, LightRed    ,0.9804, 0.5020, 0.4471);
  iscr(1, LightMagenta,0.8549, 0.4392, 0.8392);
  iscr(1, Yellow      ,1.0000, 1.0000, 0.0000);
  iscr(1, LightGray   ,0.8275, 0.8275, 0.8275);
  iscr(1, White       ,1.0000, 1.0000, 1.0000);



  iswn(2,-15.00,220.00,-40.00,40.00);
  isvp(2,0.0,1.0,0.0,1.0);
  iselnt(2);

  isfais(1);

  isfaci(LightGray);
  isplci(Red);

  igbox(-15.00,220.00,-40.00,40.00);

  iuwk(1,1);

}
/**************************************************/
/************** graph_close() *********************/
/**************************************************/

void graph_close(){
  igend();
}















