//
//  Authors:  Silvia Masciocchi                     Created:  05-Feb-99
//            Ivan Kisel                            Modified: 07-Jun-99
//            Max-Planck-Institut fuer Physik                 01-Jul-99
//            Munchen                                         19-Sep-99
//                                                            25-Oct-99
//                                                            11-Dec-99
//                                                            23-Jan-00
//  Email:    sma@mppmu.mpg.de ikisel@mppmu.mpg.de  HERA-B Collaboration
//
// fill_banks.C
// VDS reconstruction program - filling banks after tracking program Cats.
// Called from catsevnt().
//
 
//!!!!!!!!!!!!!!!!!!!!!!
using namespace std;//!!
//!!!!!!!!!!!!!!!!!!!!!!

#include <vector>

#include <iostream.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <arte/arte.hh>
#include "arte/RSEG_cmp.hh"
//#include <rang/rgfit.hh>
//#include "vdsFit.hh"

#include "Class_MGEDE.hh"

#include "arte/Chrono.hh"

//extern Chrono zeit;

int acGlobalToLocal(float *, float *, int);

const MGede& LoadGede(void);

#define NCHAMBERS  10
#define NPLANES   200

extern int   npr[NPLANES];
extern float zpl[NPLANES], coeff[NPLANES];

extern int   Nsl_Sec[NCHAMBERS+1][5][5];

extern int   GEDE_cmp[NPLANES],  GEDE_nsl[NPLANES],
  GEDE_sec[NPLANES],  GEDE_lay[NPLANES];
extern float GEDE_sxmn[NPLANES], GEDE_sxmx[NPLANES];
extern float GEDE_symn[NPLANES], GEDE_symx[NPLANES];
extern float GEDE_cos[NPLANES],  GEDE_sin[NPLANES]; 
extern float GEDE_xfw[NPLANES],  GEDE_nwi_x_pit[NPLANES];

extern int   base_plane[5];


extern "C" {

  /*
  void time_start(){
    zeit.start();
  }


  void time_stop(){
    zeit.stop();
  }
  */

  int acglobaltolocal(float * rLocal, float * rGlobal, int cmp){

    return  acGlobalToLocal(rLocal, rGlobal, cmp);
  }

  void fill_GEDE_array(void) {
    int   ipl;
    float alpha;


    MGede gedeMap = LoadGede();
    
    /* use MGedeIterator */
    for(MGedeIterator pos = gedeMap.begin(); pos!=gedeMap.end(); ++pos) {
      //cout << *pos;
      
      ipl = pos->gede();
      
      GEDE_xfw[ipl] = pos->dmin(); 
      zpl[ipl] = pos->z();
      
      
      GEDE_cos[ipl] = pos->c();
      GEDE_sin[ipl] = pos->s();

      GEDE_sxmn[ipl] = pos->sxmn()-0.1;
      GEDE_sxmx[ipl] = pos->sxmx()+0.1;
      
      GEDE_symn[ipl] = pos->symn()-0.1;
      GEDE_symx[ipl] = pos->symx()+0.1;
      
      alpha  = pos->alpha();
      npr[ipl] = 0;
      
      if(  80.0 < alpha && alpha <   90.0 ){
	npr[ipl]   =  1;
	coeff[ipl] =  1.0;
	base_plane[1] = ipl;
      }
      if( 260.0 < alpha && alpha <  270.0 ){
	npr[ipl]   =  1;
	coeff[ipl] = -1.0;
	}
      
      if(  90.0 < alpha && alpha <  100.0 ){
	npr[ipl]   =  2;
	coeff[ipl] =  1.0;
	base_plane[2] = ipl;
      }
      if( 270.0 < alpha && alpha <  280.0 ){
	npr[ipl]   =  2;
	coeff[ipl] = -1.0;
      }
      
      if( 170.0 < alpha && alpha <  180.0 ){
	npr[ipl]   =  3;
	coeff[ipl] =  1.0;
	base_plane[3] = ipl;
      }
      if( 350.0 < alpha && alpha <  360.0 ){
	npr[ipl]   =  3;
	coeff[ipl] = -1.0;
      }
      if( -10.0 < alpha && alpha <    0.0 ){
	npr[ipl]   =  3;
	coeff[ipl] = -1.0;
      }
      
      if( 180.0 < alpha && alpha <  190.0 ){
	npr[ipl]   =  4;
	coeff[ipl] =  1.0;
	base_plane[4] = ipl;
      }
      if(   0.0 < alpha && alpha <   10.0 ){
	npr[ipl]   =  4;
	coeff[ipl] = -1.0;
      }
      
      if (npr[ipl]==0)
	cout<<"*** CATS: Wrong angle for the plane "<<ipl<<": "<<alpha<<endl; 
      
      Nsl_Sec[GEDE_nsl[ipl]][GEDE_sec[ipl]][npr[ipl]] = ipl;
    }
  }
  
  int fill_mimp_points(int n_mtra, 
		  float *mimp_x, float *mimp_y, float *mimp_z){
    int nmimps;

    nmimps = 0;
    ArteTable<MTRA>::iterator mtra = (ArteTable<MTRA>::iterator) n_mtra;
    for( ArteRelationIterator<MIMP> mimp = mtra->mimp.begin();
	 mimp != mtra->mimp.end(); ++mimp ){
      if ((*mimp)->gede->cmp != 1 ) continue;
      nmimps++;
      
      mimp_x[nmimps] = (*mimp)->x;
      mimp_y[nmimps] = (*mimp)->y;
      mimp_z[nmimps] = (*mimp)->z;
    }

    return nmimps;
  }



  void fill_banks(int Ntk, int *nh_trk, 
		  float *Xpf_trk,float *X0f_trk,float *Ypf_trk,float *Y0f_trk, 
		  float *Cf_trk, 
		  float *Xpe_trk,float *X0e_trk,float *Ype_trk,float *Y0e_trk, 
		  float *Ce_trk, 
		  float *Chi2_trk, float *Pinv_trk, int *hits){
    
    int nh, itk, ih, i;
    vector<ArtePointer<RHIT> > rhits;
    RSEG r;
    RHIT rh;
   
    /*fill the RSEG table*/
    
    nh = 0;
    for (itk=0;itk<Ntk;itk++){ // loop over 3D tracks

      // a track is parametrized as a line y(or x)=A*z+B in projections
      // for x proj. A = Xp_trk[itk]; B = X0_trk[itk];   
      // for y proj. A = Yp_trk[itk]; B = Y0_trk[itk];
      
      r.zf = ArtePointer<HITB>(hits[nh])->z;
      r.xf = Xpf_trk[itk]*r.zf + X0f_trk[itk];
      r.yf = Ypf_trk[itk]*r.zf + Y0f_trk[itk];
      r.txf = Xpf_trk[itk];
      r.tyf = Ypf_trk[itk];

      r.ze = ArtePointer<HITB>(hits[nh+nh_trk[itk]-1])->z;
      r.xe = Xpe_trk[itk]*r.ze + X0e_trk[itk];
      r.ye = Ype_trk[itk]*r.ze + Y0e_trk[itk];
      r.txe = Xpe_trk[itk];
      r.tye = Ype_trk[itk];

      r.pf  = r.pe  = Pinv_trk[itk];

      for (i= 0; i<10; ++i ){
	r.cvf[i] = Cf_trk[10*itk+i];
	r.cve[i] = Ce_trk[10*itk+i];
      }
      for (i=10; i<14; ++i ) r.cvf[i] = r.cve[i] = 0.0;
      r.cvf[14] = r.cve[14] = 100;

      r.time = 0.0; 
      r.dtim = 0.0; 
      r.chi  = Chi2_trk[itk]; 
      r.fit  = 0; 
      r.cmp  = 1; 
      r.cmp  = r.cmp | Rsegc::bit16;
       
      ArteTable<RSEG>::push_back( r );
      ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );

      for (ih=nh+nh_trk[itk]-1; ih>=nh; ih--){
	ArtePointer<HITB> hitb (hits[ih]);
	createRelation( hitb, rseg, &RSEG::hitb );
      }
      
      int number_of_rhits = 0;
      ArteRelationIterator<HITB> hitb_begin = rseg->hitb.begin();
      ArteRelationIterator<HITB> hitb_end   = rseg->hitb.end();
      for( ArteRelationIterator<HITB> hitb = hitb_begin;
           hitb != hitb_end; ++hitb ) {

        rh.x  = rseg->xf+((*hitb)->z-rseg->zf)*rseg->txf;
        rh.y  = rseg->yf+((*hitb)->z-rseg->zf)*rseg->tyf;
        rh.tx = rseg->txf;
        rh.ty = rseg->tyf;
        rh.p  = rseg->pf;
        rh.q  = 0;
        ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back( rh );

        // createRelation( rseg,  rhit, &RSEG::rhit );
        rhits.push_back( rhit );
        createRelation( *hitb, rhit, &HITB::rhit );

	number_of_rhits++;
      }

      for (ih=number_of_rhits; ih>=1; ih--){
	int hitb_num = 0;
	ArteRelationIterator<HITB> hitb_begin = rseg->hitb.begin();
	ArteRelationIterator<HITB> hitb_end   = rseg->hitb.end();
	for( ArteRelationIterator<HITB> hitbIt = hitb_begin;
	     hitbIt != hitb_end; ++hitbIt ) {
	  hitb_num++;
	  if (hitb_num == ih){
	    ArteRelationIterator<RHIT> rhit_begin = (*hitbIt)->rhit.begin();
	    ArteRelationIterator<RHIT> rhit_end   = (*hitbIt)->rhit.end();
	    for( ArteRelationIterator<RHIT> rhitIt = rhit_begin;
		 rhitIt != rhit_end; ++rhitIt ) {
	      ArtePointer<RHIT> rhit(*rhitIt);
	      createRelation( rseg, rhit, &RSEG::rhit );	  
	    }
	  }
	}
      }

      /*
      int fitflag   = rseg->fit;
      int rs        = rseg - ArtePointer<RSEG>(1) + 1;
      int fitstatus = rangerFitV(rs, 0);//rgfit_(rs,0);
      rseg->fit  = fitflag;
      fitflag = fitstatus;
      */
      nh=nh+nh_trk[itk]+1; /* skip (-1): hits of diff. tracks are */
                           /* separated by (-1)                   */

    }

    return;
  }
}





