
/***********************************************************************
*
* DGEVNT
* written by Siegmund Nowak 08/93
*
* This routine steers the digitization, i.e. the transformation  
* of GEANT output hits (MIMP) into the digit (DIGB) and hit (HITB) 
* structures  foreseen in the reconstruction  program either for 
* all detectors or for selected detectors (see kadgsel.C). The 
* transformation is done in subdetector specific subroutines 
***********************************************************************/

#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "gean/gclink.h"
#include "gean/ZGESL.h"
#include "digi/zutsel.h"
#include "digi/dglev.h"
#include "digi/dgcut.h"

extern "C" {
  void dgevnt_();
  void dgbldt_();
  void alrini_();
  void grndmq_(int &,int &,int &, char *, int);
  void dgmask_(int &); 
  void mmfree_(int &);
  void dgsil_(int &);
  void dgotr_(int &);          void dgomod_(int &);
  void dgmsgc_(int &);         void dgitr_(int &);
  void dgptl_(int &);
  void dgrich_(int &);
  void dgtrds_(int &);         void dgtrdf_(int &);
  void dgmuoo_(int &);         void dgmuoi_(int &);
  void dgcal_(const unsigned int &);
}

void dgevnt_() {
/*^^^^^^^^^^^^^*/
  int i1 = 1, m1 = -1;

//check that data are indeed MC  
  if( !ArteTable<MCIN>::booked() ) return;

//define random generator numbers for digitisation
  ArtePointer<MCIN> mc(1);     
  if(mc->rdig[0] == 0 || mc->rdig[1] == 0) {
     alrini_();
     mc->rdig[0] = arteflag_.lundseed[0];
     mc->rdig[1] = arteflag_.lundseed[1];
  }
  grndmq_(mc->rdig[0],mc->rdig[1],i1,"S",1);

//mask out detector regions defined by dgcut
  if (dgcuxy_.ncutdg > 0)  dgmask_(i1);

//loop over all  superlayers

  for(ArteTableIterator<GESL> gs; gs.valid(); gs.next()){
     if(zutsel_.ngsel) {
        int found =0;
        for (int j=0; j<zutsel_.ngsel; j++) {
           if(!strncmp((*gs)->name,&zutsel_.hgsel[4*j],4)) {
              found = 1;           break;
           }
        }
        if (!found) continue;
     }

     int isltyp = (*gs)->cmp;
     if(ArteTable<MIMP>::size() >0) {
 
        for(int ipl = (*gs)->fpl;ipl != (*gs)->fpl+(*gs)->npl;ipl++) {

           if(isltyp == LGESL_SIL) {                                  //SVD
              dgsil_(ipl);
           }

           else if(isltyp == LGESL_WCH) {                             //OTR 
              if(dglev_.idglev[LGESL_WCH-1] == 1) dgotr_(ipl);
              else dgomod_(ipl);
           }

           else if(isltyp == LGESL_MSG) {                             //ITR 
              if(dglev_.idglev[LGESL_MSG-1] == 0) dgmsgc_(ipl);
              else if (dglev_.idglev[LGESL_MSG-1] == 1) dgitr_(ipl); 
           }

           else if(isltyp == LGESL_PTC) {                             //PTC
              dgptl_(ipl);
           }

           else if(isltyp == LGESL_RIC) {                             //RICH
              if(ipl  == (*gs)->fpl)  dgrich_(ipl);
           }

           else if(isltyp == LGESL_TRD) {                             //TRD 
              if(dglev_.idglev[LGESL_TRD-1] ==  0) dgtrdf_(ipl);
              else if (dglev_.idglev[LGESL_TRD-1] == 1) dgtrds_(ipl);
           }

           else if(isltyp == LGESL_MUO) {                             //MUO 
              dgmuoo_(ipl);
           }

           else if(isltyp == LGESL_MUX) {                             //MUX
              dgmuoi_(ipl);
           }
        }
     }

     if(isltyp == LGESL_ECA) {  
           if(ArteTable<MCAL>::size() >0) dgcal_(gs.index());
     }
  }

//  restore  mimps which have been masked out

  if (dgcuxy_.ncutdg > 0)  dgmask_(m1);

  mmfree_(ArtePointer<DIGB>::getLink().getRef());
  mmfree_(ArtePointer<HITB>::getLink().getRef());
  mmfree_(ArtePointer<HITC>::getLink().getRef());
}





