
// *****************************************************************************
//
// source:    DQ.C
//
// type:      source code
//
// created:   19 Oct 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ routines
//
// changes:
// 20 Oct 1999 (AG) make DQ_flag() usable for C         
// 01 Nov 1999 (AG) use LVL_M_ABC for nrclevel
// 04 Nov 1999 (AG) repair kuip
// 26 Nov 1999 (AG) introduce DQ_RECO
// 12 Jan 2000 (AG) introduce GLOBAL = CORR
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// ONLINE includes
#include "component.h"

// Arte includes
#include "dq/DQ.h"

// externals
extern int DQ_DB;
extern int DQ_EVAL;

extern int DQ_SVD;
extern int DQ_ITR;
extern int DQ_HIPT;
extern int DQ_OTR;
extern int DQ_RICH;
extern int DQ_TRD;
extern int DQ_ECAL;
extern int DQ_MUON;
extern int DQ_FLT;
extern int DQ_SLT;
extern int DQ_TLT;
extern int DQ_FARM;

extern int DQ_CORR;
extern int DQ_RECO;

extern int DQ_USR1;
extern int DQ_USR2;

extern int DQ_TARG;

extern int AC_SVD;
extern int AC_RICH;

// prototypes
extern "C" void kugetc_( char *string, int *len, int len1 );

//
// DQ_ID() function
//
extern "C" unsigned int
DQ_ID( int COMP_XYZ, int LVL_ABC, int histID )
{
  //return  ( ( COMP_XYZ<<28 ) | ( LVL_ABC<<24 ) | histID );
  return  ( ( MOD_DQ<<28 ) | ( COMP_XYZ<<24 ) | ( LVL_ABC<<20 ) | histID );
}

//
// AC_ID() create histogram id accoding to coding rules by component Number
//
extern "C" unsigned int
AC_ID( int COMP_XYZ, int LVL_ABC, int histID )
{
  return  ( ( MOD_AC<<28 ) | ( COMP_XYZ<<24 ) | ( LVL_ABC<<20 ) | histID );
}

//
// DQ_flag() function
//
extern "C" int
DQ_flag( int COMP_XYZ, int LVL_M_ABC )
{
  int rc = 0;

  //cout << "DQ_flag(): COMP_XYZ=" << COMP_XYZ << " LVL_M_ABC=" << LVL_M_ABC << endl;

  //cout << "DQ_flag(): DQ_SVD= " << DQ_SVD  << endl;
  //cout << "DQ_flag(): DQ_RICH=" << DQ_RICH << endl;

  if( ( COMP_XYZ == COMP_SVD  ) && ( DQ_SVD  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ITR  ) && ( DQ_ITR  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_OTR  ) && ( DQ_OTR  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_HIPT ) && ( DQ_HIPT & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RICH ) && ( DQ_RICH & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TRD  ) && ( DQ_TRD  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ECAL ) && ( DQ_ECAL & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_MUON ) && ( DQ_MUON & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_FLT  ) && ( DQ_FLT  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_SLT  ) && ( DQ_SLT  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TLT  ) && ( DQ_TLT  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_FARM ) && ( DQ_FARM & LVL_M_ABC ) ) rc = 1;

  if( ( COMP_XYZ == COMP_CORR ) && ( DQ_CORR & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RECO ) && ( DQ_RECO & LVL_M_ABC ) ) rc = 1;

  if( ( COMP_XYZ == COMP_USR1 ) && ( DQ_USR1 & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_USR2 ) && ( DQ_USR2 & LVL_M_ABC ) ) rc = 1;

  return rc; 
}

//
// AC_flag() by component ID
//
extern "C" int
AC_flag( int COMP_XYZ, int LVL_M_ABC )
{
  //cout << "AC_flag() "
  //   << "COMP_XYZ="  << COMP_XYZ  << " "
  //   << "LVL_M_ABC=" << LVL_M_ABC << endl;

  int rc = 0;

  if( ( COMP_XYZ == COMP_SVD  ) && ( AC_SVD  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RICH ) && ( AC_RICH & LVL_M_ABC ) ) rc = 1;

  return rc;
}

//
// dq_comp *************************************************************
//
extern "C"
void dq_comp_()
{  
  //--------------------------------------------------------------------
  //   read in subdetector
  //--------------------------------------------------------------------
  
  int len;        
  int len1 = 4;
  char input[7];
  kugetc_( input, &len, len1 );
  input[len] = 0;
  
  char *subdetector;
  subdetector  = (char *) malloc( len+1 );
  strncpy( subdetector, input, len );
  subdetector[len] = '\0';
  
  for( int i=0; i<len; i++ ) subdetector[i] = toupper( subdetector[i] );
  
  if( strncmp( subdetector,"SVD",   3 ) &&
      strncmp( subdetector,"VDS",   3 ) &&
      strncmp( subdetector,"ITR",   3 ) && 
      strncmp( subdetector,"HIPT",  4 ) &&
      strncmp( subdetector,"OTR",   3 ) && 
      strncmp( subdetector,"RICH",  4 ) &&
      strncmp( subdetector,"TRD",   3 ) && 
      strncmp( subdetector,"ECAL",  4 ) &&
      strncmp( subdetector,"MUON",  3 ) && 
      strncmp( subdetector,"SLT",   3 ) &&
      strncmp( subdetector,"TLT",   3 ) &&
      strncmp( subdetector,"FARM",  4 ) && 
      strncmp( subdetector,"TARG",  4 ) &&
      strncmp( subdetector,"CORR",  4 ) &&
      strncmp( subdetector,"GLOBAL",6 ) &&
      strncmp( subdetector,"RECO",  4 ) && 
      strncmp( subdetector,"USR1",  4 ) && 
      strncmp( subdetector,"USR2",  4 ) && 
      strncmp( subdetector,"ALL",   3 ) ){
    cout << "DQ_init(): ********* DQ/SUBD: unknown subdetector: "
	 << subdetector << endl;
  }

  //cout << "DQ_init(): subdetector=" << subdetector << endl;
  
  //--------------------------------------------------------------------
  //   read in subdetetcor status
  //--------------------------------------------------------------------

  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *status = (char *) malloc( len+1 );
  strncpy( status, input, len );
  status[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( input[i] );
  
  if(strncmp(status,"ON",2) && strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/SUBD: unknown subdetector status: "
	 << status << endl;
  }
  //cout << "DQ_init(): status=" << status << endl;
  
  //--------------------------------------------------------------------
  //   read in dq level (hit/raw/rec)
  //--------------------------------------------------------------------
  
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  int  nrclevel = 0;  
  char *rclevel  = (char *) malloc( len+1 );
  strncpy( rclevel, input, len );
  rclevel[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) rclevel[i] = toupper(input[i]);
  
  if(      !strncmp( rclevel, "RAW", 3 ) ){
    nrclevel = LVL_M_RAW;
  }
  else if( !strncmp( rclevel, "HIT", 3 ) ){
    nrclevel = LVL_M_HIT;
  } 
  else if( !strncmp( rclevel, "REC", 3 ) ){
    nrclevel = LVL_M_REC;
  }
  else {
    cout << "DQ_init(): ****** DQ/SUBD: unknown level: " << rclevel << " "
	 << "Taking default." << endl;
  }

  //cout << "DQ_init(): nrclevel=" << nrclevel << endl;

  //--------------------------------------------------------------------
  //   set dq status
  //--------------------------------------------------------------------
  
  if( !strncmp( status,"OFF", 3 ) ) nrclevel=0;
  
  if( !strncmp( subdetector, "SVD" ,   3 ) ) DQ_SVD  |= nrclevel;
  if( !strncmp( subdetector, "VDS" ,   3 ) ) DQ_SVD  |= nrclevel;
  if( !strncmp( subdetector, "ITR" ,   3 ) ) DQ_ITR  |= nrclevel;
  if( !strncmp( subdetector, "HIPT",   4 ) ) DQ_HIPT |= nrclevel;
  if( !strncmp( subdetector, "OTR" ,   3 ) ) DQ_OTR  |= nrclevel;
  if( !strncmp( subdetector, "RICH",   4 ) ) DQ_RICH |= nrclevel;
  if( !strncmp( subdetector, "TRD" ,   3 ) ) DQ_TRD  |= nrclevel;
  if( !strncmp( subdetector, "ECAL",   4 ) ) DQ_ECAL |= nrclevel;
  if( !strncmp( subdetector, "MUON",   4 ) ) DQ_MUON |= nrclevel;
  if( !strncmp( subdetector, "FLT" ,   3 ) ) DQ_FLT  |= nrclevel;
  if( !strncmp( subdetector, "SLT" ,   3 ) ) DQ_SLT  |= nrclevel;
  if( !strncmp( subdetector, "TLT" ,   3 ) ) DQ_TLT  |= nrclevel;
  if( !strncmp( subdetector, "FARM",   4 ) ) DQ_FARM |= nrclevel;
  if( !strncmp( subdetector, "CORR",   4 ) ) DQ_CORR |= nrclevel;
  if( !strncmp( subdetector, "GLOBAL", 4 ) ) DQ_CORR |= 0xffff;
  if( !strncmp( subdetector, "RECO",   4 ) ) DQ_RECO |= nrclevel;
  if( !strncmp( subdetector, "USR1",   4 ) ) DQ_USR1 |= nrclevel;
  if( !strncmp( subdetector, "USR2",   4 ) ) DQ_USR2 |= nrclevel;
  
  if (!strncmp( subdetector, "ALL" , 3 ) ){
    DQ_SVD  |= nrclevel;
    DQ_HIPT |= nrclevel;
    DQ_OTR  |= nrclevel;
    DQ_ITR  |= nrclevel;
    DQ_RICH |= nrclevel;
    DQ_TRD  |= nrclevel;
    DQ_ECAL |= nrclevel;
    DQ_MUON |= nrclevel;
    DQ_FLT  |= nrclevel;
    DQ_SLT  |= nrclevel;
    DQ_TLT  |= nrclevel;
    DQ_FARM |= nrclevel;
    DQ_CORR |= nrclevel;
    DQ_RECO |= nrclevel;
    DQ_USR1 |= nrclevel;
    DQ_USR2 |= nrclevel;
  }

  //cout << "DQ_init(): DQ_SVD= " << DQ_SVD  << endl;
  //cout << "DQ_init(): DQ_RICH=" << DQ_RICH << endl;
}

//
// dq_db ***************************************************************
//
extern "C"
void dq_db_()
{  
  //--------------------------------------------------------------------
  //   read in status for saving histograms to the DB
  //--------------------------------------------------------------------
  
  int len;        
  int len1 = 3;
  char input[7];
  kugetc_(input, &len, len1);
  input[len] = 0;
  
  char *status;
  status  = (char *) malloc(len+1);
  strncpy(status, input, len);
  status[len] = '\0';
  
  if(len>0) for(int i=0; i<len; i++) status[i] = toupper(status[i]);
  
  if(strncmp(status,"ON",2)&&strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/DB: unknown status for saving histograms to DB: "
	 << status << endl;
  }

  //--------------------------------------------------------------------
  //   set status
  //--------------------------------------------------------------------
  
  if (!strncmp(status,"OFF",3)) DQ_DB=0;
  if (!strncmp(status,"ON", 2)) DQ_DB=1;
}

//
// dq_eval *************************************************************
//
extern "C"
void dq_eval_()
{  
  //--------------------------------------------------------------------
  //   read in demand for evaluating histograms
  //--------------------------------------------------------------------
  int len;        
  int len1 = 3;
  char input[7];
  kugetc_(input, &len, len1);
  input[len] = 0;
  
  char *status;
  status  = (char *) malloc(len+1);
  strncpy(status, input, len);
  status[len] = '\0';
  
  if(len>0) for(int i=0; i<len; i++) status[i] = toupper(status[i]);
  
  if(strncmp(status,"ON",2)&&strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/EVAL: unknown demand for evaluating the histograms: "
	 << status << endl;
  }

  //--------------------------------------------------------------------
  //   set status
  //--------------------------------------------------------------------  
  if (!strncmp(status,"OFF",3)) DQ_EVAL=0;
  if (!strncmp(status,"ON", 2)) DQ_EVAL=1;
}

//
// ac_comp
//
extern "C" void
ac_comp_()
{   
  //cout << "ac_comp_()" << endl;

  //
  // read in subdetector
  //
  int len  = 0;
  int len1 = 4;
  char input[7];
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *subdetector  = (char *) malloc( len + 1 );
  strncpy(subdetector, input, len);
  subdetector[len] = '\0';
  //cout << "ac_comp_(): subdetector=" << subdetector << endl;
  
  // loop
  for( int i=0; i<len; i++ ){
    subdetector[i] = toupper( subdetector[i] );
  }

  // check
  if( strncmp( subdetector, "SVD", 3 ) && 
      strncmp( subdetector, "VDS", 3 ) &&
      strncmp( subdetector, "ITR", 3 ) && 
      strncmp( subdetector, "HIPT",4 ) &&
      strncmp( subdetector, "OTR", 3 ) && 
      strncmp( subdetector, "RICH",4 ) &&
      strncmp( subdetector, "TRD", 3 ) && 
      strncmp( subdetector, "ECAL",4 ) &&
      strncmp( subdetector, "MUON",3 ) && 
      strncmp( subdetector, "SLT", 3 ) &&
      strncmp( subdetector, "TLT", 3 ) &&
      strncmp( subdetector, "FARM",4 ) && 
      strncmp( subdetector, "ALL", 3 )    ){
    cout << "ac_comp_(): ********* AC/SUBD: unknown subdetector: "
         << subdetector << endl;
  }
  
  //
  // read in status
  //
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *status  = (char *) malloc( len + 1 );
  strncpy( status, input, len );
  status[len] = '\0';
  //cout << "ac_comp_(): status=" << status << endl;
  
  // convert to upper case letters
  if( len>0 ){
    for( int i=0; i<len; i++ ){
      status[i] = toupper( status[i] );
    }
  }  

  // check
  if( strncmp( status, "ON",  2 ) &&
      strncmp( status, "OFF", 3 )    ){
    cout << "ac_comp_(): ********* AC/SUBD: unknown subdetector status="
	 << status << endl;
  }
        
  //
  // read in rec
  //
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *rec  = (char *) malloc( len + 1 );
  strncpy( rec, input, len );
  status[len] = '\0';
  //cout << "ac_comp_(): rec=" << rec << endl;
  
  // convert to upper case letters
  if( len>0 ){
    for( int i=0; i<len; i++ ){
      rec[i] = toupper( rec[i] );
    }
  }  

  if( strncmp( rec, "TST", 3 ) &&
      strncmp( rec, "DEF", 3 )    ){
    cout << "ac_comp_(): ******* AC/SUBD: unknown subdetector level="
	 << rec << endl;
  }
  
  // check
  int level = 0;
  if( !strncmp( rec, "TST", 3 ) ) level = LVL_M_TST;
  if( !strncmp( rec, "DEF", 3 ) ) level = LVL_M_DEF;

  //
  // final print-out
  //
  //cout << "ac_comp_(): "
  //  << "subdetector=" << subdetector << " "
  //   << "status="      << status      << " "
  //   << "level="       << level       << endl;
  
  //
  // set flags
  //
  if (!strncmp( status, "OFF", 3 ) ) level=0;

  if( !strncmp( subdetector, "SVD",  3 ) ) AC_SVD  |= level;
  if( !strncmp( subdetector, "RICH", 4 ) ) AC_RICH |= level;

  //cout << "ac_comp_(): "
  //   << "AC_SVD="  << AC_SVD  << " "
  //   << "AC_RICH=" << AC_RICH << endl;
}

//
//
//
void
DQ_print( const char *txt )
{
  cout << "DQ_print(): Called from " << txt << ":" << endl;
  cout << "            DQ_SVD= " << DQ_SVD  << endl;
  cout << "            DQ_ITR= " << DQ_ITR  << endl;
  cout << "            DQ_HIPT=" << DQ_HIPT << endl;
  cout << "            DQ_OTR= " << DQ_OTR  << endl;
  cout << "            DQ_RICH=" << DQ_RICH << endl;
  cout << "            DQ_TRD= " << DQ_TRD  << endl;
  cout << "            DQ_ECAL=" << DQ_ECAL << endl;
  cout << "            DQ_MUON=" << DQ_MUON << endl;
  cout << "            DQ_FLT= " << DQ_FLT  << endl;
  cout << "            DQ_SLT= " << DQ_SLT  << endl;
  cout << "            DQ_TLT= " << DQ_TLT  << endl;
  cout << "            DQ_FARM=" << DQ_FARM << endl;
  cout << "            DQ_CORR=" << DQ_CORR << endl;
  cout << "            DQ_RECO=" << DQ_RECO << endl;
  //cout << "            DQ_USR1=" << DQ_USR1 << endl;
  //cout << "            DQ_USR2=" << DQ_USR2 << endl;
}
