
// *****************************************************************************
//
// source:    faStop.C
//
// type:      source code
//
// created:    7 Jul 1997
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    fastop_()
//
// input:     void
// return:    int  ARTE_error_code
//
// called by: ATSTOP
//
// purporse:  termination of farm stuff called for FARM mode
//
// changes:
// 25 Nov 1998 (AG) stop rpm
// 27 Nov 1998 (AG) multiple file input
// 01 Dec 1998 (AG) daq format logging
// 11 Jan 1999 (AG) includes
// 12 Jan 1999 (AG) removed emg status flag
// 14 Jan 1999 (AG) removed hbook filling
// 21 Jan 1999 (AG) more printout
// 02 Feb 1999 (AG) halt rpm
// 23 Feb 1999 (AG) removed faUtil.hh
// 25 Feb 1999 (AG) use grr error logger
// 11 Mar 1999 (AG) polishing
// 01 Jun 1999 (AG) put includes
// 09 Sep 1999 (AG) always use cout
// 08 Oct 1999 (AG) write to log file
// 18 Oct 1999 (AG) polishing
// 03 Nov 1999 (AG) add process id to logfile
// 11 Nov 1999 (AG) improve log
// 19 Nov 1999 (AG) polishing
// 24 Nov 1999 (AG) use ARTE_error()
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>

// ONLINE inludes
#include "rpm.h"
#include "rhp.h"
#include "grr.h"

// ARTE includes
#include "farm/fa.hh"
#include "farm/farm.hh"

// externals
extern int  nArg;   // line command arguments
extern char **vArg; // line command arguments

extern int  FARM;     // FARM mode
extern int  FAON;     // FARM online mode
extern char *DBNAME;  // database name
extern int  NOSPAR;   // sparsification
extern int  RPMINIT;  // rpm/rhp initialized
extern int  FAGRR;    // grr error logger     
extern char *PARNAME; // emg partition
extern int  SEG_NUM;  // emg partitions
extern int  SEG_LEN;  // emg partition size
extern char *LOGFILE; // log-file

// C prototypes
#ifdef sun
extern "C" long gethostid();
extern "C" int  gethostname( char *hostname, int length );
#endif

//
// program code
//
extern "C" int
fastop_()
{
  //
  // host name
  //
  char *hostName = new char[30];
  gethostname( hostName, 29 );
  
  //
  // current date/time
  //
  time_t t = time( NULL ); 
  char date[25];
  strcpy( date, ctime( &t ) );
  date[24] = '\0';

  //
  // process id
  //
  int pid = getpid();
 
  //
  // farm mode
  //
  int rc = ARTE_GOOD;    
  if( FARM ){

    //
    // online
    //
    if( FAON ){

      //
      // emg
      //
      if( ( rc = faStopEmg() ) != GRR_OK ){
	ARTE_error( GRR_SERROR, "STOP: Arte emg FAILED.", rc );
      }
    }
  }

  // grr error message
  ARTE_error( GRR_SINFO, "STOP: Arte faStop() DONE.", 0 );

  //
  // rhp/rpm
  //
  if( RPMINIT ){
    if( ( rc = rhp_halt() ) == GRR_OK ){
#ifdef _OUT_
      cout << "STOP: rhp_halt()" << endl;
#endif
    }
    else {
#ifdef _ERR_
      cout << "STOP: ERROR: rhp_halt() rc=" << rc << endl;
#endif
    }

    if( ( rc = rpm_halt() ) == GRR_OK ){
#ifdef _OUT_
      cout << "STOP: rpm_halt()" << endl;
#endif
    }
    else {
#ifdef _ERR_
      cout << "STOP: ERROR: rpm_halt() rc=" << rc << endl;
#endif
    }
  }

  //
  // write to log-file
  //
  if( LOGFILE ){
#ifdef _OUT_
    cout << "STOP: Write to logfile=" << LOGFILE << endl;
#endif
    FILE *log = fopen( LOGFILE, "a+" );
    if( log != NULL ){
      fprintf( log, "STOP  on %s node:%s pid:%d\n", date, hostName, pid );
      fclose( log );
    }
  }

  // return
  return rc;
}
