// ********************************************************************
//
//  Routine:      gageFillRhpSegments
//  Author:       T. Oest
//
//  Description:  fill segments pairs in rhp ring buffer
//
//  Modification:
//
// ********************************************************************

// includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"
#include "gage/gage.hh"
#include "arte/RSEG_cmp.hh"
#include "rhp.h"
#include "dq/DQ.h"

extern FILE * gageFile;

//*****************************************************************
// function
//*****************************************************************

void gageFillRhpSegments(int cmpRSEG, int cmpAlign, float * Cuts){

if (cmpRSEG & cmpAlign) return;

//-----------------------------------------------------------------
//   Target position
//-----------------------------------------------------------------
    float xTARGET = 0.0, yTARGET = 0.0 , zTARGET = 0.0;
    float diff;

    for( ArteTable<GTAR>::iterator gtar = ArteTable<GTAR>::begin();
         gtar != ArteTable<GTAR>::end();
         ++gtar ) {
      diff = (*gtar)->x * (*gtar)->x + (*gtar)->y * (*gtar)->y;
      if (diff < 1 ) {
       xTARGET = (*gtar)->x;
       yTARGET = (*gtar)->y;
       zTARGET = (*gtar)->z;
      };
    };

//-----------------------------------------------------------------
//  loop over MARPLE segments, select only requested segments
//-----------------------------------------------------------------

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

  if ((*rseg)->fit == 7 && ((*rseg)->cmp & cmpRSEG) == cmpRSEG){

//-----------------------------------------------------------------
//  find segments
//-----------------------------------------------------------------

    ArtePointer<RSEG> rVDS  = 0;
    ArtePointer<RSEG> rRICH = 0;
    ArtePointer<RSEG> rOTR  = 0;
    ArtePointer<RCCL> rECAL = 0;

    if ((cmpRSEG & Rsegc::ecal) && 
        (*rseg)->rccl.begin() != (*rseg)->rccl.end())
        rECAL = * (*rseg)->rccl.begin();

    for (ArteRelationIterator<RSEG> rseg2 = rseg->rend.begin();
         rseg2 != rseg->rend.end(); ++rseg2){

      if (((*rseg2)->fit == 0 ||  (*rseg2)->fit == 8) &&
          ((*rseg2)->cmp & cmpRSEG)){
      
       if ((*rseg2)->cmp & Rsegc::vxd ) rVDS  = *rseg2;
       if ((*rseg2)->cmp & Rsegc::rich) rRICH = *rseg2;
       if ((*rseg2)->cmp & Rsegc::patt) rOTR  = *rseg2;
       if ((*rseg2)->cmp & Rsegc::magt) rOTR  = *rseg2;
    
       };
    };


//-----------------------------------------------------------------
//  cuts on MARPLE segments
//-----------------------------------------------------------------

   float pTrack[6];

   int goodMARPLEsegment = 1;

   if ((*rseg)->cmp & (Rsegc::rich|Rsegc::ecal)){ 
     if (rRICH == 0 || rECAL == 0){
        goodMARPLEsegment = 0;
     } else {
      float drty = (rECAL->y - yTARGET) / (rECAL->z - zTARGET) - 
                    rRICH->tyf;
      if (fabs(drty)>0.02 || rECAL->wid <= 4 ||
          fabs(1./(*rseg)->pf) < Cuts[4])

        goodMARPLEsegment = 0;

     };
   };

   //   cout << " goodMARPLEsegment  = " << goodMARPLEsegment << endl;

//-----------------------------------------------------------------
//  refit MARPLE segment
//-----------------------------------------------------------------

  float chi2;

  if (goodMARPLEsegment == 1){

   gageFitEcalRich(rRICH, rECAL, pTrack, &chi2);
   if (chi2 > 20.) goodMARPLEsegment = 0;
  };


  if (goodMARPLEsegment == 1){

    
    /*
   cout << " track after fit " << endl;
   cout << "  z    " << pTrack[0] << endl;
   cout << "  x    " << pTrack[1] << endl;
   cout << "  y    " << pTrack[2] << endl;
   cout << "  tx   " << pTrack[3] << endl;
   cout << "  ty   " << pTrack[4] << endl;
   cout << "  p    " << pTrack[5] << endl;
   cout << "  chi2 " << chi2 << endl;
    */

//-----------------------------------------------------------------
//  loop over subdetector segments
//-----------------------------------------------------------------

  for( ArteTable<RSEG>::iterator rsegAlign = ArteTable<RSEG>::begin();
       rsegAlign != ArteTable<RSEG>::end(); ++rsegAlign ) {

  if (((*rsegAlign)->fit == 0 || (*rsegAlign)->fit == 8) &&
       (*rsegAlign)->cmp & cmpAlign) {

//-----------------------------------------------------------------
//  cuts on Alignment segment segments
//-----------------------------------------------------------------

  int goodAlignmentSegment = 1;

  float p[6];

  gageTraceTRACK((*rsegAlign)->zf, pTrack, p);

  /*
  cout << " track traced to segment " << endl;
  cout << " z   " << p[0] << endl;
  cout << " x   " << p[1] << endl;
  cout << " y   " << p[2] << endl;
  cout << " tx  " << p[3] << endl;
  cout << " ty  " << p[4] << endl;
  cout << " rho " << p[5] << endl;
  */

  float dx, dy, dtx, dty;

  dx    = p[1] - (*rsegAlign)->xf ;
  dy    = p[2] - (*rsegAlign)->yf ;
  dtx   = p[3] - (*rsegAlign)->txf ;
  dty   = p[4] - (*rsegAlign)->tyf ;


  /*
  cout << " cuts " << endl;
  cout << " dx  " << dx << endl;
  cout << " dy  " << dy << endl;
  cout << " dtx " << dtx << endl;
  cout << " dty " << dty << endl;
  */

  if (fabs(dx) >Cuts[0]  || fabs(dy) >Cuts[1] || 
      fabs(dtx)>Cuts[2]  || fabs(dty)>Cuts[3] )  goodAlignmentSegment = 0;


       //  cout << " goodAlignmentSegment  " << goodAlignmentSegment << endl;

//-----------------------------------------------------------------
//  fill structure and pass to buffer
//-----------------------------------------------------------------

    if (goodAlignmentSegment == 1){

     GageSegmentPair SegmentPair;

     SegmentPair.key    = ArtePointer<EVHD>(1)->dbmajor;
     SegmentPair.z      = (*rsegAlign)->zf;
     SegmentPair.p1[0]  = (*rsegAlign)->xf;
     SegmentPair.p1[1]  = (*rsegAlign)->yf;
     SegmentPair.p1[2]  = (*rsegAlign)->txf;
     SegmentPair.p1[3]  = (*rsegAlign)->tyf;

     for (int i=0; i<10; i++) SegmentPair.V1[i] = (*rsegAlign)->cvf[i];

     SegmentPair.p2[0]  = p[1];
     SegmentPair.p2[1]  = p[2];
     SegmentPair.p2[2]  = p[3];
     SegmentPair.p2[3]  = p[4];

     for (int i=0; i<10; i++) SegmentPair.V2[i] = 0.0;


//-----------------------------------------------------------------
//  write to file or copy in rhp buffer
//-----------------------------------------------------------------

     if (gageFile !=0){

      fwrite(&SegmentPair, sizeof(GageSegmentPair), 1, gageFile);
      //      cout << " track written to file " << endl;

     } else {

      int size, currentsize;
      char *buf = rbf_obtain(AC_ID( COMP_CORR, LVL_REC, 1),
                             RBF_READ, &size, &currentsize); 
 
      int newsize = sizeof(GageSegmentPair) / sizeof(char);

      memcpy(buf, &SegmentPair, newsize);

      rbf_release(AC_ID( COMP_CORR, LVL_REC, 1), 
                  buf, newsize);

      //      cout << " track written to rhp buffer " << newsize << endl;

     };

    };
    };


  };
  };
 };
 };

};
