/**********************************************************************
*-- Author :    Siegmund Nowak   14/08/96
*                                                                     *
*       Store tracking media parameters for HERA-B in JTMED                   *
*       deviated from GSTMED                                          *
*       Specialities for HERA-B:                                      *
*       FIELDM    maximum field value  set to 10 kG                   *
*       TMAXFD    maximum angle due to field deviation permitted in   *
*                 one step  set to 10 degrees                         *
*       DEEMAX    set to -1 for automatic calculation                 *
*       STMIN     minimal step due to energy loss or multiple         *
*                 scattering   set to -1 for automatic calculation    *
*       UBUF      no array of NWBUF additional parameters             *
*                                                                     *
*                                                                     *
* Tracking Medium data structure JTMED (see GSTMED)                   *
*    ==>Called by : <USER>, MEDCON,CRMIXT                             *
*       Author    R.Brun  changed by S.Nowak                          *
*                                                                     *
**********************************************************************/
#include <stdio.h>
#include <string.h>
#include <vector.h>
#include "arte/arte.hh"
#include "gean/gclink.h"
#include "gean/gcnum.h"
#include "gean/gccuts.h"
#include "gean/gcphys.h"
#include "gean/gcmzfo.h"
#include "gean/ZGDETV.h"
#include "gean/ZGESL.h"


extern "C" {
   void crtmed_(const int &);
   void mzbook_(int*, int*, int*, int*, char*, int*, int*, int*, int*, int*,const int);
   void mzpush_(int*, int*, int*, int*, char*,const int);
}

void crtmed_(const int &imat) { 
    extern vector<int*>    hg_tmed;
    static int crtmed_nmed ;
    ArtePointer<GMAT> gm(imat);
    if(gm->flag == 0) return;

    int i0=0, i1=1, i3=3, i10=10, i14=14, i40=40;
    int jtm=0, isvol=0, ifield=0;
    float stemax =10.;
    char cname[13];
    float jx;

    unsigned int len = 12;
    if(len >  strlen(gm->name)) len = strlen(gm->name);
    strncpy(cname,gm->name,len);
    for(unsigned int k=0; k<len; k++) {
      if(cname[k] == ' ') { len=k;  break;
      }
    }    
    cname[len] = '\0';

    char sl[4]; char si[1];
    strncpy(sl,"TMED",4);
    strncpy(si,"I",1);

    if(gclink_.jtmed <= 0){
       
       mzbook_(&gcbank_.ixcons,&gclink_.jtmed,&gclink_.jtmed,
                &i1,sl,&gcnum_.ntmed,&gcnum_.ntmed,&i40,&i3,&i0,4);

       memcpy(gcbank_.iq+gclink_.jtmed,&gccuts_.cutgam,40);
       for(int k=0; k< 13; k++) { 
          jx =float(*(&gcphys_.ipair+5*k)); 
          memcpy(gcbank_.iq+gclink_.jtmed+ k+10,&jx,4);
       }  

       *(gcbank_.iq + gclink_.jtmed - 6 ) = 0;
       jx =float(gcphlt_.ilabs); memcpy(gcbank_.iq + gclink_.jtmed +30,&jx,4);
       jx =float(gcphlt_.isync); memcpy(gcbank_.iq + gclink_.jtmed +31,&jx,4);
       jx =float(gcphlt_.istra); memcpy(gcbank_.iq + gclink_.jtmed +32,&jx,4);
       crtmed_nmed = 0;
    }

    int dnew = imat- hg_tmed.size();
    int *ptr;
    if(dnew >0 ){
       for (int k = 0; k< dnew; k++) {
	  ptr = new int[4];
          *ptr = 0;   *(ptr+1) = 0;   *(ptr+2) = 0;   *(ptr+3) = 0; 
          hg_tmed.push_back(ptr);
       }
    }					     
    else  ptr = hg_tmed[imat-1];


    char cnam[20];
    int medmask[4] = {1, 2, 4, 8};

    for(int j=0; j<4; j++) {
       memset(cnam,' ',20);
       int ia = gm->flag & medmask[j];
       if(ia != 0 ) {

         crtmed_nmed = crtmed_nmed + 1;
         *(ptr+j) = crtmed_nmed;

         strcpy(cnam, cname);
         if(j == 0) {
            isvol = 0;
            ifield = 0;
            stemax = 1.e+10;
         }
         else if(j == 1) {
            strncat(cnam,"_magn",5);
            isvol = 0;
            if(zgdetv_.DETV[LGESL_FLD-1] == 'S') ifield = 0;
            else if(zgdetv_.DETV[LGESL_FLD-1] == 'L') ifield = 1;
            stemax = 10.;
         }
         else if(j == 2) {
            isvol = 1;
            ifield = 0;
            stemax = 1.e+10;
         }
         else if (j == 3) {
            strncat(cnam,"_magn",5);
            isvol = 1;
            if(zgdetv_.DETV[LGESL_FLD-1] == 'S') ifield = 0;
            else if(zgdetv_.DETV[LGESL_FLD-1] == 'L') ifield = 1;
            stemax = 10.;
         }
         cnam[strlen(cnam)] = ' ';

         if(crtmed_nmed > gcnum_.ntmed) {
            int dnmed = crtmed_nmed - gcnum_.ntmed; 
            mzpush_(&gcbank_.ixcons,&gclink_.jtmed,&dnmed,&i0,si,1);
            gcnum_.ntmed=crtmed_nmed;
         }
         int mnmed = -crtmed_nmed;
         mzbook_(&gcbank_.ixcons,&jtm,&gclink_.jtmed,&mnmed,sl,&i10,&i10,&i14,
                 &gcmzfo_.iotmed,&i0,4);

         memcpy(gcbank_.iq + jtm,cnam,20);

         jx = float(imat);     memcpy(gcbank_.iq + jtm + 5,&jx,4);
         jx = float(isvol);    memcpy(gcbank_.iq + jtm + 6,&jx,4);
         jx = float(ifield);   memcpy(gcbank_.iq + jtm + 7,&jx,4);
         jx = 10.;             memcpy(gcbank_.iq + jtm + 8,&jx,4);
         jx = 10.;             memcpy(gcbank_.iq + jtm + 9,&jx,4);
         jx = stemax;          memcpy(gcbank_.iq + jtm +10,&jx,4);
         jx = -1.;             memcpy(gcbank_.iq + jtm +11,&jx,4);
         jx = .01;             memcpy(gcbank_.iq + jtm +12,&jx,4);
         jx = -1.;             memcpy(gcbank_.iq + jtm +13,&jx,4);

       }
    }
}









