      SUBROUTINE TRSGEO

************************************************************************
* Author : Dmitri Bandurin
*
* This routines creates the geometry for the TRD support
* inside TRSL superlayer after it has installed in MOTH
*
* called from trdgeo.F
************************************************************************

      IMPLICIT NONE
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
c#include "gean/ZGDETV.fi"
c#include "gean/ZGTRDE.fi"
*...
      REAL PX,PY,PZ,Z_STEP,PAR(3),R_TUBE,CO,Z0
      REAL DX0,DY0,DY1,DZ0,DZ1,DX_TSP2,DY_TSP2,DZ_TSP2,
     +DX_TSP3,DY_TSP3,DZ_TSP3, DX_TSPC,DZ_TSPC
      INTEGER IROH,ICOP,ICAR,IAIR,IVAC,IVOL,MEDNUM

#include "arte/Arte.ff"
#include "auto/GEO.ff"

      Z0 = 0.0   

      DY0 = 200.0
      DY1 =  70.0
      DX0 = 200.0
      DZ0 =   1.5
      DZ1 =   0.5

      DX_TSP2 = 20.0
      DY_TSP2 = 0.5*(DY0 + DY1)
      DZ_TSP2 = DZ0

      DX_TSP3 = 37.5
      DY_TSP3 = DY0 - DY_TSP2
      DZ_TSP3 = DZ1
      
      R_TUBE = 11.0

      DZ_TSPC =  0.05
      DX_TSPC = 10.0
      Z_STEP = 0.05

      IROH = MEDNUM(1,'ROHACELL')
      IVAC = MEDNUM(1,'VACUUM') 
      IAIR = MEDNUM(1,'AIR')
      ICOP = MEDNUM(1,'COPPER')
      ICAR = MEDNUM(1,'CARBON')

*... Creation the volumes:
      PAR(1) = DX0 - 2*DX_TSP2 
      PAR(2) = 0.5*(DY0 + DY1) 		
      PAR(3) = DZ1-2*Z_STEP
      CALL GSVOLU( 'TSP1' , 'BOX ' ,  ICAR, PAR , 3 , IVOL )
      PAR(3) = Z_STEP
      CALL GSVOLU( 'TS12' , 'BOX ' ,  IROH, PAR , 3 , IVOL )
      PAR(1) = DX_TSP2 	
      PAR(2) = DY_TSP2		
      PAR(3) = DZ_TSP2
      CALL GSVOLU( 'TSP2' , 'BOX ' ,  IROH, PAR , 3 , IVOL )
      PAR(1) = DX_TSP3 	
      PAR(2) = DY_TSP3		
      PAR(3) = DZ_TSP3
      CALL GSVOLU( 'TSP3' , 'BOX ' ,  IROH, PAR , 3 , IVOL )
      PAR(1) = 0.0 	
      PAR(2) = R_TUBE 		
      PAR(3) = DZ1-2*Z_STEP
      CALL GSVOLU( 'TST1' , 'TUBE' , IAIR, PAR , 3 , IVOL )
      PAR(3) = Z_STEP
      CALL GSVOLU( 'TST2' , 'TUBE' , IAIR, PAR , 3 , IVOL )

*... Position the volumes:
      PX = 0.
      PZ = Z0 !0.0
      PY =-0.5*(DY0 - DY1) 
      CALL GSPOS('TST1', 1, 'TSP1', PX, PY, PZ, 0, 'ONLY')
      PY = 0.5*(DY0 - DY1) 
      CALL GSPOS('TSP1', 1, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      CALL CRGWAL('TSP1',1) 

      PY =-0.5*(DY0 - DY1) 
      CALL GSPOS('TST2', 1, 'TS12', PX, PY, PZ, 0, 'ONLY')
      PY = 0.5*(DY0 - DY1) 
      PZ = Z0 - DZ1+0.5*Z_STEP
      CALL GSPOS('TS12', 1, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      CALL CRGWAL('TS12',1) 
      PZ = Z0 + (DZ1+0.5*Z_STEP)
      CALL GSPOS('TS12', 2, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      CALL CRGWAL('TS12',2) 

      PX =-(DX0 - DX_TSP2)
      PY = 0.5*(DY0 - DY1)
      PZ = Z0 
      CALL GSPOS('TSP2', 1, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      PX =-PX
      CALL GSPOS('TSP2', 2, 'TRSL', PX, PY, PZ, 0, 'ONLY')

      PX =-(DX0 - DX_TSP3) 
      PY =-(DY1 + DY_TSP3)
      CALL GSPOS('TSP3', 1, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      PX = -PX
      CALL GSPOS('TSP3', 2, 'TRSL', PX, PY, PZ, 0, 'ONLY')

      CALL GSATT('TRSL','COLO',7)
      CALL GSATT('TSP1','COLO',3)
      CALL GSATT('TSP2','COLO',4)
      CALL GSATT('TSP3','COLO',5)
      CALL GSATT('TST1','COLO',1)

*... Division along Z-axe of TSP2, TSP3:
      CO = -(DZ_TSP2-Z_STEP)
      CALL GSDVN2('TS21','TSP2',1,3,CO,ICAR)
      CO = -CO - 0.5*Z_STEP
      CALL GSDVN2('TS22','TS21',1,3,CO,IROH)

      CO = -(DZ_TSP3-Z_STEP)
      CALL GSDVN2('TS31','TSP3',1,3,CO,ICAR)
      CO = -CO - 0.5*Z_STEP
      CALL GSDVN2('TS32','TS31',1,3,CO,IROH)

*... Position the cable boxes:
      PAR(1) = DX_TSPC 	
      PAR(2) = DY0 		
      PAR(3) = DZ_TSPC
      CALL GSVOLU( 'TSPC' , 'BOX ' ,ICOP, PAR , 3 , IVOL )
      PX = DX0 - DX_TSPC - 2*DX_TSP2 - 15.
      PY = 0.
      PZ = Z0 + DZ1 + DZ_TSPC
      CALL GSPOS('TSPC', 1, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      PZ = Z0 -(DZ1 + DZ_TSPC)
      CALL GSPOS('TSPC', 2, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      PX = -PX
      CALL GSPOS('TSPC', 3, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      pz = Z0 + DZ1 + DZ_TSPC
      CALL GSPOS('TSPC', 4, 'TRSL', PX, PY, PZ, 0, 'ONLY')
      CALL GSATT('TSPC','COLO',2)

      E N D
