*CMZ :  1.01/01 09/01/96  16.20.06  by  Fedor Ratnikov
*CMZ :  1.00/01 22/03/95  21.24.30  by  Alexander Snizhko
*CMZ :  1.00/00 11/01/95  12.54.34  by  Dominik Ressing
*CMZ :  0.01/02 25/08/94  22.04.36  by  Dominik Ressing
*-- Author :    Dominik Ressing   24/08/94
************************************************************************
*
* L1CELRG
* written by Dominik Ressing, 24/08/94 10.43.03
*
* This routines action:
* Report up to nMax cell ranges belonging to the region Reg in detector iDet.
* If N exceeds nMax, N is set to nMax + 1.
* For Iflag.ne.0 a geometric cell number is computed: no detector boundaries
* or cell overlaps are in effect.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iDet        in          detector specifier
* Reg         in          region to search Cells in
* dxdz,dydz   in          region shift in x,y for detector-region z difference
* iFlag       in          0: only wires on detector recorded
*                         1: region may exceed detector
* nMax        in          maximal number of ranges to report
* N           out         number of ranges found
* iFirst      out         first Hit in Range i
* iLast       out         last Hit in Range i
*
************************************************************************
c
      SUBROUTINE L1CELRG(iDet,Reg,dxdz,dydz,iFlag,nMax,N,iFirst,iLast)
c
      IMPLICIT NONE
c
#include "gean/gcbank.fi"
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      INTEGER iDet(L1DETSPElen),iFlag,nMax,N,iFirst(*),iLast(*)
      REAL    Reg(L1GEOINFlen),dxdz,dydz
c
      REAL    Po(L1POINTlen),y0,yu,x0,xu,top
      INTEGER jInf,j,k,jGeo,jCell
      LOGICAL full
c

c
c Reset of wire arrays
c
      CALL VZERO(iFirst(1),nMax)
      CALL VZERO(iLast(1),nMax)

      N = 0
      IF ( Reg(L1GEOINFsx).le.0. ) RETURN
      IF ( nMax.lt.1 ) RETURN
      N = 1
c
      CALL l1detif(iDet,jGeo,jInf,jCell)
      IF (jGeo.le.0 ) THEN
         WRITE (L1LUNprn,*) 'L1CEREG ==> invalid detector'
     $        ,iDet(L1DETSPEdet),iDet(L1DETSPElayer)
     $        ,iDet(L1DETSPEsection)
         RETURN
      END IF
      Po(L1POINTz0) = Q(jGeo+L1GEOINFz )
      Po(L1POINTdx) = dxdz
      Po(L1POINTdy) = dydz
      y0 = Reg(L1GEOINFy0)
      x0 = Reg(L1GEOINFx0)
      xu = x0 + Reg(L1GEOINFsx)
      yu = y0 + Reg(L1GEOINFsy)
      IF ( IQ(jInf+L1DETINFnCol).gt.1 ) THEN
         yu = y0
         top = y0 + Reg(L1GEOINFsy)
      ELSE
         yu = y0 + Reg(L1GEOINFsy)
      END IF
c
      IF ( Q(jGeo+L1GEOINFsin_al).gt.0. ) THEN
         Po(L1POINTx0) = x0                          ! point to upper left corner
         Po(L1POINTy0) = yu
         IF ( iFlag.eq.0.or.IQ(jInf+L1DETINFnCol).gt.1 ) THEN ! pads
                                                              ! could be
                                                              ! treated
                                                              ! by
                                                              ! l1pohit only
            CALL l1pohit(iDet,Po,iFirst(N),iLast(N)) ! cell numbers for first point
         ELSE
            CALL l1pohi1(iDet,Po,iFirst(N))          ! cell numbers for first point
            iLast(N) = iFirst(N)
         END IF
         Po(L1POINTx0) = xu                          ! point to lower right corner
         Po(L1POINTy0) = y0
      ELSE
         Po(L1POINTx0) = x0                          ! point to lower left corner
         Po(L1POINTy0) = y0
         IF ( iFlag.eq.0.or.IQ(jInf+L1DETINFnCol).gt.1 ) THEN
            CALL l1pohit(iDet,Po,iFirst(N),iLast(N)) ! cell numbers for first point
         ELSE
            CALL l1pohi1(iDet,Po,iFirst(N))          ! cell numbers for first point
            iLast(N) = iFirst(N)
         END IF
         Po(L1POINTx0) = xu                          ! point to upper right corner
         Po(L1POINTy0) = yu
      END IF
      IF ( iFlag.eq.0.or.IQ(jInf+L1DETINFnCol).gt.1 ) THEN
         CALL l1pohit(iDet,Po,j,k)                   ! cell numbers for last point
      ELSE
         CALL l1pohi1(iDet,Po,j)                     ! cell numbers for last point
         k = j
      END IF
c
      IF ( iFlag.eq.0.or.IQ(jInf+L1DETINFnCol).gt.1  ) THEN
         IF(iFirst(N).LE.0.AND.j.LE.0)   THEN ! Nothing found
            N = 0
            GOTO 999                          !return
         ELSEIF(j.LE.0)   THEN                ! Last point is outside detector
            iLast(N) = IQ(jInf+L1DETINFnRow)
         ELSEIF(iFirst(N).LE.0)   THEN        ! First point is outside detector
            iFirst(N) = 1
            iLast(N) = k
         ELSE                                ! both points are inside
            iLast(N) = MAX(k,iLast(N))
         ENDIF
      ELSE
         IF ( MIN(j,iFirst(N)).le.-9999 ) THEN ! one of the points invalid ??
            N = 0
            GOTO 999
         ELSE
            iLast(N) = MAX(k,iLast(N))
         END IF
      END IF
c
c     --- handle pad-like detectors
      IF(IQ(jInf+L1DETINFnCol).GT.1)   THEN
c      upper left corner
       Po(L1POINTx0) = x0       ! point to upper left corner
       Po(L1POINTy0) = top
       CALL l1pohit(iDet,Po,j,k)
c???     full = ((iLast(N)-iFirst(N)+1).ge.IQ(jInf+L1DETINFnRow)) !
                                 ! covered full region?
       IF(k.GT.0)  THEN
 100    CONTINUE
        N = N+1
        IF(N.GT.nMax) GOTO 555  ! exceeds memory
        iFirst(N) = iFirst(N-1)+IQ(jInf+L1DETINFnRow) ! new range on top
        iLast(N)  = iLast(N-1)+IQ(jInf+L1DETINFnRow)
        IF(iLast(N).LT.k)         GOTO 100 ! continue
       ENDIF
      ENDIF
c
 555  CONTINUE  ! region not covered within nMax (N=nMax+1)
 999  CONTINUE
c
      RETURN
c
      END
