//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcMatchPoint class
//
//**********************************************************************/
#include <math.h>

#include <CLHEP/Matrix/Vector.h>
#include <CLHEP/Utilities/fortran.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "marple/marple.hh"

extern "C"{
  extern float FORTRAN_ROUTINE( prob )(float *, int *);
	  }

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chRcMatchPoint = 0;
#endif

bool comp_chi2( const RcMatchPoint * prm1, const RcMatchPoint * prm2)
{  return prm1->chi2() < prm2->chi2(); }

RcMatchPoint::RcMatchPoint( RcPoint &rp1, 
			    RcPoint &rp2,
			    RcMatchFlag matchFlag,
			    int lineswim)
  : RcPoint(), fMatchFlag(matchFlag), foriginPoints(), fHitbList()
{
  
# ifdef MARPLEPERFORMANCES
  if(chRcMatchPoint == 0){
    ChronoManager CM;
    chRcMatchPoint = CM.createChrono("RcMatchPoint() ");
  }
  chRcMatchPoint->start();
# endif

  RcPoint* ptPoint=0;
  int createTmp=0;
  if( fabs(rp1.z() - rp2.z()) > 0.1e-3 )
    {
      // in case rp1.z() != rp2.z(), prolongate rp1 to rp2.z()
      ptPoint = new RcPoint(rp1);
      if( lineswim )
	{
	  ptPoint->lineswim(  rp2.z() );
	}else{
	  ptPoint->prolongate( rp2.z() );
	}
      createTmp = 1;
    }else{
      ptPoint = &rp1;
    }
  setZ( rp2.z());  
  setmark(0);
  setchi2(MARPLE_INFINITY);

  //find all HITB's
  fHitbList =  rp1.hitbList() ;
  unsigned int oldSize = fHitbList.size();

  // supposed that every entry in hitbList must appear only one time in list  
  std::vector<ArtePointer<HITB> > addHitb = rp2.hitbList();
  for( unsigned int i=0; i < addHitb.size(); i++ )
    {
      int newHitb = 1;
      for(unsigned int j=0; j < oldSize && newHitb ; j++ )
	{ 
	  if( addHitb[i] == hitbList()[j] ){ newHitb = 0 ;}
	}
      if( newHitb ){ fHitbList.push_back(addHitb[i]); }
    }
  
  int ifail;
  
  if( matchFlag == 0 ) fMatchFlag = 11111;
  
  //---------------------------------------------------------------
  // constrain parameters to match 
  fNvariables = fMatchFlag.Nvariables();
  HepVector p1(fNvariables);
  HepVector p2(fNvariables);
  
  // fill covariant matrix
  HepSymMatrix invCov_1(fNvariables, 1);
  HepSymMatrix invCov_2(fNvariables, 1);

  int param = 0;
  if( fMatchFlag.ifmarked_X() )
    {
      param++;
      p1(param) = ptPoint->x();
      p2(param) = rp2.x();
      invCov_1(param,param) = ptPoint->cov()(1,1);
      invCov_2(param,param) = rp2.cov()(1,1);
      int param2 = param;
      if(  fMatchFlag.ifmarked_Y() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(1,2);
	  invCov_2(param,param2) = rp2.cov()(1,2);
	}
      if(  fMatchFlag.ifmarked_TX() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(1,3);
	  invCov_2(param,param2) = rp2.cov()(1,3);
	}

      if(  fMatchFlag.ifmarked_TY() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(1,4);
	  invCov_2(param,param2) = rp2.cov()(1,4);
	}
      if(  fMatchFlag.ifmarked_PE() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(1,5);
	  invCov_2(param,param2) = rp2.cov()(1,5);
	}
    }

  if( fMatchFlag.ifmarked_Y() )
    {
      param++;
      p1(param) = ptPoint->y();
      p2(param) = rp2.y();
      invCov_1(param,param) = ptPoint->cov()(2,2);
      invCov_2(param,param) = rp2.cov()(2,2);
      int param2 = param;
      if(  fMatchFlag.ifmarked_TX() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(2,3);
	  invCov_2(param,param2) = rp2.cov()(2,3);
	}

      if(  fMatchFlag.ifmarked_TY() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(2,4);
	  invCov_2(param,param2) = rp2.cov()(2,4);
	}
      if(  fMatchFlag.ifmarked_PE() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(2,5);
	  invCov_2(param,param2) = rp2.cov()(2,5);
	}
    }

  if( fMatchFlag.ifmarked_TX() )
    {
      param++;
      p1(param) = ptPoint->tx();
      p2(param) = rp2.tx();
      invCov_1(param,param) = ptPoint->cov()(3,3);
      invCov_2(param,param) = rp2.cov()(3,3);
      int param2 = param;
      if(  fMatchFlag.ifmarked_TY() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(3,4);
	  invCov_2(param,param2) = rp2.cov()(3,4);
	}
      if(  fMatchFlag.ifmarked_PE() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(3,5);
	  invCov_2(param,param2) = rp2.cov()(3,5);
	}
    }
  if( fMatchFlag.ifmarked_TY() )
    {
      param++;
      p1(param) = ptPoint->ty();
      p2(param) = rp2.ty();
      invCov_1(param,param) = ptPoint->cov()(4,4);
      invCov_2(param,param) = rp2.cov()(4,4);
      int param2 = param;
      if(  fMatchFlag.ifmarked_PE() )
	{
	  param2++;
	  invCov_1(param,param2) = ptPoint->cov()(4,5);
	  invCov_2(param,param2) = rp2.cov()(4,5);
	}
    }
  if( fMatchFlag.ifmarked_PE() )
    {
      param++;
      p1(param) = ptPoint->pe();
      p2(param) = rp2.pe();
      invCov_1(param,param) = ptPoint->cov()(5,5);
      invCov_2(param,param) = rp2.cov()(5,5);
    }

  //---------------------------------------------------------
  // calculate invariant matrix

  //cout << " 1 point " << p1 << " " << invCov_1 << endl;
  //cout << " 2 point " << p2 << " " << invCov_2 << endl;

  invCov_1.invert(ifail);
  if( ifail != 0 )
    {
      error_message();   
      setmark(ifail); 
      if( createTmp ) delete ptPoint;
# ifdef MARPLEPERFORMANCES
    chRcMatchPoint->stop();
# endif
      return;
    }

  invCov_2.invert(ifail);
  if( ifail != 0 )
    {
      error_message();   
      setmark(ifail); 
      if( createTmp ) delete ptPoint;
# ifdef MARPLEPERFORMANCES
    chRcMatchPoint->stop();
# endif
      return;
    }

  // calculate the matrix 1/Cov = 1/Cov1 + 1/Cov2 

  HepSymMatrix cov_match(invCov_1);
  cov_match += invCov_2;
  cov_match.invert(ifail);
  if( ifail != 0 )
    {
      error_message();   
      setmark(ifail); 
      if( createTmp ) delete ptPoint;
# ifdef MARPLEPERFORMANCES
    chRcMatchPoint->stop();
# endif
      return;
    }

  // calculate the mean parameters

  HepVector pmean = invCov_1 * p1  ;
  pmean += (invCov_2*p2);
  pmean = cov_match * pmean; 

  // calculate the chi2 of matching
  p1 -= pmean;
  p2 -= pmean;
  HepDouble chi2 = invCov_1.similarity( p1) +  invCov_2.similarity( p2);
  setchi2( chi2 );

  //  cout << " chi2 "<< chi2 << " pmean " << pmean << " " << cov_match << endl;


  //---------------------------------------------------
  // save calculated values :

  param = 0;
  if( fMatchFlag.ifmarked_X())
    { 
      param++;
      setX(pmean(param));
      setcov(1,1, cov_match(param, param) );
      int param2 = param;
      if( fMatchFlag.ifmarked_Y())
	{
	  param2++;
	  setcov(1,2, cov_match(param, param2) );	  
	}
      if( fMatchFlag.ifmarked_TX())
	{
	  param2++;
	  setcov(1,3, cov_match(param, param2) );	  
	}
      if( fMatchFlag.ifmarked_TY())
	{
	  param2++;
	  setcov(1,4, cov_match(param, param2) );	  
	}
      if( fMatchFlag.ifmarked_PE())
	{
	  param2++;
	  setcov(1,5, cov_match(param, param2) );	  
	}
    }
  if( fMatchFlag.ifmarked_Y())
    { 
      param++;
      setY(pmean(param));
      setcov(2,2, cov_match(param, param) );
      int param2 = param;
      if( fMatchFlag.ifmarked_TX())
	{
	  param2++;
	  setcov(2,3, cov_match(param, param2) );	  
	}
      if( fMatchFlag.ifmarked_TY())
	{
	  param2++;
	  setcov(2,4, cov_match(param, param2) );	  
	}
      if( fMatchFlag.ifmarked_PE())
	{
	  param2++;
	  setcov(2,5, cov_match(param, param2) );	  
	}
    }
  if( fMatchFlag.ifmarked_TX())
    { 
      param++;
      settx(pmean(param));
      setcov(3,3, cov_match(param, param) );
      int param2 = param;
      if( fMatchFlag.ifmarked_TY())
	{
	  param2++;
	  setcov(3,4, cov_match(param, param2) );	  
	}
      if( fMatchFlag.ifmarked_PE())
	{
	  param2++;
	  setcov(3,5, cov_match(param, param2) );	  
	}
    }
  if( fMatchFlag.ifmarked_TY())
    { 
      param++;
      setty(pmean(param));
      setcov(4,4, cov_match(param, param) );
      int param2 = param;
      if( fMatchFlag.ifmarked_PE())
	{
	  param2++;
	  setcov(4,5, cov_match(param, param2) );	  
	}
    }
  if( fMatchFlag.ifmarked_PE())
    { 
      param++;
      setpe(pmean(param));
      setcov(5,5, cov_match(param, param) );
    }

  // save origin
  //RcPoint* address = &rp1;
  foriginPoints.push_back( &rp1 );
  //address = &rp2;
  foriginPoints.push_back( &rp2 );

  if( createTmp )  delete ptPoint;

  checkAndCorrectCov();
  ptPoint = 0;
# ifdef MARPLEPERFORMANCES
    chRcMatchPoint->stop();
# endif
}

//----------------------------------------------------------------
// call CERNLIB function probability

HepDouble RcMatchPoint::quality() const
{
  if( fNvariables == 0 ) return 0;
  else
    {
      float getchi2 = chi2();
      int Nvar = fNvariables; // not beauty.... have to avoid const
      return (HepDouble) FORTRAN_ROUTINE(prob)( &getchi2, &Nvar);
    }
} 


//----------------------------------------------------------------
//  correct RcMatchPoint by measured HITB
void RcMatchPoint::addHitb( const ArtePointer<HITB> hitb, int & iErr )
{
  iErr = 0;
  int newHit = 1;
  for( unsigned int i=0; i < fHitbList.size(); i++ )
    {
      if( fHitbList[i] == hitb ){ newHit = 0; }
    }
  if( !newHit )
    {
      int index = hitb - *ArteTable<HITB>::begin();
      arteMsg << "MARPLE: HITB " << index 
	   << " already connected with that RcMatchPoint";
      arteMsg.allp();
      return;
    }
  
  HepDouble deltaChi2 = RcPoint::filter( hitb, iErr);
  if( !iErr )
    {
      fHitbList.push_back(hitb);
      setchi2( chi2() + deltaChi2 );
      //fNvariables++;
      // svd_f_chi2 is taken from Joerg note = 0.2
      //HepDouble svd_f_chi2 = 0.2;
      //setquality( quality() + 1 - svd_f_chi2 * deltaChi2 );  
    }else{
      int index = hitb - *ArteTable<HITB>::begin();
      arteMsg << "MARPLE: addHitb: unable to filter RcMatchPoint to HITB" 
	      << index;
      arteMsg.allp();
    }
}
//----------------------------------------------------------------

//  Kalman refit of the one of original RcPoint's using RcMatchPoint parameters
HepDouble RcMatchPoint::refitOrig(int & iErr, int iOrig )
{
  if( iOrig >= (int)origin().size() ) 
    {
      arteMsg << "MARPLE: refitOrig : only " << origin().size() 
	      << " RcPoint connected with RcMatchPoint object" ;
      arteMsg.allp();
      return 0;
    }
  HepDouble deltaChi2 = 0;
  iErr = 0;
  int iErr1 = 0;
  if( iOrig >= 0 )
    {
      deltaChi2 += origin()[iOrig]->refit(iErr1);
      if( iErr1 ) iErr=iErr1;
    }else{
      for( unsigned int iCount=0; iCount < origin().size(); iCount++ )
	{
	  deltaChi2 += origin()[iCount]->refit(iErr1);
	  if( iErr1 ) iErr=iErr1;
	}
    }
  return deltaChi2;
}

//----------------------------------------------------------------

// returns relation with MTRA 
std::vector<ArtePointer<MTRA> > RcMatchPoint::mtraList() const
{
  std::vector<ArtePointer<MTRA> > mList;
  if( ArteTable<MTRA>::size() == 0 ) return mList;
  for( unsigned int i=0; i < origin().size(); i++ )
    {
      std::vector<ArtePointer<MTRA> > nextList( (*origin()[i]).mtraList() );
      for( unsigned int j=0; j < nextList.size(); j++ )
  	{
	 if( !RcMtraOk(nextList[j], mList ) ) mList.push_back(nextList[j]);
	}
    }

  std::vector<ArtePointer<HITB> > hitbL = hitbList();  
  for( unsigned int ihit = 0; ihit<hitbL.size(); ihit++ )
    for(ArteRelationIterator<MIMP> riMimp = (hitbL[ihit])->mimp.begin();
	riMimp != (hitbL[ihit])->mimp.end();
	++riMimp )
      {
	ArtePointer<MTRA> mEntry( (*riMimp)->mtra);
	if( !RcMtraOk( mEntry, mList ) )
	  mList.push_back(mEntry); 
      }
  return mList;
}

//----------------------------------------------------------------

// returns relation with HITC 
std::vector<ArtePointer<HITC> > RcMatchPoint::hitcList() const
{
  std::vector<ArtePointer<HITC> > hList;
  for( unsigned int i=0; i < origin().size(); i++ )
    {
      std::vector<ArtePointer<HITC> > nextList( (*origin()[i]).hitcList() );
      for( unsigned int j=0; j < nextList.size(); j++ )
  	{
	  hList.push_back(nextList[j]);
	}
   }
  return hList;
}

ostream & operator << (ostream & os, const RcMatchPoint &rp)
{
  os << "Match Point with  chi2 = " << rp.chi2() 
     << " , Quality = " << rp.quality() <<  flush;
  os << " Variables used for match : " << rp.matchFlag() <<  endl;
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  return os ;
}

