#ifndef _RCSEGCLUSBUILDER_HH_
#define _RCSEGCLUSBUILDER_HH_
//---------------------------------------------------------------------
// Created: 21.04.98 by   Olya Igonkina

// MARPLE builder class for match Segments and Clusters

//---------------------------------------------------------------------


#include <vector>
#include <iterator>

#include "marple/RcSegPoint.hh"
#include "marple/RcClusPoint.hh"
#include "marple/RcMatchBuilder.hh"


class RcSegClusBuilder: public RcMatchBuilder<RcSegPoint, RcClusPoint>
{
public:
  ///              default construtor
  RcSegClusBuilder(); 

  ///              make match of two vectors with rcConst constants
  RcSegClusBuilder( std::vector<RcSegPoint  *> vecLeft,
		    std::vector<RcClusPoint *> vecRight,
		    RcMatchConst & rcConst,
		    RcMatchConst & rcConstBrem);

  ///               destructor
  ~RcSegClusBuilder();

  ///               set constants for matching of Bremstralung
  inline void setBremConst( RcMatchConst & rcConstBrem)
    { fptRcConstBrem = &rcConstBrem; }; 
  ///               returns constants for matching
  inline RcMatchConst  * ptToBremConst() const { return fptRcConstBrem; };

  inline int Nbrem()              const{ return fNbrem;              }
  inline int NbadBrem()           const{ return fNbadBrem;           }
  // number of real pair correspond to the :
  inline int rNmtra()             const{ return frNmtra;             }
  inline int rNelectron()         const{ return frNelectron;         }
  inline int rNjpsiElectron()     const{ return frNjpsiElectron;     }
  inline int rNmtraBrem()         const{ return frNmtraBrem;         }
  inline int rNelectronBrem()     const{ return frNelectronBrem;     }
  inline int rNjpsiElectronBrem() const{ return frNjpsiElectronBrem; }

  // number of pair correspond to the :
  inline int Nmtra()             const{ return fNmtra;             }
  inline int Nelectron()         const{ return fNelectron;         }
  inline int NjpsiElectron()     const{ return fNjpsiElectron;     }
  inline int NmtraBrem()         const{ return fNmtraBrem;         }
  inline int NelectronBrem()     const{ return fNelectronBrem;     }
  inline int NjpsiElectronBrem() const{ return fNjpsiElectronBrem; }

  // MatchPoint.mark() = 0 standard match
  //                   - 1 matching with segment momentum estimation
  //                   - 2 matching using energy of cluster
  //                   - 3 bremstralung
  //                   - 4 match rings

  typedef  int (RcSegClusBuilder::* MatchFunction) ( RcSegPoint  * segment, 
						     RcClusPoint * cluster,
						     RcMatchPoint* &result);

  void doMatch(); // default one as in RcmatchBuilder

  void doMatch(  MatchFunction matchFunction,
		 int markCombination = 1);

  void matchBrem(          int markCombination = 1);
  void matchWithMomentum(  int markCombination = 1);
  void matchWithEcluster(  int markCombination = 1);
  void matchRings(         int markCombination = 1);

  void saveRSEG(int & iErr ) const;

  void statMC();
  //   if iType = 0  - print whole statistics
  //            = 1  - only charged statictics
  //            = -1 - only bremsstrahlung statictics
  void stat( int iLong, int iType = 0 ) const;

  ///               fill ntuple with statistics
  void fillMatch( int iNtup = 0) const; 
  void fillDeltas( int iNtup = 0, int pulsNtup = 0) const; 


  /*
  void deleteMatch( int iStat = 1);
  void deleteLeft( int iStat  = 1);
  void deleteRight( int iStat = 1);
  
  void deleteAll( int iStat = 1);
  */

protected:

  int matchBrem(         RcSegPoint  * segment, 
			 RcClusPoint * cluster,
			 RcMatchPoint* &result);

  int matchWithMomentum( RcSegPoint  * segment, 
			 RcClusPoint * cluster,
			 RcMatchPoint* &result);
  int matchWithEcluster( RcSegPoint  * segment, 
			 RcClusPoint * cluster,
			 RcMatchPoint* &result);
  int matchRingsMagnet(  RcSegPoint  * segment, 
			 RcClusPoint * cluster,
			 RcMatchPoint* &result);
  int matchRingsNoMagnet(RcSegPoint  * segment, 
			 RcClusPoint * cluster,
			 RcMatchPoint* &result);

private:

  RcMatchConst *  fptRcConstBrem;

  // number of found pairs
  int    fNmtra;
  int    fNelectron;
  int    fNjpsiElectron;
  int    fNmtraBrem;
  int    fNelectronBrem;
  int    fNjpsiElectronBrem;

  // real number of pair
  int    frNmtra;
  int    frNelectron;
  int    frNjpsiElectron;
  int    frNmtraBrem;
  int    frNelectronBrem;
  int    frNjpsiElectronBrem;

  // found by bremstralung estimation
  int    fNbrem;
  int    fNbadBrem;

};

#endif
