//--------------------------------------------------------------------
// Created: 17.12.99 by   Olya Igonkina
/* MARPLE function for matching OTR segments and ECAL clusters

   FORTRAN_ROUTINE(matchotrecal)
   // ( float * threshold_cut ) - the threshold cut is not used now
   // if threshold_cut > 200 - it is Ktrig cut, otherwise it is pt_cut  

   Note that in general reconstruction scheme the function matchRsegCare
   from CARE package should be used instead matchotrecal.
 */
//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcSegClusBuilder.hh"
#include "arte/RSEG_cmp.hh"

#include "marple/RcMcFunction.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchOtrEcal = 0;
#endif


//definition of constants for SUSI-CARE matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
//use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2

// match with P estimation
RcMatchConst OCmatchConst = { 
 11000,     1335.,   -30.,  1500.,      0,          2,            2,
    1,       9.,      9.,   1.e+10,  1.e+10,   1. ,   10000.
};


RcMatchConst OCmatchSoftConst = { 
  11000,     1335.,   -30.,  1500.,      0,          2,            2,
  1,       20.,      20.,   1.e+10,  1.e+10,   1.,   10000.
};

extern "C"  
void FORTRAN_ROUTINE(matchotrecal)() // float * threshold_cut ) 
{
# ifdef MARPLEPERFORMANCES
  if(chMatchOtrEcal == 0){
    ChronoManager CM;
    chMatchOtrEcal = CM.createChrono("match Otr Ecal ");
  }
  chMatchOtrEcal->start();
# endif

  // ----------------------------------------------------
  
  RcSegClusBuilder gIOtrEcalMatch;  // builder for charge track search


  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------

  if( MARPLE_SOFT )
    {
      gIOtrEcalMatch.setConst(     OCmatchSoftConst);
    }else{ 
      gIOtrEcalMatch.setConst(     OCmatchConst);
    }      

  //--------------------------------------------------------------------
  // create vector of segments from Outer Tracker
  //--------------------------------------------------------------------

  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if( ( (*irseg)->cmp & Rsegc::bitsign ) != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::patt) == 0  )  continue;
      if( gIOtrEcalMatch.ptToMatchConst()->
	  geomAccept(HepDouble((*irseg)->ze)) )
	{ 	  
	  RcSegPoint * ptToPoint = new RcSegPoint( *irseg , 1); // endPoint
	  // !!!!!!!!!!!! CAREFULLL !!!!!!!!!!!!!!!
	  ptToPoint->setty(ptToPoint->y()/ptToPoint->z() );
	  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	  ptToPoint->lineswim(gIOtrEcalMatch.ptToMatchConst()->z_plane);
	  gIOtrEcalMatch.addPointLeft( *ptToPoint);
	  ptToPoint = 0;
	}
    }


  // no candidate found --- no reason go through clusters
  if( gIOtrEcalMatch.vectorLeft().size() == 0 ) 
    {
# ifdef MARPLEPERFORMANCES
      chMatchOtrEcal->stop();
# endif
      return;
    }

  //--------------------------------------------------------------------
  // create vector of clusters from CARE
  //--------------------------------------------------------------------

  static std::vector<RcClusPoint *> nextClusterSet;
      
  if( MARPLE_BKGR )
    {
      // clusters from previous event
      for( unsigned int clus= 0; clus < nextClusterSet.size(); clus++ )
	gIOtrEcalMatch.addPointRight( *(nextClusterSet[clus]) );
      
      nextClusterSet.erase(nextClusterSet.begin(), nextClusterSet.end());
      
    }
  

  int iErr  = 0;
  for( ArteTable<RCCL>::iterator irccl = ArteTable<RCCL>::begin();
       irccl != ArteTable<RCCL>::end(); ++irccl )
    {
      if( (*irccl)->flag < 0 ) continue;
      RcClusPoint * ptPoint = 0;
      //if( (*irccl)->wid<6 || (*irccl)->wid> 10 ) takeRCCL = 0;
      //      if( maTakeRccl(*threshold_cut, *irccl) )
	{
	  ptPoint = new RcClusPoint( (*irccl));
	  if( MARPLE_BKGR )
	    nextClusterSet.push_back(ptPoint);
	  else
	    gIOtrEcalMatch.addPointRight( *ptPoint);
	  ptPoint = 0;
	}
      ptPoint = 0;
    }  

  //--------------------------------------------------------------------
  // I step : 
  //--------------------------------------------------------------------

  gIOtrEcalMatch.doMatch();

  //--------------------------------------------------------------------
  // estimate momentum for some of matched pairs
  //--------------------------------------------------------------------
  for( unsigned int iMatch = 0; 
       iMatch < gIOtrEcalMatch.matchList().size(); iMatch++ )
    {
      RcPoint * otr  = gIOtrEcalMatch.leftOrigin(iMatch);
      RcPoint * matchPt = gIOtrEcalMatch.matchList()[iMatch];
      RcPoint tmp = *otr ;
 
      static float bp = 288;
      float overMomentum = (otr->z()*otr->tx() - otr->x())/bp;
      // in principle we should correct it by factor 
      // sqrt((1+tx**2)/(1+tx**2+ty**2)) 
      // but it does not make big difference since the angles are small
 
      if( fabs( overMomentum ) > 2 ) // Pt < 0.5 GeV
        {
          continue;
        }
 
      tmp.setpe( overMomentum );
 
      if ( fabs(tmp.pe()) < 1 )
        {
          tmp.estimatePEfromTAR(6, iErr);
        }else{
          if( tmp.pe() < 0 ) tmp.setpe(-1);
          else               tmp.setpe(1);
        }
      matchPt->setpe( tmp.pe() );
      matchPt->setcov(5,5, tmp.cov()(5,5) );
    }
 

  //--------------------------------------------------------------------
  // outprint
  //--------------------------------------------------------------------


  if( MARPLE_DEBUG >= 20 )
    {
      gIOtrEcalMatch.fillMatch(8136);
      gIOtrEcalMatch.fillMatch(8136);
    }else  if( MARPLE_DEBUG >= 10 ){
      gIOtrEcalMatch.fillDeltas(8236); // deltas, pulses
    }
  if( MARPLE_DEBUG > 0 && MARPLE_DEBUG != 10 && MARPLE_DEBUG != 20 )
    {
      static int firstOTRCARE = 1;
      if( firstOTRCARE )
	{
	  cout << "================= match ============="<< endl; 
	  gIOtrEcalMatch.stat(1);
	  cout << endl << endl << endl;
	  firstOTRCARE = 0;
	}
      gIOtrEcalMatch.stat(MARPLE_DEBUG,0);
    }
  
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) 
    {
      gIOtrEcalMatch.saveRSEG(iErr);
    }

  if( !MARPLE_BKGR )
    {
      gIOtrEcalMatch.saveRelation(iErr);
      if( iErr ){ cout << "MARPLE: some relations were not saved " << endl;}
    }
  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  gIOtrEcalMatch.deleteAll();  

# ifdef MARPLEPERFORMANCES
    chMatchOtrEcal->stop();
# endif
}
