
// Boris Fominykh  23.12.1999
// member functions of class MuDbAlign

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "murec/MuDbAlign.h"

//** Constructor */
MuDbAlign::MuDbAlign()
  :  MuDbTable::MuDbTable()
{
  Ini();
  strncpy(DBtable,"MuonAlignment",14);
  strncpy(Desc,"I:allint F:allfloat",20);
  strncpy(Title,"Muon Alignment Constants",25);
}

//** Destructor */
MuDbAlign::~MuDbAlign()
{
  Release();
}

//## Methods

//** Initialization */
void MuDbAlign::Ini()
{

  Nframe=50;
  Status = 0;  
  Ncollect = 0;
  SizeW  = 0;
  ElIntW = NULL;
  ElFlW  = NULL;
  return;
}

//** Book temporary space */
int MuDbAlign::Book()
{
  if(Nframe <= 0) {
    cout << "MuDbAlign::Book - Nframe= " << dec << Nframe;
    cout << " is NOT set" << endl;
    return 1; 
  }
  if(SizeW == Nframe) return 0;
  if(SizeW > 0) Release();
  ElIntW = new int[NelInt*Nframe];
  if(ElIntW == NULL) return 2;
  ElFlW = new float[NelFloat*Nframe];
  if(ElFlW == NULL) {
    delete []ElIntW;
    return 3;
  }
  SizeW = Nframe;
  Ncollect = 0;
  int* pel = ElIntW;
  for(int i=0; i<NelInt*SizeW; i++) *(pel+i)=0;
  float* pelf = ElFlW;
  for(int i=0; i<NelFloat*SizeW; i++) *(pelf+i)=0.;
  return 0;
}

//** Release the temporary space*/
void MuDbAlign::Release()
{
  if(SizeW > 0) {
    if(ElIntW != NULL) delete []ElIntW;
    if(ElFlW != NULL) delete []ElFlW;
    Status = 0;
    Ncollect = 0;
    SizeW  = 0;
    ElIntW = NULL;
    ElFlW  = NULL;
  }
  return;
}
 
//** Read the table from DB into booked temporary space*/
int MuDbAlign::Read()
{
  Status = 0;
  Ncollect = 0;
  int ret=0;
  int nel=0;
// open DB
  ret=MuDbTable::Open(0);
  if(ret > 0) return ret;
// fetch the table
  ret=MuDbTable::Fetch();
  if(ret != 0) goto labelexit;
// book the space
  Nframe=Size/NelFloat;
  ret=MuDbAlign::Book();
  if(ret != 0) return 21;
// read the table from DB
  Nf=SkipTableField(X,1);
  nel = GetTableField(X,TABLEint,NelInt*SizeW,ElIntW);
  if(nel <= 0) {
    ret=31;
    goto labelexit;
  }
  Nf=SkipTableField(X,2);
  nel = GetTableField(X,TABLEfloat,NelFloat*SizeW,ElFlW);
  if(nel <= 0) {
    ret=32;
    goto labelexit;
  }
// close DB;
  Status = 1;
  ret = 0;
labelexit:
  MuDbTable::Close();
  return ret;
}

//** Write the table from temporary space to DB */
int MuDbAlign::Write()
{
  if(SizeW <= 0) return 1;
  if(Status != 2) return 2;
  if(Ncollect < 1) return 3;
  // open DB
  int ret=MuDbTable::Open(1);
  if(ret > 0) return ret;
  // write the table to DB
  SkipTableField(X,1);
  int ret1=PutTableField(X,TABLEint,NelInt*Ncollect,ElIntW);
  int ret2=PutTableField(X,TABLEfloat,NelFloat*Ncollect,ElFlW);
  int ret4=StoreTable(X,T,0);
  // release the temporary space
  MuDbAlign::Release();
  // close DB;
  MuDbTable::Close();
  //
  if(ret1 > 0 && ret2 > 0 && ret4 > 0) return 0;
  if(ret1 <= 0) return 11; 
  if(ret2 <= 0) return 12; 
  if(ret4 <= 0) return 13; 
  return 14;
}

//** Get the alignment constants of one frame from temporary space*/
int MuDbAlign::Get(int* nframe,MuAlign* palign)
{
  palign->Ini();
  if(Status != 0 && Status != 1) {
    cout << "MuDbAlign::Get - Status= " << Status;
    cout << " No actions" << endl;
    return 41;
  }
  if(Status == 0) {  
    int ret=MuDbAlign::Read();
    if(ret != 0) return ret;
  }
  if(*nframe < 0 || *nframe >= SizeW) {  
    cout << "MuDbAlign::Get - Nframe= " << *nframe;
    cout << " is OUT of range 0-" << SizeW-1 << endl;
    return 42;
  }
  int* elint =ElIntW+(*nframe)*NelInt;
  float* elfl=ElFlW+(*nframe)*NelFloat;

// extract the integers
  palign->nframe = *nframe;
  palign->type = *elint;
  palign->msl  = *(elint+1);
  palign->nlay = *(elint+2);
  for(int j=0; j<3; j++) {
    palign->nx1u[j]=*(elint+j+3);
    palign->nx2u[j]=*(elint+j+6);
    palign->nx99u[j]=*(elint+j+9);
    palign->nx1d[j]=*(elint+j+12);
    palign->nx2d[j]=*(elint+j+15);
    palign->nx99d[j]=*(elint+j+18);
  }

// extract the floats
  palign->dx = *elfl;
  palign->dy  = *(elfl+1);
  palign->dz  = *(elfl+2);
  palign->xc  = *(elfl+3);
  palign->yc  = *(elfl+4);
  palign->angl= *(elfl+5);

  palign->Valid = 1;
  return 0;
}

//** Print the alignment constants */
void MuDbAlign::Print()
{
  ofstream fout("MuAlignDump.out");
  if (!fout) 
  {
    cout << "MuDbAlign::Print - can not open file MuAlignDump.out";
    cout << endl;
    return;
  }

  int ret;
  MuAlign A;
  A.Ini();
  if(Status == 0) {  
    ret=MuDbAlign::Read();
  }
  int nfr=MuDbAlign::getNframe();

  cout << "print table MuonAlignment into file MuAlignDump.out" << endl;
  cout << "DataBase: " << DBname << " Table: " << DBtable;
  cout << " Title: <" << Title << "> " << endl;
  cout << " Desc: <" << Desc << "> Size= " << SizeW;
  cout << " Run " << Run << " Minor " << Minor << " frames " << nfr << endl;

  fout << "DataBase: " << DBname << " Table: " << DBtable;
  fout << " Title: <" << Title << "> " << endl;
  fout << " Desc: <" << Desc << "> Size= " << SizeW;
  fout << " Run " << Run << " Minor " << Minor << " frames " << nfr << endl;

  if(nfr > 0) {
    for(int i=0; i<nfr; i++) {
      int nfr = i;
      int ret=MuDbAlign::Get(&nfr,&A);
      if(ret == 0) {
        fout << " " << endl;
	fout << "#frame " << nfr << " type " << A.type;
	fout << " msl " << A.msl << " nlay " << A.nlay;
        fout << " A.Valid= " << A.Valid << endl;
        if(A.nlay > 0 && A.nlay < 4) {
	for(int j=0; j<A.nlay; j++) {
         fout << "Lay " << j+1;
         if(A.type == 1) {
	  fout << " Up: nx1 " << A.nx1u[j] << " nx2 " << A.nx2u[j] << " nx99 " << A.nx99u[j];
	  fout << " Down: nx1 " << A.nx1d[j] << " nx2 " << A.nx2d[j] << " nx99 " << A.nx99d[j];
         }
         if(A.type == 2) {
	  fout << " Quadrants - nqu1 " << A.nx1u[j] << " nqu2 " << A.nx2u[j];
         }
         fout << endl;
	}
	}
	fout << " dx " << A.dx << " dy " << A.dy << " dz " << A.dz;
	fout << " xc " << A.xc << " yc " << A.yc << " angl " << A.angl << endl;
      }
      else {
        fout << " " << endl;
	fout << "#frame " << i << " is NOT read" << endl;
      }
    }
  }
  return;
}

//** Add one element to the temporary space */
int MuDbAlign::Add(MuAlign* palign)
{
  if(Status != 0 && Status != 2) {
    cout << "MuDbAlign::Add - Status= " << Status;
    cout << " No actions" << endl;
    return 41;
  }
  if(Status != 2) {  
    Nframe=50;
    int ret=MuDbAlign::Book();
    if(ret != 0) return ret;
    Status=2;
  }
  if(Ncollect >= SizeW) {
    cout << "MuDbNframe::Add - Ncollect= " << Ncollect;
    cout << " SizeW= " << SizeW << " is NOT enough" << endl;
    return 42;
  }

  // pointers
  int* elint=ElIntW+Ncollect*NelInt;
  float* elfl=ElFlW+Ncollect*NelFloat;

// add the integers
  *elint=palign->type;
  *(elint+1)=palign->msl;
  *(elint+2)=palign->nlay;
  for(int j=0; j<3; j++) {
    *(elint+j+3)=palign->nx1u[j];
    *(elint+j+6)=palign->nx2u[j];
    *(elint+j+9)=palign->nx99u[j];
    *(elint+j+12)=palign->nx1d[j];
    *(elint+j+15)=palign->nx2d[j];
    *(elint+j+18)=palign->nx99d[j];
  }

// add the floats
  *elfl=palign->dx;
  *(elfl+1)=palign->dy;
  *(elfl+2)=palign->dz;
  *(elfl+3)=palign->xc;
  *(elfl+4)=palign->yc;
  *(elfl+5)=palign->angl;

  Ncollect += 1;
  return 0;
}

//** Create the table in DB from the ascii file*/
int MuDbAlign::Create()
{
  FILE *hl = fopen(FileName, "r");
  if(!hl) {
    cout << "MuDbAlign::Create - no input File" << endl;
    return 1;
  }
  char numb[11] = "0123456789";
  int control=0;
  MuAlign A;
  int Nfr=0;
// loops over lines: max 1000 lines
  int nline=0;
  char line[121];
  for(int il=0; il<5000; il++) {
// read the line
    if(fgets(line,sizeof(line),hl) == NULL) {
      fclose(hl);
      if(feof(hl)) goto goodexit;
      else { 
        cout << "MuDbAlign::Create -  error of file read";
        return 2;
      }
    }
    ++nline;
    char *card = &line[0];
// read the first word from the line
    char word[10];
    int nitem=sscanf(card,"%s",word);
// pass  blank lines and commented cards
    if(nitem == -1) continue;
    if(word[0]=='#' || word[0]=='*' || word[0]=='/') continue;
    card+=Itl(card,nitem);

// end of file
    if(!strcmp(word,"END")) {
      fclose(hl);
      goto goodexit;
    }

// MU - control card
    if(word[0]=='M' || word[1]=='U') {
      A.Ini();
      for(int i=0; i<11; i++) {
        if(word[2] == numb[i]) {
          A.msl = i;
          break;
        }
      }
      char type[10];
      nitem=sscanf(card,"%s",type);
      if(nitem == -1) continue;
      card+=Itl(card,nitem);
      if(!strcmp(type,"tube"))  A.type=1;
      if(!strcmp(type,"pad"))   A.type=1;
      if(!strcmp(type,"pixel")) A.type=2;
      char nlay[10];
      nitem=sscanf(card,"%s",nlay);
      if(nitem == -1) continue;
      card+=Itl(card,nitem);
      if(!strcmp(nlay,"Nlay=")) {
        int nl=0;
        nitem=sscanf(card,"%d",&nl);
        if(nitem == -1) continue;
        card+=Itl(card,nitem);
        Nfr += 1;
        A.nframe=Nfr;
        A.nlay=nl;
        control=1;
      }
      continue;
    }

// lay
    if(!strcmp(word,"lay")) {
    if(control == 1)        {
      int lay  =0;
      int nx1u =0;
      int nx2u =0;
      int nx99u=0;
      int nx1d =0;
      int nx2d =0;
      int nx99d=0;
      if (sscanf(line, " lay %d nx1u %d nx2u %d nx99u %d nx1d %d nx2d %d nx99d %d " ,
               &lay,&nx1u,&nx2u,&nx99u,&nx1d,&nx2d,&nx99d) == 7) {
        if(lay > 0 && lay <= A.nlay) {
          A.nx1u[lay-1] =nx1u;
          A.nx2u[lay-1] =nx2u;
          A.nx99u[lay-1]=nx99u;
          A.nx1d[lay-1] =nx1d;
          A.nx2d[lay-1] =nx2d;
          A.nx99d[lay-1]=nx99d;
        } 
        else {
          cout << " NOT correct lay= " << lay << " Nlay= " << A.nlay << endl;
          cout << line << endl;
        }
      }
      else {
        cout << "ERROR: " << line << endl;
      }
      continue;
    }
    }

// nqu1
    if(!strcmp(word,"nqu1")) {
    if(control == 1)        {
      int nqu1=0;
      int nqu2=0;
      if (sscanf(line, " nqu1 %d nqu2 %d " ,
               &nqu1,&nqu2) == 2) {
        if(nqu1 > 0 && nqu1 < 5 && nqu2 > 0 && nqu2 < 5) {
          A.nx1u[0] =nqu1;
          A.nx2u[0] =nqu2;
        } 
        else {
          cout << " NOT correct nqu1= " << nqu1 << " nqu2 " << nqu2 << endl;
          cout << line << endl;
        }
      }
      else {
        cout << "ERROR: " << line << endl;
      }
      continue;
    }
    }

// dx
    if(!strcmp(word,"dx")) {
    if(control == 1)        {
      float dx  =0.;
      float dy  =0.;
      float dz  =0.;
      float xc  =0.;
      float yc  =0.;
      float angl=0.;
      if (sscanf(line, " dx %f dy %f dz %f xc %f yc %f angl %f " ,
               &dx,&dy,&dz,&xc,&yc,&angl) == 6) {
        A.dx  =dx;
        A.dy  =dy;
        A.dz  =dz;
        A.xc  =xc;
        A.yc  =yc;
        A.angl=angl;
	//	A.Print();
        int ret=MuDbAlign::Add(&A);
        if(ret != 0) cout << "can not add the frame, ret= " << ret << endl;
      } 
      else {
        cout << "ERROR: " << line << endl;
      }
      continue;
    }
    }
  }
goodexit:
  int ret=MuDbAlign::Write();
  //  Print();
  Release();
  return ret;
}
