
// Boris Fominykh 9.01.2000
// fill GESL, GEDE from muon DB

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbNede.h"
#include "murec/MuDbNesl.h"

extern "C" {
  void MuCheckGesl(MuNesl*);
  void MuFillGesl(ArteTableIterator<GESL>,MuNesl*);
  void MuAddGesl(MuNesl*);
  void MuCheckGede(int,MuNesl*,MuNede*);
  void MuFillGede(ArteTableIterator<GEDE>,MuNede*);
  void MuAddGede(MuNede*);
  void MuGeoPoint();
}

extern "C" {
int MuGeometry(int run,char* DBname)
{
  int ret;
  MuNesl   lnesl;  
  MuDbNesl Nesl;  
  Nesl.setDBname(DBname);
  Nesl.setRun(run);
  Nesl.setMinor(0);
  ret=Nesl.Read();
  if(ret != 0) {
    cout << "MuGeometry: NESL table is NOT read from DB, ret= " << ret;;
    cout << " DBname= " << DBname << " RUN= " << run << endl;
    return 1;
  }

  MuNede   lnede;  
  MuDbNede Nede;  
  Nede.setDBname(DBname);
  Nede.setRun(run);
  Nede.setMinor(0);
  ret=Nede.Read();
  if(ret != 0) {
    cout << "MuGeometry: NEDE table is NOT read from DB, ret= " << ret;;
    cout << " DBname= " << DBname << " RUN= " << run << endl;
    return 2;
  }

// loops over muon superlayers
  for(int i=0; i<8; i++) {
    int j=i/2;
// superlayer key
    int neslkey = 18+10*j+i-2*j;
    ret=Nesl.Get(&neslkey,&lnesl);
    if(ret != 0) {
      cout << "MuGeometry: no NESL for key " << neslkey << endl;
    }
    else {
// parameters of chamber structure
      int nxmem[2][3];
      int nx99m[2][3];
      for(int i=0; i<2; i++) {
        for(int j=0; j<3; j++) {
          nxmem[i][j]=0;
          nx99m[i][j]=0;
        }
      }
      int msl = lnesl.nsl;
      int cmp = lnesl.cmp;
      int laym = lnesl.lay;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          for(int i=0; i<2; i++) {
            nxmem[i][l]=lnesl.chamb[2*l+i];
            nx99m[i][l]=lnesl.nx99[2*l+i];
          }
        }
      }

// cross-check
      int nvol=lnesl.npl;
      int nvl=0;
      if(cmp == 8 && laym > 0) {
        for(int l=0; l<laym; l++) {
          for(int i=0; i<2; i++) {
            nvl+=2*nxmem[i][l];
            nvl+=2*nx99m[i][l];
          }
        }
      }
      if(cmp == 9) nvl=laym;
      if(nvl != nvol) {
        cout << "MuGeometry: cross-check: nvol= " << nvol;
        cout << " is not equal nvl= " << nvl << endl;
      }
// check GESL in ARTE
      MuCheckGesl(&lnesl);

// loops over cmp=8 nede elements
      if(cmp == 8 && laym > 0) {
        for(int lay=0; lay<laym; lay++) {
          for(int ud=0; ud<2; ud++) {
            if(nxmem[ud][lay] > 0) {
              for(int nx=0; nx<nxmem[ud][lay]+1; nx++) {
                int nxx=0;
                if(nx < nxmem[ud][lay]) nxx=nx+1;
                if(nx == nxmem[ud][lay] && nx99m[ud][lay] == 1) nxx=99;
                if(nxx > 0) {
                  for(int sd=0; sd<2; sd++) {
// key word
  int nedekey = 100000*msl+10000*(lay+1)+1000*ud+10*nxx+sd;
                    ret=Nede.Get(&nedekey,&lnede);
                    if(ret != 0) {
		      cout << "MuGeometry: no NEDE for key " << nedekey << endl;
                    }
                    else {
// check GEDE in ARTE
                      MuCheckGede(sd,&lnesl,&lnede);
                    }
                  }
                }
              }
            }
          }
        }
      }

// loops over cmp=9 nede elements
      if(cmp == 9 && laym > 0) {
        for(int qu=0; qu<laym; qu++) {
// key word
          int nedekey = 1000*msl+100*(qu+1)+cmp;
          ret=Nede.Get(&nedekey,&lnede);
          if(ret != 0) {
	    cout << "MuGeometry: no NEDE for key " << nedekey << endl;
          }
          else {
// check GEDE in ARTE
            MuCheckGede(0,&lnesl,&lnede);
          }
        }
      }
    }
  }
  MuGeoPoint();
  return 0;
}

void MuCheckGesl(MuNesl* pnesl)
{
// superlayer name
  char namsl[5];
  strncpy(namsl,&(pnesl->name[0]),5);
  namsl[4]='\0';
  int check=0;
// loops over ARTE superlayers
  for ( ArteTableIterator<GESL> isl = ArteTable<GESL>::begin(); 
	isl != ArteTable<GESL>::end();
	++isl ) {
    int cmp  = (*isl)->cmp;
    if(cmp == 8 || cmp == 9) {
// ARTE superlayer name
      char name[5];
      strncpy(name,(*isl)->name,5);
      name[4]='\0';
      if(!strcmp(namsl,name)) {
        check=1;
        MuFillGesl(isl,pnesl);
      }
    }
  }
  if(check == 0) MuAddGesl(pnesl);
  return;
}

void MuFillGesl(ArteTableIterator<GESL> isl,MuNesl* pnesl)
{
  (*isl)->cmp = pnesl->cmp;
  (*isl)->shp = pnesl->shp;
  (*isl)->nsl = pnesl->nsl;
  (*isl)->mfl = pnesl->mfl;
  (*isl)->lay = pnesl->lay;
  (*isl)->npl = pnesl->npl;
  (*isl)->fpl = pnesl->fpl;
  (*isl)->nwl = pnesl->nwl;
  (*isl)->fwl = pnesl->fwl;
  (*isl)->ltyp= pnesl->ltyp;
  (*isl)->x1  = pnesl->x1;
  (*isl)->x2  = pnesl->x2;
  (*isl)->y1  = pnesl->y1;
  (*isl)->y2  = pnesl->y2;
  (*isl)->z1  = pnesl->z1;
  (*isl)->z2  = pnesl->z2;
  (*isl)->dx  = pnesl->dx;
  (*isl)->dy  = pnesl->dy;
  return;
}

void MuAddGesl(MuNesl* pnesl)
{
  struct GESL gesl;
  char nam[4];
  strncpy(&nam[0],pnesl->name,4);
  strncpy(gesl.name,nam,4);

  gesl.cmp = pnesl->cmp;
  gesl.shp = pnesl->shp;
  gesl.nsl = pnesl->nsl;
  gesl.mfl = pnesl->mfl;
  gesl.lay = pnesl->lay;
  gesl.npl = pnesl->npl;
  gesl.fpl = pnesl->fpl;
  gesl.nwl = pnesl->nwl;
  gesl.fwl = pnesl->fwl;
  gesl.ltyp= pnesl->ltyp;
  gesl.x1  = pnesl->x1;
  gesl.x2  = pnesl->x2;
  gesl.y1  = pnesl->y1;
  gesl.y2  = pnesl->y2;
  gesl.z1  = pnesl->z1;
  gesl.z2  = pnesl->z2;
  gesl.dx  = pnesl->dx;
  gesl.dy  = pnesl->dy;
  ArtePointer<GESL>::append(gesl);
  return;
}

void MuCheckGede(int sd,MuNesl* pnesl,MuNede* pnede)
{
// chamber name
  char name[9];
  char namde[9];
  strncpy(namde,&(pnede->hwc[0]),9);
  namde[8]='\0';
  int sdch = 0;
  for ( ArteTableIterator<GEDE> iv = ArteTable<GEDE>::begin(); 
	iv != ArteTable<GEDE>::end();
	++iv) 
  {
    int cmp  = (*iv)->cmp;
    if(cmp == 8 || cmp == 9) {
// ARTE chamber name
      strncpy(name,(*iv)->hwc,9);
      if(cmp == 9) {
        for(int i=5; i< 8; i++) name[i]=' ';
      }
      name[8]='\0';

      if(!strcmp(namde,name)) {
        if(sd == sdch) {
          MuFillGede(iv,pnede);
          return;
        }
        if(sd == 1 && sdch == 0) sdch=1;
      }
    }
  }
  MuAddGede(pnede);
  return;
}

void MuFillGede(ArteTableIterator<GEDE> iv,MuNede* pnede)
{
  (*iv)->cmp = pnede->cmp;
  (*iv)->lay = pnede->lay;
  (*iv)->sec = pnede->sec;
  (*iv)->shp = pnede->shp;
  (*iv)->xmin = pnede->xmin;
  (*iv)->xmax = pnede->xmax;
  (*iv)->ymin = pnede->ymin;
  (*iv)->ymax = pnede->ymax;
  (*iv)->zmin = pnede->zmin;
  (*iv)->zmax = pnede->zmax;
  (*iv)->sxmn = pnede->sxmn;
  (*iv)->sxmx = pnede->sxmx;
  (*iv)->symn = pnede->symn;
  (*iv)->symx = pnede->symx;
  (*iv)->cos  = pnede->cos;
  (*iv)->sin  = pnede->sin;
  for(int j=0; j<9; j++) (*iv)->rot[j]=pnede->rot[j];
  (*iv)->acc  = pnede->acc;
  (*iv)->eff  = pnede->eff;
  (*iv)->dod  = pnede->dod;
  (*iv)->pit  = pnede->pit;
  (*iv)->ms   = pnede->ms;
  for(int j=0; j<4; j++) (*iv)->xfw[j]=pnede->xfw[j];
  for(int j=0; j<4; j++) (*iv)->zfw[j]=pnede->zfw[j];
  for(int j=0; j<4; j++) (*iv)->nwi[j]=pnede->nwi[j];
  (*iv)->nsl = pnede->nsl;
  (*iv)->nwl = pnede->nwl;
  (*iv)->fwl = pnede->fwl;
  (*iv)->xmad= pnede->xmad;
  (*iv)->ymad= pnede->ymad;
  (*iv)->zmad= pnede->zmad;
  for(int j=0; j<4; j++) (*iv)->y1w[j]=pnede->y1w[j];
  for(int j=0; j<4; j++) (*iv)->y2w[j]=pnede->y2w[j];
  (*iv)->dedx = pnede->dedx;
  (*iv)->gcop = pnede->gcop;
  (*iv)->noise= pnede->noise;
  (*iv)->ares = pnede->ares;

  char buf[4];
  strncpy(buf,&(pnede->gvol[0]),4);
  strncpy((*iv)->gvol,buf,4);

  return;
}

void MuAddGede(MuNede* pnede)
{
  struct GEDE gede;

  gede.cmp = pnede->cmp;
  gede.lay = pnede->lay;
  gede.sec = pnede->sec;
  gede.shp = pnede->shp;
  gede.xmin = pnede->xmin;
  gede.xmax = pnede->xmax;
  gede.ymin = pnede->ymin;
  gede.ymax = pnede->ymax;
  gede.zmin = pnede->zmin;
  gede.zmax = pnede->zmax;
  gede.sxmn = pnede->sxmn;
  gede.sxmx = pnede->sxmx;
  gede.symn = pnede->symn;
  gede.symx = pnede->symx;
  gede.cos  = pnede->cos;
  gede.sin  = pnede->sin;
  for(int j=0; j<9; j++) gede.rot[j]=pnede->rot[j];
  gede.acc  = pnede->acc;
  gede.eff  = pnede->eff;
  gede.dod  = pnede->dod;
  gede.pit  = pnede->pit;
  gede.ms   = pnede->ms;
  for(int j=0; j<4; j++) gede.xfw[j]=pnede->xfw[j];
  for(int j=0; j<4; j++) gede.zfw[j]=pnede->zfw[j];
  for(int j=0; j<4; j++) gede.nwi[j]=pnede->nwi[j];
  gede.nsl = pnede->nsl;
  gede.nwl = pnede->nwl;
  gede.fwl = pnede->fwl;
  gede.xmad= pnede->xmad;
  gede.ymad= pnede->ymad;
  gede.zmad= pnede->zmad;
  for(int j=0; j<4; j++) gede.y1w[j]=pnede->y1w[j];
  for(int j=0; j<4; j++) gede.y2w[j]=pnede->y2w[j];
  gede.dedx = pnede->dedx;
  gede.gcop = pnede->gcop;
  gede.noise= pnede->noise;
  gede.ares = pnede->ares;

  char name[9];
  strncpy(name,pnede->hwc,9);
  if(pnede->cmp == 8) {
    name[8]='\0';
    strncpy(gede.hwc,name,9);
  }
  if(pnede->cmp == 9) {
    name[5]='\0';
    strncpy(gede.hwc,name,6);
  }

  char buf[4];
  strncpy(buf,pnede->gvol,4);
  strncpy(gede.gvol,buf,4);

  ArtePointer<GEDE>::append(gede);

  return;
}

// update GESL -> GEDE pointers
void MuGeoPoint()
{
  for ( ArteTableIterator<GESL> isl = ArteTable<GESL>::begin(); 
        isl != ArteTable<GESL>::end();
        ++isl ) {
    int   cmp  = (*isl)->cmp;
    if(cmp == 8 || cmp == 9) {
      (*isl)->fpl=0;
      (*isl)->npl=0;
    }
  }
// loops over GEDE
  int gede=0;
  for ( ArteTableIterator<GEDE> iv = ArteTable<GEDE>::begin(); 
        iv != ArteTable<GEDE>::end();
        ++iv ) {
    gede++;
    int cmp = (*iv)->cmp;
    if(cmp == 8 || cmp == 9) {
      ArtePointer<GESL> gesl = (*iv)->gesl;
      if(gesl->fpl == 0) gesl->fpl=gede;
      gesl->npl += 1;
    }
  }
  return;
}

}


