// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/State.hh"
#include "rang/Node.hh"
#include "rang/ECClus.hh"
#include "rang/ScatThick.hh"
#include "rang/Geom.hh"
#include "rang/zfpar.hh"
#include "rang/utfeld.hh"
#include "rang/RadEl.hh"

//-- Constructor
rangerECClus::rangerECClus() {
  cout << "This is the rangerECClus default constructor" << endl;
  rcl = 0;
  radThick = 0;
};

rangerECClus::rangerECClus(ArtePointer<RCCL> rccl,int charge,rangerFitMode fm,
			   const float radTh) 
: rangerHitBase(rccl,charge,fm) {
#ifdef FULL
  cout << "This is the rangerECClus(rhit) constructor, hitb=" 
       << rhit->hitb << endl;
#endif
  rcl = rccl;
  radThick = radTh;
  m = new HepVector(3);
  (*m)(1) = rccl->xcog;
  (*m)(2) = rccl->ycog;
  if (rccl->esum > 0) {
    (*m)(3) = 1/rccl->esum;
  }
  else {
    (*m)(3) = 9999;
  }
  V = new HepSymMatrix(3);
  (*V)(1,1) = rccl->vcog[0];
  (*V)(1,2) = rccl->vcog[1];
  (*V)(2,2) = rccl->vcog[2];
  (*V)(1,3) = 0;
  (*V)(2,3) = 0;
  (*V)(3,3) = (*m)(3) * (*m)(3) * 0.0004;     // assume flat error of 2 pc at the moment
};

rangerECClus::~rangerECClus(){
  delete m;
  m = 0;
  delete V;
  V = 0;
};


void rangerECClus::projectP(HepVector& resid,
			     HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectP" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();
  resid(3) = (*m)(3) - fabs( s->qP() );

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;

  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
  HH(3,5) = (s->qP()>=0)? 1 : -1;

}

void rangerECClus::projectPI(HepVector& resid,
			     HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectP" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();
  resid(3) = (*m)(3) - fabs( s->qP() );

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;
  HH(1,6) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;
  HH(2,6) = 0;

  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
  HH(3,5) = (s->qP()>=0)? 1 : -1;
  HH(3,6) = 0;

}


void rangerECClus::projectL(HepVector& resid,
			     HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectL" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();
  resid(3) = 0;

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;

  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;

}



void rangerECClus::projectLI(HepVector& resid,
			     HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerECClus::projectLI" << endl;
#endif
  resid(1) = (*m)(1) - s->x();
  resid(2) = (*m)(2) - s->y();
  resid(3) = 0;

  HH(1,1) = 1;
  HH(1,2) = 0;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

  HH(2,1) = 0;
  HH(2,2) = 1;
  HH(2,3) = 0;
  HH(2,4) = 0;
  HH(2,5) = 0;

  HH(3,1) = 0;
  HH(3,2) = 0;
  HH(3,3) = 0;
  HH(3,4) = 0;
  HH(3,5) = 0;

}



