// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "CLHEP/Matrix/Matrix.h"
#include "CLHEP/Matrix/SymMatrix.h"
#include "rang/StatePI.hh"
#include "rang/StateP.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
#include "rang/zfpar.hh"
#include "rang/rkmain.hh"
#include "rang/utfeld.hh"
#include "rang/FitMode.hh"
#include "rang/hfill.hh"
#include "rang/zfconst.hh"
#include "rang/RadEl.hh"
#include "rang/FLAGHist.hh"

//-- Constructor
rangerStatePI::rangerStatePI() : p(6), c(6) {
//  cout << "rangerState default constructor" << endl;
  zState = 0;
  qPEstimate = 0.1;
#ifdef FULL
  cout << "rangerState rhit constructor" << endl;
  cout << "rangerState: the state vector is now " << endl;
  cout << p << endl;
#endif
}

rangerStatePI::rangerStatePI(ArtePointer<RHIT> rhit) 
  : rangerState(rhit), p(6), c(6) {
  setX() = rhit->x;
  setY() = rhit->y;
  setTx() = rhit->tx;
  setTy() = rhit->ty;
  setQp() = rhit->p;
  setI() = 0;
}

rangerStatePI::rangerStatePI(ArtePointer<HITB> hitb) 
  : rangerState(hitb), p(6), c(6) {
  setX() = 0;
  setY() = 0;
  setTx() = 0;
  setTy() = 0;
  setQp() = 0;
  setI() = 0;
}

rangerStatePI::rangerStatePI(ArtePointer<RCCL> rccl,int charge) 
  : rangerState(rccl), p(6), c(6) {
  setX() = rccl->x;
  setY() = rccl->y;
  setTx() = rccl->tx;
  setTy() = rccl->ty;
  if (rccl->esum > 0) {
    setQp() = charge/rccl->esum;
  }
  else {
    setQp() = charge * 9999;
  }
  setI() = 0;
}

rangerStatePI::rangerStatePI(const rangerStatePI& s) 
  : rangerState(s), p(6), c(6) {
#ifndef FULL
  cout << "rangerStatePI::rangerStatePI: here is the copy constructor" << endl;
#endif
  zState = s.zState;
  p = s.p;
  c = s.c;
  qPEstimate = s.qPEstimate;
#ifdef FULL
  cout << "rangerStatePI::rangerStatePI: Copy constructor done " << endl;
#endif
}

rangerStatePI::~rangerStatePI(){
//  cout << "rangerStatePI destructor has been called" << endl;
}

rangerState& rangerStatePI::operator=(const rangerStatePI& s) {
#ifdef FULL
  cout << "rangerState& rangerStatePI::operator=(const rangerStatePI& s): "
       << " here is the assignment operator" << endl;
  double zz = s.z();
  cout << "   the given z is " << zz << endl;
#endif
  zState = s.zState;
  qPEstimate = s.qPEst();
  p = s.p;
  c = s.c;
  return *this;
}

rangerState& rangerStatePI::operator=(const rangerState& s) {
#ifdef FULL
  cout << "rangerState& rangerStatePI::operator=(const rangerState& s)" << endl;
#endif
  zState = s.z();
  qPEstimate = s.qPEst();
  p = s.vec();
  c = s.cov();
  return *this;
}

rangerState& rangerStatePI::operator=(const rangerStateP& s) {

  zState = s.z();
  qPEstimate = s.qPEst();
  for (int i=0; i<5; ++i) {
    p[i] = s.vec()[i];
    for (int j=i; j<5; ++j) {
      c[i][j] = s.cov()[i][j];
    }
    c[i][5] = 0;
  }
  p[5] = 0;
  c[5][5] = 0;
  return *this;
}

rangerState& rangerStatePI::operator=(const rangerStateL& s) {

  zState = s.z();
  qPEstimate = s.qPEst();
  for (int i=0; i<4; ++i) {
    p[i] = s.vec()[i];
    for (int j=i; j<4; ++j) {
      c[i][j] = s.cov()[i][j];
    }
    c[i][4] = 0;
    c[i][5] = 0;
  }
  p[4] = 0;
  p[5] = 0;
  c[4][4] = 0;
  c[4][5] = 0;
  c[5][5] = 0;
  return *this;
}

rangerState& rangerStatePI::operator=(const rangerStateLI& s) {

  zState = s.z();
  qPEstimate = s.qPEst();
  for (int i=0; i<4; ++i) {
    p[i] = s.vec()[i];
    for (int j=i; j<4; ++j) {
      c[i][j] = s.cov()[i][j];
      c[5][j] = s.cov()[5][j];
    }
    c[i][4] = 0;
  }
  p[4] = 0;
  p[5] = s.vec()[4];
  c[4][4] = 0;
  c[5][4] = 0;
  c[5][5] = s.cov()[4][4];
  return *this;
}

void rangerStatePI::covInit() {
#ifdef FULL
  cout << "rangerStatePI::covInit: z=" << z << endl;
  cout << "                     state matrix has " 
       << c.num_row() << " " << c.num_col() << endl;
#endif
  c(1,1) = zfpar_.cxxrf;
  c(2,2) = zfpar_.cyyrf;
  c(3,3) = zfpar_.ctxtxrf;
  c(4,4) = zfpar_.ctytyrf;
  c(5,5) = zfpar_.cqqrf;
  //6,6) = 1600.;
  c(6,6) = zfconst_.sigI * zfconst_.sigI;
  for (int i=1; i<=6; i++) {
    for (int j=1; j<i; j++) {
//      cout << "initialize c(" << i << "," << j << ")" << endl;
      c(i,j) = 0;
    }
  }
#ifdef FULL
  cout << "cov matrix is " << endl << c << endl;
#endif
}



int rangerStatePI::predict(const double& zDestination,
			 const double& qPExp,rangerState*& s,
			 HepMatrix*& F) {

  int newState     = (s == 0);  // denote if memory allocations exist
  int newTransport = (F == 0);   //   .    .    .        .         .

#ifdef FULL
  cout << "rangerStatePI::predict: zDestination=" << zDestination << endl;

  cout << "rangerStatePI::predict: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStatePI();
    if (!s) {
      cout << "rangerState::predict: allocation error" << endl; 
      return -10;
    }
  }
  if (!F) {
    F = new HepMatrix(6,6);
    if (!F) {
      cout << "rangerState::predict: allocation error" << endl; 
      return -10;
    }
  }

#ifdef FULL
  cout << "rangerStatePI::predict: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  double deltaZ = fabs(zDestination - zState);
  if ( (zDestination>zfpar_.zmaxMag && zState>zfpar_.zmaxMag)
       || (zDestination<zfpar_.zminMag && zState<zfpar_.zminMag)
       || deltaZ < 0.001) {
    s->setZ() = zDestination;      // linear transport
    s->setX() = x() + (zDestination - zState) * tx();
    s->setY() = y() + (zDestination - zState) * ty();
    s->setTx() = tx();
    s->setTy() = ty();
    s->setQp() = qP();
    s->setI() = I();   // assume that TOF is not yet implemented
    s->setQpEst() = qPEst();

    for (int i=1; i<=6; i++) {
      for (int j=1; j<=6; j++) {
	(*F)(i,j) = 0.;
      }
    }
    (*F)(1,1) = 1;
    (*F)(2,2) = 1;
    (*F)(3,3) = 1;
    (*F)(4,4) = 1;
    (*F)(5,5) = 1;
    (*F)(6,6) = 1;
    (*F)(1,3) = zDestination - zState;
    (*F)(2,4) = zDestination - zState;
  }
  else {
    double zout = zDestination;
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pin[6];
    pin[0] = zState;
    pin[1] = x();
    pin[2] = y();
    pin[3] = tx();
    pin[4] = ty();
    pin[5] = qPExp;
    double prec[4];
    double dzTrunc = fabs(zDestination - zState);
    if (dzTrunc<1) dzTrunc = 1;

    prec[0] = 0.001 / dzTrunc;
    prec[1] = 0.001 / dzTrunc;
    prec[2] = 0.0001 / dzTrunc;
    prec[3] = 0.0001 / dzTrunc;

    int icovar = 1;
    //int icovar = 2;
    double pout[6];
    double rkd[25];
    int ierror;
    rkmain_(icovar,zout,fstep,pin,prec,pout,rkd,ierror);

    //cout << "state::predict: fake rkmain error " << endl;
    //ierror = 1;

    if (ierror == 0) {
      if (pout[0] != zDestination) 
	cout << "state::predict: zDest changed from "
	     << zDestination << " to " << pout[0] << endl;
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
	(*F)(i,6) = 0;
      }
      for (int j=1; j<=5; j++) {
	(*F)(6,j) = 0;
      }
      (*F)(6,6) = 1;

      s->setZ() = zDestination;
      s->setX() = pout[1] + (*F)(1,5) * (qP() - qPExp);
      s->setY() = pout[2] + (*F)(2,5) * (qP() - qPExp);
      s->setTx() = pout[3] + (*F)(3,5) * (qP() - qPExp);
      s->setTy() = pout[4] + (*F)(4,5) * (qP() - qPExp);
      s->setQp() = qP();  // only valid in case momentum is unchanged
      s->setI() = I();    // assume that TOF is not yet implemented
      s->setQpEst() = qPEst();
    }
    else {
      cout << "state::predict: rkmain returns ierror=" << ierror 
	   << " during transport from " << zState << " to "
	   << zDestination << ", qP=" << qP() << endl;
      return -1;
    }
  }


#ifdef FULL
  cout << "rangerStatePI::predict: after calculation, *F=" << *F << endl;

  cout << "rangerStatePI::predict: the transported vector is"
       << s->vec() << endl;
#endif

  s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStatePI::predict: the transported cov is"
       << s->cov() << endl;
#endif

  if (1) {                // everything OK
    return 0;
  }
  else {
    cout << " This should not happen!!!" << endl;
    if (newState) delete s;
    if (newTransport) delete F;
    return -1000;
  }
}


void rangerStatePI::applyScat(const rangerScatThin& scat) {

  cov()(3,3) += scat.cTxTx;
  cov()(4,4) += scat.cTyTy;
  cov()(3,4) += scat.cTxTy;   // exploit symmetric matrix

  float xField[3];
  xField[0] = x();
  xField[1] = y();
  xField[2] = zState;
  float hField[3];
  utfeld_(xField,hField);
  
  if (zState > zfpar_.zminMag && zState < zfpar_.zmaxMag) {
    if (fabs(hField[1]) > 0.1) {
      cov()(5,5) += zfpar_.fP * fabs(hField[1]) * scat.dScat;
    }
  }
}

void rangerStatePI::applyScat(const rangerScatThick& scat) {

  cov()(3,3) += scat.cTxTx;
  cov()(4,4) += scat.cTyTy;
  cov()(3,4) += scat.cTxTy;   // exploit symmetric matrix

  cov()(1,1) += scat.cXX;     // correction for 
  cov()(1,2) += scat.cXY;     // thick scatterer
  cov()(2,2) += scat.cYY;
  cov()(1,3) += scat.cXTx;
  cov()(1,4) += scat.cXTy;
  cov()(2,3) += scat.cYTx;
  cov()(2,4) += scat.cYTy;

  float xField[3];
  xField[0] = x();
  xField[1] = y();
  xField[2] = zState;
  float hField[3];
  utfeld_(xField,hField);
  
  if (zState > zfpar_.zminMag && zState < zfpar_.zmaxMag) {
    if (fabs(hField[1]) > 0.1) {
      cov()(5,5) += zfpar_.fP * fabs(hField[1]) * scat.dScat;
    }
  }
}


void rangerStatePI::applyRadEl(const rangerRadEl& rad) {
  if (zState > zfpar_.zminEc && zState < zfpar_.zmaxEc) {
    p(5) *= rad.qPFactor;
    cov()(5,5) += rad.cQpQp;
  }
}


void rangerStatePI::applyDeDx(const float rl) {
  if (zState < zfpar_.zmaxMag && qP() != 0) {
    //float eLoss = 0.075 * rl;
    float eLoss = 0.048 * rl;     // tuned for radThickEff
    setQp() = (qP() > 0)? (1./(1./qP() + eLoss)) : (1./(1./qP() - eLoss));
  }
}


void rangerStatePI::applyRadPhoton(const float e,const float dE) {
  if (qP() != 0) {
    double qPOld = qP();
    setQp() = (qP() > 0)? (1./(1./qP() + e)) : (1./(1./qP() - e));
    double efac = qP() / qPOld;
    double dCov = (efac * efac * efac * efac - 1) * cov()(5,5)
      + dE * dE * qP() * qP() * qP() * qP();
    if (dCov >0) cov()(5,5) += dCov;
  }
}


void rangerStatePI::storeRseg(ArtePointer<RSEG> rseg,int firstLast){
 switch (firstLast) {
 case 1:
   rseg->zf = zState;
   rseg->xf = x();
   rseg->yf = y();
   rseg->txf = tx();
   rseg->tyf = ty();
   rseg->pf = qP();
   rseg->cvf[ 0] = cov()(1,1);
   rseg->cvf[ 1] = cov()(1,2);
   rseg->cvf[ 2] = cov()(2,2);
   rseg->cvf[ 3] = cov()(3,1);
   rseg->cvf[ 4] = cov()(3,2);
   rseg->cvf[ 5] = cov()(3,3);
   rseg->cvf[ 6] = cov()(4,1);
   rseg->cvf[ 7] = cov()(4,2);
   rseg->cvf[ 8] = cov()(4,3);
   rseg->cvf[ 9] = cov()(4,4);
   rseg->cvf[10] = cov()(5,1);
   rseg->cvf[11] = cov()(5,2);
   rseg->cvf[12] = cov()(5,3);
   rseg->cvf[13] = cov()(5,4);
   rseg->cvf[14] = cov()(5,5);
   rseg->time = I();
   rseg->dtim = cov()(6,6)>0 ? sqrt(cov()(6,6)) : 0;

   // just for monitoring: fill interaction time into hist
#ifdef rangerHIST
   {
     int id=129;
     float xxx = I() - zfconst_.tI;
     float yyy = 0;
     float www=1;
     hfill_(id,xxx,yyy,www);
     id=128;
     xxx = zfconst_.tI;
     yyy = I();
     hfill_(id,xxx,yyy,www);
     id=139;
     if ( cov()(6,6) <= 0 ) {
       cout << "StatePI::storeRseg: cov()(6,6)=" << cov()(6,6) << endl;
       return;
     }
     xxx = (I() - zfconst_.tI) / sqrt( cov()(6,6) );
     hfill_(id,xxx,yyy,www);
   }
#endif
   break;
 case 2:
   rseg->ze = zState;
   rseg->xe = x();
   rseg->ye = y();
   rseg->txe = tx();
   rseg->tye = ty();
   rseg->pe = qP();
   rseg->cve[ 0] = cov()(1,1);
   rseg->cve[ 1] = cov()(1,2);
   rseg->cve[ 2] = cov()(2,2);
   rseg->cve[ 3] = cov()(3,1);
   rseg->cve[ 4] = cov()(3,2);
   rseg->cve[ 5] = cov()(3,3);
   rseg->cve[ 6] = cov()(4,1);
   rseg->cve[ 7] = cov()(4,2);
   rseg->cve[ 8] = cov()(4,3);
   rseg->cve[ 9] = cov()(4,4);
   rseg->cve[10] = cov()(5,1);
   rseg->cve[11] = cov()(5,2);
   rseg->cve[12] = cov()(5,3);
   rseg->cve[13] = cov()(5,4);
   rseg->cve[14] = cov()(5,5);
   break;
 default:
   cout << "rangerStatePI::storeRseg: invalid firstLast " << firstLast << endl;
   return;
   break;
 }
}


void rangerStatePI::storeRtra(ArtePointer<RTRA> rtra,int firstLast){
 switch (firstLast) {
 case 1:
   rtra->zf = zState;
   rtra->xf = x();
   rtra->yf = y();
   rtra->txf = tx();
   rtra->tyf = ty();
   rtra->pf = qP();
   rtra->cvf[ 0] = cov()(1,1);
   rtra->cvf[ 1] = cov()(1,2);
   rtra->cvf[ 2] = cov()(2,2);
   rtra->cvf[ 3] = cov()(3,1);
   rtra->cvf[ 4] = cov()(3,2);
   rtra->cvf[ 5] = cov()(3,3);
   rtra->cvf[ 6] = cov()(4,1);
   rtra->cvf[ 7] = cov()(4,2);
   rtra->cvf[ 8] = cov()(4,3);
   rtra->cvf[ 9] = cov()(4,4);
   rtra->cvf[10] = cov()(5,1);
   rtra->cvf[11] = cov()(5,2);
   rtra->cvf[12] = cov()(5,3);
   rtra->cvf[13] = cov()(5,4);
   rtra->cvf[14] = cov()(5,5);
   break;
 case 2:
   break;
 default:
   break;
 }
}



void rangerStatePI::storeRhit(ArtePointer<RHIT> rhit) {
  rhit->x = x();
  rhit->y = y();
  rhit->tx = tx();
  rhit->ty = ty();
  rhit->p = qP();
}


void rangerStatePI::copyVec( const rangerState& s) {
  zState = s.z();
  p = s.vec();
  qPEstimate = s.qPEst();
}




void rangerStatePI::linExtrapVec( const rangerState& s, float z) {

  zState = z;

  setX() = s.x() + (z - s.z()) * s.tx();
  setY() = s.y() + (z - s.z()) * s.ty();
  setTx() =  s.tx();
  setTy() =  s.ty();
  setQp() =  s.qP();
  setI() = s.I();
  setQpEst() =  s.qPEst();
}


void rangerStatePI::dump() {
  cout << "     zState=" << zState << "  vec=" << p << endl;
  cout << "                              cov=" << c << endl;
}
