// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerNode
// node base class

#ifndef rangerNode_HH
#define rangerNode_HH

#include "arte/arte.hh"
#include "CLHEP/Matrix/Matrix.h"
#include "rang/State.hh"
#include "rang/ParticleType.hh"

class rangerNode {

public:
  rangerNode();
  rangerNode(rangerFitMode fm);
  rangerNode(ArtePointer<RHIT> rhit,rangerFitMode fm);
  rangerNode(ArtePointer<HITB> hitb,rangerFitMode fm);
  rangerNode(ArtePointer<RCCL> rccl,int charge,rangerFitMode fm);
  virtual ~rangerNode();

  static rangerNode* makeNode(ArtePointer<RHIT>& rhit,
			      rangerFitMode fm); // create new node
  static rangerNode* makeNode(ArtePointer<RCCL>& rccl,
			      int charge,rangerFitMode fm);
  static rangerNode* makeNode(const ArtePointer<GEDE>& ge,
			      rangerNode* n,const float path=1); // create new node
  static rangerNode* makeNode(const ArtePointer<GWAL>& gw,
			      rangerNode* n,const float path=1); // create new node
  virtual int filter();              // filter a node from scratch
  virtual int filter(rangerNode* n,
		     rangerParticleType pType); // filter a node 
                                     //using state from other mode

  int smooth(rangerNode*n);  // smooth a node using information of following
                             // node
  virtual void residUpdate(rangerNode*, 
			   HepMatrix&){};
                             // update residual and chiSq

  int predict(const double& zDestination,    // z of prediction plane
	      rangerState*& predState,       // result of prediction
	      HepMatrix*& transpMatrix);    // resulting transport matrix
                             // calculate predicted state at z
                             // (forwarding function to state::predict)
  virtual void processNoise(rangerParticleType){};  
                             // apply process noise to state of this node
  virtual void predProcessNoise(rangerParticleType){};  
                             // apply process noise to prediction
  void list();
  void dump();
  virtual int dim() const { return 0; }  // dimension of measurement
  virtual int plane(){
    cout << "This is just the base plane()" << endl;
    return 0;
  }
  virtual float ms() const {
    cout << "This is just the base ms()" << endl;
    return 0;
  }
  virtual float msPlain() const=0;
  virtual float dz(){ return 0; }   // thickness of plane
  float radiationThickness();
  float radiationThickEff();
  void covInit();
  virtual void storeRhit(ArtePointer<RHIT>){}; 
  virtual void mergeRhit(ArtePointer<RHIT>,float){};
  inline void storeRseg(ArtePointer<RSEG> rseg,int fl) 
                              { state->storeRseg(rseg,fl); };
  inline void storeRtra(ArtePointer<RTRA> rtra,int fl) 
                              { state->storeRtra(rtra,fl); };
  ArtePointer<RPNT> storeRpnt(ArtePointer<RPNT> rpnt)
                              { return state->storeRpnt(rpnt); };
  inline const rangerState* getPrediction() { return predictedState; };
  inline const HepMatrix* getF() { return transportMatrix; };
  inline const double& zNode() const { return z; };
  inline rangerState& getState() { return *state; } // accessor for state
  virtual inline const double getChiSq() const { return 0; };
  virtual inline int hasHitb() { return 0;}
  virtual const ArtePointer<HITB> getHitb() const { return 0; };
  virtual void updInternal(){};   // update internal degrees of freedom

protected:
  double z;
  double qPExpand;                 // Q/p to be used as expansion point
  //rangerNode* nextNode;
  //rangerNode* previousNode;
  rangerState* state;
  rangerState* predictedState;
  HepMatrix* transportMatrix;

  friend ostream& operator<<(ostream &os,const rangerNode &n);
};

#endif
