// ********************************************************************
//
//  Routine:      rc99Arefit 
//  Author:       T.Oest
//
//  Description:  refit for RTRA tracks
//
//  Modification:
//   -- call the refit only if RHITs are available  9/09/99 T.Oest
//   -- call global alignment transformation       29/10/99 T.Oest
//   -- no global alignment transformation, this 
//      is done now for RHITs                      13/01/00 T.Oest
//   -- call rcSetRTRArelations                    18/01/00 T.Oest
//
// ********************************************************************
#include <algo.h>
#include <arte/arte.hh>
#include <arte/artefunc.h>

#include <rang/Track.hh>
#include <rang/Node.hh>
#include <rang/Geom.hh>
#include <rang/rgfit.hh>

void acRSEGinGlobalCoordinates(ArtePointer<RSEG> rseg, int flag );
void rcSetRTRArelations(ArtePointer<RTRA>);

//****************************************************************
// function
//****************************************************************
void rc99Arefit(){

  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {
    if( (*rtra)->hsi>0) {

//------------------------------------------------------------------
// get the silicon segment and do a refit
//------------------------------------------------------------------

  for( ArteRelationIterator<RSEG> rseg = (*rtra)->rseg.begin();
       rseg != (*rtra)->rseg.end(); ++rseg ) {
    if (((*rseg)->cmp == 1 || (*rseg)->cmp == 65537) &&
        (*rseg)->rhit.begin() != (*rseg)->rhit.end()){

       //==== keep the information on the number of hits in rseg->time/dtim
           float np=(*rseg)->time;
           float nn=(*rseg)->dtim;
 
 for( ArteRelationIterator<RHIT> rhit = (*rseg)->rhit.begin();
            rhit != (*rseg)->rhit.end(); ++rhit ) {
        (*rhit)->p = (*rtra)->pf;
       };

      (*rseg)->pf = (*rtra)->pf;
      (*rseg)->cvf[14] = (*rtra)->cvf[14];


      //     cout << endl;
      //      cout << " new track            " << (*rseg)->xf << " " << (*rseg)->yf << " "<< 
      //	(*rseg)->txf << " "<< (*rseg)->tyf << " " << endl;

      int rsegFit   = (*rseg)->fit;
      int rs        = *rseg - ArtePointer<RSEG>(1) + 1;
      int fitstatus = rgfit_(rs,0);
      (*rseg)->fit  =  rsegFit;

      //     cout << " after refit          " << (*rseg)->xf << " " << (*rseg)->yf << " "<< 
      //	(*rseg)->txf << " "<< (*rseg)->tyf << " " << endl;

          (*rseg)->time=np;
          (*rseg)->dtim=nn;


      if( fitstatus >= 0 ) {

        int flag = 0;
	//	acRSEGinGlobalCoordinates(*rseg, flag);

        (*rtra)->lth  =  (*rseg)->chi;
        (*rtra)->zf   =  (*rseg)->zf;
        (*rtra)->xf   =  (*rseg)->xf;
        (*rtra)->yf   =  (*rseg)->yf;
        (*rtra)->txf  =  (*rseg)->txf;
        (*rtra)->tyf  =  (*rseg)->tyf;

        (*rtra)->cvf[10] = (*rtra)->cvf[10] * sqrt((*rseg)->cvf[0]/(*rtra)->cvf[0]);
        (*rtra)->cvf[11] = (*rtra)->cvf[11] * sqrt((*rseg)->cvf[2]/(*rtra)->cvf[2]);
        (*rtra)->cvf[12] = (*rtra)->cvf[12] * sqrt((*rseg)->cvf[5]/(*rtra)->cvf[5]);
        (*rtra)->cvf[13] = (*rtra)->cvf[13] * sqrt((*rseg)->cvf[9]/(*rtra)->cvf[9]);

        for (int i=0; i<=9 ; i++) (*rtra)->cvf[i] = (*rseg)->cvf[i];
      } else {
	//	cout << " ************** refit not sucessfull " << fitstatus << endl;
        (*rtra)->fit  = 1000+(*rtra)->fit; 
      };
//------------------------------------------------------------------
// set the missing RTRA relation
//------------------------------------------------------------------
     rcSetRTRArelations(*rtra);
    };
  };
  };
};
};
