 
// ********************************************************************
//
//  Routine:      rcSetTrackPoints
//
//  Description:  set track points
//
//  Modification:
//
// ********************************************************************
 
 
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include "arte/arte.hh"

extern int rc99SelectTracksStatus;


extern "C" void rkmain_(const int    & icovar, 
			const double & zout, 
			const double & fstep, 
			const double * parameters, 
			const double * prec, 
			const double * p_track, 
			const double * derivatives, 
			const int    & ierror);


//********************************************************************
// function
//********************************************************************

int rcSetTrackPoints() {

  const int index[5][5] = { {0, 1,  3, 6,10},
                            {1, 2,  4, 7,11},
                            {3, 4,  5, 8,12},
                            {6, 7,  8, 9,13},
                            {10,11,12,13,14}};
//-------------------------------------------------------------------
//   new points only for /RECON/RTRA ON
//-------------------------------------------------------------------

  if (rc99SelectTracksStatus == 0) return 0; 

//-------------------------------------------------------------------
//   loop over tracks
//-------------------------------------------------------------------


  for(ArteTable<RTRA>::iterator rtra=ArteTable<RTRA>::begin();
      rtra!=ArteTable<RTRA>::end();rtra++)
    {

//-------------------------------------------------------------------
//   one point for the RICH, to be exchanged with OTR information
//   the covariance matrix is not correct since in RTRA the
//   correlation between momentum error and other parameters are not
//   set.
//-------------------------------------------------------------------
      double zout = 1000.;
      
      const int    icovar=2;    
      double       prec[4];
      double       derivatives[5][5]={};
      double       p_track[6], parameters[6];
      double       fstep=50.;
      
      parameters[0] = double((*rtra)->zf);
      parameters[1] = double((*rtra)->xf);
      parameters[2] = double((*rtra)->yf);
      parameters[3] = double((*rtra)->txf);
      parameters[4] = double((*rtra)->tyf);
      parameters[5] = double((*rtra)->pf);


      if (zout != parameters[0]) {
	
	int    ierror;
	prec[0] = 0.001/fabs(zout-parameters[0]);
	prec[1] = prec[0];
	prec[2] = 0.0001/fabs(zout-parameters[0]);
	prec[3] = prec[2];
      
	rkmain_(icovar, zout, fstep, parameters, prec, p_track, 
		& derivatives[0][0], ierror);

        if (ierror == 0) {
         RPNT rpnt;

         rpnt.zf  = float(p_track[0]);
         rpnt.xf  = float(p_track[1]);
         rpnt.yf  = float(p_track[2]);
         rpnt.txf = float(p_track[3]);
         rpnt.tyf = float(p_track[4]);
         rpnt.pf  = float(p_track[5]);

         rpnt.cvf[index[4][0]] = 0.0;
         rpnt.cvf[index[4][1]] = 0.0;
         rpnt.cvf[index[4][2]] = 0.0;
         rpnt.cvf[index[4][3]] = 0.0;
         rpnt.cvf[index[4][4]] = (*rtra)->cvf[index[4][4]];

         for (int i=0; i<=3 ; i++){
         for (int j=i; j<=3 ; j++){
	   //         for (int i=0; i<=4 ; i++){
	   //         for (int j=i; j<=4 ; j++){

          float sigma = 0.0;
	  for (int k=0; k<=3; k++) { for (int l=0; l<=3; l++){
	    //	  for (int k=0; k<=4; k++) { for (int l=0; l<=4; l++){
           sigma = sigma + float(derivatives[k][i])
                         * (*rtra)->cvf[index[k][l]]
                         * float(derivatives[l][j]);
          };};

          rpnt.cvf[index[i][j]] = sigma;
         };};

         ArtePointer<RPNT> Prpnt = ArteTable<RPNT>::push_back(rpnt);
         createRelation( *rtra, Prpnt, &RPNT::rtra );

        };
      };

     };
      
       
  return 0;      
};
