
// *****************************************************************************
// 
//  Author:  Rainer Mankel                     Created:  17-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
// rcevnt.C
// reconstruction main program
// dispatch selecting the appropriate main program according to
// time or geometry version
//
// changes:
// 17 Dec 1999 (AG) introduce timing
// 13 Jan 2000      call rc99Aevnt for all data 
//
// *****************************************************************************

// standard includes
#include <math.h>

// ONLINE includes
#include "rhp.h"

// ARTE includes
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"

// prototypes
extern "C" void rcZevnt_();

void rc99Aevnt();

//
// program code
//
extern "C" void
rcevnt_()
{
  //-------------------------------------------------------------------
  //     start timing
  //-------------------------------------------------------------------
  static Chrono *cron_rcevnt = 0;
  if( !cron_rcevnt ){
    ChronoManager CM;
    cron_rcevnt = CM.createChrono( "cron_rcevnt" );
  }
  cron_rcevnt->start();

  //    
  // test if MC data
  //
  if (ArteTable<MCIN>::booked()) {
    
    // MC data: check geometry
    int igeo =  (int) (10000 * (*(ArteTable<MCIN>::begin()))->vgeo + 0.5);
    
    if (igeo >= 00700 && igeo < 10000) {
      
      rcZevnt_();
      
    }
    else {
      if (igeo >= 980000 && igeo <= 999999) {
	rc99Aevnt();
      }
      else {
	arteMsg 
	  << "rcevnt: reconstruction for this geometry not yet included"
	  << endl;
	static int id1;
	arteMsg.allp(id1,10);
      }
    }
  }
  else {
    //
    // real data: check date
    //
    if (! ArteTable<EVHD>::booked()) {
      arteMsg << "rcevnt: event header missing" << endl;
      arteMsg.warn();
      return;
    }
    int date = (*ArteTable<EVHD>::begin())->date;
    
    rc99Aevnt();

  }

  //-------------------------------------------------------------------
  //     stop timing
  //-------------------------------------------------------------------
  cron_rcevnt->stop();
  float cputime = cron_rcevnt->lastTime();
  //cout << "rcevnt_(): cputime=" << cputime << " sec" << endl;
  if( DQ_flag( COMP_FARM, LVL_M_HIT ) )
    rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 5 ), cputime, 1 );
  
  //-------------------------------------------------------------------
  //     EVRC
  //-------------------------------------------------------------------
  //faFillEVRC( cputime ); // > ARTE-03-05-r1
  int size = ArteTable<EVRC>::size();
  mmbook_( ArtePointer<EVRC>::getLink().getRef(), 1 );
  ArtePointer<EVRC> evrc( size + 1 );
  evrc->ipaddr  = arteflag_.ipaddr;
  evrc->errmask = 0x0;
  evrc->date    = time( NULL );
  evrc->cputime = cputime;
}
