#ifndef __RICUTIL_H__
#define __RICUTIL_H__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#ifndef M_PI
#define M_PI 3.141592654
#endif

#ifdef NOARTE
#  include "myArte.hh"
#include "auto/RIC.hh"
#else
//#  define bool int
#  include "arte/arte.hh"
//#  undef bool
#endif

#define F77NAME(name) name##_
//#include "hist.h"

// -------------------------------------------------------------------------

extern "C" void atstopc_();

/// Exit function for RICH: atstopc_() called for ARTE run, 
/// exit(n) for stand-alone run.
inline void Exit(int n) {
#ifdef NO_ARTE
  exit(n);
#else
  fprintf(stderr," $$$ ATSTOPC called from RICH, exiting with code %d\n",n);
  atstopc_();
#endif
}

// -------------------------------------------------------------------------

class RichRun {
 public:
  static int               type;
  static ArtePointer<GEDE> gede;
  static int               magnet;
  static int               bad_channels;
  static int               bad_channels_reference;
  static float             photon_fraction;
};

// -------------------------------------------------------------------------
 
#include <time.h>

#ifdef NO_ARTE
extern "C" {

  extern void  F77NAME(poissn)(float *, int *, int *);
  extern void  F77NAME(poiset)(float *);
  extern float F77NAME(rndm)(float *);
  extern void  F77NAME(rdmin)(float*);
  extern void  F77NAME(polint)(float *, float*,int*,float *, float *);
  inline double polint(float* f,float* a,int k,float x) {
    float ret;
    F77NAME(polint)(f,a,&k,&x,&ret);
    return ret;
    }
  } 
//@Man: random numbers
//@{
/// Initalisation of the random number generator
/**
 Initalisation of the CERNLIB-random number generator rndms and
 poissn. If the function is called without specifying any argument,
 the initial random number is derived from the actual time. If
 the first argument is specified, the generator starts with this
 seed. The second argument is used fro the call of the CERNLIB-routine
 'poiset'.
 */
  inline void rnd_init(int n=(int)time(0)*100,float amax=0) { 
    printf("Random: start rndm with: %d\n",n);
    float num=(float)n;
    F77NAME(rdmin)(&num);
    if (amax>0) F77NAME(poiset)(&amax);
    } 
    
/// generation of normal random numbers
/**
 Generation of flat random numbers from 0 ... 1. See CERNLIB-routine rndm.
 */
  inline float rnd_normal()  {   float dummy; return F77NAME(rndm)(&dummy); }

///  generation a poisson distributed random numbers
/**
  Generation of poisson-distributed random numbers with average mu. 
  See CERNLIB-routine 'poissn'.
 */
 
  inline int   rnd_poisson(float mu){
    int err, nav;
    F77NAME(poissn)(&mu, &nav, &err);
    if (err) {
      fprintf(stderr,"Random::poisson: Error in poissn. Code %d\n",err);
      Exit(1);
      }
    return nav;
    }

//@}
#else
extern "C" {
  extern void  F77NAME(gpoiss)(float *, int *, int *);
  extern void  F77NAME(grndm)(float *,int *);
  extern void  F77NAME(polint)(float *, float*,int*,float *, float *);
  inline double polint(float* f,float* a,int k,float x) {
    float ret;
    F77NAME(polint)(f,a,&k,&x,&ret);
    return ret;
    }
  } 
  
  inline float rnd_normal()  {   
    float ret; 
    int n=1;
    F77NAME(grndm)(&ret,&n);
    return ret;
    }
  inline int rnd_poisson(float mu){
    int ret;
    int n=1;
    F77NAME(gpoiss)(&mu,&ret,&n);
    return ret;
    }
#endif


// -------------------------------------------------------------------------
//@Man: operations with three-dimensional vectors
//@{
///  
/**
  The class Vec3 allows the declaration of three dimensional vectors.
  Assume a,b and c are of type Vec3 and s,r,theta,phi are of type double. 
  Then the following operations are 
  defined:\\[4mm]
  \begin{tabular}{ll}
   equality:   &if (a==b) { }\\
   inequality: & if (a!=b) {}\\
   assignment: & a=b; \\
   addition:   & a=b+c; a+=b; \\
   subtraction: & a=b-c; a-=b; \\
   multiplication with a scalar: & a=s*b; a=b*s; a*=s;\\
   division by a scalar:  & a=b/s; a/=s;\\
   scalar product: & s=a*b; \\
   cross product:  & a=cross(b,c); \\
   calculation of the norm: & s=norm(a); \\
   normalisation : & a=normalize(b); \\
   get polar coordinates: & a.ToPolar(\&r,\&theta,\&phi);\\
   set from polar coordinates: & a=FromPolar(r,theta,phi);\\
  \end{tabular}
  */
class Vec3 {
double v[3];

public:
/// 
  inline int operator==(const Vec3 &v1) const {
    return (v[0]==v1[0]&&v[1]==v1[1]&&v[2]==v1[2]);
    }
///     
  inline int operator!=(const Vec3 &v1) const {
    return (v[0]!=v1[0]||v[1]!=v1[1]||v[2]!=v1[2]);
    }
///
  inline Vec3& operator+=(const Vec3 &v1)  {
    v[0]+=v1[0];v[1]+=v1[1];v[2]+=v1[2];
    return *this;
    }
///
  inline Vec3& operator-=(const Vec3 &v1)  {
    v[0]-=v1[0];v[1]-=v1[1];v[2]-=v1[2];
    return *this;
    }
///
  inline Vec3& operator*=(const double a)  {
    v[0]*=a,v[1]*=a;v[2]*=a;
    return *this;
    }
///
  inline Vec3& operator/=(const double a)  {
    v[0]/=a,v[1]/=a;v[2]/=a;
    return *this;
    }
/// constructor allowing to set all three components
  inline Vec3(const double x=0,const double y=0,const double z=0) { 
    v[0]=x; v[1]=y; v[2]=z; 
    }
/// constructor using a float[3]-array
  inline Vec3(const float x[3]) { 
    v[0]=x[0]; v[1]=x[1]; v[2]=x[2]; 
    }
/// constructor using a double[3]-array
  inline Vec3(const double x[3]) { 
    v[0]=x[0]; v[1]=x[1]; v[2]=x[2]; 
    }
/// copy constructor
  inline Vec3(const Vec3& v1)           { memcpy(v,v1.v,sizeof(v)); }
///  assignment
  inline void operator=(const Vec3& v1) { memcpy(v,v1.v,sizeof(v)); }

/// returns the i'th (i=0,1,2) component: Vec3 a; double x=a[0];
  inline const double& operator[](const int i) const { 
#ifdef INDEXCHECK
    if (i<0||i>3) {
      fprintf(stderr,"Vec3[int]: illegal index %d\n",i);Exit(1);
      }
#endif
    return v[i];
    }
    

/// returns the x-component:  Vec3 a; double x=a.x();
  inline double  x() const { return v[0]; }
/// returns the y-component:  Vec3 a; double y=a.y();
  inline double  y() const { return v[1]; }
/// returns the z-component:  Vec3 a; double z=a.z();
  inline double  z() const { return v[2]; }
  
/// setting the i'th (i=0,1,2) component: Vec3 a; a[0]=0.3;
  inline  double& operator[](const int i)  { 
#ifdef INDEXCHECK
    if (i<0||i>3) {
      fprintf(stderr,"Vec3[int]: illegal index %d\n",i);Exit(1);
      }
#endif
    return v[i];
    }
/// setting the x-component:  Vec3 a; a.x()=0.3;
  inline double& x()      { return v[0]; }
/// setting the y-component:  Vec3 a; a.y()=0.3;
  inline double& y()      { return v[1]; }
/// setting the z-component:  Vec3 a; a.z()=0.3;
  inline double& z()      { return v[2]; }
  
/// print the vector
  inline void print(char* fmt="%12.6lf %12.6lf %12.6lf ",FILE *fp=stdout) const { 
    fprintf(fp,fmt,v[0],v[1],v[2]);
    }
/// conversion to polar coordinates
  inline void ToPolar(double *r,double *costhe,double *phi) {
    *r=sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);
    *costhe=v[2]/ *r;
    if (v[0]!=0) {
      *phi=atan2(v[1],v[0]);
      if (*phi<0) *phi=2*M_PI+*phi;
      }
    else if (v[1]>0) {
      *phi=M_PI/2;
      }
    else if (v[1]<0) {
      *phi=3*M_PI/2;
      }
    else {
      *phi=0;
      }
    }

  };
//@Man: additional operations
//@{
/// addition of two vectors
inline Vec3 operator+(const Vec3&v1,const Vec3&v2) {
  return Vec3(v1.x()+v2.x(),v1.y()+v2.y(),v1.z()+v2.z());
  }

/// subtraction of two vectors 
inline Vec3 operator-(const Vec3&v1,const Vec3&v2) {
  return Vec3(v1.x()-v2.x(),v1.y()-v2.y(),v1.z()-v2.z());
  }
  
/// scalar product of two vectors
inline double operator*(const Vec3&v1,const Vec3&v2) {
  return v1.x()*v2.x()+v1.y()*v2.y()+v1.z()*v2.z();
  }
/// multiplication with a scalar
inline Vec3 operator*(const double a,const Vec3&v) {
  return Vec3(a*v.x(),a*v.y(),a*v.z());
  }

/// multiplication with a scalar
inline Vec3 operator*(const Vec3&v,const double a) {
  return a*v;
  }
  
/// division with a scalar
inline Vec3 operator/(const Vec3&v,const double a) {
  return Vec3(v.x()/a,v.y()/a,v.z()/a);
  }

/// cross product of to vectors
inline Vec3 cross(const Vec3& v1,const Vec3& v2) {
  return Vec3(v1.y()*v2.z()-v1.z()*v2.y(),
              v1.z()*v2.x()-v1.x()*v2.z(),
              v1.x()*v2.y()-v1.y()*v2.x() );
  }

/// returns the norm 
inline double norm(const Vec3& v) {
  return sqrt(v*v);
  }
/// returns the normalized vector
inline Vec3  normalize(const Vec3& v) {
  double n=norm(v);
  return Vec3(v.x()/n,v.y()/n,v.z()/n);
  }
/// calculate a carthesian vector from polar coordinates
inline Vec3 FromPolar(double r,double costhe,double phi) {
  double sinthe=0;
  if (fabs(costhe)<=double(1)) sinthe=sqrt(double(1)-costhe*costhe);
  else {
    fprintf(stderr,"Vec3::FromPolar: abs(costhe)>1 %f\n",costhe);
    Exit(1);
    }
  if (fabs(sinthe)<2e-8) {sinthe=0;costhe=1;};
  return Vec3(r*cos(phi)*sinthe,r*sin(phi)*sinthe,r*costhe);
  }

inline ostream & operator << (ostream &o, const Vec3 &v) {
    o<<"("<<v.x()<<","<<v.y()<<","<<v.z()<<")";
    return o;
  }
//@}
//@}

// ----------------------------------------------------------------------
/// coordinate transformation
/**
 For the definition of a coordinate system three carthesian normalized
 direction vectors has to be specified together with the origin. Example:\\[4mm]
 CoordinateSystem A;\\
 A.ex()=Vec3(0,1,0);\\
 A.ey()=Vec3(0,0,1);\\
 A.ez()=cross(A.ex(),A.ey());\\
 A.x0()=Vec3(0.5,0.5,0);\\[4mm]
 If the coordinate system is set up correctly, a coordinate transformation 
 consisting out of a translation to the origin of A and a rotation into the 
 A coordinatesystem can be executed, as well as the pure rotation. The
 inverse operations are also available. The so called old system is alwayes
 defined by: ex=Vec3(1,0,0),ey=Vec3(0,1,0),ez=Vec3(0,0,1),x0=Vec3(0,0,0));
 */
class CoordinateSystem {
Vec3 eex,eey,eez,xx0;
public:
/// returns the origin
  inline const Vec3& x0()    const { return xx0; }
/// sets the origin
  inline       Vec3& x0()          { return xx0; }
/// returns ex
  inline const Vec3& ex()    const { return eex; }
/// sets ex  
  inline       Vec3& ex()          { return eex; }
/// returns ey
  inline const Vec3& ey()    const { return eey; }
/// sets ey
  inline       Vec3& ey()          { return eey; } 
/// return ez
  inline const Vec3& ez()    const { return eez; }
/// sets ez
  inline       Vec3& ez()          { return eez; }
  
/// default construtor setting up a unit transformation
  inline CoordinateSystem() {
    eex=Vec3(1,0,0);
    eey=Vec3(0,1,0);
    eez=Vec3(0,0,1);
    xx0=Vec3(0,0,0);
    }
/// copy constructor
  inline CoordinateSystem(const CoordinateSystem& A) { 
    x0()=A.x0();     
    ex()=A.ex();
    ey()=A.ey();
    ez()=A.ez();
    }
/// assignment operator (CoordinateSytem A,B; A=B;)
  inline void operator=(const CoordinateSystem& A) { 
    x0()=A.x0();     
    ex()=A.ex();
    ey()=A.ey();
    ez()=A.ez();    
    }
//
//  x,y arbitrary but perpendicular to z
//
  inline void setrotation(const Vec3& x1) {
    eez=normalize(x1);
    if (fabs(x1[1])>1E-10) {
      eey=normalize(cross(eez,Vec3(1,-x1[0]/x1[1],0)));
      }
    else {
      eey=Vec3(0,1,0);    
      eez[1]=0;
      eez=normalize(eez);
      }
    eex=normalize(cross(eey,eez));
    }
 
/// check if the rotation is a unit matrice 
  inline int UnitTransformation() {
    return eex[0]>(1-1E-14)&&eey[1]>(1-1E-14)&&eez[2]>(1-1E-14);
    }
/// rotate vector into the 'A'-system
  inline Vec3 RotateFromOld(const Vec3& v)  const{
    return Vec3(eex[0]*v[0]+eex[1]*v[1]+eex[2]*v[2],
                eey[0]*v[0]+eey[1]*v[1]+eey[2]*v[2],
                eez[0]*v[0]+eez[1]*v[1]+eez[2]*v[2]);
    }
/// coordinate transformation: translate + rotate into 'A'-system
  inline Vec3 TransformFromOld(const Vec3& v) const {
    return RotateFromOld(v-xx0);
    }

/// inverse rotation
  inline Vec3 RotateToOld(const Vec3& v)  const{
    return Vec3(eex[0]*v[0]+eey[0]*v[1]+eez[0]*v[2],
                eex[1]*v[0]+eey[1]*v[1]+eez[1]*v[2],
                eex[2]*v[0]+eey[2]*v[1]+eez[2]*v[2]);
    }
/// inverse coordinate tranformation
  inline Vec3 TransformToOld(const Vec3& v) const {
    return RotateToOld(v)+xx0;
    }
/// print CoordinateSystem
  inline void print(char *fmt="%12.9lf %12.9lf %12.9lf",FILE *fp=stdout) {
    xx0.print(fmt,fp);
    eex.print(fmt,fp);
    eey.print(fmt,fp);
    eez.print(fmt,fp);
    }
  };
  

// -----------------------------------------------------------------------

#define P_HQUERC 197.327053E-18 // GeV*m
#define P_C      299792458      // m/s
//@Man: conversions energy-frequency-wavelength
//@{
/// $\omega$[Hz] to wavelength[m] 
inline double omega2lambda(double w) {     // [lambda]=m
  return (2*M_PI*P_C)/w;
  }
/// wavelength[m] to $\omega$[HZ] 
inline double lambda2omega(double l) {     // [lambda]=m
  return (2*M_PI*P_C)/l;
  }
/// $\omega$[HZ] to energy [GeV]
inline double omega2energy(double w) {     // [E]=GeV
  return P_HQUERC/P_C*w;
  }
/// energy [GeV] to $\omega$[HZ] 
inline double energy2omega(double e) {     // [E]=GeV
  return P_C/P_HQUERC*e;
  }
/// wavelength[m] to energy [GeV]
inline double lambda2energy(double l) {    // [E]=GeV [lambda]=m
  return (2*M_PI*P_HQUERC)/l;
  }
/// energy [GeV] to wavelength[m]
inline double energy2lambda(double e) {    // [E]=GeV [lambda]=m
  return (2*M_PI*P_HQUERC)/e;
  }
inline double m2GeV(double l) {    // [E]=GeV [lambda]=m
  return (2*M_PI*P_HQUERC)/l;
  }
inline double GeV2m(double e) {    // [E]=GeV [lambda]=m
  return (2*M_PI*P_HQUERC)/e;
  }
//@}

/// description of a straight line
/** A line is describe by a position and a direction vector
 */
class StraightLine {
Vec3   xx0,nn;
public:
/// constructor: nothing is done here
  inline StraightLine() {};
  inline StraightLine(Vec3& x0,Vec3& n) { xx0=x0;nn=normalize(n);}
/// return the position vector
  inline const Vec3& x() const { return xx0; }
/// sets the position vector: Photon p; p.x()=Vec3(0,0,0);
  inline       Vec3& x()       { return xx0; }
 
/// return the position vector
  inline const Vec3& n()  const { return nn; }
/// sets the flight direction: Photon p; p.n()=Vec3(0,0,1);
  inline       Vec3& n()        { return nn; }

  } ;

/// description of a photon
/** A photon is describe by defining the energy [GeV](like in GEANT)
    the position x(), the flight direction n() and the polarisation
    vector. The class Photon provides the combined information.
 */
class Photon : public StraightLine {
Vec3    pol;
double  ee;   // [GeV]
public:
/// constructor: nothing is done here
  inline Photon() {};

  
/// return the position vector
  inline const Vec3& polarisation()  const { return pol; }
/// sets the polarisation vector: Photon p; p.polarisation()=Vec3(0,1,0);
  inline       Vec3& polarisation()        { return pol; }
  
/// return the energy[GeV]
  inline double  e()  const { return ee; }
/// sets the energy[GeV]
  inline double& e()        { return ee; }
   
/// returns the wavelength[m]
  inline double  lambda()  const  { return energy2lambda(ee); }
/// returns the frequency [1/sec]
  inline double  omega()  const   { return energy2omega(ee); }
  
  };

/// description of a massive particle without spin
class Track : public StraightLine {
double  ee,pp, mm;
public:
/// constructor: nothing is done here
  inline Track(double e,double px,double py,double pz) {
    ee=e;
    n()=normalize(Vec3(px,py,pz));
    pp=px*px+py*py+pz*pz;
    mm=sqrt(ee*ee-pp);
    pp=sqrt(pp);
    };

/// returns the momentum of the particle
  inline double  p()    const { return pp; }
/// sets the momentum of the particle
  inline double  mass() const { return mm; }
/// returns the energy of the particle
  inline double  e()    const { return ee; }
  };

// -------------------------------------------------------------------------
/// utility for scanning the ASCII input files
/**
 This class provide some function to read from an ASCII-Scriptfile,
 where comment line starting with a '\#' are allowed. This lines
 gets automatically removed during the calls. 
 */
class ScriptFile {
char *bfr,*s;
int  bufsize;
FILE *fp;
public:
/// construcor: open the ASCII-File
  ScriptFile(char *filename);
/// returns the file pointer
  inline FILE* file() { return fp; }
/// determine the length in bytes of the next n-items
  int  itemlength(char *s,int nitem);
  int scan(char *fmt, ... );
/// reads a line from the file removing comments and empty lines
  int getline(char s[256]);
/// reads a line from the file removing comments
  int skipline(char s[256]);
/// reads the next string from the file removing comments
  int gettag(char s[256]);
/// destructor
  ~ScriptFile();
  };

// -----------------------------------------------------------------------

/// description of a tube
/** 
  This class is used for the description of the proton and electron
  beam shroud.
  */
class Tube : public CoordinateSystem {

double r,l;

public:
/// sets tube radius:  Tube t; t.radius()=200;
  inline        double& radius()       { return r; }
/// return the tube radius
  inline        double  radius() const { return r; }
/// set the tube length: Tube t; t.length=30;
  inline        double& length()       { return l; }
/// returns the length of tube.
  inline        double  length() const { return l; }
/// assginment operator: Tube A,B; A=B;
  inline void operator=(const Tube& t)  { 
    CoordinateSystem::operator=(t);
    r=t.r;
    l=t.l;
    }
/// Intersection of a photon with a tube. See also page:
  int intersection(const StraightLine& p,double *l);
/// print the contents of the class to an ASCII-File
  void print(FILE* fp=stdout,char *name="Tube");
///  constructor reading information from an ASCII-File produced using 'print'
  Tube(ScriptFile &f);
///  copy constructor
  inline Tube(const Tube& t)  { *this=t; }
///  constructor: nothing is done here.
  inline Tube() {};
  };

// -----------------------------------------------------------------------
/// description of an rectangle
/**
  A rectangular surface in space is described. The rectangle
  can have a division of the x and y axis (in the local coordinates),
  like it is the case for th RICH-supermodules. The position of 
  the intersection of a photon with the surface can be calculated.
  The reflection of a photon on the surface (planar mirrors) is also
  described.
 */
class Rectangle : public CoordinateSystem {


double xcell,xdim;
double ycell,ydim;
int    nnx,nny;
double dd;
Vec3   lasthit,lastlhit;

public:

/// returns the number of division along the x-axis
  inline        int     nx()    const { return nnx; } 
/// returns the number of division along the x-axis
  inline        int     numcol()const { return nnx; } 
/// returns the length of the rectangle in x-direction
  inline        double  xsize() const { return xdim; } 
/// returns the length of one division the x-axis
  inline        double  xdiv()  const { return xcell; } 

/// sets the number of division along the x-axis
  inline        int&    nx()          { return nnx; } 
/// sets the number of division along the x-axis
  inline        int&    numcol()      { return nnx; } 
/// sets the length of the rectangle in x-direction
  inline        double& xsize()       { return xdim; } 
/// sets the length of one division the x-axis
  inline        double& xdiv()        { return xcell; } 

/// returns the number of division along the y-axis
  inline        int     ny()    const { return nny; } 
/// returns the number of division along the y-axis
  inline        int     numrow()    const { return nny; } 
/// returns the length of the rectangle in y-direction
  inline        double  ysize() const { return ydim; } 
/// returns the length of one division the y-axis
  inline        double  ydiv()  const { return ycell; } 

/// sets the number of division along the y-axis
  inline        int&    ny()          { return nny; } 
/// sets the number of division along the y-axis
  inline        int&    numrow()      { return nny; } 
/// sets the length of the rectangle in y-direction
  inline        double& ysize()       { return ydim; } 
/// sets the length of one division the y-axis
  inline        double& ydiv()        { return ycell; } 

/// returns the column (index of the division in x-direction)
  inline int col(double x) const { return ((int)(x/xcell));}
/// returns the row (index of the division in y-direction)
  inline int row(double y) const { return ((int)(y/ycell));}
/// calculated a linear index from the row and column
  inline int index(int row,int col) const { return row*numcol()+col;  }
/// calculated a linear index from the position (local coordinates) on the surface
  inline int index(double x,double y) const { return index(row(y),col(x));  }
/// center in x of the column number 'col'  
  inline double x(int col) const {
    return (col+0.5)*xcell;
    }
/// center in y of the column number 'col'  
  inline double y(int row) const {
    return (row+0.5)*ycell;
    }

/// Intersection of a photon with the surface. See also page:
/**
  The intersection point of a photon with the surface of the rectangle
  is calculated. If there is a intersection the function returns 1 else
  it returns 0. In the first case the distance between the intersection
  point and the photon postion is return using the second argument.
  You can move the photon to the intersection point simply by: p.x()+=l*p.n();
  */
  int intersection(const StraightLine& p,double *l);
/// return the last hit position
  inline const Vec3& hit() const { return lasthit;}
/// return the last hit position in local coordinates
  inline const Vec3& localhit() const { return lastlhit;}
/// reflextion of a photon on the surface.
  int reflect(Photon* p);


/// assginment operator: Rectangle A,B; A=B;
  inline void operator=(const Rectangle& r)  { 
    CoordinateSystem::operator=(r);
    xcell  =r.xcell;
    xdim   =r.xdim;
    nnx    =r.nnx;
    ycell  =r.ycell;
    ydim   =r.ydim;
    nny    =r.nny;
    dd     =r.dd;
    lasthit=r.lasthit;
    }

/// print the contents of the class to an ASCII-File
  void print(FILE *fp=stdout,char *name="rectangle") const;

///  constructor reading information from an ASCII-File produced using 'print'
  Rectangle(ScriptFile&);
///  constructor : x0=origin, vx=vector&length defining x-direction ,vy same in y
  Rectangle(const Vec3& x0,const Vec3& vx,const Vec3& vy,
            int numcol=0,double xcell=0,
            int numrow=0,double ycell=0);
/// constructor 
  inline Rectangle() {  
    nnx=nny=1;
    xcell=ycell=0;
    }
/// copy constructor
  inline Rectangle(const Rectangle& r)  { *this=r; }
  };
  
  
// ---------------------------------------------------------------------------

class Polygon {
double *xx,*yy,*nnx,*nny,*dd;
int    num;
public:
  inline Polygon() { num=0;nnx=nny=xx=yy=dd=0;}
  ~Polygon();
  void set(double x,double y);
  inline int numpoint() { return num; }
  inline       double& x(int i)       { return xx[i]; }
  inline const double& x(int i) const { return xx[i]; }
  inline       double& y(int i)       { return yy[i]; }
  inline const double& y(int i) const { return yy[i]; }
  int inside(double x,double y);
  };
  
// ---------------------------------------------------------------------------
  
template<class T> class ListElement {
public:
  ListElement<T> *next;
  T* ptr;
  inline ListElement(T *t) { next=0; ptr=t; }
  };

/// a very simple List
/**
  To have the ability to store a elements, where the total number
  isn't defined from the beginning (PMTypes,PMT's) this List-class
  is introduced. The List can be created using the append function,
  one can iterate over all elements using:\\[4mm]
  List<double> l;\\
  double *x;\\
  l.newiterator();\\
  while ((x=l.iterate())) { ; }\\
  or \\
  for (l.newiterator();(x=l.iterate();) {}\\[4mm]
  The constructor delete the List. A template class is used
  to allow the usage for differnt kind of data, like List<PMType>
  and List<PM>.
  */
template<class T> class List {
ListElement<T> *first,*last,*iter;
public:
/// constructor
  inline List(){ first=last=0;}
/// destructor
  inline ~List(){ 
    iter=first;
    while (iter) {
      ListElement<T>* t=iter;
      iter=iter->next;
      delete t;
      }
    }

/// append an element of type T to the list.
  void append(T* t) {
    ListElement<T>* l=new ListElement<T>(t);
    if (last) last->next=l;
    else first=l;
    last=l;
    }
/// initialize iteration over the list.
  inline void newiterator() { iter=first; }
/// iterate over the list, return a zero pointer, when finished
  inline T* iterate() {
  T* l;
  if (iter) { 
      l=iter->ptr;
      iter=iter->next; 
      }
    else l=0;
    return l;
    }
  };
  
// ---------------------------------------------------------------------------
/// two dimensional table
/**
  This class uses the structure CPOI to store its (x,y)-values for 
  every point in the table. A typical use is the refractive index
  of a dieletric material, which is specified in the input file
  using a set of points (lambda,refractive index). This class
  provide then function like a linear interpolation, so that this
  code haven't to be repeated everywhere, where the data is only
  know on discrete points, which of cause also allow an easy change
  of the input the the MC by changing the input ASCII-File.
*/
class Table   {
int    ifirst,num;
int    ffl;
double *h;
double xxmin,xxmax,dx;
int    nx;

public:
/// index in the CPOI table, where the first entry can be found
  inline int    firstpoint() const { return ifirst; }
/// number of points in this table
  inline int    numpoints()  const { return num; }
/// return the x-value of the i'th point  
  inline float  x(int i)     const { return ArtePointer<CPOI>(ifirst+i)->x[0]; }
/// return the y-value of the i'th point
  inline float  y(int i)     const { return ArtePointer<CPOI>(ifirst+i)->x[1]; }
/// lowest=first defined x-value
  inline float  xmin()  const { return x(0); }
/// largest=last defined x-value
  inline float  xmax()  const { return x(num-1); }  
/// return the result of a linear interpolation with the points around x
  double LinearInterpolation(const double x) const;
/// return the result of a third order polynominal  interpolation around x
  double CubicInterpolation(const double x) const;
/// default functional access (now linear interpolation).
double  f(const double x) ;
/// assignment operator: Table a,b; a=b;  
  inline void operator=(const Table& t)  {
    num=0;
    int i;
    for (i=0;i<t.numpoints();i++) append(t.x(i),t.y(i));
    h=0;
    finit();
    }
/// print the contents of the class to an ASCII-File
  void print(FILE *fp=stdout,char *name="table") const;
///  constructor reading information from an ASCII-File produced using 'print'
  Table(ScriptFile& f);
///  constructor producing an empty table
  inline Table() {    num=0; h=0;}
/// filling of an empty table (in frequent call the x has to be ordered)
  void append(float x,float y);
/// setting of existing x-points (special purposes)
  inline float& x(int i)           { ffl=1;return ArtePointer<CPOI>(ifirst+i)->x[0]; }
/// setting of existing y-points (special purposes)
  inline float& y(int i)           { ffl=1;return ArtePointer<CPOI>(ifirst+i)->x[1]; }
/// for an Table() table finit has to be called before using f
  void finit();
/// constructor to be used if ARTE table CPOI is already filled
  inline Table(int i,int n) { 
    ifirst=i;
    num=n;
    h=0;
    finit();
    }
/// copy constructor
  inline Table(const Table& t)  {
    *this=t;
    }
/// destructor
  ~Table();
  };
  

#endif 

