//**********************************************************************/
//   RICH reconstruction (rire), version 2.00
//
// Created: 08.03.99, by Ignacio Ari\~no. 
//        
//  * RichHits class to hold the RICH hits without looping through ARTE 
//       tables.
//  * The class Richhits has to be initialised before using RichTrack class
//       (in future versions it will be automatic)
//
//
//**********************************************************************/

#ifndef _RICHHITS_HH_
#define _RICHHITS_HH_



#include "CLHEP/Vector/ThreeVector.h"
#include "CLHEP/Matrix/SymMatrix.h"

#include <vector.h>

// ------------------------------------------------------------------------

class Pcand
{
public:
  inline Pcand(float th, float sig)
    {
      theta=th;
      sigma=sig;
    }
  inline float Theta(){return theta;}
  inline float Sigma(){return sigma;}

private:
  float theta;
  float sigma;
};

////////////////////////////////////////////////////////////////////

class RHit
{
public:
RHit(); 
//~RHit();
inline RHit(double inphi,double inlambda,double incellsize, 
	    int inup=0, int inmctrack=0)
    {
      phi=inphi;
      lambda=inlambda;
      cellsize=incellsize;
      up = inup;
      mctrack=inmctrack;
    }
  inline  double Phi(){return phi;}
  inline  double Lambda(){return lambda;}
  inline  double Cellsize(){return cellsize;}
  inline  int Mctrack(){return mctrack;}
  inline  int Up() {return up;}
 
private:
  //  int x,y,assigned,mctrack;
  double phi;
  double lambda;
  double cellsize;
  int mctrack;
  int up;
} ;


///////////////////////////////////////////////////

class RichHits
{
public:
  RichHits(); // fill the vector with all the rich hits.
  ~RichHits();
  inline RHit rhit(int i){return *VecRichHits[i];}
  inline double Phi(int i){return VecRichHits[i]->Phi();}
  inline double Lambda(int i){return VecRichHits[i]->Lambda();}
  inline double Cellsize(int i){return VecRichHits[i]->Cellsize();}
  inline int McTrack(int i){return VecRichHits[i]->Mctrack();}
  inline int Up(int i){ return VecRichHits[i]->Up();}

  inline void append(RHit hit){
    RHit* rh=new RHit(hit);
    VecRichHits.push_back(rh);
  }

  inline const int numhits(){return VecRichHits.size();}

private:

vector<RHit*> VecRichHits;

};


#endif





