
//**********************************************************************/
// Created: 08.03.99, by Ignacio Ari\~no   for the RICH reconstruction 
//         from tracks
//
// implementation of RichTrack class
//
//**********************************************************************/


#include <arte/arte.hh>

#include "rire/richreco.hh"

#include "rich/PhiLambda.h"
#include "rich/rich.h"

#include "CLHEP/Hist/HBookTuple.h"


extern "C" float geomeff_(float *x,float *p,float *costhe,int *n,
			  float *mina,float *maxa);

RichTrack::RichTrack()
{
  track= new RcSegPoint();
  Readconstantsfromfile();
    //default constructor
}

RichTrack::RichTrack(RcSegPoint rcsp)
{  
  Readconstantsfromfile();
  track= new RcSegPoint(rcsp);
  track->prolongate(ZMidRich()); //prolongate to middle of RICH
}


RichTrack::RichTrack(RcMatchPoint rcmp)
{
  Readconstantsfromfile();
  track = new RcSegPoint(rcmp);
  track->prolongate(ZMidRich()); //prolongate to middle of RICH
}

RichTrack::RichTrack(Hep3Vector point, Hep3Vector direction, float momentum )
{
  Readconstantsfromfile();
  HepSymMatrix cov(5,0); //make zero cov matrix

  RcPoint rcp(point,direction.x(),direction.y(),1/momentum,cov,0);
  track = new RcSegPoint(rcp);

  //track->setcov(3,3,0.000001);
  //track->setcov(4,4,0.000001);
  track->setcov(3,3,0.00000025);  //0.0005^2
  track->setcov(4,4,0.00000025);

  track->prolongate(ZMidRich()); //prolongate to middle of RICH

  //beampipe shadow
  SetShadow();

}


RichTrack::RichTrack(ArtePointer<RTRA> aprtra)
{
  Readconstantsfromfile();

  //look for RPNT closer to the begin of RICH
  ArtePointer<RPNT> closerpnt;
  ArtePointer<RPNT> closerpnt2;

  //closerpnt2->zf=2000;
  for(ArteRelationIterator<RPNT> rpntit=aprtra->rpnt.begin();
      rpntit!=aprtra->rpnt.end();rpntit++)
    {
      if((*rpntit)->zf>closerpnt->zf && (*rpntit)->zf<zRichmax)
	closerpnt=*rpntit;

      if(closerpnt2->zf < zRichmax) closerpnt2=*rpntit;
      if((*rpntit)->zf<closerpnt2->zf && (*rpntit)->zf>zRichmax)
	closerpnt2=*rpntit;
    }

  Hep3Vector point(closerpnt->xf,closerpnt->yf,closerpnt->zf);

  //cout<<point<<closerpnt2->xf<<" "<<closerpnt2->yf<<" "<<closerpnt2->zf<<endl;

  //if there is no point after magnet give momentum 0
  //so that no likelihoods are calculated (provisional)
  if(closerpnt->zf>300){
    float tx,ty;
    if(closerpnt2->zf > zRichmax){
      //if(closerpnt2->zf<2000){
      tx=(closerpnt2->xf-closerpnt->xf)/(closerpnt2->zf-closerpnt->zf);
      ty=(closerpnt2->yf-closerpnt->yf)/(closerpnt2->zf-closerpnt->zf);
    }else{
      tx=closerpnt->txf;
      ty=closerpnt->tyf;
    }

    //RcPoint rcp(point,closerpnt->txf,closerpnt->tyf,closerpnt->pf,
    //	closerpnt->cvf,0);
    RcPoint rcp(point,tx,ty,closerpnt->pf,closerpnt->cvf,0);
    track = new RcSegPoint(rcp);
    track->prolongate(ZMidRich());
  }
  else{
    RcPoint rcp(point,closerpnt->txf,closerpnt->tyf,10000001,
		closerpnt->cvf,0);
    track = new RcSegPoint(rcp);
    //  track->prolongate(ZMidRich());
  }

  //beampipe shadow
  SetShadow();

  for(int i=0;i<6;i++)  lnlikelihoods[i]=0;

  backslope=0;

}


RichTrack::~RichTrack()
{
  delete track;
}





void RichTrack::Readconstantsfromfile()
{
  //find vessel dimensions from ARTE tables.
  for(ArteTable<GESL>::iterator geslit=ArteTable<GESL>::begin();
      geslit!=ArteTable<GESL>::end();geslit++){
    if((*geslit)->cmp==5){
      zRichmin=(*geslit)->z1;
      zRichmax=(*geslit)->z2;
      break;
    }
  }

  if(Meanrefrindex==0){
    if(ArteTable<MTRA>::begin()!=ArteTable<MTRA>::end())
       Meanrefrindex=1.00138;
    else{
      int run=(*ArteTable<EVHD>::begin())->run;
      float radbeta1=getRadius(run,0);
      Meanrefrindex=1/cos(radbeta1);
    }
  }
             
  //provisional: constants set here

  maxtheta=0.060;
  numsigmas=3;

  Sigma_spherical=0.00016;//not fully understood 
  Sigma_cromatic=0.0005;
  Sigma_cellsize=2/(1140*sqrt(12)); //to be multiplied by the cellsize

}




void RichTrack::SetShadow()
{
  float azi0=atan(track->y()/track->x());
  if(track->x() < 0) azi0+=M_PI;
  if(track->x() > 0 && track->y() < 0) azi0+=2*M_PI;

  //only for tracks
  if(track->y()>0) 
    azi0+=M_PI;
  else
    azi0-=M_PI;

  float dist=sqrt(track->x()*track->x() + track->y()*track->y());
  ArtePointer<CVES> apcves(1);
  float bprad=(apcves->prad)/10;
    
  if(dist>bprad){
    float theta=asin(bprad/dist);
    minazi=azi0-theta;
    maxazi=azi0+theta;

  }
  else
    {
      minazi=10;
      maxazi=10;
    }

  // minazi=0;
  // maxazi=0;

}





double RichTrack::NumExpPhotons(int hyp)
{

  //beam pipe shadow
  if(minazi==10 && maxazi==10) return 0;
  //  float shadow=(maxazi-minazi)/(2*M_PI);

  double beta=1/sqrt(1+pow(masshyp(hyp)*track->pe(),2));
  if(beta<1/Meanrefrindex) return 0;

  float theta_cer=ThetaCer(hyp);
  
  double nphotons= Nphotbeta1 * geomefi(theta_cer) * 
    (1-1/pow(Meanrefrindex*beta,2))/(1-1/pow(Meanrefrindex,2));

  return nphotons;  
}



 
double RichTrack::RelTheta(double hitphi,double hitlambda,int up=0)
{

  //corrections for spherical aberrations (up=0, down=1)
  
  float dphi= RICH_LambdaPhi->Correct()->dPhi(hitphi,hitlambda,1-up);
  float dlam=RICH_LambdaPhi->Correct()->dLambda(hitphi,hitlambda,1-up);

  //alignment corrections?
  //  float dy=(TrackLambda()-hitlambda-P_verti+0.001)/D_verti;

  return sqrt(dphi*dphi + dlam*dlam);
}




double RichTrack::BackSlope(float sigmamean,float binsize,
			    int numback,int *histoback)
{
  if(numback==0) {
    //cout<<"warning: numback=0 "<<TrackLambda()<<" "<<TrackPhi()<<endl;
    numback=1;
    //    backslope=0;
    //    return 0;
  }

  // Calculate accessible area
  
  float thetal=0;
  float thetah=0;
  float lastthetah=0;
  float area=0;
  float theta_calc=0;

  for(int j=5;j>0;j--){
    theta_calc=ThetaCer(j);
    if(theta_calc==0) continue;
    thetal=theta_calc-numsigmas*sigmamean;
    thetah=theta_calc+numsigmas*sigmamean;
    if(lastthetah<thetal){
      area+=3.141592*(pow(thetal,2)-pow(lastthetah,2))*
	geomefi((thetal+lastthetah)/2);
    }
    lastthetah=thetah;      
  }
  if(lastthetah<maxtheta){
    area+=3.141592*(pow(maxtheta,2)-pow(lastthetah,2))*
      geomefi((maxtheta+lastthetah)/2);
  }  

  int nbins=int(maxtheta/binsize);
      
  float alpha=1;
  float beta=0;

  double chi=0;

  if(area!=0){

    double lastchi=10000;

    backslope=double(numback)/area;

    float alphaprobe=0.02;
    float betaprobe=-0.1;
    //float alphastep=0.1;
    //float betastep=-5;
    
    float chi=back_chi(backslope,alpha,beta,nbins,binsize,histoback);
    int steps=0;
    do {
      steps++;
      lastchi=chi;
      float chialpha=back_chi(backslope,alpha+alphaprobe,beta,nbins,binsize,
			      histoback);
      float chibeta=back_chi(backslope,alpha,beta+betaprobe,nbins,binsize,
			     histoback);
      alpha+=100*(lastchi-chialpha)*alphaprobe/lastchi;
      beta+=100*(lastchi-chibeta)*betaprobe/lastchi;
      chi=back_chi(backslope,alpha,beta,nbins,binsize,histoback);
      //cout<<"chi "<<steps<<" "<<alpha<<" "<<beta<<" "<<chi<<endl;
    }while((lastchi-chi)>chi*0.01 && steps<10);
     
  }
  
  //  cout<<endl<<"lik, chi "<<alpha<<" "<<beta<<" "<<alpha2<<" "<<beta2<<endl;
  //cout<<" chi2 "<<alpha2<<" "<<beta2<<" "<<chi<<endl;


  back_alpha=alpha;
  back_beta=beta;

  if(area!=0) {
    backslope=double(numback)/area;
    return double(numback)/area;
  }
  else {
    //cout<<"error : area=0 "<<lastthetah<<" "
    //	<<geoeff((maxtheta+lastthetah)/2)
    //	<<" "<<Track()->x()<<" "<<Track()->y()<<endl;
    backslope=0;
    return 0;
  }
}


double RichTrack::back_chi(float slope, float alpha, float beta, int nbins,
			   float binsize,int *histoback)
{
  double chi=0;
  for(int i=0;i<nbins;i++){
    if(histoback[i]==-5) continue;
    float th=(i+0.5)*binsize;
    float fun=backslope*binsize*2*3.141592*(alpha*th+beta*th*th)*
      geomefi(th);
    if(histoback[i]!=0) chi+=pow(histoback[i]-fun,2)/histoback[i];
    else chi+=fun*fun;
    //cout<<"        "<<i<<" "<<fun<<" "<<histoback[i]<<" "<<chi<<endl;
  }
  return chi;
}

float RichTrack::geoeff(float theta)
{

  float x[3];
  float p[3];
  x[0]=track->x();
  x[1]=track->y();
  x[2]=track->z();

  p[0]=RichDir().x();
  p[1]=RichDir().y();
  p[2]=RichDir().z();
  //  int n=int(40*theta/maxtheta);
  //  if(n<20) n=20;
  int n=20+int(20*theta/maxtheta);

  float ctheta=cos(theta);

  //cout<<"theta "<<theta<<" "<<ctheta<<endl;

  //float geoef=geomeff_(x, p, &ctheta,&n);

  float mina=minazi;
  float maxa=maxazi; 

  float geoef=geomeff_(x, p, &ctheta,&n,&mina,&maxa);
  return geoef;
}


void RichTrack::Fillgeomefi()
{
  float maxefi=0;
  int imaxefi=0;
  for(int i=0;i<15;i+=2){
    //for(int i=0;i<15;i++){
    geomefis[i]=geoeff(maxtheta*float(i+0.5)/15);
    if(maxefi<geomefis[i]) {
      maxefi=geomefis[i];
      imaxefi=i;
    }
  }
  
  for(int i=1;i<15;i+=2){
    geomefis[i]=(geomefis[i-1]+geomefis[i+1])/2;
  }
  
  /*
  cout<<"geomefis"<<endl;
  for(int i=0;i<15;i++){
    cout<<"      "<<i<<" "<<int(i/2)<<" "<<maxtheta*float(i+0.5)/15<<" "<<geomefis[i]<<endl;
    }*/
  
  for(int i=0;i<15;i++){
    if(geomefis[i]<maxefi/2.5 && i>imaxefi) geomefis[i]=maxefi/2.5;
    if(geomefis[i]<maxefi/1.5 && i<imaxefi) geomefis[i]=maxefi/1.5;
  }
}
    





void RichTrack::FindLikelihoods(RichHits *richhits)
{

  if(fabs(1/Track()->pe())<0.0001) return;

  Fillgeomefi();

  //find the list of candidates for any hypothesis and the list of 
  //background photons.

  vector<Pcand*> VCand;

  double sig_center=sqrt(pow(SigmaPhiTrack(),2)+pow(SigmaLambdaTrack(),2)); 
  //double sig2=pow(sig_center,2)+pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);
  double sig2=pow(Sigma_spherical,2)+pow(Sigma_cromatic,2);

  double theta_el=ThetaCer(1);
  //Optical corrections
  //displacements at z=0 (m)
  float x0=(RichEntryPoint().x()-RichDir().x()*RichEntryPoint().z())*0.01;
  float y0=(RichEntryPoint().y()-RichDir().y()*RichEntryPoint().z())*0.01;
  RICH_LambdaPhi->Correct()->newRing(atan(RichDir().x()),atan(RichDir().y())
				     ,theta_el,x0,y0);
  float sigmamean=0;
  int numcand=0;
  int numback=0;

  float binsize=0.002;
  //int nbins=int(maxtheta/binsize);
  int nbins=30;
  int *histoback=new int[nbins]; 
  for(int i=0;i<nbins;i++){
    histoback[i]=0;
  }

  double sigmaphi=SigmaPhiTrack();
  double sigmalambda=SigmaLambdaTrack(); 
  double tphi=TrackPhi();
  double tlambda=TrackLambda();

  for(int i=0;i<richhits->numhits();i++)
    {

      float thetai=RelTheta(richhits->Phi(i),richhits->Lambda(i),
			    richhits->Up(i));
      if(thetai>maxtheta) continue;

      //beam pipe shadow
      int shadow=0;
      float dphi=richhits->Phi(i)-tphi;
      float dlambda=richhits->Lambda(i)-tlambda;

      float azi=atan(dlambda/dphi);
      if(dphi < 0) azi+=M_PI;
      if(dphi > 0 && dlambda <0) azi+=2*M_PI;

      if(azi>minazi && azi<maxazi) shadow=1;
      if(minazi<0 && azi-2*M_PI>minazi) shadow=1;
      

      double sigmai=sqrt(sig2 + pow(sigmaphi*dphi/thetai,2)+
			 pow(sigmalambda*dlambda/thetai,2)+
			 pow(Sigma_cellsize*richhits->Cellsize(i),2));

      /*
      if(azi>minazi && azi<maxazi && thetai>sqrt(pow(TrackLambda(),2)
      						 +pow(TrackPhi(),2))) 
	shadow=1; //shadow
      if(minazi<0 && azi-2*M_PI>minazi&&thetai>sqrt(pow(TrackLambda(),2)
						    +pow(TrackPhi(),2))) 
	shadow=1;
      */

      if(shadow) continue;

      int ring=0;
      for(int j=1;j<6;j++){
	float theta_calc=ThetaCer(j);
	if(fabs(thetai-theta_calc)<numsigmas*sigmai && theta_calc!=0) {
	  ring=1;
	  break;
	}
      }

      sigmamean+=sigmai;
      if(ring){
	Pcand *pcand=new Pcand(thetai,sigmai);
	VCand.push_back(pcand);
      }
      else{ 
	numback++;
	int ibin=int(thetai*nbins/maxtheta);
	histoback[ibin]++;
      }
    }

  if(VCand.size()+numback>0)
    sigmamean/=float(VCand.size()+numback);
  else
    sigmamean=sqrt(sig2 + pow(Sigma_cellsize*0.9,2));
  

  for(int j=1;j<6;j++){
    float theta_calc=ThetaCer(j);
    for(int i=0;i<nbins;i++){
      if(fabs(i*maxtheta/nbins-theta_calc)<numsigmas*sigmamean  &&
         fabs((i+1)*maxtheta/nbins-theta_calc)<numsigmas*sigmamean
	 && theta_calc!=0) {
	//ring=1;
	histoback[i]=-5;
	break;
      }
    }
  }    
  /*
  cout<<"backhisto ";
  for(int i=0;i<nbins;i++){
    cout<<histoback[i]<<" ";
  }
  cout<<endl;
  */

  double bslope=BackSlope(sigmamean,binsize,numback,histoback);
  //  cout<<"b "<<numback<<" "<<backslope<<" "<<back_alpha<<" "<<back_beta<<endl;

  int imin=5;
  int imax=5;
  
  double lnlike=0;
  double likeback=0;

  

  for(int j=5;j>0;j--){
    float thetai=ThetaCer(j);
    if(thetai==0) {
      imin=j-1;
      imax=j-1;
      continue;
    }
    if(j>1) {
      float thetanext=ThetaCer(j-1);
      if((thetai+3*sigmamean)>(thetanext-3*sigmamean)){
	imin=j-1;
      }else{
	likeback=0;
	for(int ihyp=imin;ihyp<=imax;ihyp++){
	  //lnlike+=like(ihyp,imin,imax,nexpp,sigmamean,VCand);
	  lnlikelihoods[ihyp]+=like(ihyp,imin,imax,sigmamean,&likeback,VCand);
	}
	for(int i=1;i<6;i++){
	  if(i<imin || i>imax) lnlikelihoods[i]+=likeback;
	}
	//cout<<"b   "<<imin<<" "<<imax<<" "<<j<<" "<<likeback<<endl;

	imin=j-1;
	imax=j-1;
      }
    }else{	
      likeback=0;
      for(int ihyp=1;ihyp<=imax;ihyp++){
	lnlikelihoods[ihyp]+=like(ihyp,1,imax,sigmamean,&likeback,VCand);
      }
      for(int i=1;i<6;i++){
	if(i>imax) lnlikelihoods[i]+=likeback;
      }	
      //cout<<"c   "<<1<<" "<<imax<<" "<<j<<" "<<likeback<<endl;
    }
  }
//likelihood normalisation
  float hlik=0;
  for(int i=1;i<6;i++){
    if(hlik<lnlikelihoods[i]) hlik=lnlikelihoods[i];
  }
  float sum=0;
  for(int i=1;i<6;i++){
    sum+=exp(lnlikelihoods[i]-hlik);
  }
  for(int i=1;i<6;i++){
    lnlikelihoods[i]=exp(lnlikelihoods[i]-hlik)/sum;
  }


  /*cout<<" likelihood "<<Mom()<<" ";
  for(int i=1;i<6;i++) cout<<lnlikelihoods[i]<<" ";    
  cout<<endl;*/

  
  //delete VCand
  for(vector<Pcand*>::iterator vcit=VCand.begin();
      vcit!=VCand.end();vcit++)
    {
      delete *vcit;
    }
  VCand.erase(VCand.begin(),VCand.end());

  delete[] histoback;

}
  




double RichTrack::like(int ihyp, int imin, int imax,float sigmamean,
		       double *likeback,vector<Pcand*> VCand)
{
  double nexpp=NumExpPhotons(ihyp);
  double theta_calc=ThetaCer(ihyp);
  double theta_min=ThetaCer(imax);
  double theta_max=ThetaCer(imin);
  
  float nback=2*3.141592*backslope*
    ((back_alpha/2.0)*(pow(theta_max+3*sigmamean,2)-
		      pow(theta_min-3*sigmamean,2))
    +(back_beta/3.0)*(pow(theta_max+3*sigmamean,3)-
		      pow(theta_min-3*sigmamean,3)))*
    geomefi((theta_max+theta_min)/2);
  
  //test
  if(nback<0.5) nback=0.5;
  
  /*
    cout<<"nback "<<nback<<" "<<imin<<" "<<imax<<" "<<backslope
      <<" "<<VecCand.size()<<" "<<nexpp<<" "
      <<pow(theta_max+3*sigmamean,2)-pow(theta_min-3*sigmamean,2)
      <<" "<<sigmamean<<" "<<theta_max<<" "<<theta_min<<endl;
  */
  
  int ncand=0;
  for(unsigned int i=0;i<VCand.size();i++)
    {
      double thetai=VCand[i]->Theta();
      double sigmai=VCand[i]->Sigma();
      if(thetai>theta_min-3*sigmai && thetai < theta_max+3*sigmai) ncand++;
    }

  double like=0;
  if(ncand==0){
    //if(imin<=ihyp && ihyp<=imax) 
    like=-(nexpp+nback);
    *likeback=-nback;

    return like;
  }

  float fracs=0;
  float fracb=1;
  *likeback=ncand*log(nback)-(nback)-(ncand*log(ncand)-ncand);

  like=ncand*log(nexpp+nback)-(nexpp+nback)-(ncand*log(ncand)-ncand);
  fracs=nexpp/(nexpp+nback);
  fracb=nback/(nexpp+nback);

  for(unsigned int i=0;i<VCand.size();i++)
    {
      double thetai=VCand[i]->Theta();
      double sigmai=VCand[i]->Sigma();
      
      if(thetai<theta_min-3.*sigmai || thetai >theta_max+3.*sigmai) continue;

      float normback=(pow(theta_max+3.*sigmai,2)-pow(theta_min-3.*sigmai,2))*
	back_alpha/2.
	+(pow(theta_max+3.*sigmai,3)-pow(theta_min-3.*sigmai,3))*back_beta/3.;

      like+=log(fracs*exp(-0.5*pow((thetai-theta_calc)/sigmai,2))/
		(2.48156*sigmai)+
		fracb*(thetai*back_alpha+thetai*thetai*back_beta)/normback);
      //(sqrt(2.*M_PI)*sigmai*0.99)=2.48156;
      *likeback+=log((thetai*back_alpha+thetai*thetai*back_beta)/normback);

    }

  return like;
}
















