#include "rise/RICHEventMC.h"
#include "rich/vessel.h"
#include "arte/RSEG_cmp.hh"
#include "rich/PhiLambda.h"


/////////////////////// RICH photons ////////////////////////////////////////
ArtePointer<GEDE> RichGedePointer=0;

RICHphotons::RICHphotons() {
  if (RichGedePointer==0)
    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp==5) {
        RichGedePointer=(*igede);
        break;
      }
    }
  
  for (ArteTable<MIMP>::iterator imimp=RichGedePointer->mim1;
       imimp!=RichGedePointer->miml; ++imimp) {
    unsigned imtra=(*imimp)->mtra.index();
    photonList.insert( pair<const unsigned, unsigned>(imtra,(*imimp).index()));
  }
  it=photonList.begin();
  itl=photonList.end();
}

void RICHphotons::print() {
  for (begin(); valid(); next()) cout<<(*it).first<<" "<<(*it).second<<" "<<id()<<endl;
}

void RICHphotons::print(unsigned imtra) {
  for (begin(imtra); valid(); next()) cout<<(*it).first<<" "<<(*it).second<<" "<<id()<<endl;
}



//////////////////////// MC RICH RINGS /////////////////////////////////////
RICHrings::RICHrings() {
  //cout<<"RICHrings::RICHrings()"<<endl;
  extern RICHVessel *RICH_Vessel;
  RICHVessel &rv=*RICH_Vessel;
  photons=new RICHphotons;
  unsigned oldMtra=0;
  data.mimps=0;
  float sumxz=0,sumyz=0,sumz=0,sumz2=0,sumx=0,sumy=0;
  for (photons->begin(); photons->valid(); photons->next()) {
    if (photons->mtra()!=oldMtra) {
      if (data.mimps>3) {
	// -- direction
	float delta=data.mimps*sumz2-sumz*sumz;
	data.tx=(data.mimps*sumxz-sumx*sumz)/delta;
	data.ty=(data.mimps*sumyz-sumy*sumz)/delta;
	//cout<<data.tx<<" "<<data.ty<<endl;
	// -- index of refraction
	data.nrefr/=float(data.mimps);
	// -- radius
	data.radius=0;
	for (photons->begin(oldMtra); photons->valid(); photons->next()) {
	  float dtx=data.tx-photons->mimp()->tx/photons->mimp()->p;
	  float dty=data.ty-photons->mimp()->ty/photons->mimp()->p;
	  float rad=sqrt(dtx*dtx+dty*dty);
	  data.radius+=rad;
	}
	data.radius/=float(data.mimps);

	ArtePointer<MTRA> imtra(oldMtra); // obsolete!!!!
	data.vertex=imtra->bver;
	data.topp  =imtra->topp;
	data.mother=imtra->btra->topp;

	ringList.insert(pair<const unsigned, RichRingData>(oldMtra,data));
      }
      
      // -- new ring
      if (!photons->end()) {
	oldMtra=photons->mtra();
	float x=photons->mimp()->x;
	float y=photons->mimp()->y;
	float z=photons->mimp()->z;
	sumxz=x*z;
	sumyz=y*z;
	sumz=z;
	sumz2=z*z;
	sumx=x;
	sumy=y;
	data.mimps=1;
	if (photons->detected()) data.hitbs=1;
	else data.hitbs=0;
	float e=norm(Vec3(photons->mimp()->tx,photons->mimp()->ty,photons->mimp()->p));
	float l=energy2lambda(e);
	data.nrefr=rv.Radiator().RefractiveIndex(l);
      }
    }
    else {
      // -- direction
      float x=photons->mimp()->x;
      float y=photons->mimp()->y;
      float z=photons->mimp()->z;
      //cout<<x<<" "<<y<<" "<<z<<endl;
      sumxz+=x*z;
      sumyz+=y*z;
      sumz+=z;
      sumz2+=z*z;
      sumx+=x;
      sumy+=y;
      // -- index of refraction
      float e=norm(Vec3(photons->mimp()->tx,photons->mimp()->ty,photons->mimp()->p));
      float l=energy2lambda(e);
      data.nrefr+=rv.Radiator().RefractiveIndex(l);
      // -- number of photons
      data.mimps++;
      if (photons->detected()) data.hitbs++;
      // -- coordinate
      data.x=Vec3(x,y,z);
    }
  }
  begin();
}


void RICHrings::print() {
  for (begin(); valid(); next()) {
    cout<<endl
	<<"mtra="<<mtra()<<endl
	<<"type="<<topp()->lund<<endl
	<<"mother="<<mother()->lund<<endl
	<<"hitbs="<<DetectedPhotons()<<endl
	<<"mimps="<<RadiatedPhotons()<<endl
	<<"RICH position="<<x()<<endl
	<<"vertex z="<<vertex()->z<<endl
	<<"direction="<<phi()<<","<<lambda()<<endl
	<<"radius="<<radius()<<endl
	<<"nrefr="<<RefractionIndex()<<endl
	<<"p="<<momentum()<<endl;
  }
}

void RICHrings::fill(HBookTuple *tuple) {
  for (begin(); valid(); next()) {
    tuple->column("tx",tx());
    tuple->column("ty",ty());
    tuple->column("rad",radius());
    tuple->column("p",momentum());
    tuple->column("x",x().x());
    tuple->column("y",x().y());
    tuple->column("z",x().z());
    tuple->column("hitb",DetectedPhotons());
    tuple->column("mimp",RadiatedPhotons());
    tuple->column("z0",vertex()->z);
    tuple->dumpData();
  }
}



float RICHrings::momentum() {
  float px=mtra()->px;
  float py=mtra()->py;
  float pz=mtra()->pz;
  return sqrt(px*px+py*py+pz*pz);
}



/////////////// RECONSTRUCTED RINGS //////////////////////////////////

RICHreco::RICHreco() {

  if (int((*ArteTable<RSEG>::end()).index())-
      int((*ArteTable<RSEG>::begin()).index())<0) return;
  mcRings= new RICHrings;
  for (ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin();
       irseg!=ArteTable<RSEG>::end(); ++irseg) {
    if ((*irseg)->cmp!=Rsegc::rich) continue;
    unsigned imtra=unsigned((*(*irseg)->mtra.begin()).index());
    // -- find correspoding MC particle
    int fFlag=0;
    if (mcRings->find(imtra)) fFlag=1;
    // -- ring parameters and errors
    data.tx      = (*irseg)->txf;
    if (fFlag) key.dtx=data.tx-mcRings->tx();
    else key.dtx=0;
    data.ty      = (*irseg)->tyf;
    if (fFlag) key.dty=data.ty-mcRings->ty();
    else key.dty=0;
    data.radius  = (*irseg)->time;
    if (fFlag) key.dradius=data.radius-mcRings->radius();
    else key.dradius=0;
    data.photons = (*irseg)->ze;
    if (fFlag) key.dphotons=data.photons-mcRings->DetectedPhotons();
    else key.dphotons=0;
    key.mtra=imtra;
    recoList.insert( pair<const RecoKey,RichRecoData>(key,data) );
    }
  begin();
}


void RICHreco::print() {
  for (begin(); valid(); next()) {
    cout<<endl
	<<"mtra="<<(*it).first.mtra<<endl
	<<"phi="<<phi()<<", lambda="<<lambda()<<endl
	<<"dphi="<<dtx()<<", dlambda="<<dty()<<endl
	<<"radius="<<radius()<<endl
	<<"dradius="<<dradius()<<endl
	<<"photons="<<photons()<<endl
	<<"dphotons="<<dphotons()<<endl;
  }
}

void RICHreco::fill(HBookTuple *tuple) {
  unsigned oldMtra=0;
  int      multi=1;
  for (begin(); valid(); next()) {
    unsigned imtra=mtra().index();
    if (imtra!=oldMtra) {
      oldMtra=imtra;
      multi=1;
    }
    else multi++;
    mcRings->find(imtra);
    tuple->column("phi",mcRings->phi());
    tuple->column("phif",phi());
    tuple->column("lam",mcRings->lambda());
    tuple->column("lamf",lambda());
    tuple->column("rad",mcRings->radius());
    tuple->column("radf",radius());
    tuple->column("pho",mcRings->DetectedPhotons());
    tuple->column("phof",photons());
    tuple->column("p",mcRings->momentum());
    tuple->column("x",mcRings->x().x());
    tuple->column("y",mcRings->x().y());
    tuple->column("z",mcRings->x().z());
    tuple->column("z0",mcRings->vertex()->z);
    tuple->column("multi",multi);
    tuple->dumpData();
  }
}


bool RICHreco::find(unsigned imtra) {
  //cout<<"requested mtra="<<imtra<<endl;
  it=itl=recoList.end();
  RecoKey search;
  search.mtra=imtra;
  search.dtx=search.dty=search.dradius=0; // minimum
  search.dphotons=0;
  it =recoList.lower_bound(search);
  search.dtx=search.dty=search.dradius=100; // sanity maximum
  itl=recoList.upper_bound(search);
  //cout<<"after: first mtra="<<(*it).first.mtra<<"  last mtra="<<(*itl).first.mtra<<endl;
  return (it!=recoList.end());
}








