#include <stdio.h>
#include "rise/RingSearchDB.h"
#include "rich/ricutil.h"

extern "C" {
#include "table.h"
}

int RingSearchDB::distribute=0;

void RingSearchDB::getSearchParameters(int run_no, int verb) {
  static int oldRun_no=0;
  static int oldRev_no=-1;
  if (oldRun_no==run_no) return; /////////////////////// BUG: one run cannot have >1 entries, rev number is also needed !!!!!!!!!!!!

  verb=1;////////////////////////////////////////////////////////////////////////////////////

  if (verb>0) cout<<" $$$ RISE [Inform]: Database will be accesed "<<endl;

  char dbsite[256];
  dbsite[0]=0;
  char *getdbsite=getenv("RICH_RECODB");
  if (!getdbsite) {
    if (verb>0) {
      fprintf(stderr," $$$ RISE Inform: RICH_RECODB not defined, using default values for RingSearch\n");
      fprintf(stderr," $$$ RISE Inform: %f %f %f %d %d\n",MinSearchRadius,MaxSearchRadius,SearchStep,Threshold,OccupancyCut);
    }
    return;
  }
  strcpy(dbsite,getdbsite);

  Tables x=OpenTables(dbsite,0);
  if (!x) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(1); }  

  int tx=BookTable(x,"RICH_RADIUS",0,0);
  if (!tx) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(2); }

  TableStamp(x,run_no,TABLEmaxrev);
  if (!FetchTable(x,tx,TABLEle)) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(3); }

  SkipTableField(x,1);
  int   major=TableMajor(x);
  int   minor=TableMinor(x);
  float radius,temperature,pressure,time;
  GetTableField(x,TABLEfloat,10000,&pressure);
  GetTableField(x,TABLEfloat,10000,&temperature);
  GetTableField(x,TABLEfloat,10000,&radius);
  GetTableField(x,TABLEfloat,10000,&time);
  GetTableField(x,TABLEfloat,10000,&MinSearchRadius);
  GetTableField(x,TABLEfloat,10000,&MaxSearchRadius);
  GetTableField(x,TABLEfloat,10000,&SearchStep);
  GetTableField(x,  TABLEint,10000,&OccupancyCut);
  GetTableField(x,  TABLEint,10000,&Threshold);

  //printf(" major minor x TABLEint %d %d %d %d \n",major,minor,(int)x,TABLEint);
  //printf("RUN %d:   P=%f  T=%f  r=%f  t=%f : %f %f %f : %d %d\n",
  //run_no,pressure,temperature,radius,time,MinSearchRadius,MaxSearchRadius,SearchStep,OccupancyCut,Threshold);

  if (verb>0) {
    fprintf(stdout," $$$ RISE Inform: RingSearch parameters changed (new reference run %d):\n",major); 
    fprintf(stdout," $$$ RISE Inform: lower radius=%f  upper radius=%f search step=%f threshold=%d occupancy cut=%d\n",
	    MinSearchRadius,MaxSearchRadius,SearchStep,Threshold,OccupancyCut);
  }

  if (verb>1) PrintTables(x,1);

  CloseTables(x);

  oldRun_no=run_no;
  oldRev_no=minor;
}



void RingSearchDB::fillSearchParameters(int run_no, float *parameters,int verb) {

  char dbsite[256];
  dbsite[0]=0;
  char *getdbsite=getenv("RICH_RECODB");
  if (!getdbsite) {
    if (verb>0) {
      fprintf(stderr," $$$ RingSearchDB [Warning]: RICH_RECODB not defined, using default values for RingSearch\n");
      fprintf(stderr," $$$ RingSearchDB [Warning]: %f %f %f %d %d\n",MinSearchRadius,MaxSearchRadius,SearchStep,Threshold,OccupancyCut);
    }
    return;
  }
  strcpy(dbsite,getdbsite);

  Tables x=OpenTables(dbsite,0);
  if (!x) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(1); }  

  if (distribute) {
    if (verb>0) cout<<" $$$ RingSearchDB [Inform]: >>>>>>> Distributing table <<<<<<<<"<<endl;
    DistributeTables(x);
  }

  int tx=BookTable(x,"RICH_RADIUS",0,0);
  if (!tx) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(2); }

  TableStamp(x,run_no,TABLEmaxrev);
  if (!FetchTable(x,tx,TABLEle)) { fprintf(stderr,"Cannot fatch a table for run %d/table not found\n",run_no); Exit(3); }

  SkipTableField(x,1);
  int   major=TableMajor(x);
  int   minor=TableMinor(x);  
  int   run_ver=0;
  if (major==run_no) run_ver=minor+1;

  TableStamp(x,run_no,run_ver);
  SkipTableField(x,1);

  if (verb>1) {
    fprintf(stderr," $$$ RISE Inform: New parameters for run %d (minor=%d)\n",run_no,run_ver);
    if (major==run_no)
      fprintf(stderr," $$$ RISE Inform: Old version run %d (minor=%d)\n",run_no,run_ver);
  }

  float pressure   =parameters[0];
  float temperature=parameters[1];
  float radius     =parameters[2];
  float time       =parameters[3];

  MinSearchRadius=parameters[4];
  MaxSearchRadius=parameters[5];
  SearchStep     =parameters[6];
  OccupancyCut   =int(parameters[7]);
  Threshold      =int(parameters[8]);

  PutTableField(x,TABLEfloat,1,&pressure);
  PutTableField(x,TABLEfloat,1,&temperature);
  PutTableField(x,TABLEfloat,1,&radius);
  PutTableField(x,TABLEfloat,1,&time);
  PutTableField(x,TABLEfloat,1,&MinSearchRadius);
  PutTableField(x,TABLEfloat,1,&MaxSearchRadius);
  PutTableField(x,TABLEfloat,1,&SearchStep);
  PutTableField(x,TABLEint,1,&OccupancyCut);
  PutTableField(x,TABLEint,1,&Threshold);

  StoreTable(x,tx,0);
  if (verb>1) PrintTables(x,1);
  CloseTables(x);
}

extern "C"
int UpdateRadiusDatabase(int major, float *parameters,int verb=0) {
  RingSearchDB db;
  db.fillSearchParameters(major, parameters, verb);
  return 0;
}
