#include "rise/RingSearchEfficiency.h"
#include "arte/arte.hh"

#ifdef USE_ROOT

RingSearchEfficiency *RICH_Efficiency=0;


RingSearchEfficiency::RingSearchEfficiency() { ;
  char *fname=getenv("ROOTFILE");
  if (!fname) fprintf(stderr,"RingSearchEfficiency: [Error]: Variable ROOTFILE not defined!!! Exiting... \n");
  char rname[256];
  rname[0]=0;
  sprintf(rname,"%srun%d_rise_efficiency.root",fname,ArtePointer<EVHD>(1)->run);
  ring_file = new TFile(rname,"RECREATE","Ringsearch efficiency File");   
  ring_tree=new TTree("ring_tree","Ringsearch efficiency tree");
  ring_tree->Branch("ring",&ring,"tx:ty:rad:txf:tyf:radf:x:y:z:Tx:Ty:p:pho:phof:type:moth:multi");
}

RingSearchEfficiency::~RingSearchEfficiency() {
  cout<<" Writing Ringsearch efficiency histos ... ";
  ring_file->Write();
  cout<<"done"<<endl;
}


void RingSearchEfficiency::fillNtuple() {

  RICHrings *rich_MC = new RICHrings; 
  RICHreco  *rich_RC = new RICHreco; 

  for (rich_MC->begin(); rich_MC->valid(); rich_MC->next()) {
    ring.mc_tx  =rich_MC->tx();
    ring.mc_ty  =rich_MC->ty();
    ring.mc_rad =rich_MC->radius();
    ring.mc_p   =rich_MC->momentum();
    ring.mc_pho =rich_MC->DetectedPhotons();
    ring.mc_x[0]=rich_MC->x().x();
    ring.mc_x[1]=rich_MC->x().y();
    ring.mc_x[2]=rich_MC->x().z();
    ring.mc_type=(float)rich_MC->topp()->lund;
    ring.mc_moth=(float)rich_MC->mother()->lund;
    ring.mc_Tx  =rich_MC->T().x();
    ring.mc_Ty  =rich_MC->T().y();

    ring.rc_tx=ring.rc_ty=ring.rc_rad=ring.rc_pho=ring.multi=0;
    if (rich_RC->find(rich_MC->mtra().index())) {
      for (;rich_RC->valid();rich_RC->next()) {
	if (rich_MC->mtra()!=rich_RC->mtra()) {
	  fprintf(stderr,"RingSearchEfficiency: [Error] rc MTRA != mc MTRA,  skipping ring...\n");
	  continue;
	}
	ring.multi++;
	ring.rc_tx =rich_RC->tx();
	ring.rc_ty =rich_RC->ty();
	ring.rc_rad=rich_RC->radius();
	ring.rc_pho=rich_RC->photons();
	ring_tree->Fill();
      }
    }
    else ring_tree->Fill();
  }

  delete rich_MC;
  delete rich_RC;

}

#endif
