/*** This version allows for a single event only, no checking is performed ***/
/*** Furthermore it only works with a single slp in simulation mode        ***/
/* To make it work with more than one slp, top, trigs, and tracks would
   have to be attached to MOD.
*/
#include "CLHEP/Hist/HBookTuple.h"
#include <math.h>
#include "rich/RICHDAQ.h"
#include "rise/ring.h"
#include <fstream.h> 
extern "C" {
#define _BOOL   // nice.h complains during compilation

#ifndef L2_FARM_MODE
#define rpm_addr int
#else
#include "pce.h"
#include "rhp.h"
#endif


#ifndef L2_FARM_MODE
  typedef struct {
    float x1,y1,z1;
    float x1_err,y1_err,z1_err;
    float x2,y2,z2;
    float x2_err,y2_err,z2_err;
    float energy;
    int   type,mother,charge;
  } FLT_TRACK;
#else
#include "slp_events.h"
#include "slp_error.h"
#include "flte.h"
#endif
}

int RICH_getTracks(int eventno,EVENT_HDR* evt,FLT_TRACK* track,int ntrack);
int RICH_cut();
int RICH_init(char *dir,float radius,int threshold,int magnetflag);
void rich_stop();
void RICHfilename();
#ifndef L2_FARM_MODE
void RICH_check(HBookTuple *tuple);
#endif



static int flte_pref = (COMP_N_SLT << 24);
#define FLTE_PRE(ID) (flte_pref | 6<<16 | (ID))

static Hits      h;
static DualSpace d;


typedef struct {
  float tx,ty,tx_err,ty_err,rad,EcalE,Ex,Ey,Ez,Ecaltx,Ecalty;
  int   charge, flte_charge, nPhotons, eventNo;
  } RICHtrack;
  
RICHtrack rtrack[100];

void pub_sinit();
void pub_sbind();
#ifndef L2_FARM_MODE
void rhp_msinit();
void rhp_msbind();
#endif
static char outfilename[30];
FILE *richfp;
FILE *lamphi;
FILE *angles;

// -------------------------------------------------------------------------

int RICH_init(char *dir,float radius,int thresh,int mflag) {  

  RICH_DAQ=new RICHDAQ(dir);
  if (!RICH_DAQ) return 0;

  RichRun::magnet=mflag;

#ifdef L2_FARM_MODE
  char* DAQRUN = getenv("DAQRUN");
  char* DAQFILENUM = getenv("DAQFILENUM");
  
  strcopy(outfilename,DAQRUN);
  if (outfilename[strlen(outfilename)-1]=='/') 
    outfilename[strlen(outfilename)-1]='_';
  strcat(outfilename,"file");
  strcat(outfilename, DAQFILENUM);
  strcat(outfilename,".dat");
  richfp = fopen(outfilename, "w");
  lamphi = fopen("lamphi.dat", "w");
  angles = fopen("angles.dat", "w");
#endif

  d.define(radius*1000,thresh);
/************  book histogramms *******************************/
#ifdef L2_FARM_MODE
  int call = rhp_hbook1(FLTE_PRE(1),"RICH mass", 60, 2.5, 4.0);
  if( call != GRR_OK) 
    SLP_logError(call ,"FLTE_bookObjects: rhp_hbook1 not pleased with id 1");
#endif
  return 1;
  }

// -----------------------------------------------------------------------------------------
 
int RICH_getTracks(int eventno,EVENT_HDR *event, FLT_TRACK *track,int ntracks) {

  RICHDAQ& daq=*RICH_DAQ;


#ifndef L2_FARM_MODE
  daq.NextEvent();
#else
  daq.NextEvent(event);
#endif
 

  //if (daq.NumHitChannel()>1500) return 0;
  printf("numhit1: tracks=%d channels=%d\n",ntracks,daq.NumHitChannel());

  float r=49.5E-3;
  float dr=1.5 * 0.3E-3;
  //  float bp    = - 288;
  
  float ri    = r-dr;
  float ro    = r+dr;
  float ri2   = ri*ri;
  float ro2   = ro*ro;
  float root2 = sqrt(2);


  for (int k=0;k<ntracks;k++) {

    cout<<endl<<"new track: " << endl ;fflush(stdout);
    int nh=0;
    float radius;
    h.reset();

    float tx=(track[k].x2-track[k].x1)/(track[k].z2-track[k].z1);      
    float ty=(track[k].y2-track[k].y1)/(track[k].z2-track[k].z1);
    float dx=3*(track[k].x2_err+track[k].x1_err)/(track[k].z2-track[k].z1);

    rtrack[k].ty=0;
    rtrack[k].tx=0;
    rtrack[k].ty_err=0;
    rtrack[k].tx_err=0;

    for (int i=0;daq.HitChannelExist(i);i++) {
      float phi=RICH_LambdaPhi->phi(daq.ChannelIndex());
      float lam=RICH_LambdaPhi->lambda(daq.ChannelIndex());
      int plane=(daq.sm()>6) ? 1 : 0;
      
      float x=phi-tx;
      //cout << "x, phi, tx :" << x << " " << phi << " " << tx << endl;
      float y=lam-ty;
      //cout << "y, lam, ty :" << y << " " << lam << " " << ty << endl;
      // inner and outer rectangular bound
      if ((fabs(x)<ri/root2-dx&&fabs(y)<ri/root2)||(fabs(x)>ro+dx&&fabs(y)>ro)) {
	continue;
      }
      // detailed circular cut
      float rp2=(x-dx)*(x-dx)+y*y;
      float rm2=(x+dx)*(x+dx)+y*y;
      if (x<0&&rp2>ri2||x>0&&rm2>ri2) {
	int ok=0;
	if (x>dx) {
	  if (rp2<ro2) ok=1;
	}
	else if (x<-dx) {
	  if (rm2<ro2) ok=1;
	}
	else if (fabs(y)<ro) ok=1;
	if (ok) { 
	  h.append(d.dualx(phi*1000),d.dualy(lam*1000),plane);
	}
      }
    } // hit loop
    
    d.nextevent();
    d.ringsearch(h);
    float dold=1;
    float phi=1E37,lam=1E37;
    cout<< "Found rings numring() = " << d.numring() << "\n";
    for (int n=0;n<d.numring();n++) {
      Ring ring=d.getRing(n);
      float d2=(ring.phi()-tx)*(ring.phi()-tx)+(ring.lambda()-ty)*(ring.lambda()-ty);
      if (d2<dold) {
	phi=ring.phi();
	lam=ring.lambda();
	nh=int(ring.numhit_refit);
	radius=ring.radius;
	dold=d2;
      }
    }
    if (dold<1E-5) {
      rtrack[k].ty_err=0.3E-3;
      rtrack[k].tx_err=0.5E-3;
      rtrack[k].Ex = track[k].x2;
      rtrack[k].Ey = track[k].y2;
      rtrack[k].Ez=track[k].z2;
      rtrack[k].ty =lam;
      rtrack[k].tx =phi;
      rtrack[k].rad=radius;
      rtrack[k].EcalE=track[k].energy;
      rtrack[k].Ecaltx = tx;
      rtrack[k].Ecalty = ty;
      rtrack[k].nPhotons = nh;
      rtrack[k].flte_charge = track[k].charge;
      rtrack[k].eventNo = eventno;
      if( (track[k].x2 - phi * track[k].z2) < 0)
	{
	  rtrack[k].charge = 1;
	}
      else
	{
	  rtrack[k].charge = -1;
	}
    }
  } // track loop
  return 0;
}

// ------------------------------------------------------------------------//

int RICH_cut(FLT_TRACK *track,int ntracks)
{
  RICHDAQ& daq=*RICH_DAQ;

#ifdef L2_FARM_MODE
  int rhpActive = rhp_active(FLTE_PRE(0));
#else
  int rhpActive=0;
#endif
  int returnValue, chi, chj;
  rhpActive = 1;
  static int triggerCalls=0, triggerAccepts=0;
  const static float p0zbend = 288.;
  returnValue = 0;
  printf("numhit %d %d\n",ntracks,daq.NumHitChannel());
  float biy, bjy;
  triggerCalls++;
  if (daq.NumHitChannel()<15000)
    {
      for (int i=0;i<ntracks;i++)
	{
	  if (rtrack[i].tx_err==0) continue;
 	  if ( fabs(biy = (rtrack[i].Ey - tan(rtrack[i].ty) * rtrack[i].Ez))
 	       > 30.) continue;
	  for (int j=i+1;j<ntracks;j++)
	    {
	      if (rtrack[j].tx_err==0) continue;
	      if ((chi=rtrack[j].charge) == (chj=rtrack[i].charge)) continue;
	      if ((chi != rtrack[i].flte_charge)) continue;
	      if ((chj != rtrack[j].flte_charge)) continue;
 	      if ( fabs(bjy=(rtrack[j].Ey-tan(rtrack[j].ty)*rtrack[j].Ez))
 		   > 30.) continue;
	      float bix, bjx, rpi, rpix, rpiy, rpiz, rpj, rpjx, rpjy, rpjz;
	      float ei, ej, deltaI, deltaJ;
	      deltaI = fabs(rtrack[i].Ey - tan(rtrack[i].ty)*rtrack[i].Ez);
	      deltaJ = fabs(rtrack[j].Ey - tan(rtrack[j].ty)*rtrack[j].Ez);
	      // Impact parameter for particle i //

	      bix = rtrack[i].Ex - 
		(float(tan(double(rtrack[i].tx)))) * rtrack[i].Ez;
	      
	      // Impact parameter for particle j //
	      
	      bjx = rtrack[j].Ex - 
		(float(tan(double(rtrack[j].tx)))) * rtrack[j].Ez;
	      
	      // Momentum magnitude for particle i //
	      
	      if (bix != 0)
		{
		  rpi = p0zbend / fabs(bix);
		}
	      else
		{
		  rpi = 0.0;
		}

	      // Momentum magnitude for particle j //
	      
	      if (bjx != 0)
		{
		  rpj = p0zbend / fabs(bjx);
		}
	      else
		{
		  rpj = 0.0;
		}
	      
	      // Energy for particle i //

	      ei = fabs(rpi);
	      if (ei>920.) continue;
	      // Energy for particle j //

	      ej = fabs(rpj);
	      if (ej>920.) continue;

	      // phi, part. i, before magnet //

	      float txi = ( ( rtrack[i].Ex - 
			      float(tan(double(rtrack[i].tx))) * 
			      (rtrack[i].Ez - 450.))
			    / 450.); 

	      float tyi=rtrack[i].ty; // lam, part. i //
	      
	      // phi, part. j, before magnet //
	      
	      float txj = ( ( rtrack[j].Ex - 
			      float(tan(double(rtrack[j].tx))) * 
			      (rtrack[j].Ez - 450.))
			    / 450.); 

	      float tyj=rtrack[j].ty; // lam, part. j //
	      
	      rpix = rpi * float( cos( double(tyi)) * sin( double(txi)));
	      rpiy = rpi * float( sin( double(tyi)));
	      rpiz = rpi * float( cos( double(tyi)) * cos( double(txi)));

	      rpjx = rpj * float( cos( double(tyj)) * sin( double(txj)));
	      rpjy = rpj * float( sin( double(tyj)));
	      rpjz = rpj * float( cos( double(tyj)) * cos( double(txj)));

	      float m2 = 2.0 * (ei*ej - (rpix*rpjx + rpiy*rpjy + rpiz*rpjz));
	      float richmass = float(sqrt(double(m2)));

	      if (m2>4)
		{ 
		  printf("ACCCCCEPPTPTTT\n");

		  fprintf(richfp, "%f ", richmass);    /* 1 */
		  fprintf(richfp, "%f ", bix);         /* 2 */
		  fprintf(richfp, "%f ", biy);         /* 3 */
		  fprintf(richfp, "%f ", bjx);         /* 4 */
		  fprintf(richfp, "%f ", bjy);         /* 5 */
		  fprintf(richfp, "%f ", rpi);         /* 6 */
		  fprintf(richfp, "%f ", rpj);         /* 7 */
		  fprintf(richfp, "%f ", rtrack[i].rad);  /* 8 */
		  fprintf(richfp, "%d ", rtrack[i].nPhotons); /* 9 */
		  fprintf(richfp, "%f ", rtrack[j].rad);   /* 10 */
		  fprintf(richfp, "%d ", rtrack[j].nPhotons); /* 11 */
		  fprintf(richfp, "%d ", chi);         /* 12 */
		  fprintf(richfp, "%d ", chj);         /* 13 */
		  fprintf(richfp, "%d ", chi+chj);     /* 14 */
		  fprintf(richfp, "%d ", rtrack[i].flte_charge); /* 15 */
		  fprintf(richfp, "%d ", rtrack[j].flte_charge); /* 16 */
		  fprintf(richfp, "%f ", ei);          /* 17 */
		  fprintf(richfp, "%f ", ej);          /* 18 */
		  fprintf(richfp, "%f ", rtrack[i].EcalE);  /* 19 */
		  fprintf(richfp, "%f ", rtrack[i].Ex);   /* 20 */
		  fprintf(richfp, "%f ", rtrack[i].Ey);   /* 21 */
		  fprintf(richfp, "%f ", rtrack[i].Ecaltx); /* 22 */
		  fprintf(richfp, "%f ", rtrack[i].Ecalty); /* 23 */
		  fprintf(richfp, "%f ", rtrack[j].EcalE);  /* 24 */
		  fprintf(richfp, "%f ", rtrack[j].Ex);     /* 25 */
		  fprintf(richfp, "%f ", rtrack[j].Ey);     /* 26 */
		  fprintf(richfp, "%f ", rtrack[j].Ecaltx); /* 27 */
		  fprintf(richfp, "%f ", rtrack[j].Ecalty);  /* 28 */
		  fprintf(richfp, "%d\n", rtrack[j].eventNo); /* 29 */
		  returnValue = 1;
		}
	    }
	}
    }
  if (returnValue == 1) triggerAccepts++;
  printf(" RICH:  %d trigger calls, %d trigger Accepts.\n", 
	 triggerCalls, triggerAccepts);
  return returnValue;
}
// -----------------------------------------------------------------------------------------

void rich_stop() {

  if (RICH_DAQ)       { delete RICH_DAQ;        RICH_DAQ=0; }
  if (RICH_Detector)  { delete RICH_Detector;   RICH_Detector=0;} 
  if (RICH_Vessel)    { delete RICH_Vessel;     RICH_Vessel=0; }
  if (RICH_LambdaPhi) { delete RICH_LambdaPhi;  RICH_LambdaPhi=0; }
  fclose(richfp);
  }


#ifndef L2_FARM_MODE

extern "C" void printrich_();

void RICH_check(HBookTuple *tuple) {

  RichRun::magnet=1;


  if (!RICH_DAQ)  {

    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp==5) { RichRun::gede=(*igede); break; }
    }

    for (ArteTable<GESL>::iterator igesl=ArteTable<GESL>::begin();
	 igesl!=ArteTable<GESL>::end(); ++igesl) {
      if ((*igesl)->cmp==12) { RichRun::magnet=(*igesl)->mfl; break; }
    } 

    RICH_init("/afs/desy.de/group/hera-b/ARTE/geo/RICH/",.052,7,0);
    printrich_();
  }


  static int myEventCount=1;

  FLT_TRACK TriggerTrack[100];
  int countTracks=0;
  for (ArteTable<RCCL>::iterator irccl=ArteTable<RCCL>::begin(); 
       irccl!=ArteTable<RCCL>::end(); ++irccl) {

    if (!RichRun::magnet) {
      // --- check pt for magnet OFF
      float tx=(*irccl)->tx;
      float ty=(*irccl)->ty;
      float pt=(*irccl)->esum*sqrt(tx*tx+ty*ty);
      if (pt<0.5) continue;
    }
    else { // for magnet ON
      float x=(*irccl)->x;
      float y=(*irccl)->y;
      float R=sqrt(x*x+y*y);
      float D=sqrt(x*x+fabs(y*y*y));
      float Ktrig=500;
      float Eth=Ktrig*(1/R+1/D);
      if ((*irccl)->esum<Eth) continue;
    }

    // --- find mother
    TriggerTrack[countTracks].mother=(*(*irccl)->mtra.begin())->btra->topp->lund;
    // --- find type
    TriggerTrack[countTracks].type=(*(*irccl)->mtra.begin())->topp->lund;

    // --- track parameters
    TriggerTrack[countTracks].x1=(*irccl)->x;
    TriggerTrack[countTracks].y1=(*irccl)->y;
    TriggerTrack[countTracks].z1=(*irccl)->z;
    TriggerTrack[countTracks].x1_err=.1;
    TriggerTrack[countTracks].y1_err=.1;
    TriggerTrack[countTracks].z1_err=.1;
    TriggerTrack[countTracks].energy=(*irccl)->esum;

    // --- target
    ArtePointer<GTAR> gtar(1);
    TriggerTrack[countTracks].x2=gtar->x;
    TriggerTrack[countTracks].y2=gtar->y;
    TriggerTrack[countTracks].z2=gtar->z;
    TriggerTrack[countTracks].x2_err=.003;
    TriggerTrack[countTracks].y2_err=.003;
    TriggerTrack[countTracks].z2_err=.003;

    if (++countTracks > 9) break;
  }
  cout<<"Track candidates="<<countTracks<<endl;
  
  // --- call ring reconstruction
  if (!countTracks) return;
  EVENT_HDR *event=0;
  int eventno=0;

  RICH_getTracks(eventno,event,TriggerTrack, countTracks);

  for (int i=0; i<countTracks; i++) {

    tuple->column("event",myEventCount);
    tuple->column("richtx",rtrack[i].tx);
    tuple->column("richty",rtrack[i].ty);
    tuple->column("charge",rtrack[i].charge);
    tuple->column("ecaltx",(TriggerTrack[i].x1-TriggerTrack[i].x2)/(TriggerTrack[i].z1-TriggerTrack[i].z2));
    tuple->column("ecalty",(TriggerTrack[i].y1-TriggerTrack[i].y2)/(TriggerTrack[i].z1-TriggerTrack[i].z2));
    tuple->column("p",TriggerTrack[i].energy);
    tuple->column("type",TriggerTrack[i].type);
    tuple->column("mother",TriggerTrack[i].mother);
    tuple->dumpData();

    //cout<<"ECAL: "<<TriggerTrack[i].x1/TriggerTrack[i].z1<<" "
    //<<TriggerTrack[i].y1/TriggerTrack[i].z1
    //<<"    RICH: "<<rtrack[i].tx<<" "<<rtrack[i].ty<<endl;
  }

  myEventCount++;
}

#endif




