//--*-C++-*--
#ifndef _RINGSEARCHTIMING_H_
#define _RINGSEARCHTIMING_H_

#include "rich/rich.h"

#ifdef RISE_TIMING

#include "TStopwatch.h"
#include "TH2.h"
#include "TFile.h"


class RingSearchTiming {

 private:
  TStopwatch sw;
  double     start_time,  start_search_time,  start_fit_time;
  double     stop_time,    stop_search_time,   stop_fit_time;
  double     delta_time,  delta_search_time,  delta_fit_time;
  TH2F       *occup_time,     *occup_search,      *occup_fit;
  TFile      *ftime;

 public:
  ///  Constructor
  inline RingSearchTiming()    {

    // timing file
    char *fname=getenv("ROOTFILE");
    char qfile[256];
    qfile[0]=0;
    sprintf(qfile,"%srun%d_rich_timing.root",fname,ArtePointer<EVHD>(1)->run);
    ftime=new TFile(qfile,"RECREATE","Timing file");

    occup_time  =new TH2F(  "occup_time","Total RingSearch Time vs. Occupancy",200,0,2000,200,0,5);
    occup_search=new TH2F("occup_search",     "Ring Search Time vs. Occupancy",200,0,2000,200,0,5);
    occup_fit   =new TH2F(   "occup_fit",        "Ring Fit Time vs. Occupancy",200,0,2000,200,0,5);

    // reset times
    start_time=start_search_time=start_fit_time=0;
    stop_time = stop_search_time= stop_fit_time=0;
    delta_time=delta_search_time=delta_fit_time=0;
  }

  /// Destructor
  inline ~RingSearchTiming() {
    cout<<" Writing stopwatch histograms... ";
    ftime->Write(); 
    cout<<" done!"<<endl;
  }

  /// Start Stopwatch
  inline void   startTiming()  { 
    sw.Start(); 
    start_time=sw.GetRealTime();
    delta_time=0;
    delta_search_time=0;
    delta_fit_time=0;
  }

  /// Stop Stopwatch
  inline void   stopTiming()   { 
    sw.Stop(); 
    delta_time=sw.GetRealTime()-start_time;
  }

  /// Start ringsearch timing
  inline void startSearchTiming() {
    start_search_time=sw.GetRealTime();
  }

  /// Stop ringsearch timing
  inline void stopSearchTiming() {
    delta_search_time+=sw.GetRealTime()-start_search_time;
  }

  /// Start ringfit timing
  inline void startFitTiming() {
    start_fit_time=sw.GetRealTime();
  }

  /// Stop ringfit timing
  inline void stopFitTiming() {
    delta_fit_time+=sw.GetRealTime()-start_fit_time;
  }


  /// Fill occupancy-time histogram
  inline void   fillTimingHistograms(float occupancy) {
    occup_time->  Fill(occupancy,       delta_time,1);
    occup_search->Fill(occupancy,delta_search_time,1);
    occup_fit->   Fill(occupancy,   delta_fit_time,1);
    //cout<<delta_time<<" "<<delta_search_time<<" "<<delta_fit_time<<endl;
  }

  /// Fill occupancy-time histogram, occupancy taken from HITB table
  inline void   fillTimingHistograms() {
    int n0=RichRun::gede->hit1.index();
    int n1=RichRun::gede->hitl.index();
    fillTimingHistograms(float(n1-n0+1));
  }
};

extern RingSearchTiming *RICH_SearchTiming;

#endif
#endif
