#ifndef __Ring_H__
#define __Ring_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <iostream.h>
#include "rich/ricutil.h"
#include "rich/PhiLambda.h"

#define MAXHITS        10000
#define MAXHITSPERRING 1000
#define MAXRINGS       500



// ------------------------------------------------------------------------

typedef struct {
  int x,y,assigned,mctrack,plane;
  } Hit;

// ------------------------------------------------------------------------

class Hits {
public:
int numhit;
Hit hit[MAXHITS];

inline void reset() { numhit=0; }
inline Hits()       { reset();  }
inline void append(int xx,int yy,int plane,int mctrack=0) {
  if (numhit==MAXHITS) {
    printf("Hits::append(): Too many hits: numhit=%d>dimension=%d\n",
       numhit,MAXHITS);
    return;
  }
  Hit& h=hit[numhit++];
  h.x=xx;
  h.y=yy;
  h.plane=plane;
  h.assigned=0;
  h.mctrack=mctrack;
}



inline void randomize() {
  int i;
  Hit h;
  for (i=0;i<10*numhit;i++) {
    int n=int(rnd_normal()*numhit);
    int m=int(rnd_normal()*numhit);
    if (n!=m) {
      h.x            =hit[n].x,
      h.y            =hit[n].y,
      h.plane        =hit[n].plane;
      h.assigned     =hit[n].assigned,
      h.mctrack      =hit[n].mctrack;
      hit[n].x       =hit[m].x,
      hit[n].y       =hit[m].y,
      hit[n].plane   =hit[m].plane;
      hit[n].assigned=hit[m].assigned,
      hit[n].mctrack =hit[m].mctrack;
      hit[m].x       =h.x,
      hit[m].y       =h.y,
      hit[m].plane   =h.plane;
      hit[m].assigned=h.assigned,
      hit[m].mctrack =h.mctrack;
      }
    }
  }
  

inline void print(FILE *fp=stdout) {
  int i;
  for (i=0;i<numhit;i++) {
    Hit& h=hit[i];
    fprintf(fp,"hit %4d %6d %6d %2d %2d %4d\n",
	    i,h.x,h.y,h.plane,h.assigned,h.mctrack);
    }
  }
};


// ------------------------------------------------------------------------

class Ring  {
  double cov[3][3];
  int checkSN(Hits &h,float x0,float y0,float r0);
public:
  inline Ring() {;}
  int ix0;
  int iy0;
  float x0,y0;
  int numhit;
  Hit* hit[MAXHITSPERRING];
  float radius,numhit_refit,numhit_3sigma;
  int niter;
  float chi2,N,sigma,mu,f,numhit_bgrd,sigma_signal;
  
  // track
  inline double phi()    const { return x0*1E-5; }
  inline double lambda() const { return y0*1E-5; }
  inline double thetaC() const { return radius*1E-5; }
  inline double covariance(int i,int j) const { return cov[i][j]*1E-10; }
  inline double tx() const     { return tan(phi()); }
  inline double ty() const     { return tan(lambda()/cos(phi())); }

  // hits @@@
  inline double  hitPhi(int j)    const {return hit[j]->x/100000.;}
  inline double  hitLambda(int j) const {return hit[j]->y/100000.;}
  inline int     numHits()        const {return numhit;} 

  int  mctrack() const;
  int calcring(Hits& hh);
};

// ------------------------------------------------------------------------

class Center {
public:
  int x0;
  int y0;
  int numhit;
  Hit* hit[MAXHITSPERRING];
  inline Center() {
    x0=y0=numhit=0;
    }
  };

// ------------------------------------------------------------------------


class DualSpace {
Center center[300];
Center *assignedcenter[300],*abovethresh[300];
Ring   rings[MAXRINGS];

int   threshold;
int   r;
int   r2;
float ri;

int   nsqrtf;
float sqrtbini;
float sqrtf[10001];

int     nphi;
float   phibin;
int     phitab[3001];

int   numassigned,numabovethresh,numrings,ringfl;
float dualfac,dualfaci;


  int findmax();
  void findring(Hit* hi,Hits* h);

public:

// ------------------------------------------------------------------------

  inline DualSpace() {
    //cout<<"DualSpace::DualSpace()"<<endl;
    memset(center,0,sizeof(center));
    numassigned=0;
    numrings=0;

    // @@@
    dualfac=100;                // 1/hit pos precision
    dualfaci=1/dualfac;

    }

// ------------------------------------------------------------------------
  inline void nextevent() {
    numassigned=0;
    numrings=0;
    }

  void define(float rad,int t);

// ------------------------------------------------------------------------


  inline int dualx(float x) const {
    return (int)(dualfac*x+0.5);
    }
  inline int dualy(float y) const {
    return (int)(dualfac*y+0.5);
    }

  inline float x(int dualx) const {
    return dualx*dualfaci;
    }
  inline float y(int dualy) const {
    return dualy*dualfaci;
    }


  inline int   numring()      const { return numrings; }
  inline float x0(int i)      const { return x(rings[i].ix0); }
  inline float y0(int i)      const { return y(rings[i].iy0); }
  inline float x0refit(int i) const { return rings[i].x0*dualfaci; }
  inline float y0refit(int i) const { return rings[i].y0*dualfaci; }
  inline float radius(int i)  const { return rings[i].radius*dualfaci; }
  inline int   numhit(int i)  const { return rings[i].numhit; }
  inline int   numiter(int i) const { return rings[i].niter; }
  inline float chi2(int i)    const { return rings[i].chi2*1e-2; }
  inline Hit&  hit(int i,int j) const {return *rings[i].hit[j];}
  inline Ring &getRing(int i)   {return rings[i];}


  inline double ax(int i) const { return x0refit(i)/1000; }
  inline double ay(int i) const { 
    return atan(tan(y0refit(i)/1000)/cos(x0refit(i)/1000)); 
  }

  inline double tx(int i)      const { return tan(x0refit(i)/1000); }
  inline double ty(int i)      const { 
    return tan(y0refit(i)/1000)/cos(x0refit(i)/1000); 
  }

  inline int    mctrack(int i) const { return rings[i].mctrack(); }


  /*@@@
  inline Hist1 plot() {
    Hist1 h(nphi,0,nphi*phibin);
    int i;
    for (i=0;i<nphi;i++) h[i]=center[i].numhit;
    return h;
    }
    */
// ------------------------------------------------------------------------

  inline void append(int  dualx,int dualy,Hit* hit) {
    if (abs(dualx)>6000) return;
    register int i=phitab[int(ri*(dualx+r))];
    if (dualy<0) i=nphi-i-1;
    if (i<0||i>3000) {printf("dualx %d %d %f\n",dualx,r,ri);
//       getchar();
       return;
       }
    register Center& c=center[i];
    c.x0+=dualx,
    c.y0+=dualy,
    c.hit[c.numhit]=hit;
    if (c.numhit++==0) assignedcenter[numassigned++]=&c;
    else if (c.numhit==threshold) ringfl=1;
    }
// ------------------------------------------------------------------------
  void ringsearch(Hits& h);
  };


// ------------------------------------------------------------------------
#endif
