#ifndef _riter_h_
#define _riter_h_
#include <arte/arte.hh>
#include <rich/rich.h>
#include "rise/RICHEventMC.h"  

#ifdef RITER_DEBUG

#ifdef USE_ROOT

#include "TROOT.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TH1.h"
#include "TH2.h"
#include "TProfile.h"
#include "TNtuple.h"
#include "TRandom.h"
#include "TGraph.h"
#include "TGraphErrors.h" 

extern TFile *rfile;
#endif 

#endif

extern float RITERrefind;
extern int   RITERnhip;
extern int   RITERinput;

#ifdef RITER_DEBUG
#include "CLHEP/Hist/HBookFile.h"
extern   HBookTuple *mtuple1; 
extern   HBookTuple *mtuple2; 
extern   HBookTuple *mtuple3; 
extern   HBookTuple *mtuple4; 

#endif
#include "CLHEP/Vector/ThreeVector.h"

#define pi M_PI
#define RITER_ERROR 1
#define RITER_OK 0

#define NCHAN 86016
#define NWCHA (NCHAN-1)/32+1
extern "C" {
  int CherenkovAngle(int nbins, float min, float max, float *xarray);
}
#define NDEL 512       //track_buffer size
#define NFOTB 10000    // photon buffer size
#define NTB 100000     // rich buffer size
#define NHIP  7        // number of hipothesis

#define ifix(x) (int)x 
#define max(x,y) (x>y) ? x : y
#define min(x,y) (x<y) ? x : y
#define sqr(x) x*x

extern "C" {
  extern float alogam_(float&);
}



// Marko Staric iterative search identification routines
//-------------------------------------------------------------

typedef struct {
  Hep3Vector r;
  float sig,eff;
  int type;
  int nh;
} channel;


class RICHGeometry {

 public:
  float sigthc;   
  float sig_sfa;  
  float r_mi;
  
  // sigma^2 =sig0^2+sigthc^2+sig_sfa^2  
  // sig0 =nn*sig_pad; nn={4-m1,2-m4, 1-m16}
  float sig_pad;
  float f_pad;   // f_pad=f/d_pad

#define NUMROW 24
#define NUMCOL 12
#define NUMSM  32
#define NUMCH  16
  
  int idet[128][48][14];
  channel ixyz[NUMSM][NUMROW][NUMCOL][NUMCH];
  int  bit_map[NWCHA];
  channel  ch[NCHAN];

  float an_zero;  
  float ref_ind;  // refractive index -average
  
  float  th_0,fi_0; 
  Hep3Vector rc_mu,rc_md;    // spherical mirrors CoC(nonsegmented)
  Hep3Vector CoCSphericalMirror[80];  // spherical mirrors CoC
  Hep3Vector rp_mu,rp_md;    // planar mirrors -position
  Hep3Vector ezp_mu,ezp_md;  // planar mirrors -normal
  float rmax2;    // square of the maximum Ch.radius 
  
  Hep3Vector  r0[14],vx[14],vy[14],vn[14];  // geometry of supermodules
  float       dvx[14],dvy[14],dxsm[14],acc_lens;
  float z_rich_min;                         // start and and of the rich
  float z_rich_max;
  
  float x_cut, y_cut, z_cut, z_magnet,p_min;
  int print();
  int    set_pmt(int type,int ism,int ir,int ic,int &ichan);
  void   read_pmt(FILE *fp1,int &ism,int &irow,int &icol,int &itp,long &line,int &ok);
  void   to_xyz(int ism,int ir,int ic,int nn,Hep3Vector &r,float &sig);  

  Hep3Vector src2xyz(int ism,int ir,int ic,int ch);
  int pmtype(int ism,int ir,int ic,int ch);
  RICHGeometry();
  ~RICHGeometry(){};

};

/* --------------------------------------------------------------------------*/
typedef struct { 
  Hep3Vector r,dir;      //! entrance point, direction
  float p ;              //! momentum
  HepDouble sig;
  int  itype;            //! particle type (unknown=0,e=1,mu=2,pi=3,k=4,p=5)
  Hep3Vector  r0;        //! mean photon emission point
  Hep3Vector r1u, r1d;   //! particle hit on (up, down) sph. mirror
  Hep3Vector diru, dird; //! direction after reflection (up, down) 
  Hep3Vector rcu, rcd;   //! ring centers (up and down detector)
  short  wm ;            //! =1 if particle hits upper sph. mirror
  float thc[NHIP];          //! expected cerenkov angles for {0,e,mu,pi,k,p,e+e-}   
  float sig_mean[NHIP];          //!  sigmas  for {0,e,mu,pi,k,p,e+e-} 
  float  stf[NHIP];         //! expected number of signal photons for {..}
  float bgr[NHIP];          //! expected number of bgr photons for {..}
  ArtePointer<RTRA> rtra;
} track_buffer;


typedef struct { 
  Hep3Vector rf;  // ! photon hits
  float  sig;     // ! angular error (sigma)
  float  utez;    // ! photon weight (channel efficiency, occupancy)
} photon_buffer;

typedef struct { 
  int ir;
  int ihit;
  int ifot;
  float utez;
  float thc;
  float w;                 //! result of iteration procedure
} richtab;
     
/* --------------------------------------------------------------------------*/
class RICHFaktor{
 public:
float bgrfak[10];
float faktor[100];
 RICHFaktor();
~ RICHFaktor(){};
};
/* --------------------------------------------------------------------------*/
class Riter {

protected:
  RICHGeometry *RICH_geo;
  RICHFaktor   *Faktor;  
  int track_selected[NDEL];  
  track_buffer  track[NDEL];
  photon_buffer foton[NFOTB];
  richtab       rich[NTB];
   
  int nhip;                  // number of hypothesis
  int mode;                  // 0 - RTRA-RPNT 1 - ECAL VDS
  int  itip_mc[NDEL];        //! particle type from montecarlo //
  int  iu[NDEL];             //! nonzero if from b
  float   plkh[NDEL][NHIP];     //! likelihood
  float   sfot[NDEL][NHIP];     //! measured number of photons
  int irm;                   //! max radius in bins
  int niter;                   // number od iterations
  int ntracks;                 // number of tracks in the event
  int nfot;                    // number of photons in the event
  float anzero_mul;            // NzeroNormalize
  float bgr_mul;               // BgrNormalize
  float pf[256][NDEL];         //! histogram (radius,track)
  float p_mass[NHIP];
  float bin;                   //! bin size

#define  nnn 100 
  float cfi[nnn], sfi[nnn];    // for exp_no_phot
  float gean_mass(int ipart);
  int   itip_gean(int ipart);
  int   itip_lund(int ipart);
  
  int   read_ini( const char *);
  Hep3Vector   toRICH( int x, int y, int z, Hep3Vector s);
  int   print_tracks_par( track_buffer *t );
  int   calc_tracks_par(  track_buffer *t );
  void  zero_iter(  int ihit, float pf[256][NDEL], int &irm );
  int   calc_thcer( photon_buffer &f, track_buffer  &t,
		    float &th_cer,    Hep3Vector &dirf);
  
  int   calc_full_thcer( photon_buffer &f,track_buffer  &t,float &th_cer,Hep3Vector &dirf);
  int   ring_tab(   int ihit,int ifot,float thc,float bin,float pf[256][NDEL],float utez,richtab *rich,int &n);
  void  ring_iter(richtab *rich,int n,float pf[256][NDEL],int nh,int nf);
  int   fill_lkh(float x,float wi,float x0,float sig,float sph,float bph,float &plkh,float &sfot);
  int  Which_SFM( Hep3Vector  &r); 
  
  float  fun_bgr(float x);
  int    presek_detector( Hep3Vector &r, Hep3Vector &dir, int kje, Hep3Vector &r1, float  &alfa,float  &beta,int &ism);
  float  cher_angle(float p,float mass,float ref_ind);
  int    exp_no_phot(float thc,float path,float theta, float a, float b, int ism,  float &stfs, float &sig_mean, float &acg);
  float  exp_no_bgr(int i,float *thc,float *sig,float *acg);
  void   oznaci_bine( float *xxx, float *yyy, int nch,float bin, short *hip, float * sig, float *th, float *acg);
  
  float  acc_the(float theta);
  void   measured_dir(track_buffer &t);
  int    part_type(float p_mass);
  int    test_hyp(float *thc,int itype,float th_cer,float dthc);
  int    cerenkov_kot(Hep3Vector &r0,Hep3Vector &dir,Hep3Vector &rh,Hep3Vector &rc,float r_mi,float &th_cer,Hep3Vector &dirf);
  int    cerenkov_kot_0( Hep3Vector &r0, Hep3Vector &dir, Hep3Vector &rh, Hep3Vector &rc,HepDouble r_mi,float  &th_cer,Hep3Vector &dirf);
  int    resi_sin(float a,float b,float c,float &x, float &y,int & iter,int max_iter);
  int    resi_sin_0(float a,float b,float c,float &x,float &y);
  // ********************** geometrical routines
  
  float  com(Hep3Vector &dir,int izb);
 void   smer(HepDouble theta,HepDouble fi,Hep3Vector &dir);
  
  void   premica(Hep3Vector &rinit,Hep3Vector &dir,Hep3Vector &rfinl,float &path,HepInt izb);
  
  void   presek_sfera( Hep3Vector  &r,Hep3Vector  &dir,HepDouble radij, Hep3Vector  &rc, Hep3Vector  &r1, Hep3Vector  &r2, int &seka);

  void  presek_shroud( Hep3Vector  &r,
                       Hep3Vector  &dir,
                       HepDouble radij, 
                       Hep3Vector  &rc, 
                       Hep3Vector  &r1, 
                       float &path, 
                       int &seka);


  void   odboj_na_sferi(Hep3Vector &r,Hep3Vector &dir,HepDouble radij,Hep3Vector &rc);
  
  // ROOT histograming
  
#ifdef RITER_DEBUG 
  int Riter::bgr_calc_parameters(int nh,int ntra,richtab *rich );
#endif
  
  // selection routines
  
  int    select_track(track_buffer *t);
  
  // ARTE interface
  int    Filltracks_MIMP1();
  int    Filltracks_MIMP();
  int    Filltracks_RCCL();
  int    Filltracks_RPNT();
  int    Filltracks_RTRA();
  int    Filltracks_RPNT2(ArtePointer<RTRA> rtra,int &itra);
  int    Fillfotons_HITB();
  void   FillRTRA();
  
  int    main(int idog,int ntra,int neve);
  // *********************** public members 
public:  
  int  CherenkovAngle(int nbins, float min, float max, float *xarray);
  int  ResetTracks();
  int  AddTrack(ArtePointer<RTRA> rtra,
		Hep3Vector &dir,float err,Hep3Vector &point, float p);
  int    Iterate();
  void    FillRTRA(int mode);
  int  Riter_evnt();
  Riter();
  ~Riter();
};
// ---------------------------------------------------------------------

inline Riter::~Riter(){
   delete RICH_geo;
   delete Faktor;

#ifdef RITER_DEBUG
 
#ifdef USE_ROOT
   rfile->Write();
   delete rfile;
#endif
#endif
}

inline float Riter::com(Hep3Vector &dir,int izb)
{
  if (izb==1) return dir.x();  
  if (izb==2) return dir.y();  
  if (izb==3) return dir.z();
  return 0;
};

void fill_ntuple_4(int iev,track_buffer *t,int iu,int itip,float *plkh,float *sfot);
void fill_ntuple_5(int ism,int icol,int irow,channel *ch);
float pois_log(float mean,float n);

#include <riter/riter.icc>
#endif 












