#include <riter/riter.hh>

#include "arte/RSEG_cmp.hh"
#include "arte/Chrono.hh"


extern "C" void rkmain_(const int    & icovar, 
			const double & zout, 
			const double & fstep, 
			const double * parameters, 
			const double * prec, 
			const double * p_track, 
			const double * derivatives, 
			const int    & ierror);




Chrono *Ritertime;

int myneve=0;

// ---------------------------------------------------------------------------
int IsInside(float x,float y)
{
  static float   th_0= 0.145;  // maximum acceptance  y direction
  static float   fi_0= 0.195;  // maximum acceptance  x  direction

  float fx =fabs(x);
  float fy =fabs(y);
 
  if  ( ( fy > th_0 ) ||  (fx > fi_0)  )  return 0;
  if  (th_0-fx*0.3 < fy) return 0;
  return 1;
}               

// ---------------------------------------------------------------------------
int  Riter::Fillfotons_HITB(){
  // fill photon structure ****************************************
  int nhitb=0;
  for(ArteTable<HITB>::iterator hitbit=ArteTable<HITB>::begin();
      hitbit!=ArteTable<HITB>::end();++hitbit) {
    ArtePointer<HITB> hitb(hitbit);
    ArteRelationIterator<MIMP> hbmimp = hitb->mimp;
    //    if (imtra->topp->lund!=211) continue; // select only pions 
 
    ArtePointer<GEDE> gede(hitb->gede); 
    if(gede->cmp==5){        // 5 = RICH 
      int id=(*hitb->digb.begin())->id;      
      int ch =  id%100; id=id/100;
      int col=  id%100; id=id/100;
      int row=  id%100; 
      int  sm=  id/100;
      //  cout  << RICH_geo->src2xyz(sm,row,col,ch); 
      //  cout << hitb->x << " " <<  hitb->y<< " " <<  hitb->z <<  endl;
      
      foton[nhitb].rf =10.*Hep3Vector(hitb->x,hitb->y,hitb->z);
      if (hitb->y>0) {
         foton[nhitb].rf=virtual_position(foton[nhitb].rf, 
					  RICH_geo->rp_mu, 
					  RICH_geo->ezp_mu);
      } else {
	foton[nhitb].rf=virtual_position(foton[nhitb].rf, 
					 RICH_geo->rp_md, 
					 RICH_geo->ezp_md);
      }  
                
      float sig  = sqrt(sqr(  RICH_geo->pmtype(sm,row,col,ch)*
                              RICH_geo->sig_pad )+ 
                        sqr( RICH_geo->sigthc  )+
                        sqr( RICH_geo->sig_sfa ));
      foton[nhitb].sig  = sig;
      foton[nhitb].utez = 1;
      nhitb++;  

    }
  }   
return nhitb;
}  
// -------------------------------------------------------------------------
#ifdef RITER_DEBUG

int  Riter::Filltracks_MIMP1(){
   int itra=0;
   // DENIS's function
  RICHrings *rich=new RICHrings;
  // loop over RICH MC particles (= at least one photon is radiated)
  

  for ( rich->begin(); rich->valid(); rich->next() ) {
     float tx=rich->tx();
     float ty=rich->ty();
     Hep3Vector r=toHep3Vector(rich->x());
     Hep3Vector mver=10*Hep3Vector(rich->vertex()->x,rich->vertex()->y,rich->vertex()->z);
     int lund=rich->topp()->lund;
     

     //     if (lund !=211) continue; // select only pions
     //     if (rich->momentum()<15) continue; // cut  15 Gev
     //     cout<< "lund:"<< lund<< r<< mver <<  endl;

     //      if  (fabs(ty)> RICH_geo->th_0) continue;
     //      if  (fabs(tx)> RICH_geo->fi_0) continue;
      if  (mver.z()> RICH_geo->z_rich_min) continue; // particle should be 
     itip_mc[itra]=itip_lund(lund);
 
     track[itra].dir = Hep3Vector(tx,ty,1).unit();
     track[itra].r    = 10.*r+((RICH_geo->z_rich_min-10*r.z())/track[itra].dir.z())* track[itra].dir;
    
     track[itra].p=min(1000,rich->momentum()); 
     track[itra].itype =0;    //  ???? particle type from MC
     track[itra].sig   = 0;      
     //measured_dir(track[itra]);

     mtuple4->column("iev",myneve);	
     mtuple4->column("rx",track[itra].r.x()  );	
     mtuple4->column("ry",track[itra].r.y()  );
     mtuple4->column("rz",track[itra].r.z()  );
     mtuple4->column("kx",track[itra].dir.x()  );	
     mtuple4->column("ky",track[itra].dir.y()  );
     mtuple4->column("kz",track[itra].dir.z()  );
     mtuple4->column("p", track[itra].p );
     mtuple4->column("itip", itip_mc[itra]);
     mtuple4->dumpData() ; //   fill ntuple 
 

    if (select_track(track)) itra++;
  }
  delete rich;  
  cout <<    itra << "MIMP1  =>" << endl;
  return itra;
}
#endif

// ---------------------------------------------------------------------------

int  Riter::Filltracks_MIMP(){  
  return 0;
}  
// -------------------------------------------------------------------------
float GetGlobalAlignmentCorrection(int i){

  for( ArteTable<ACGA>::iterator acga = ArteTable<ACGA>::begin();
        acga != ArteTable<ACGA>::end(); ++acga ) {
    if ((*acga)->cmp == 67108864){
      if (i)
	return (*acga)->dty;
      else
	return (*acga)->dtx;
    };
  };
  return 0;
}


// -------------------------------------------------------------------------
int Riter::Filltracks_RPNT2(ArtePointer<RTRA> aprtra,int &itra)
{

//-------------------------------------------------------------------
//   get global alignment correction
//-------------------------------------------------------------------
  static float dtyGlobal =  GetGlobalAlignmentCorrection(1);
  static float dtxGlobal =  GetGlobalAlignmentCorrection(0);

//-------------------------------------------------------------------

  static  float zmin= 1001; 
  //look for RPNT closer to the middle of RICH
  ArtePointer<RPNT> closerpnt;
  for(ArteRelationIterator<RPNT> rpntit=aprtra->rpnt.begin();
      rpntit!=aprtra->rpnt.end();rpntit++)
    {
      if( ((*rpntit)->zf > closerpnt->zf) && 
          ((*rpntit)->zf < zmin) )	closerpnt=*rpntit;
    }
  Hep3Vector point(closerpnt->xf,closerpnt->yf,closerpnt->zf);

  float txf= closerpnt->txf - dtxGlobal;
  float tyf= closerpnt->tyf - dtyGlobal;

  if((closerpnt->zf>300)&& IsInside( txf , tyf ) ){
    Hep3Vector  dir = Hep3Vector(txf,tyf,1).unit();
    track[itra].dir = dir;
    track[itra].r    = 10.*point+ (( zmin-10*point.z()) / dir.z())*dir; 
    if (closerpnt->pf!=0)    
      track[itra].p=min(1000,fabs(1/closerpnt->pf)); 
    else 
      track[itra].p=0;          
    
    ArtePointer<MTRA> imtra(aprtra->mtra.begin());
    itip_mc[itra]=itip_lund(imtra->topp->lund);
    
    track[itra].rtra  = aprtra;
    track[itra].sig   = sqrt(closerpnt->cvf[5]+closerpnt->cvf[9]); 

    itra++;
    return 1; 
  }
  return 0;
}

// -------------------------------------------------------------------------

int  Riter::Filltracks_RPNT(){
  int itra=0;
  // fill track structure **************************************
  int alltracks=0;
  for(ArteTable<RTRA>::iterator rtra1it=ArteTable<RTRA>::begin();
      rtra1it!=ArteTable<RTRA>::end();++rtra1it) {
    ArtePointer<RTRA> rtra(rtra1it);
    if (rtra->fit) continue;     
    if (Filltracks_RPNT2(rtra,itra)) track_selected[alltracks]=1;
    else                             track_selected[alltracks]=0;
    alltracks++;         
  }  
  // ************************************************************
#ifdef RITER_DEBUG
  cout <<    itra << "RICH tracks " << alltracks<< "all tracks"<<endl;
#endif
  return itra;
}

// -------------------------------------------------------------------------

int  Riter::Filltracks_RTRA(){
  int itra=0;
  for(ArteTable<RTRA>::iterator rtrait=ArteTable<RTRA>::begin();
      rtrait!=ArteTable<RTRA>::end();rtrait++)
    {
      double zout;

      ArteRelationIterator<RCCL> rccl     = (*rtrait)->rccl.begin();
      
      if ( rccl != (*rtrait)->rccl.end()) {
	zout = double((*rccl)->z);
      } else {
	zout = 800.;
      };
      

      const int    icovar=2;    
      double       prec[4];
      double       derivatives[5][5]={};
      double       p_track[6], parameters[6];
      double       fstep=50.;
      
      parameters[0] = double((*rtrait)->zf);
      parameters[1] = double((*rtrait)->xf);
      parameters[2] = double((*rtrait)->yf);
      parameters[3] = double((*rtrait)->txf);
      parameters[4] = double((*rtrait)->tyf);
      parameters[5] = double((*rtrait)->pf);


   //
   // tracing returns derivatives, p_track
   //
      if (zout != parameters[0]) {
	
	int    ierror;
	
	prec[0] = 0.001/fabs(zout-parameters[0]);
	prec[1] = prec[0];
	prec[2] = 0.0001/fabs(zout-parameters[0]);
	prec[3] = prec[2];
	
      
	rkmain_(icovar, zout, fstep, parameters, prec, p_track, 
		& derivatives[0][0], ierror);
	
	
	if (ierror == 0) continue;// ?????
      }
      
      Hep3Vector point(p_track[1],p_track[2],p_track[0]);
      
      if(p_track[0]==0 && p_track[1]==0 && p_track[2]==0 ) 
	continue;
      
      Hep3Vector direction(p_track[3],p_track[4],1);
      direction=direction.unit();
      
      float momentum=fabs(p_track[5]);
      float err=derivatives[2][2]+derivatives[3][3];
      if (AddTrack((*rtrait),direction, err,point,momentum)) itra++;
    }
  
  return itra;
}

// -------------------------------------------------------------------------

int  Riter::Filltracks_RCCL(){
  int itra=0;
  
  // fill track structure **************************************
  for(ArteTable<RCCL>::iterator rtra1it=ArteTable<RCCL>::begin();
      rtra1it!=ArteTable<RCCL>::end();++rtra1it) {
    
    ArtePointer<RCCL> rtra(rtra1it);
      // ************************************************************
      // track selection is done here
      // ************************************************************
      if  (fabs(rtra->y/rtra->z)> RICH_geo->th_0) continue;
      if  (fabs(rtra->x/rtra->z)> RICH_geo->fi_0) continue;
      Hep3Vector drx = Hep3Vector(rtra->x/rtra->z,rtra->y/rtra->z,1).unit();
      track[itra].dir = drx;
      track[itra].r    = 10.*Hep3Vector(rtra->x,rtra->y,rtra->z)+
	((RICH_geo->z_rich_min-10*rtra->z)/drx.z())* drx;
      if (rtra->esum!=0)    track[itra].p=min(1000,rtra->esum); 
      else track[itra].p=0;          
      track[itra].itype = 0;    //  ???? particle type from MC
      track[itra].sig   = sqrt(rtra->cve[5]+rtra->cve[9]); 

      itra++;
      // ************************************************************
  
  }
#ifdef RITER_DEBUG
  cout <<    itra << "clusters found  =>" << endl;
#endif
  return itra;
}

// ---------------------------------------------------------------

void Riter::FillRTRA() {
  for (int itra=0;itra<ntracks;itra++) { 
      float sum=0;
      float max=plkh[itra][0];
      for (int i=0;i<6;i++) if (plkh[itra][i]>max ) max=plkh[itra][i];
      for (int i=0;i<6;i++) plkh[itra][i]=exp(plkh[itra][i]-max);
      for (int i=0;i<6;i++) sum+=plkh[itra][i];
      if (sum!=0) sum=1/sum;  else sum=0;
      track[itra].rtra->lre =plkh[itra][1]*sum;
      track[itra].rtra->lrmu=plkh[itra][2]*sum;
      track[itra].rtra->lrpi=plkh[itra][3]*sum;
      track[itra].rtra->lrk =plkh[itra][4]*sum;
      track[itra].rtra->lrp =plkh[itra][5]*sum;
      track[itra].rtra->flr=1;
      itra++;
  } 
}
// ---------------------------------------------------------------

void Riter::FillRTRA(int mode) {
  
  for (int itra=0;itra<ntracks;itra++) { 
    float sum=0;
    float max=plkh[itra][0];
    for (int i=0;i<6;i++) if (plkh[itra][i]>max ) max=plkh[itra][i];
    for (int i=0;i<6;i++) plkh[itra][i]=exp(plkh[itra][i]-max);
    for (int i=0;i<6;i++) sum+=plkh[itra][i];
    if (sum!=0) sum=1/sum;  else sum=0;
    track[itra].rtra->lrpi=plkh[itra][1]*sum;
    track[itra].rtra->lrk =plkh[itra][3]*sum;
    track[itra].rtra->lrp =(plkh[itra][2]+plkh[itra][4]+plkh[itra][5])*sum;
  }
}
// --------------------------------------------------------------------------
int  Riter::Riter_evnt(){

  int itra=0;

  if (!mode)  itra=Filltracks_RPNT();
  else itra=Filltracks_RTRA();
  int nhitb=Fillfotons_HITB();
#ifdef RITER_DEBUG
   cout << "RITER : Ntracks :" << itra << endl;
   cout << "RITER : Nhits :" << nhitb << endl;
   if (!Ritertime) {
     Ritertime=new Chrono("Riter"); 
   } 
   Ritertime->start();
#endif
   if (itra) main(nhitb,itra,myneve);
#ifdef RITER_DEBUG

   Ritertime->stop();
   cout << myneve << " RITER : Average time :" << Ritertime->averageTime() << endl;
#endif

  if (!mode) FillRTRA(); else FillRTRA(1);
  return RITER_OK;
};

int  Riter::AddTrack(ArtePointer<RTRA> rtra,
  Hep3Vector &dir,float err,Hep3Vector &point, float p){

  register float zmin= RICH_geo->z_rich_min;
  if((point.z()>300)&& IsInside(dir.x(),dir.y() ) ) {
    track[ntracks].dir = dir;
    track[ntracks].r    = 10.*point+ ((zmin-10*point.z())/dir.z())*dir; 
    track[ntracks].p=p;
    itip_mc[ntracks]=0;
    track[ntracks].sig   = sqrt(err); 
    track[ntracks].rtra=rtra;
    ntracks++;
    return 1; 
  }
  return 0;
}


int  Riter::ResetTracks(){
  ntracks=0; 
  return RITER_OK;
}

int  Riter::Iterate(){
  int nhitb=Fillfotons_HITB();
  if (ntracks) main(nhitb,ntracks,myneve);
  myneve++;
  return RITER_OK;
}

int  Riter::CherenkovAngle(int nbins, float min, float max, float *xarray){
  int ntra  = Filltracks_RPNT();
  int nhitb = Fillfotons_HITB();
  for  (int itra=0;itra<ntra;itra++) {
    calc_tracks_par(&track[itra]);
  }
  Hep3Vector dirf;
  int bin=0;
  float th_cer=0;
  float dx=(max - min) /nbins;
  for  (int itra=0;itra<ntra;itra++){
    for  (int ifot=0;ifot<nhitb;ifot++){
      if (foton[ifot].utez>0) {
	if (calc_full_thcer(foton[ifot],track[itra],th_cer,dirf)!=RITER_OK) 
	  continue;
	
	if ((th_cer>=max) || (th_cer<min)) continue;
	bin= int((th_cer-min) / dx);
        xarray[bin]++;
      }
    }
  }

  return nbins;
}


Riter *reco_Riter;


extern "C" {
  void Riter_init_(){

  float z_rich_min=0, z_rich_max=0;

  //find vessel dimensions from ARTE tables.
  for(ArteTable<GESL>::iterator geslit=ArteTable<GESL>::begin();
      geslit!=ArteTable<GESL>::end();geslit++){
    if((*geslit)->cmp==5){
      z_rich_min=(*geslit)->z1;
      z_rich_max=(*geslit)->z2;
      break;
    }
  }          
  cout << "_______________________________________________\n";
  cout << " z_rich_min " << z_rich_min;
  cout << " z_rich_max " << z_rich_max << endl;

     
  }
  
  void Riter_evnt_(){
#ifdef RITER_DEBUG
    cout << "RITER start_evnt" << endl;
     char tmpc[10];
     sprintf(tmpc,"%d",myneve);
     if (myneve < MAXHIST)  
     hRICH[myneve]=new   HBookHistogram(tmpc,200,-.25,.25,200,-.25,.25,1000+myneve); 
 
#endif
     //    printf("reco_Riter=%d\n",reco_Riter); fflush(stdout);
     if  (!reco_Riter )  {
       cout << "RITER initialisation \n" ;
       reco_Riter=new Riter();
     } 
     reco_Riter->Riter_evnt();
     myneve++;
     
     
#ifdef RITER_DEBUG
    cout << "RITER end_evnt" << endl;
#endif
  }
  
  void Riter_stop_(){
   delete reco_Riter;
  }

  int CherenkovAngle(int nbins, float min, float max, float *xarray){
    if  (!reco_Riter )  {
      cout << "RITER initialisation \n" ;
      reco_Riter=new Riter();
    } 
    return reco_Riter->CherenkovAngle(nbins, min, max, xarray);
  }
  

}
  
	   







