#include <stdio.h>
#include <arte/arte.hh>

#include "grr.h"

#include "aframelib.h"
#include "holmes.h"

// -----------------------------------------------------
//
//  Get target position from GTAR bank
//
//  Return: Number of targets found
//
// -----------------------------------------------------
int HolmesGetTargets( float *xtarget[], float *ytarget[], float *ztarget[] )
{
  char *routine = "HolmesGetTargets";

  // target positions
  static float target_x[8];
  static float target_y[8];
  static float target_z[8];
  int          ntarget = 0;

  // if iol target x/y/z parameter are given, take target spot from there...
  if (   holmes_iol.target_x.value < 10. && 
	 holmes_iol.target_y.value < 10. &&
	 holmes_iol.target_z.value < 10. ) {
    target_x[0] = holmes_iol.target_x.value;
    target_y[0] = holmes_iol.target_y.value;
    target_z[0] = holmes_iol.target_z.value;
    ntarget = 1;
  } else
    // Get target positions from the GTAR bank
    // (accept target if closer as 10 mm to the beam axis)
    for( ArteTable<GTAR>::iterator iGTAR = ArteTable<GTAR>::begin();
	 iGTAR != ArteTable<GTAR>::end(); iGTAR++ ) {
      ArtePointer<GTAR> gtar = *iGTAR;
      if ( gtar->x * gtar->x + gtar->y * gtar->y < 1.0 && ntarget < 8 ) {
	target_x[ntarget] = gtar->x;
	target_y[ntarget] = gtar->y;
	target_z[ntarget] = gtar->z;
	ntarget++;
      }
    }

  if ( holmes_iol.debug_mode.value ) {
    aframe_mess(routine,GRR_SINFO,"Target Positions (%d targets):", ntarget);
    for ( int i=0; i<ntarget; i++ ) {
      aframe_mess(routine,GRR_SINFO, "   (x,y,z) = %f, %f, %f",
	       target_x[i], target_y[i], target_z[i]);
    }
  }
  
  // check if we have a target at all
  if ( ntarget == 0 )
    aframe_mess(routine,GRR_SWARNING,"No target spot given!");

  *xtarget = target_x;
  *ytarget = target_y;
  *ztarget = target_z;

  return ntarget;
}
