//
// Some startup procedures which have to be executed once in the beginning
// of an ARTE job. This was previously part of SvdEvt().
//
// T. Glebe, September 1999
// 

// some Arte stuff
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "silarte/silarte.hh"
#include "silarte/EVinfo.hh"

// return codes
#include "grr.h"

// sparse event definition
#include "event.h"
#include "etb.h"

// prototype declarations
#include "aframelib.h"
#include "vdsleda.h"
#include "silraw.h"
#include "marvin.h"
#include "vdsrhp.h"

int SvdStartup() {

  // protection against multiple calls
  static bool firsttime = true;

  if(!firsttime)
    return GRR_OK;
  else
    firsttime = false;

  //
  // No SVD Digitization in case of MC  
  //
  if ( CheckMC() )
    return GRR_OK;

  //
  // REAL DATA
  //

  char *routine = "SvdStartup";

  // get event pointer
  EVENT_HDR *event = ARTE_get_event();
  if ( !event ) {
    aframe_mess(routine,GRR_SERROR,"Could not find event!");
    return GRR_FISHY;
  }      

  //    Get constants from database
  //
  // Time stamp is evaluated in this order:
  //  1. date/time   from vdsleda IOL
  //  2. major/minor from vdsleda IOL
  //  3. event time from DDAQ (if all IOL parameters are =0)
  int major = 0;
  int minor = 0;
  int ndaq;

  EVT_DATA_RECORD daq;
  ETB_get_daqs( event, 1, &daq, &ndaq );
  if ( ndaq > 0 ) {
    major = daq.data.event_time;
    minor = 0;
  }

  int rc = vdsleda_startprogram( major, minor );
  if ( rc != GRR_OK )
    return rc;
  
  // start silraw
  rc = silraw_startprogram();
  if ( rc != GRR_OK )
    return rc;
  
  // start marvin
  marvin_init();
  
  // vdsrhp init
  rc = vdsrhp_init();
  if ( rc != GRR_OK )
    return rc;
  
  // fill GEDE table with geometry from file
  rc = silarte_updateGEDE();
  if ( rc != GRR_OK )
    return rc;

  return GRR_OK;

}
