#ifndef SILARTE_HH
#define SILARTE_HH

class GEDE_class;
class HOLMES_class;

#ifdef __cplusplus
extern "C" {
#endif

/* include iol */
#include "iol.h"

typedef struct {
  IOL_MNAME  module[IOL_MLEN];       /* The module name must be first        */
  IOL_SIZE   size            ;       /* The number of verbs must be second   */
  IOL_STRING version         ;       /* version of this package              */
  IOL_FLOAT  hit_resolution  ;       /* hit resolution                       */
  IOL_FLOAT  efficiency      ;       /* average hit efficiency (for MC gen.) */
  IOL_FLOAT  double_hit_resolution;  /* double hit resolution  (for MC gen.) */
  IOL_FLOAT  noise_rate      ;       /* rate of noise hits     (for MC gen.) */
  IOL_FLAG   debug_mode      ;       /* debug mode                           */
} Silarte_Iol_t;
extern Silarte_Iol_t silarte_iol;

/* define maximum allowed number of clusters */
#define DIM_NCLMX   1000


/* Subroutine templates */
int silarte_param_init();


/* sparsification for orte */
int sparseDSVDinit();
int sparseDSVD( int* src_recid,  char *srcdat,
		int* dest_recid, char *destdat, int srclength );
int sparseDSVDstop();


#ifdef __cplusplus
	   }
#endif

/* C++ subroutine templates */
int  fillHITBfromMIMP();
int  fillHITBfromSPARSEVT();
int  fillSPARSEVTfromHITB();
int  fillSPARSEVTfromDSVD();
int  fillSILRAWfromDSVD();
int  silarte_updateGEDE();
int  silarte_returnGEDE( char* slname, int lay, int sec );

/* user calls from ARTE */
int  SvdInit();
int  SvdEvnt();
int  SvdStop();
int  SvdStartup();

/* initialization */
int  SvdIolInit();

/* user call to holmes */
int         HolmesInit();
int         HolmesEvent();
int         HolmesGetTargets( float *xtarget[], float *ytarget[], float *ztarget[] );
void        HolmesFillHITBfromARTE( HOLMES_class *HOLMESp );
GEDE_class *HolmesFillGEDEfromARTE();

/* user calls to alignment code */
int  SvdAlignEvent();
int  SvdDoAlign();
int  SvdAlignStop();

/* user calls to efficiency code */
int  SvdEffEvent();
int  SvdEffStop();

/* connection to VDS database */
int silarte_store_VDS_Side ( int major, int minor, int flag );
int silarte_update_VDS_Side( int major, int minor, int flag );
int fillVDSSidefromGEDE    ( int major, int minor, int flag );

#endif
