#include <list.h>
#include <stdio.h>

#include "L3_Parameter.hh"
#include "L3_SLP.hh"
#include "L3_Vtx.hh"
#include "L3_Origin.hh"

#ifdef L3_ARTE_FRAME
#include "L3_Arte.hh"
#endif

#ifdef L3_SLP_FRAME
#include "L3_VTstruct.hh"
#endif

vector<L3_Track>::iterator L3_Origin::BestMatch(L3_Track &tofind,vector<L3_Track> &tr,float *dist)
{
  
  float dummy;
  if(dist==NULL) dist = &dummy;
  vector<L3_Track>::iterator result = tr.end();
  *dist = -1;
  for(vector<L3_Track>::iterator i=tr.begin();i!=tr.end();i++)
    {
      float newdist = TrackDist(*i,tofind);
      if( *dist<0 || newdist<*dist )
	{
	  *dist = newdist;
	  result = i;
	}
    }
  return result;
}

const int ntmin = 3;        //at least three tracks on a primary vertex
const float scale = 0.024;   //scale is 240 microns

class TrkInfo {
public:
  TrkInfo(L3_Track* a,float longi,float verti) : t_track(a),t_longi(longi),t_verti(verti) {}
  TrkInfo() : t_track(NULL),t_longi(0.0),t_verti(0.0) {}    
  TrkInfo(const TrkInfo &a){ operator=(a); }
  TrkInfo& operator = (const TrkInfo &a)
    {
      t_track = a.t_track;
      t_longi = a.t_longi;
      t_verti = a.t_verti;
      return *this;
    }
  float longi() const { return t_longi; }
  float verti() const { return t_verti; }
  L3_Track* track() const { return t_track; }
private:
  L3_Track* t_track;
  float t_longi,t_verti;
};

ostream& operator<<(ostream &o,const TrkInfo &a)
{
  //o << a.longi() << ',' << a.verti() << '[';
  o.form("(%.0f,%.0f)",a.longi()*1E4,a.verti()*1E4);
  o << '[';
  if(a.track()==NULL) o << "NULL"; else o << a.track()->nr();
  o << ']';
  return o;
}

class CL_lessLongi : public binary_function< TrkInfo, TrkInfo, bool > {
public:
  inline
  bool operator() (const TrkInfo& R , const TrkInfo& S) const {
    return ( R.longi() < S.longi() ); 
  }
};

class Cluster {
public:
  Cluster(){}
  Cluster(int p) : target(p) {}
  Cluster(const Cluster &a){ operator=(a); }
  void add(const TrkInfo &a){ data.push_back(a); }
  unsigned int Ntracks() const { return data.size(); }
  Cluster& operator=(const Cluster& a)
    {
      data = a.data;
      target = a.target;
      return *this;
    }
  vector<TrkInfo> getUniqueTracks(const Cluster &a) const {  
    vector<TrkInfo> result;
    for(vector<TrkInfo>::const_iterator i = data.begin();i!=data.end();i++) 
      {
	L3_Track *tr1 = i->track(); 
	int exists = 0;
	for(vector<TrkInfo>::const_iterator ia = a.data.begin();ia!=a.data.end();ia++) 	
	  {
	    L3_Track *tr2 = ia->track();
	    if( (void*)tr1 == (void*)tr2 ){ exists=1; break; }
	  }
	if(!exists) result.push_back( *i );
      }
    return result;
  }
  vector<TrkInfo> data;  
  int target;
};

class CL_greater : public binary_function< Cluster, Cluster, bool > {
public:
  inline
  bool operator() (const Cluster& R , const Cluster& S) const {
    return ( R.Ntracks() > S.Ntracks() ); 
  }
};

ostream& operator<<(ostream &o,const Cluster &a)
{
  o << L3_Geometry::getTarget(a.target).name() << ' ' << a.data.size() << " : ";
  for(vector<TrkInfo>::const_iterator i = a.data.begin();i!=a.data.end();i++) 
    {
      o << *i << ' ';
    }
  o << endl;
  return o;
}

void DoCluster(vector<TrkInfo>& List,int start,int end,vector<Cluster> &result,int target)
{
  //cout << "DoCluster " << start << " .. " << end << endl;
  float maxDiff = (List[end].longi() - List[start].longi() + scale ) * 10000;
  float diff = maxDiff;
  int von = 0;
  for(int k=start;k<end;k++)
    {
      float d = List[k+1].longi() - List[k].longi();
      if( d<diff )
	{
	  von = k;
	  diff = d;
	}
    }
  if( diff<scale )
    {
      int bis = von+1;
      do {
       float low = maxDiff;
       float high = maxDiff;
       if( von-1>=start )
	 {
	   low = List[von].longi()-List[von-1].longi();
	   //cout << "low " << low << endl;
	 }
       if( bis+1<=end ) 
	 {
	   high = List[bis+1].longi()-List[bis].longi();
	   //cout << "high " << high << endl;
	 }
       //cout << "low=" << low << ",  high=" << high << ",  diff=" << diff << endl;
       if( low<=high ){
	 if(diff + low < scale ){
	   diff += low;
	   von--;
	 } else 	   
	   break;	 
       } else {
	 if(diff + high < scale ){
	   diff += high;
	   bis++;
	 } else break;
       }       
      } while(1);
      if( bis-von+1 >=ntmin )
	{
	  Cluster c(target);
	  //cout << "Cluster ";
	  for(int h=von;h<=bis;h++)
	    {
	      //cout << h << '=' << List[h].longi() << ' ';
	      c.add(List[h]);
	    }
	  //cout << endl;	  
	  result.push_back(c);
	}
      if( start < von-1 ) DoCluster(List,start,von-1,result,target);
      if( bis+1 < end   ) DoCluster(List,bis+1,end,result,target);
    }
}
 
void L3_Origin::MYreco(vector<L3_Track> &tr)
{
  if( tr.size() < 1 ) return;
  int N = L3_Geometry::nTarget();
  vector<Cluster> clusters;

  for(int i=0;i<N;i++) //loop over all wires 
    {      
      vector<TrkInfo> cand_list;
      const L3_Target &targ = L3_Geometry::getTarget(i);
      cout << "Target " << targ.name() << ' ' << targ.z() << ' ' << endl;
      for(vector<L3_Track>::iterator tra = tr.begin();tra!=tr.end();tra++)      
	{	  	  
	  tra->kalTransport(targ.z());
	  HepVector d(3);
	  d[0] = tra->x(); 
	  d[1] = tra->y();
	  d[2] = targ.z();
	  targ.distance(d);
	  float d0 = fabs(d[0]);
	  float d1 = fabs(d[1]);
	  int take = 0;
	  float v=0.0,h=0.0;
	  if( targ.dx() > targ.dy() ){  //horizontal wire
	    if( d0<targ.dx() && d1<L3_Parameter::SearchBandSize() ){ take = 1; h=d[0]; v=d[1]; }
	  } else { 
	    if( d1<targ.dy() && d0<L3_Parameter::SearchBandSize() ){ take = 1; h=d[1]; v=d[0]; }
	  }
	  if(take)
	    {
	      if(L3_Parameter::Info())
		cout << " " << tra->nr() << ' ' << d[0] << ' ' << d[1] << ' ' << h << endl;
	      //cout << " " << tra->MinDistanceToTarget(targ).mag() << " away from target" << endl;
	      cand_list.push_back( TrkInfo(&(*tra),h,v) );
	    }	
	}
      //do the clustering on this wire
      sort(cand_list.begin(),cand_list.end(),CL_lessLongi());      
      int n = cand_list.size();
      if(n-1>0) DoCluster(cand_list,0,n-1,clusters,i);
    }  
  //sort cluster according to their size, so large clusters are more likely to rule out small clusters
  sort(clusters.begin(),clusters.end(),CL_greater());
  if( L3_Parameter::Info() ){
    cout << "*** candidates ***" << endl;
    for(vector<Cluster>::iterator i = clusters.begin();i!=clusters.end();i++)
      {
	cout << *i;
      }  
  }
  //look for clusters which share too many tracks  
  {
    int N = clusters.size();
    for(int i=0;i<N-1;)
      {
	for(int k=N-1;k>i;)
	  {
	    if( clusters[i].target == clusters[k].target )  //do not compare clusters on the same wire
	      {
		k--;
		if( k<=i ){ i++; k=i; }
		continue;
	      }
	    vector<TrkInfo> List1 = clusters[i].getUniqueTracks(clusters[k]);
	    vector<TrkInfo> List2 = clusters[k].getUniqueTracks(clusters[i]);
	    int take1=1,take2=1;	    
	    if( List1.size() < clusters[i].Ntracks() )  //not all tracks unique
	      {
		vector<Cluster> res;
		if(List1.size()>1) DoCluster(List1,0,List1.size()-1,res,-1);
		if( res.size()<1 ) take1=0;
	      }
	    if( List2.size() < clusters[k].Ntracks() )
	      {
		vector<Cluster> res;
		if( List2.size()>1 ) DoCluster(List2,0,List2.size()-1,res,-1);
		if( res.size()<1 ) take2=0;
	      }
	    if( take1==0 && take2==0 )
	      {
		//if both cannot survive we take the one with more tracks
		//(take both if same number of tracks)
		//MAYBE this is NOT A GOOD IDEA for several IA in the detector - to be studied
		if( clusters[i].Ntracks() > clusters[k].Ntracks() )
		  take1=1;
		else if( clusters[k].Ntracks() > clusters[i].Ntracks() )
		  take2=1;	     
	      }	
	    //cout << "take1=" << take1 << ", take2=" << take2 << endl;
	    if( take1==take2 ){  //take both or both failed	      
	      k--;
	      if( k<=i ){ 
		i++;  //next outer
		k=i;  //leave inner loop
	      }
	    } else if(take1){	 
	      clusters.erase(clusters.begin()+k);
	      N--;
	      //k remains as it is
	      break;
	    } else if(take2){	 
	      clusters.erase(clusters.begin()+i);
	      N--;
	      //i remains as it is
	      break;//leave loop
	    } else {
	      cout << "LOGICAL MISTAKE" << endl;
	    }
	  }
      }
  }  
  if(L3_Parameter::Info()) cout << "*** found vertices ***" << endl;
  for(vector<Cluster>::iterator i=clusters.begin();i!=clusters.end();i++) 
    {
      const int tindx = i->target;  //target index
      const L3_Target& target = L3_Geometry::getTarget(tindx);
      if(L3_Parameter::Info()) cout << *i;
      PCovMatrix m;
      for(int x=0;x<3;x++)
	{
	  m.a[x][x] = 4*vtwire_.cw[tindx].a[x][x];
	  for(int y=0;y<x;y++)
	    {
	      m.a[x][y] = m.a[y][x] = 0.0;
	    }
	}
      Hep3Vector guess = target.get3center();
      L3_Vtx vtx(guess,m);
      for(vector<TrkInfo>::iterator ii = i->data.begin();ii!=i->data.end();ii++)
	{
	  vtx.add( *(ii->track()) );
	}	    
      vtkalm_.smooth[ vtx.Nr()-1 ] = 1;
      if( vtx.fit() == 0 ) 
	{
	  if(L3_Parameter::Info()) 
	    cout << "vertex with " << vtx.nTrack() << " tracks is (before smoothing) " << vtx.vertex() << endl;
	  vtx.smooth(14);		
	  if( vtx.nTrack() >= ntmin ) {
	    if(L3_Parameter::Info()) cout << "vertex with " << vtx.nTrack() << " tracks is (after  smoothing) " << vtx.vertex() << endl;
	    primVertexListNew.push_back( L3_Vertex(vtx) );
	  } else {
	    if(L3_Parameter::Info()) cout << "Vertex deleted" << endl;
	  }
	}	    
    }
}

void L3_Origin::SLTreco(vector<L3_Track> &l3tracks,vector<L3_Track> &l2tracks)
{
  //Hep3Vector primVertex = primVertexList[0].vertex();
  //const L3_Target& target = L3_Geometry::ClosestTarget( primVertex );
  for(vector<L3_Track>::iterator q = l2tracks.begin();q!=l2tracks.end();q++)
    {
      if(L3_Parameter::Info()) cout << "L2 track " << q->nr() << endl;
      int added = 0;
      L3_Vtx vtx;
      for(vector<L3_Track>::iterator q3 = l3tracks.begin();q3!=l3tracks.end();q3++)      
	{
	  float diff = q3->TrackDistance(*q).mag();
	  if( diff < 0.06 ){ 
	    vtx.add(*q3); 
	    added++; 	    
	    if(L3_Parameter::Info()) cout << q3->nr() << ' ' << diff << endl;
	  }
	}
      if(L3_Parameter::Info()) cout << added << " of " << l3tracks.size() << " tracks to vertex" << endl;
      if( added>0 ){
	vtx.add(*q);
	if( vtx.fit() == 0 )
	  {	      
	    if(L3_Parameter::Info()) {
	      cout << " vertex is " << vtx.vertex() << endl;
	      //cout << "vertex with " << vtx.nTrack() << " tracks is (before smoothing) " << vtx.vertex() << endl;
	    }
	    if( vtx.nTrack() >= 2 ) {
	      if(L3_Parameter::Info()) cout << "vertex with " << vtx.nTrack() << " tracks is " << vtx.vertex() << endl;
	      vtx.associate(*q);
	      addTracks(vtx,l3tracks);
	      secVertexList.push_back( L3_Vertex(vtx) );
	    } else {
	      if(L3_Parameter::Info()) cout << "Vertex deleted" << endl;
	    }
	  }	    
      }
    }
}

#ifdef L3_ARTE_FRAME
void showPrimVertices()
{
   int n=1;
   for (ArteTable<MVER>::iterator mi=ArteTable<MVER>::begin(); mi!=ArteTable<MVER>::end(); mi++ ) {    
     ArtePointer<MVER>& m = *mi;
     if (m->flag == 1) {
       const char *name = L3_Geometry::ClosestTarget(Hep3Vector(m->x,m->y,m->z)).name();
       printf("%i\t%.4f\t%.4f\t%.4f , %s\n",n,m->x,m->y,m->z,name);
       n++;
     }
   }
}
#endif

//we get a copy of the track array here
void L3_Origin::reconstruct(vector<L3_Track>& l3tracks, L3_Timer& Timer) {
#ifdef L3_ARTE_FRAME
  if(ArteTable<MVER>::booked()) showPrimVertices();
#endif
  Timer.start_Vertex();  
  vector<L3_Track> l2tracks = L3_SLP::getL2Tracks();
  if(L3_Parameter::Trace()) cout << "# of tracks from L2: " << l2tracks.size() << endl;
  if(L3_Parameter::Trace()) cout << "# of tracks from L3: " << l3tracks.size() << endl;    
  {
    int nextID = l3tracks.size() + 1;    
    int extra = 0;
    for(vector<L3_Track>::iterator i = l2tracks.begin();i!=l2tracks.end();i++)
      {
	i->kalTransport(0);  //vt uses z=0 as reference plane
	float d;     
	vector<L3_Track>::iterator p = BestMatch(*i,l3tracks,&d);
	if( p!=l3tracks.end() && d<0.001 ){ //good match
	  //cout << "Match for SLT track " << d << endl;
	  //cout << "SLT-track:" << endl << *i << endl;
	  //cout << "TLT-track:" << endl << *p << endl;
	  i->setnr( p->nr() );
	  l3tracks.erase( p );
	} else {
	  //no match for this l2 tracks -> give it the next number
	  i->setnr( nextID++ );
	  extra++;
	}
      }  
    if( l3tracks.size()+l2tracks.size() > (unsigned int)ntrtot ){
      int max = ntrtot;
      for(vector<L3_Track>::iterator i=l2tracks.begin();i!=l2tracks.end();i++){
	if( i->nr() > max ) i->setnr(max--);
      }
      for(vector<L3_Track>::iterator i=l3tracks.begin();i!=l3tracks.end();){
	if( i->nr() > max )
	  l3tracks.erase(i);
	else
	  i++;
      }
    }
  }
  L3_Vtx::NewEvent();
  for(vector<L3_Track>::iterator q = l3tracks.begin();q!=l3tracks.end();q++){  
    L3_Vtx::toVt(*q);
  }  
  for(vector<L3_Track>::iterator q = l2tracks.begin();q!=l2tracks.end();q++){
    L3_Vtx::toVt(*q);
  }
  VTreco(l3tracks,l2tracks);
  {
    //vector<L3_Track> all = l3tracks + l2tracks;    
    //MYreco(all);
  }

  //if(primVertexList.size()==1)   
  SLTreco(l3tracks,l2tracks);    
  Timer.stop_Vertex();
}

void showVertex(L3_Vtx &vtx)
{
  //display on screen
  printf("%i\t%.4f\t%.4f\t%.4f\t, %i tracks={ ",vtx.Nr(),vtx.x(),vtx.y(),vtx.z(),vtx.nTrack());
  for(int i=0;i<vtx.nTrack();i++)
    {
      int trnr = vtx.TrackNr(i);
      cout << trnr << ' ';
    }
  cout << '}' << endl;
}

void L3_Origin::init()
{
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Origin::init() has been called" << endl;
#endif
  int i;
  for(i=0;i<L3_Geometry::nTarget();i++)
    {
      //#ifdef L3_ARTE_FRAME      
      //int n = i+1;
      //vtwir_(n);
      //if( n!=i+1 ) cout << "strange *** " << endl;
      //#endif
      //#ifdef L3_SLP_FRAME
      const L3_Target &targ = L3_Geometry::getTarget(i);
      int verti = 0; //assume horizontal wire
      const double pihalf=M_PI_2;
      const double sqrt12=sqrt(12.0);
      // Wire orientation
      if( targ.dy() > targ.dx() ) {
        if( targ.x() < 0 ) 
          verti = -1;
        else
          verti = 1;
      } 
      // Wire Position and rotation angles
      vtwire_.xw[i]   = Point( targ.x(),targ.y(),targ.z() );
      vtwire_.thew[i] = 0.0;
      vtwire_.phiw[i] = verti*pihalf;
      // RMS Width and length of wire (in beam direction)
      double sigmaXY = L3_Parameter::sigmaxy();
      if( sigmaXY <= 0.0 ) sigmaXY = (targ.dx()<targ.dy()) ? targ.dx()/sqrt12 : targ.dy()/sqrt12;
      vtwire_.sigtw[i] = sigmaXY;
      double sigmaZ = L3_Parameter::sigmaz();
      if( sigmaZ<=0.0 ) sigmaZ = targ.dz()/sqrt12; 
      vtwire_.siglw[i] = sigmaZ;
      if( L3_Parameter::Info() ){
	int sigxy=int(1E4*sigmaXY+0.5);
	int sigz =int(1E4*sigmaZ+0.5);
	cout<<"L3_Origin::init: "<<targ.name()<<" sigmaxy/z = "<<sigxy<<'/'<<sigz<<" microns"<<endl;
      }
      //#endif
    }
  vtwire_.nw = i;
  //cout << vtwire_.nw << " wires in buffer" << endl;
  i=0; //initialize all wires
  vtwini_(i);
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Origin::init() is done" << endl;
#endif
}

void L3_Origin::addTracks(L3_Vtx &vtx,vector<L3_Track> &tr)
{
  vector<int> trackNr = vtx.OutgoingTracks();
  for(vector<L3_Track>::const_iterator tit = tr.begin();tit!=tr.end();tit++)
    {
      int tracknr = tit->nr();
      for(vector<int>::const_iterator iit = trackNr.begin();iit!=trackNr.end();iit++)
	{
	  if( tracknr == *iit )
	    {
	      vtx.associate(*tit);
	      break; // next track
	    }
	}
    }
}

void L3_Origin::VTreco(vector<L3_Track> &l3tracks,vector<L3_Track> &l2tracks)
{
  int status = 42;
  //call vt's reconstruction routine
  {
    //vtreco_(status);
    //if(status!=0) cout << "vtreco_ returns " << status << endl;
  }
  {
    int Bindex = 0; //no B track found (B -> Jpsi/K)
    vtprim_(Bindex,status);
    if(status!=0 && L3_Parameter::Debug()) cout << "vtprim_ returns " << status << endl;
  }

  //fill results
  //primary vertices
  int n = ivprim_.nvprim;
  if(L3_Parameter::Trace()) cout << n << " PRIMARY vertices found" << endl;
  for(int i=0;i<n;i++)
    {      
      int index = ivprim_.ivprim[i];
      L3_Vtx vtx(index);      
      addTracks(vtx,l3tracks);
      addTracks(vtx,l2tracks);
      if(L3_Parameter::Trace()) showVertex(vtx);
      primVertexList.push_back( L3_Vertex(vtx) );
    }
  //secondary vertices
  n = ivsec_.nvsec;
  if(L3_Parameter::Trace()) cout << n << " SECONDARY vertices found" << endl;
  for(int i=0;i<n;i++) 
    {
      int index = ivsec_.ivsec[i];
      L3_Vtx vtx(index);
      addTracks(vtx,l3tracks);
      addTracks(vtx,l2tracks);
      if(L3_Parameter::Trace()) showVertex(vtx);
      //secVertexList.push_back( L3_Vertex(vtx) );
      cout << "~~~ secondary vertex found by VT is IGNORED ~~~" << endl;
    }
}






