#include "L3_Parameter.hh"
#ifdef L3_GCC
#include <iostream.h>
#include <math.h>
#endif
#ifdef L3_EGCS
#include <iostream>
#include <cmath>
#include <stdlib.h>
#endif
#include "L3_Defines.hh"
#include "L3_Macros.hh"
#include "L3_Mathlib.hh"
#include "L3_Geometry.hh"

//online lib include
#include "par.h"
#include "grr.h"

float L3_Parameter::the_StereoResol = 0.1;
float L3_Parameter::the_MinMomentumOfRefTracks = 1.0; //GeV
float L3_Parameter::the_margin = 0.1; 
int L3_Parameter::the_run = 0;
int L3_Parameter::the_maxCommonHits = 7;
float L3_Parameter::the_maxProbDiff = 0.4;
float L3_Parameter::the_minProb = 0.00001;
float L3_Parameter::the_Offset_TargetBox_zmax; // Absolut z-distance (FirstTarget-FrontSideOfTargetBox)
float L3_Parameter::the_Offset_TargetBox_zmin; // Absolut z-distance (LastTarget-BackSideOfTargetBox)
float L3_Parameter::the_TargetBox_Width;       // Absolut width of quadratic (x-y plane) TargetBox
float L3_Parameter::the_SearchWindow_HalfMinimalWidth; // 0.5 * Minimal Width of SearchWindow 
float L3_Parameter::the_SearchWindow_HalfMaximalWidth; // 0.5 * Maximal Width of SearchWindow 
float L3_Parameter::the_SearchWindow_HalfOpeningAngle; // 0.5 * Opening Angle of SearchWindow
vector<float> L3_Parameter::chi2cuts(maxNdof-minNdof+1);
int L3_Parameter::tlt_debug = 3;
float L3_Parameter::the_SearchBandSize = 0.1;
int L3_Parameter::the_Ana = 0xffffffff;
float L3_Parameter::the_sigmaxy = -1.0; //wire constraint in x/y for primary vertex finding
float L3_Parameter::the_sigmaz = -1.0; //wire constraint in z for primary vertex finding
// negative values mean that the default wire-dimension/sqrt(12) is used

void L3_Parameter::user_init()
{
  tlt_debug = 0; //2 would enable tracing 
  //the_StereoResol = 0.08; //-- MC data
  //  the_StereoResol = 0.20;
  //value for online data:
  //  the_StereoResol = 0.12;
  //higher value in 98 run, to correct missalignment
  the_StereoResol = 0.10;
  
  the_MinMomentumOfRefTracks = 1.0;
  
  //the_margin = 0.01; //-- MC data 100 microns
  //higher value in 98 run, to correct missalignment
  the_margin = 0.1;
  //value for online data:
  //  the_margin = 0.04;
  //  the_margin = 0.025;
  the_run = 0;
  // Parameters for Clone and Ghost Killer
  // Run 98, real data:
  if (the_run == 98 ) {
    the_maxCommonHits = 7;
    the_maxProbDiff = 0.4;
  } else {
    // MC, full SiVD:
    the_maxCommonHits = 5;
    the_maxProbDiff = 0.4;
  }
  setminProb(0.00001);
  //these settings are good for data
  the_Offset_TargetBox_zmax = 6.0; 
  the_Offset_TargetBox_zmin = 6.0;
  the_TargetBox_Width = 3.5;
  //Coarse alignment good to +- 300 mum
  //the_SearchWindow_HalfMinimalWidth = 0.35;
  //the_SearchWindow_HalfMaximalWidth = 0.40;
  //Fine alignment good to +- 50 mum
  the_SearchWindow_HalfMinimalWidth = 0.10;
  the_SearchWindow_HalfMaximalWidth = 0.25;
  //Online run:
  // the_SearchWindow_HalfMinimalWidth = 0.12;
  //overwrite if MC in ARTE
#ifdef L3_ARTE_FRAME
  if( ArteTable<MCIN>::booked() ){
    the_Offset_TargetBox_zmax = 3.0; 
    the_Offset_TargetBox_zmin = 2.0;
    the_TargetBox_Width = 0.8;
    the_SearchWindow_HalfMinimalWidth = 0.006;
    the_SearchWindow_HalfMaximalWidth = 0.035;
    //the_SearchWindow_HalfMinimalWidth = 0.035;
    //the_SearchWindow_HalfMaximalWidth = 0.070;
  }
#endif
  the_SearchWindow_HalfOpeningAngle = 0.002;
  //the_SearchWindow_HalfOpeningAngle = 0.003;
  the_SearchBandSize = 0.1;
  the_Ana = 0xffffffff;
}

void L3_Parameter::init(int argc,char** argv) {

  user_init(); // hard coded initialization
  int dump=0;
  //look for arguments on the command line 
  char *param_file = getenv("TLT_FILE");
  for (int i=1;i<argc;i++) {		    
    if (!strncmp(argv[i],"-tltdebug:",10)) {
      tlt_debug = atoi(argv[i]+10);
    } else if(!strncmp(argv[i],"-tltfile:",9)){
      param_file = argv[i]+9;
    } else if(!strcmp(argv[i],"-tltopt")){
      compileInfo();
    } else if(!strcmp(argv[i],"-tltdump"))
      dump=1;
  }    
  static int first = 1;
  if( first ){
    first = 0;
    par_sinit();
    //        123456789012345 --- only 16 characters (including trailing 0) are allowed
    par_book("L3StereoResol",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_StereoResol);
    par_book("L3MinMomentum",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_MinMomentumOfRefTracks);
    par_book("L3Margin"     ,PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_margin);
    par_book("L3Run"        ,PAR_T_INT   | PAR_T_PUBLISH,1,&the_run);
    par_book("L3maxCommHits",PAR_T_INT   | PAR_T_PUBLISH,1,&the_maxCommonHits);
    par_book("L3maxProbDiff",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_maxProbDiff);
    par_book("L3minProb"    ,PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_minProb);
    par_book("L3OffTBoxZmax",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_Offset_TargetBox_zmax);
    par_book("L3OffTBoxZmin",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_Offset_TargetBox_zmin);
    par_book("L3TBoxWidth"  ,PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_TargetBox_Width);
    par_book("L3SWinMinWid2",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_SearchWindow_HalfMinimalWidth);
    par_book("L3SWinMaxWid2",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_SearchWindow_HalfMaximalWidth);
    par_book("L3SWinOpAngl2",PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_SearchWindow_HalfOpeningAngle);
    par_book("L3debug"      ,PAR_T_INT   | PAR_T_PUBLISH,1,&tlt_debug);
    par_book("L3SearchBand" ,PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_SearchBandSize);
    //L3_Analysis NOT made public -> it cannot be changed while running
    par_book("L3Analysis"   ,PAR_T_INT                 ,1,&the_Ana);
    par_book("L3sigmaxy"    ,PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_sigmaxy);
    par_book("L3sigmaz"     ,PAR_T_FLOAT | PAR_T_PUBLISH,1,&the_sigmaz);
  }
  if( param_file ) 
    {
      if( readFile(param_file)==GRR_OK && Trace() )
	cout << "L3_Paramter::init: read configuration file " << param_file << endl;
    }
  if(dump) showParameter();
}

float L3_Parameter::calcChi2cut(int ndof)
{
  float lochi = 0.0;
  float hichi = (float)ndof*10;
  //make sure the initial chi2 is large enough
  while( L3_prob(hichi,ndof) > the_minProb )  hichi *= 10.0;   
  //find the chi2 iteratively
  float guess;
  do {   
    guess = (lochi+hichi)/2;
    //cout << lochi << ' ' << guess << ' ' << hichi << " : ";
    float val = L3_prob(guess,ndof);
    //cout << val << endl;
    if( fabs(val-the_minProb)/the_minProb < 0.0001 ) 
      return guess;
    else if(val>the_minProb)
      lochi = guess;
    else 
      hichi = guess;    
  } while(1);
}


void L3_Parameter::setminProb(float p)
{
  if(p<0 || p>1)
    {
      cout << " !!!!!!!!! TLT Warning !!!!!!!!! " << endl;
      cout << "L3_Parameter::setminProb: argument should be in [0,1], but is " << p << endl;
      cout << "L3_Parameter::setminProb: TLT is taking the default value!" << endl;
      return;
    }
  the_minProb = p;

  if (p == 0.0) {
    for(int ndof=minNdof;ndof<=maxNdof;ndof++) 
      {      
	chi2cuts[ndof-minNdof] = 1e17;
      }
    return;
  }
  if (p == 1.0) {
    for(int ndof=minNdof;ndof<=maxNdof;ndof++) 
      {      
	chi2cuts[ndof-minNdof] = 0.0;
      }
    return;
  }    

#ifdef L3_VERBOSE
  if(L3_Parameter::Info()){
    cout << "Chi2-cut table for minProb=" << the_minProb << endl;
    cout << "\tndof\tchi2" << endl;
    cout << "-----------------------------------------" << endl;
  }
#endif
  for(int ndof=minNdof;ndof<=maxNdof;ndof++) 
    {      
      float cut = calcChi2cut(ndof); 
      chi2cuts[ndof-minNdof] = cut;
#ifdef L3_VERBOSE
      if(L3_Parameter::Info()) cout << '\t' << ndof << '\t' << cut << endl; 
#endif
    }
}

void L3_Parameter::showParameter()
{
  cout << "****** current third level trigger settings *******************" << endl;
  cout << "L3_Parameter::StereoResol = " << the_StereoResol << endl;
  cout << "L3_Parameter::MinMomentumOfRefTracks = " << the_MinMomentumOfRefTracks << endl;
  cout << "L3_Parameter::margin = " << the_margin << endl;
  cout << "L3_Parameter::run = " << the_run << endl;
  cout << "L3_Parameter::maxCommonHits = " << the_maxCommonHits << endl;
  cout << "L3_Parameter::maxProbDiff = " << the_maxProbDiff << endl;
  cout << "L3_Parameter::minProb = " << the_minProb << endl;
  cout << "L3_Parameter::Offset_TargetBox_zmax = " << the_Offset_TargetBox_zmax << endl;
  cout << "L3_Parameter::Offset_TargetBox_zmin = " << the_Offset_TargetBox_zmin << endl;
  cout << "L3_Parameter::TargetBox_Width = " << the_TargetBox_Width << endl;
  cout << "L3_Parameter::SearchWindow_HalfMinimalWidth = " << the_SearchWindow_HalfMinimalWidth << endl;
  cout << "L3_Parameter::SearchWindow_HalfMaximalWidth = " << the_SearchWindow_HalfMaximalWidth << endl;
  cout << "L3_Parameter::SearchWindow_HalfOpeningAngle = " << the_SearchWindow_HalfOpeningAngle << endl;
  cout << "L3_Parameter::L3Debug = " << tlt_debug << endl;
  cout << "L3_Parameter::SearchBandSize = " << the_SearchBandSize << endl;
  cout << "L3_Parameter::L3Analysis = " << the_Ana << " (0x" << hex << the_Ana << dec << ")" << endl;
  cout << "L3_Parameter::sigmaxy = " << the_sigmaxy << endl;
  cout << "L3_Parameter::sigmaz = " << the_sigmaz << endl;
  cout << "***************************************************************" << endl;
}

int L3_Parameter::readFile(char *name)
{
#define NN "L3_Parameter::readFile: "
  int rc = par_fetch(name);
  if( rc!=GRR_OK ) {
    P5(NN"par_fetch returns ",grr_stript(rc)," for file ",name,endl);   
  } else {
    L3_Geometry::calculate_TargetBox(); 
    setminProb(the_minProb);
  }
  return rc;
#undef NN
}

int L3_Parameter::saveFile(char *name)
{
#define NN "L3_Parameter::saveFile: "
  int rc = par_save(name);
  if( rc!=GRR_OK ) P5(NN"par_save returns ",grr_stript(rc)," for file ",name,endl);   
  return rc;
#undef NN
}

void L3_Parameter::compileInfo(){
  cout << "TLT compiled on " << __DATE__ << ' ' << __TIME__ << endl;
  vector<char*> v;
#ifdef L3_EXTERNAL
  v.push_back("EXTERNAL");
#endif      
#ifdef L3_EGCS
  v.push_back("EGCS");
#endif      
#ifdef L3_GCC
  v.push_back("GCC");
#endif      
#ifdef L3_ARTE_FRAME
  v.push_back("ARTE_FRAME");
#endif      
#ifdef L3_SLP_FRAME
  v.push_back("SLP_FRAME");
#endif      
#ifdef L3_SVD_NPPN
  v.push_back("SVD_NPPN");
#endif      
#ifdef L3_SVD_NPNP
  v.push_back("SVD_NPNP");
#endif      
#ifdef L3_ONE_MIMP_PER_DOUBLESIDED_MODULE
  v.push_back("ONE_MIMP_PER_DOUBLESIDED_MODULE");
#endif
#ifdef L3_ARTE2
  v.push_back("ARTE2");
#endif
#ifdef L3_ARTE3
  v.push_back("ARTE3");
#endif
#ifdef L3_ONLINE_MODE
  v.push_back("ONLINE_MODE");
#endif
#ifdef L3_OFFLINE_MODE
  v.push_back("OFFLINE_MODE");
#endif
#ifdef L3_SVD_ROTATE_GEO
  v.push_back("SVD_ROTATE_GEO");
#endif      
#ifdef L3_NPNP
  v.push_back("NPNP");
#endif      
#ifdef L3_NPPN
  v.push_back("NPPN");
#endif      
#ifdef L3_NO_RHP
  v.push_back("NO_RHP");
#endif
#ifdef L3_NO_PAW
  v.push_back("NO_PAW");
#endif
#ifdef L3_PRINT_EVENT_SUMMARY
  v.push_back("PRINT_EVENT_SUMMARY");
#endif
#ifdef L3_VERBOSE
  v.push_back("VERBOSE");
#endif
#ifdef L3_MDEBUG
  v.push_back("MDEBUG");
#endif
  for(vector<char*>::iterator i=v.begin();i!=v.end();i++)
    cout << "\tL3_" << *i << endl;
}

#ifdef L3_ARTE_FRAME
#ifdef L3_GCC
#include <string.h>
#include <ctype.h>
#endif
#ifdef L3_EGCS
#include <cstring>
#include <cctype>
#endif

int L3_Parameter::setParameter(char *name,float val)
{
  int rc = 1;
  if( !strcmp(name,"stereoresol")){
    the_StereoResol = val;
  } else if(!strcmp(name,"minmomentum")){
    the_MinMomentumOfRefTracks = val;
  } else if(!strcmp(name,"margin")){
    the_margin = val;
  } else if(!strcmp(name,"run")){
    the_run = (int)val;
  } else if(!strcmp(name,"maxcomhits")){
    the_maxCommonHits = (int)val;
  } else if(!strcmp(name,"maxprobdiff")){
    the_maxProbDiff = val;
  } else if(!strcmp(name,"minprob")){
    setminProb(val);
  } else if(!strcmp(name,"offset_targetbox_zmax")){
    the_Offset_TargetBox_zmax = val;
  } else if(!strcmp(name,"offset_targetbox_zmin")){
    the_Offset_TargetBox_zmin = val;
  } else if(!strcmp(name,"targetbox_width")){
    the_TargetBox_Width = val;
  } else if(!strcmp(name,"searchwindow_halfminimalwidth")){
    the_SearchWindow_HalfMinimalWidth = val;
  } else if(!strcmp(name,"searchwindow_halfmaximalwidth")){
    the_SearchWindow_HalfMaximalWidth = val;
  } else if(!strcmp(name,"searchwindow_halfopeningangle")){
    the_SearchWindow_HalfOpeningAngle = val;
  } else if(!strcmp(name,"l3debug")){
    tlt_debug = (int)val;
  } else if(!strcmp(name,"l3searchband")){
    the_SearchBandSize = val;
  } else if(!strcmp(name,"l3sigmaxy")){
    the_sigmaxy = val;
  } else if(!strcmp(name,"l3sigmaz")){
    the_sigmaz = val;
  } else rc=0;
  return rc;
}

void L3_Parameter::helpARTE()
{
  cout << "tltparam <parametername> <value>, ";
  cout << "with <parametername> one of:" << endl;
  cout << "{StereoResol, MinMomentum, margin, run, maxComHits, maxProbDiff, minProb}" << endl;
  cout << "{Offset_TargetBox_zmax, Offset_TargetBox_zmin, TargetBox_Width}" << endl;
  cout << "{SearchWindow_HalfMinimalWidth, SearchWindow_HalfMaximalWidth, SearchWindow_HalfOpeningAngle, L3Debug}" << endl;
  cout << "{SearchBandSize, SigmaXY, SigmaZ }" << endl;
  showParameter();
}

extern "C" void kugeti_(int*);
extern "C" void kugetr_(float*);
extern "C" void kugetc_(char[], int*, int);

inline void trans(char *s,int len)
{
  s[len]=0;
  for(int i=0;i<len;i++) s[i] = tolower(s[i]);
}

extern "C" void tltparam_(void) {
  #define NN "tltparam: "

  int cmd_len=999;
  char command[40];
  float arg1=999;
  kugetc_(command, &cmd_len,sizeof(command)-1);
  trans(command,cmd_len);
  kugetr_(&arg1);

  //cout << "command = ~" << command << "~" << endl; 
  //cout << "cmd_len = " << cmd_len << endl;
  //cout << "arg1 = " << arg1 << endl;

  if (!strcmp(command, "help")) {
    L3_Parameter::helpARTE();
  } else {
    if( L3_Parameter::setParameter(command,arg1)==0 )
      {
	cout << NN"unknown parameter name " << command << endl;
      }
  }
  #undef NN
}

extern "C" void tltio_(void) {
  #define NN "tltio: "

  int cmd_len=999;
  char command[40],filename[40];
  kugetc_(command, &cmd_len,sizeof(command)-1);  
  trans(command,cmd_len);
  kugetc_(filename, &cmd_len,sizeof(filename)-1);  
  trans(filename,cmd_len);
  
  //cout << "command = ~" << command << "~" << endl; 
  //cout << "filename = ~" << filename << "~" << endl;

  if (!strcmp(command, "read")) {
    L3_Parameter::readFile(filename);
  } else if(!strcmp(command,"write")) {
    L3_Parameter::saveFile(filename);
  } else if(!strcmp(command,"help")){
    cout << "tltio <read|write> <filename>" << endl;
  } else {
    cout << NN"unknown command " << command << endl;
  }
  #undef NN
}

#endif







