#include "L3_Silicon.hh"
#include "L3_Window.hh"
#include "L3_Parameter.hh"
#include "L3_Geometry.hh"
#ifdef L3_EGCS
#ifdef L3_VERBOSE
#include <iostream>
#endif
#include <set>
#include <map>
#include <algorithm>
#include <functional>
#endif
#ifdef L3_GCC
#ifdef L3_VERBOSE
#include <iostream.h>
#endif
#include <algo.h>
#include <function.h>
#endif

#ifdef L3_SLP_FRAME
#include "vdsHITB.h"
extern "C" {
  VDS_hitb* vdsslt_fillHITB( int* nhitb);
}
#endif

int xcnt = 0;

L3_Silicon::L3_Silicon(const L3_Silicon& a) {
  TrackList = a.TrackList;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      SpacePointList[i][j] = a.SpacePointList[i][j];
      SpacePoint2List[i][j] = a.SpacePoint2List[i][j];
      SpacePoint2XList[i][j] = a.SpacePoint2XList[i][j];
      for (int k=0; k<4; k++) { //Locate Plane
	HitbList[i][j][k] = a.HitbList[i][j][k];
      }
    }
  }
}

L3_Silicon& L3_Silicon::operator = (const L3_Silicon& a) {
  TrackList = a.TrackList;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      SpacePointList[i][j] = a.SpacePointList[i][j];
      SpacePoint2List[i][j] = a.SpacePoint2List[i][j];
      SpacePoint2XList[i][j] = a.SpacePoint2XList[i][j];
      for (int k=0; k<4; k++) { //Locate Plane
	HitbList[i][j][k] = a.HitbList[i][j][k];
      }
    }
  }
  return *this;
}

void L3_Silicon::release() {
  xcnt = 0;
#ifdef L3_EGCS
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      (SpacePointList[i][j]).clear();
      (SpacePoint2List[i][j]).clear();
      (SpacePoint2XList[i][j]).clear();      
      for (int k=0; k<4; k++) { //Locate Plane
	(HitbList[i][j][k]).clear();
      }
    }
  }
  TrackList.clear();
#endif
#ifdef L3_GCC
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      (SpacePointList[i][j]).erase((SpacePointList[i][j]).begin(), (SpacePointList[i][j]).end());
      (SpacePoint2List[i][j]).erase((SpacePoint2List[i][j]).begin(), (SpacePoint2List[i][j]) .end());
      (SpacePoint2XList[i][j]).erase((SpacePoint2XList[i][j]).begin(), (SpacePoint2XList[i][j]).end());
      for (int k=0; k<4; k++) { //Locate Plane
	(HitbList[i][j][k]).erase((HitbList[i][j][k]).begin(), (HitbList[i][j][k]).end());
      }
    }
  }
  TrackList.erase(TrackList.begin(), TrackList.end());
#endif
}
  
int L3_Silicon::nHitbs() const {
  int Count=0;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      for (int k=0; k<4; k++) { //Locate Plane
	Count += (HitbList[i][j][k]).size();
      }
    }
  }
  return Count;
}
int L3_Silicon::nPoints() const {
  int Count=0;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      Count += (SpacePointList[i][j]).size();
    }
  }
  return Count;
}

vector<L3_Hitb> 
L3_Silicon::HList() const {
  vector<L3_Hitb> tmpList;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      for (int k=0; k<4; k++) { //Locate Plane
	for (vector<L3_Hitb>::const_iterator hit = (HitbList[i][j][k]).begin();
	     hit != (HitbList[i][j][k]).end(); ++hit) {
	  tmpList.push_back(*hit);
	}
      }
    }
  }
  return tmpList;
}

vector<L3_SpacePoint>
L3_Silicon::Points() const{
  vector<L3_SpacePoint> TmpSpList;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      for (vector<L3_SpacePoint>::const_iterator SpacePointListIt
             = (SpacePointList[i][j]).begin();
           SpacePointListIt != (SpacePointList[i][j]).end();
           ++SpacePointListIt) {
        TmpSpList.push_back(*SpacePointListIt);
      }
    }
  }
  return TmpSpList;
}

void L3_Silicon::reconstruct(L3_Timer& Timer) {
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::reconstruct START" << endl;
#endif
  if (L3_Parameter::run() == 98) {
    Timer.start_PointReco();
    SpacePointReco();
    SpacePairReco();
    PurgeSpacePoints();
    Timer.stop_PointReco();
    Timer.start_TrackReco();
    SortSpacePoints();
    SortSpacePairs();
    TrackReconstruction98();
    findValidTracks();
    Timer.stop_TrackReco();
  }
  if (L3_Parameter::run() == 0) {
    Timer.start_PointReco();
//    SpacePointReco();
//    SpacePointRecoTest();
    SpacePointRecoChi2();
    PurgeSpacePoints();
    Timer.stop_PointReco();
    Timer.start_TrackReco();
    SortSpacePoints();    
    TrackReconstruction();
    findValidTracks();
    Timer.stop_TrackReco();    
  }
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::reconstruct STOP" << endl;
#endif
}

void L3_Silicon::SpacePointReco() {
//Reconstruct SpacePoints
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePointReco START" << endl;
#endif
  float dst=0.;
  float x1p=0.;
  float x3p=0.;
  float PredictHit3x=0.;
  float PredictHit4x=0.;
  float Distanz=0.0;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      (SpacePointList[i][j]).reserve(500);
      for (int P=0; P<=1; P++) {
	vector<L3_SpacePoint> SpacePairList;
	SpacePairList.reserve(500);
#ifdef lala
#ifdef L3_SVD_NPNP
	int StP = P+2;
        int P3  = 1-P;
        int P4  = 3-P;
#endif
#ifdef L3_SVD_NPPN
	int StP = 3-P;
        int P3  = 1-P;
        int P4  = P+2;
#endif	
#endif
	int StP = L3_Geometry::getPlaneIndex(i,j,P,0);
	int P3  = L3_Geometry::getPlaneIndex(i,j,P,1);
	int P4  = L3_Geometry::getPlaneIndex(i,j,P,2);
	const L3_Plane& Plane   = L3_Geometry::getPlane(i,j,P);
	const L3_Plane& StPlane = L3_Geometry::getPlane(i,j,StP);
	const L3_Plane& Plane3  = L3_Geometry::getPlane(i,j,P3);
	const L3_Plane& Plane4  = L3_Geometry::getPlane(i,j,P4);
//  	cout << "Plane  :"  << Plane << endl;
//  	cout << "StPlane:"  << StPlane << endl;
//  	cout << "Plane3 :"  << Plane3 << endl;
//  	cout << "Plane4 :"  << Plane4 << endl;
	Distanz = ((Plane.zfw(0)+StPlane.zfw(0)) * 0.5) - L3_Geometry::ztarget();
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 1 fuer vertikale Draehte (+/- 2.5 grad zur y-Achse)
// 	    cout << " TEST : dst= " << dst << endl;
	if ( fabs( Plane.cos() ) > 0.5 ) { // +-2.5 degree layers
	  dst = fabs(2.0 * ( Plane.symx() - Plane.symn() )
		     * 0.0436609429);
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    x1p = Plane.icos()
	      * ( ( (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget())) * hit->x() )
		  + (Plane.symn() * Plane.sin() ) );
// 	    cout << "> Hit : " << hit->hit() << "   x1p = " << x1p << endl;
	    for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		 Sthit!= (HitbList[i][j][StP]).end(); ++Sthit) {
	      x3p = StPlane.icos() 
		* ( (( Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) * Sthit->x())
		    + (StPlane.symn() * StPlane.sin()) );
// 	      cout << ">>>  StHit : " 
// 		   << Sthit->hit() << "   x3p = " << x3p << endl;
	      if ( (x3p >= (x1p - dst - L3_Parameter::margin() ) ) 
		   && ( x3p <= (x1p + L3_Parameter::margin())  )) {
		//-HitsInStereo.push_back(*Sthit);
		L3_SpacePoint Paar(*hit, *Sthit);
		// cout << " Paar: " << Paar << endl;
	        SpacePairList.push_back(Paar);
	      }
	    }
	  }
	} else { // 90+-2.5 degree layers
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 0 fuer horizontale Draehte (90+/-2.5 grad zur y-Achse)
	  dst = fabs(2.0 * ( Plane.sxmx() - Plane.sxmn() )
		       * 0.0436609429);
// 	    cout << " DST 2=" << dst << endl;
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    x1p = Plane.isin()
	      * ( (Plane.sxmn() * Plane.cos())
		  - (hit->x() * (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget()))) );
	    for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		 Sthit != (HitbList[i][j][StP]).end(); ++Sthit) {
	      x3p = StPlane.isin() * 
		( (StPlane.sxmn() * StPlane.cos())
		  - (Sthit->x() * (Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) ) );
	      if ((x3p <= (x1p + dst + L3_Parameter::margin())) 
		  && (x3p >= (x1p - L3_Parameter::margin()))) {
		L3_SpacePoint Paar(*hit, *Sthit);
	        //cout << " Paar: " << Paar << endl;
	        SpacePairList.push_back(Paar);
	      }
	    }
	  }
	}
//  	cout << "TLTRECO: SpacePairList SL, Quadrant, Pair, Size: " 
//  	     << i << " "  << j << " " << P << " "
//  	     << SpacePairList.size() << " " << endl;
	for (vector<L3_SpacePoint>::iterator SpacePairListIt = SpacePairList.begin();
	     SpacePairListIt != SpacePairList.end(); ++SpacePairListIt) {
	  L3_SpacePoint& Paar = *SpacePairListIt;
	  vector<L3_Hitb> HitListLayer3;
	  vector<L3_Hitb> HitListLayer4;
//  	  cout << "TLTRECO: Debug Paar: x= " << Paar.x() 
// 	       << " y= " << Paar.y() << endl;
//  	  cout << "TLTRECO: Hitblist(Plane3).size " 
	  /// 	  << HitbList[i][j][P3].size() << endl;
	  PredictHit3x = ( (Plane3.zfw(0) - L3_Geometry::ztarget()) / Distanz) *
	    ( (Paar.x() * Plane3.cos()) -
	      (Paar.y() * Plane3.sin()) );
// 	  cout << " Predict Hit3x = " << PredictHit3x << endl;
	  for (vector<L3_Hitb>::iterator hit3= (HitbList[i][j][P3]).begin();
	       hit3 != (HitbList[i][j][P3]).end(); ++hit3) {
// 	    cout << " hit3->x() = " << hit3->x() << endl;
	    if ( fabs( PredictHit3x - hit3->x())<= L3_Parameter::StereoResol()) {
	      HitListLayer3.push_back(*hit3);
	    }
	  }
	  PredictHit4x = ( (Plane4.zfw(0) - L3_Geometry::ztarget()) / Distanz) * 
	    ( (Paar.x() * Plane4.cos()) -
	      (Paar.y() * Plane4.sin()) );
	  for (vector<L3_Hitb>::iterator hit4 = (HitbList[i][j][P4]).begin();
	       hit4 != (HitbList[i][j][P4]).end(); ++hit4) {
	    if ( fabs( PredictHit4x - hit4->x())<= L3_Parameter::StereoResol()) {
	      HitListLayer4.push_back(*hit4);
	    }
	  }
// 	  cout << "TLTRECO: # Hits in 3rd /4th  Layer "
//  	       << HitListLayer3.size() << " / "
// 	       << HitListLayer4.size() << endl;
	  if (HitListLayer3.size() > 0 ) {
	    if (HitListLayer4.size() > 0 ) {
	      for (vector<L3_Hitb>::iterator hit3= HitListLayer3.begin();
		   hit3 != HitListLayer3.end(); ++hit3) {
		for (vector<L3_Hitb>::iterator hit4 = HitListLayer4.begin();
		     hit4 != HitListLayer4.end(); ++hit4) {
		  L3_SpacePoint Paerchen(Paar);
		  Paerchen.add(*hit3, *hit4);
		  (SpacePointList[i][j]).push_back(Paerchen);
		}
	      }
	    } else {
	      for (vector<L3_Hitb>::iterator hit3= HitListLayer3.begin();
		   hit3 != HitListLayer3.end(); ++hit3) {
		L3_SpacePoint Paerchen(Paar);
		Paerchen.add(*hit3);
		(SpacePointList[i][j]).push_back(Paerchen);
	      }
	    }
	  } else if (HitListLayer4.size() > 0 ) {
	    for (vector<L3_Hitb>::iterator hit4 = HitListLayer4.begin();
		 hit4!= HitListLayer4.end(); ++hit4) {
	      L3_SpacePoint Paerchen(Paar);
	      Paerchen.add(*hit4);
	      (SpacePointList[i][j]).push_back(Paerchen);
	    }	
	  }
	}
      }
    } //sector    
  }//superlayer 
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePointReco STOP, # of points is " << nPoints() << endl;
#endif
}

void L3_Silicon::SpacePointRecoTest() {
//Reconstruct SpacePoints
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePointReco START" << endl;
#endif
  float dst=0.;
  float x1p=0.;
  float x3p=0.;
  float PredictHit3x=0.;
  float PredictHit4x=0.;
  float Distanz=0.;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      (SpacePointList[i][j]).reserve(500);
      for (int P=0; P<=1; P++) {
	vector<L3_SpacePoint> SpacePairList;
	SpacePairList.reserve(500);
#ifdef lala
#ifdef L3_SVD_NPNP
	int StP = P+2;
        int P3  = 1-P;
        int P4  = 3-P;
#endif
#ifdef L3_SVD_NPPN
	int StP = 3-P;
        int P3  = 1-P;
        int P4  = P+2;
#endif	
#endif
	int StP = L3_Geometry::getPlaneIndex(i,j,P,0);
	int P3  = L3_Geometry::getPlaneIndex(i,j,P,1);
	int P4  = L3_Geometry::getPlaneIndex(i,j,P,2);
	const L3_Plane& Plane   = L3_Geometry::getPlane(i,j,P);
	const L3_Plane& StPlane = L3_Geometry::getPlane(i,j,StP);
	const L3_Plane& Plane3  = L3_Geometry::getPlane(i,j,P3);
	const L3_Plane& Plane4  = L3_Geometry::getPlane(i,j,P4);
//  	cout << "Plane  :"  << Plane << endl;
//  	cout << "StPlane:"  << StPlane << endl;
//  	cout << "Plane3 :"  << Plane3 << endl;
//  	cout << "Plane4 :"  << Plane4 << endl;
	Distanz = ((Plane.zfw(0)+StPlane.zfw(0)) * 0.5) - L3_Geometry::ztarget();
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 1 fuer vertikale Draehte (+/- 2.5 grad zur y-Achse)
// 	    cout << " TEST : dst= " << dst << endl;
	if ( fabs( Plane.cos() ) > 0.5 ) { // +-2.5 degree layers
	  dst = fabs(2.0 * ( Plane.symx() - Plane.symn() )
		     * 0.0436609429);
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    x1p = Plane.icos()
	      * ( ( (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget())) * hit->x() )
		  + (Plane.symn() * Plane.sin() ) );
// 	    cout << "> Hit : " << hit->hit() << "   x1p = " << x1p << endl;
	    for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		 Sthit!= (HitbList[i][j][StP]).end(); ++Sthit) {
	      x3p = StPlane.icos() 
		* ( (( Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) * Sthit->x())
		    + (StPlane.symn() * StPlane.sin()) );
// 	      cout << ">>>  StHit : " 
// 		   << Sthit->hit() << "   x3p = " << x3p << endl;
	      if ( (x3p >= (x1p - dst - L3_Parameter::margin() ) ) 
		   && ( x3p <= (x1p + L3_Parameter::margin())  )) {
		//-HitsInStereo.push_back(*Sthit);
		L3_SpacePoint Paar(*hit, *Sthit);
		// cout << " Paar: " << Paar << endl;
	        SpacePairList.push_back(Paar);
	      }
	    }
	  }
	} else { // 90+-2.5 degree layers
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 0 fuer horizontale Draehte (90+/-2.5 grad zur y-Achse)
	  dst = fabs(2.0 * ( Plane.sxmx() - Plane.sxmn() )
		       * 0.0436609429);
// 	    cout << " DST 2=" << dst << endl;
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    x1p = Plane.isin()
	      * ( (Plane.sxmn() * Plane.cos())
		  - (hit->x() * (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget()))) );
	    for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		 Sthit != (HitbList[i][j][StP]).end(); ++Sthit) {
	      x3p = StPlane.isin() * 
		( (StPlane.sxmn() * StPlane.cos())
		  - (Sthit->x() * (Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) ) );
	      if ((x3p <= (x1p + dst + L3_Parameter::margin())) 
		  && (x3p >= (x1p - L3_Parameter::margin()))) {
		L3_SpacePoint Paar(*hit, *Sthit);
	        //cout << " Paar: " << Paar << endl;
	        SpacePairList.push_back(Paar);
	      }
	    }
	  }
	}
//  	cout << "TLTRECO: SpacePairList SL, Quadrant, Pair, Size: " 
//  	     << i << " "  << j << " " << P << " "
//  	     << SpacePairList.size() << " " << endl;
	for (vector<L3_SpacePoint>::iterator SpacePairListIt = SpacePairList.begin();
	     SpacePairListIt != SpacePairList.end(); ++SpacePairListIt) {
	  L3_SpacePoint& Paar = *SpacePairListIt;
	  vector<L3_Hitb> HitListLayer3;
	  vector<L3_Hitb> HitListLayer4;
//  	  cout << "TLTRECO: Debug Paar: x= " << Paar.x() 
// 	       << " y= " << Paar.y() << endl;
//  	  cout << "TLTRECO: Hitblist(Plane3).size " 
	  /// 	  << HitbList[i][j][P3].size() << endl;
	  PredictHit3x = ( (Plane3.zfw(0) - L3_Geometry::ztarget()) / Distanz) *
	    ( (Paar.x() * Plane3.cos()) -
	      (Paar.y() * Plane3.sin()) );
// 	  cout << " Predict Hit3x = " << PredictHit3x << endl;
	  for (vector<L3_Hitb>::iterator hit3= (HitbList[i][j][P3]).begin();
	       hit3 != (HitbList[i][j][P3]).end(); ++hit3) {
// 	    cout << " hit3->x() = " << hit3->x() << endl;
	    if ( fabs( PredictHit3x - hit3->x())<= L3_Parameter::StereoResol()) {
	      HitListLayer3.push_back(*hit3);
	    }
	  }
	  PredictHit4x = ( (Plane4.zfw(0) - L3_Geometry::ztarget()) / Distanz) * 
	    ( (Paar.x() * Plane4.cos()) -
	      (Paar.y() * Plane4.sin()) );
	  for (vector<L3_Hitb>::iterator hit4 = (HitbList[i][j][P4]).begin();
	       hit4 != (HitbList[i][j][P4]).end(); ++hit4) {
	    if ( fabs( PredictHit4x - hit4->x())<= L3_Parameter::StereoResol()) {
	      HitListLayer4.push_back(*hit4);
	    }
	  }
// 	  cout << "TLTRECO: # Hits in 3rd /4th  Layer "
//  	       << HitListLayer3.size() << " / "
// 	       << HitListLayer4.size() << endl;
	  if (HitListLayer3.size() > 0 ) {
	    if (HitListLayer4.size() > 0 ) {
	      for (vector<L3_Hitb>::iterator hit3= HitListLayer3.begin();
		   hit3 != HitListLayer3.end(); ++hit3) {
		for (vector<L3_Hitb>::iterator hit4 = HitListLayer4.begin();
		     hit4 != HitListLayer4.end(); ++hit4) {
		  L3_SpacePoint Paerchen(Paar);
		  Paerchen.add(*hit3, *hit4);
		  L3_SpacePoint Paerchen3(Paar);
		  L3_SpacePoint Paerchen4(Paar);
		  Paerchen3.add(*hit3);
		  Paerchen4.add(*hit4);
		  (SpacePointList[i][j]).push_back(Paerchen);
		  (SpacePointList[i][j]).push_back(Paerchen3);
		  (SpacePointList[i][j]).push_back(Paerchen4);
		}
	      }
	    }
	  }
	}
      }
    } //sector    
  }//superlayer 
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePointReco STOP, # of points is " << nPoints() << endl;
#endif
}

void L3_Silicon::SpacePointRecoChi2() {
//Reconstruct SpacePoints
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePointReco START" << endl;
#endif
  float dst=0.;
  float x1p=0.;
  float x3p=0.;
  float PredictHit3x=0.;
  float PredictHit4x=0.;
  float Distanz=0.;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      (SpacePointList[i][j]).reserve(500);
      for (int P=0; P<=1; P++) {
	vector<L3_SpacePoint> SpacePairList;
	SpacePairList.reserve(500);
#ifdef lala
#ifdef L3_SVD_NPNP
	int StP = P+2;
        int P3  = 1-P;
        int P4  = 3-P;
#endif
#ifdef L3_SVD_NPPN
	int StP = 3-P;
        int P3  = 1-P;
        int P4  = P+2;
#endif	
#endif
	int StP = L3_Geometry::getPlaneIndex(i,j,P,0);
	int P3  = L3_Geometry::getPlaneIndex(i,j,P,1);
	int P4  = L3_Geometry::getPlaneIndex(i,j,P,2);
	const L3_Plane& Plane   = L3_Geometry::getPlane(i,j,P);
	const L3_Plane& StPlane = L3_Geometry::getPlane(i,j,StP);
	const L3_Plane& Plane3  = L3_Geometry::getPlane(i,j,P3);
	const L3_Plane& Plane4  = L3_Geometry::getPlane(i,j,P4);
//  	cout << "Plane  :"  << Plane << endl;
//  	cout << "StPlane:"  << StPlane << endl;
//  	cout << "Plane3 :"  << Plane3 << endl;
//  	cout << "Plane4 :"  << Plane4 << endl;
	Distanz = ((Plane.zfw(0)+StPlane.zfw(0)) * 0.5) - L3_Geometry::ztarget();
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 1 fuer vertikale Draehte (+/- 2.5 grad zur y-Achse)
// 	    cout << " TEST : dst= " << dst << endl;
	if ( fabs( Plane.cos() ) > 0.5 ) { // +-2.5 degree layers
	  dst = fabs(2.0 * ( Plane.symx() - Plane.symn() )
		     * 0.0436609429);
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    x1p = Plane.icos()
	      * ( ( (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget())) * hit->x() )
		  + (Plane.symn() * Plane.sin() ) );
// 	    cout << "> Hit : " << hit->hit() << "   x1p = " << x1p << endl;
	    for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		 Sthit!= (HitbList[i][j][StP]).end(); ++Sthit) {
	      x3p = StPlane.icos() 
		* ( (( Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) * Sthit->x())
		    + (StPlane.symn() * StPlane.sin()) );
// 	      cout << ">>>  StHit : " 
// 		   << Sthit->hit() << "   x3p = " << x3p << endl;
	      if ( (x3p >= (x1p - dst - L3_Parameter::margin() ) ) 
		   && ( x3p <= (x1p + L3_Parameter::margin())  )) {
		//-HitsInStereo.push_back(*Sthit);
		L3_SpacePoint Paar(*hit, *Sthit);
		// cout << " Paar: " << Paar << endl;
	        SpacePairList.push_back(Paar);
	      }
	    }
	  }
	} else { // 90+-2.5 degree layers
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 0 fuer horizontale Draehte (90+/-2.5 grad zur y-Achse)
	  dst = fabs(2.0 * ( Plane.sxmx() - Plane.sxmn() )
		       * 0.0436609429);
// 	    cout << " DST 2=" << dst << endl;
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    x1p = Plane.isin()
	      * ( (Plane.sxmn() * Plane.cos())
		  - (hit->x() * (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget()))) );
	    for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		 Sthit != (HitbList[i][j][StP]).end(); ++Sthit) {
	      x3p = StPlane.isin() * 
		( (StPlane.sxmn() * StPlane.cos())
		  - (Sthit->x() * (Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) ) );
	      if ((x3p <= (x1p + dst + L3_Parameter::margin())) 
		  && (x3p >= (x1p - L3_Parameter::margin()))) {
		L3_SpacePoint Paar(*hit, *Sthit);
	        //cout << " Paar: " << Paar << endl;
	        SpacePairList.push_back(Paar);
	      }
	    }
	  }
	}
//  	cout << "TLTRECO: SpacePairList SL, Quadrant, Pair, Size: " 
//  	     << i << " "  << j << " " << P << " "
//  	     << SpacePairList.size() << " " << endl;
	for (vector<L3_SpacePoint>::iterator SpacePairListIt = SpacePairList.begin();
	     SpacePairListIt != SpacePairList.end(); ++SpacePairListIt) {
	  L3_SpacePoint& Paar = *SpacePairListIt;
	  vector<L3_Hitb> HitListLayer3;
	  vector<L3_Hitb> HitListLayer4;
//  	  cout << "TLTRECO: Debug Paar: x= " << Paar.x() 
// 	       << " y= " << Paar.y() << endl;
//  	  cout << "TLTRECO: Hitblist(Plane3).size " 
	  /// 	  << HitbList[i][j][P3].size() << endl;
	  PredictHit3x = ( (Plane3.zfw(0) - L3_Geometry::ztarget()) / Distanz) *
	    ( (Paar.x() * Plane3.cos()) -
	      (Paar.y() * Plane3.sin()) );
// 	  cout << " Predict Hit3x = " << PredictHit3x << endl;
	  for (vector<L3_Hitb>::iterator hit3= (HitbList[i][j][P3]).begin();
	       hit3 != (HitbList[i][j][P3]).end(); ++hit3) {
// 	    cout << " hit3->x() = " << hit3->x() << endl;
	    if ( fabs( PredictHit3x - hit3->x())<= L3_Parameter::StereoResol()) {
	      HitListLayer3.push_back(*hit3);
	    }
	  }
	  PredictHit4x = ( (Plane4.zfw(0) - L3_Geometry::ztarget()) / Distanz) * 
	    ( (Paar.x() * Plane4.cos()) -
	      (Paar.y() * Plane4.sin()) );
	  for (vector<L3_Hitb>::iterator hit4 = (HitbList[i][j][P4]).begin();
	       hit4 != (HitbList[i][j][P4]).end(); ++hit4) {
	    if ( fabs( PredictHit4x - hit4->x())<= L3_Parameter::StereoResol()) {
	      HitListLayer4.push_back(*hit4);
	    }
	  }
// 	  cout << "TLTRECO: # Hits in 3rd /4th  Layer "
//  	       << HitListLayer3.size() << " / "
// 	       << HitListLayer4.size() << endl;
	  if (HitListLayer3.size() > 0 ) {
	    if (HitListLayer4.size() > 0 ) {
	      for (vector<L3_Hitb>::iterator hit3= HitListLayer3.begin();
		   hit3 != HitListLayer3.end(); ++hit3) {
		for (vector<L3_Hitb>::iterator hit4 = HitListLayer4.begin();
		     hit4 != HitListLayer4.end(); ++hit4) {
		  L3_SpacePoint Paerchen(Paar);
		  Paerchen.add(*hit3, *hit4);
		  if (Paerchen.chiSquare() > 75.) {
		    L3_SpacePoint Paerchen3(Paar);
		    L3_SpacePoint Paerchen4(Paar);
		    Paerchen3.add(*hit3);
		    Paerchen4.add(*hit4);
		    if (Paerchen3.chiSquare() < Paerchen4.chiSquare()) {
		      (SpacePointList[i][j]).push_back(Paerchen3);
		    } else {
		      (SpacePointList[i][j]).push_back(Paerchen4);
		    }
		  } else {
		    (SpacePointList[i][j]).push_back(Paerchen);
		  }
		}
	      }
	    } else {
	      for (vector<L3_Hitb>::iterator hit3= HitListLayer3.begin();
		   hit3 != HitListLayer3.end(); ++hit3) {
		L3_SpacePoint Paerchen(Paar);
		Paerchen.add(*hit3);
		(SpacePointList[i][j]).push_back(Paerchen);
	      }
	    }
	  } else if (HitListLayer4.size() > 0 ) {
	    for (vector<L3_Hitb>::iterator hit4 = HitListLayer4.begin();
		 hit4!= HitListLayer4.end(); ++hit4) {
	      L3_SpacePoint Paerchen(Paar);
	      Paerchen.add(*hit4);
	      (SpacePointList[i][j]).push_back(Paerchen);
	    }	
	  }
	}
      }
    } //sector    
  }//superlayer 
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePointReco STOP, # of points is " << nPoints() << endl;
#endif
}

void L3_Silicon::PurgeSpacePoints() {
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::PurgeSpacePoints" << endl;
#endif
  for (int i=0; i<7; i++) { //Locate SuperLayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      for (vector<L3_SpacePoint>::iterator outer = (SpacePointList[i][j]).begin();
	   outer != (SpacePointList[i][j]).end(); outer++) {
	for (vector<L3_SpacePoint>::iterator inner = outer +1;
	    inner != (SpacePointList[i][j]).end(); ) {
	  if ( *inner == *outer ) (SpacePointList[i][j]).erase(inner);
	  else inner++;
	}
      }
    }
  }
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::PurgeSpacePoints STOP, # of points is " << nPoints() << endl;
#endif
}

class RP_less : public binary_function< L3_SpacePoint, L3_SpacePoint, bool > {
public:
  inline
  bool operator() (const L3_SpacePoint& R , const L3_SpacePoint& S) const {
    return ( R.x() < S.x() ); 
  }
};

void L3_Silicon::SortSpacePoints() {
// Sort SpacePointList
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SortSpacePoints" << endl;
#endif
  for (int i=0; i<7; i++) { //Locate SuperLayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      sort( (SpacePointList[i][j]).begin(), (SpacePointList[i][j]).end(),
	    RP_less() );
#ifdef L3_DEBUG_SILICON
      for (vector<L3_SpacePoint>::iterator RIt 
 	     = (SpacePointList[i][j]).begin();
 	   RIt != (SpacePointList[i][j]).end(); ++RIt) {
	cout << *RIt;
      }
#endif
    }
  }
}
  
class RPWin_less : public binary_function< L3_SpacePoint, L3_Window, bool > {
public:
  inline
  bool operator() (const L3_SpacePoint& R , const L3_Window& P) const {
    return ( R.x() < P.xmin() ); 
  }
};

class RPWin_greater : public binary_function< L3_SpacePoint, L3_Window, bool > {
public:
  inline
  bool operator() (const L3_SpacePoint& R , const L3_Window& P) const {
    return ( R.x() < P.xmax() );
  }
};

void L3_Silicon::SpacePairReco() {
//Reconstruct SpacePairs
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SpacePairReco START" << endl;
#endif
  float dst=0.;
  float x1p=0.;
  float x3p=0.;
  float Distanz=0.;
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      if ((SpacePointList[i][j]).size() == 0) {
	vector<L3_SpacePoint> SpacePairList;
	SpacePairList.reserve(500);
	for (int P=0; P<=1; P++) {
#ifdef lala
#ifdef L3_SVD_NPNP
	  int StP = P+2;
#endif
#ifdef L3_SVD_NPPN
	  int StP = 3-P;
#endif	
#endif
	  int StP = L3_Geometry::getPlaneIndex(i,j,P,0);
	  const L3_Plane& Plane   = L3_Geometry::getPlane(i,j,P);
	  const L3_Plane& StPlane = L3_Geometry::getPlane(i,j,StP);
//          cout << "Plane  :"  << Plane << endl;
// 	    cout << "StPlane:"  << StPlane << endl;
	  Distanz = ((Plane.zfw(0)+StPlane.zfw(0)) * 0.5) - L3_Geometry::ztarget();
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    vector<L3_Hitb> HitsInStereo;
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 1 fuer vertikale Draehte (+/- 2.5 grad zur y-Achse)
	    if ( fabs( Plane.cos() ) > 0.5 ) { // +-2.5 degree layers
	      dst = fabs(2.0 * ( Plane.symx() - Plane.symn() )
			 * 0.0436609429 );
// 	    cout << " TEST : dst= " << dst << endl;
	      x1p = Plane.icos() 
		* ( ( (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget())) * hit->x() )
		    + (Plane.symn() * Plane.sin() ) );
// 	    cout << "> Hit : " << hit->hit() << "   x1p = " << x1p << endl;
	      for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		   Sthit!= (HitbList[i][j][StP]).end(); ++Sthit) {
		x3p = StPlane.icos() 
		  * ( (( Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) * Sthit->x())
		      + (StPlane.symn() * StPlane.sin()) );
// 	      cout << ">>>  StHit : " 
// 		   << Sthit->hit() << "   x3p = " << x3p << endl;
		if ( (x3p >= (x1p - dst - L3_Parameter::margin() ) ) 
		     && ( x3p <= (x1p + L3_Parameter::margin())  )) {
		  HitsInStereo.push_back(*Sthit);
		}
	      }
	    } else { // 90+-2.5 degree layers
// Anmerkung zum Stereowinkel
// fabs(Plane.cos()) ~ 0 fuer horizontale Draehte (90+/-2.5 grad zur y-Achse)
	      dst = fabs(2.0 * ( Plane.sxmx() - Plane.sxmn() )
			 * 0.0436609429 );
// 	    cout << " DST 2=" << dst << endl;
	      x1p = Plane.isin() 
		* ( (Plane.sxmn() * Plane.cos())
		    - (hit->x() * (Distanz / (Plane.zfw(0) - L3_Geometry::ztarget()))) );
	      for (vector<L3_Hitb>::iterator Sthit = (HitbList[i][j][StP]).begin();
		   Sthit != (HitbList[i][j][StP]).end(); ++Sthit) {
		x3p = StPlane.isin() * 
		  ( (StPlane.sxmn() * StPlane.cos())
		    - (Sthit->x() *	(Distanz / (StPlane.zfw(0) - L3_Geometry::ztarget())) ) );
		if ((x3p <= (x1p + dst + L3_Parameter::margin())) 
		    && (x3p >= (x1p - L3_Parameter::margin()))) {
		  HitsInStereo.push_back(*Sthit);
		}
	      }
	    }
	    for (vector<L3_Hitb>::iterator Sthit = HitsInStereo.begin();
		 Sthit != HitsInStereo.end(); ++Sthit) {
	      L3_SpacePoint Paar(*hit, *Sthit);
	      SpacePairList.push_back(Paar);
	    }
	  }
	}
	SpacePoint2List[i][j] = SpacePairList;
      }
      if ((SpacePoint2List[i][j]).size() == 0) {
	vector<L3_SpacePoint> SpacePairList;
	SpacePairList.reserve(500);
	for (int p=0; p<=1; p++) {
	  int P = 2*p;
	  int StP = 1 + 2*p;
	  const L3_Plane& Plane   = L3_Geometry::getPlane(i,j,P);
	  const L3_Plane& StPlane = L3_Geometry::getPlane(i,j,StP);
	  Distanz = ((Plane.zfw(0)+StPlane.zfw(0)) * 0.5) - L3_Geometry::ztarget();
	  for (vector<L3_Hitb>::iterator hit = (HitbList[i][j][P]).begin();
	       hit != (HitbList[i][j][P]).end(); ++hit) {
	    for (vector<L3_Hitb>::iterator Sthit=(HitbList[i][j][StP]).begin();
		 Sthit != (HitbList[i][j][StP]).end(); ++Sthit) {
	      L3_SpacePoint Paar(*hit, *Sthit, 1);
	      SpacePairList.push_back(Paar);
	    }
	  }
	}
	SpacePoint2XList[i][j] = SpacePairList;
      }
    }
  }
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << endl << "L3_Silicon::SpacePairReco STOP" << endl;
#endif
}

void L3_Silicon::SortSpacePairs() {
// Sort SpacePointList
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SortSpacePairs START" << endl;
#endif
  for (int i=0; i<7; i++) { //Locate SuperLayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      sort( (SpacePoint2List[i][j]).begin(), (SpacePoint2List[i][j]).end(),
	    RP_less() );
#ifdef L3_DEBUG_SILICON
      for (vector<L3_SpacePoint>::iterator RIt 
 	     = (SpacePoint2List[i][j]).begin();
 	   RIt != (SpacePoint2List[i][j]).end(); ++RIt) {
	cout << *RIt;
      }
#endif
      sort( (SpacePoint2XList[i][j]).begin(), (SpacePoint2XList[i][j]).end(),
	    RP_less() );
#ifdef L3_DEBUG_SILICON
      for (vector<L3_SpacePoint>::iterator RIt 
 	     = (SpacePoint2XList[i][j]).begin();
 	   RIt != (SpacePoint2XList[i][j]).end(); ++RIt) {
	cout << *RIt;
      }
#endif
    }
  }
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::SortSpacePairs STOP" << endl;
#endif
}

void L3_Silicon::TrackReconstruction(){
//Track reconstruction
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::TrackReconstruction START" << endl;
#endif 
  TrackList.reserve(5000);
  int Trackcounter =0;
  for (int SuperLayer=6; SuperLayer>=2; SuperLayer--) {
    for (int Sector=0; Sector<4; Sector++) {
      for (vector<L3_SpacePoint>::iterator SeedIt 
	     = (SpacePointList[SuperLayer][Sector]).begin();
	   SeedIt != (SpacePointList[SuperLayer][Sector]).end();
	   ++SeedIt) {
	L3_Track Track(Trackcounter, *SeedIt);
	Trackcounter++;
	SearchNextSL(Track,SuperLayer, 0);	
      }
    }
  }
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::TrackReconstruction STOP, # of tracks " << nTracks() << endl;
#endif 
}

void L3_Silicon::TrackReconstruction98() {
//Track reconstruction
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::TrackReconstruction98 START" << endl;
#endif 
  TrackList.reserve(3000);
  int Trackcounter =0;
  for (int layer=6; layer>=2; layer--) {
    vector<L3_Track> tmpTrackList;
    tmpTrackList.reserve(2000);
    for (int i=0; i<4; i++) {
      for (vector<L3_SpacePoint>::iterator SeedIt 
	     = (SpacePointList[layer][i]).begin();
	   SeedIt != (SpacePointList[layer][i]).end();
	   ++SeedIt) {
	L3_Track Track(Trackcounter, *SeedIt);
	Trackcounter++;
	//	cout << " Trackseed : " << Track << endl;
	int depth = 0;
	for (int j=0; j<4; j++) {
	  SearchNextSL98(Track, layer, j, depth, tmpTrackList);
	}
      }
    }
#ifdef L3_VERBOSE
    cout << " " << tmpTrackList.size() 
	 << " trackssegments starting in superlayer " << layer << endl;
#endif
    for (vector<L3_Track>::iterator TrackIt = tmpTrackList.begin();
	 TrackIt != tmpTrackList.end(); ++TrackIt) {
//      TrackIt->refit();
      TrackList.push_back(*TrackIt);
//      cout << Track;
    }
  }
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::TrackReconstruction98 STOP" << endl;
#endif 
}

int npointsOld(vector<L3_SpacePoint> &l,L3_Window &SwinMean,int show)
{
  int n = 0;
  for (vector<L3_SpacePoint>::iterator
	 SpIt = lower_bound( l.begin(),
			     l.end(),
			     SwinMean, RPWin_less() ) ;
       SpIt != lower_bound( l.begin(),
			    l.end(),
			    SwinMean, RPWin_greater() ) ; 
       ++SpIt)
    {
      if(show) cout << SpIt->x() << "\t" << SpIt->y() << endl;
      n++;
    }
  return n;
}

int npointsNew(vector<L3_SpacePoint> &l,L3_Window &SwinMean,int show)
{
  int from = 0;
  int to = -1;
  int n = l.size();
  if( n>0 )
    {
      int N = n+10;
      int low = 0;
      int high = n-1;      
      int guess=-1,oldguess;
      do {    
	oldguess = guess;
	guess = (low+high)/2;
	//cout << low << ' ' << guess << ' ' << high << endl;
	if( l[guess].x() < SwinMean.xmin() )	  
	  low = guess+1;	  
	else if( l[guess].x() > SwinMean.xmax() )	  
	  high = guess-1;	  
	else
	  {	
	    from = guess-1;
	    to = guess+1;
	    //cout << from << ' ' << guess << ' ' << to << endl;
	    for(;from>=0 && l[from].x()>=SwinMean.xmin();from--);
	    from++;
	    for(;to<n && l[to].x()<=SwinMean.xmax();to++);
	    to--;
	    //cout << from << ' ' << to << endl;
	    break;
	  }
	if(N==1) 
	  {
	    cout << "**** fishy start ****" << endl;
	    cout << low << ' ' << guess << ' ' << high << endl;
	    for(vector<L3_SpacePoint>::iterator i = l.begin();i!=l.end();i++)
	      {
		int t = i-l.begin();
		cout << t << "\t" << i->x() << "\t" << i->y() << endl;
	      }
	    cout << SwinMean << endl;	   
	    cout << "**** fishy ****" << endl;
	  }
      } while(--N>0 && guess!=oldguess);   
    }
  n = (from>to) ? 0 : (to-from+1);
  if(show)
    {
      for(int i=from;i<=to;i++)
	{
	  L3_SpacePoint *SpIt = &l[i];
	  cout << SpIt->x() << "\t" << SpIt->y() << endl;
	}
    }
  return n;
}

inline void findHits(int &from,int &to,vector<L3_SpacePoint> &l,L3_Window &SwinMean)
{
  int n = l.size();
  if( n>0 )
    {
      int N = n+10;
      int low = 0;
      int high = n-1;      
      int guess=-1,oldguess;
      do {    
	oldguess = guess;
	guess = (low+high)/2;
	//cout << low << ' ' << guess << ' ' << high << endl;
	if( l[guess].x() < SwinMean.xmin() )	  
	  low = guess+1;	  
	else if( l[guess].x() > SwinMean.xmax() )	  
	  high = guess-1;	  
	else
	  {	
	    from = guess-1;
	    to = guess+1;
	    //cout << from << ' ' << guess << ' ' << to << endl;
	    for(;from>=0 && l[from].x()>=SwinMean.xmin();from--);
	    from++;
	    for(;to<n && l[to].x()<=SwinMean.xmax();to++);
	    to--;
	    //cout << from << ' ' << to << endl;
	    break;
	  }
	if(N==1) 
	  {
	    cout << "**** fishy start ****" << endl;
	    cout << low << ' ' << guess << ' ' << high << endl;
	    for(vector<L3_SpacePoint>::iterator i = l.begin();i!=l.end();i++)
	      {
		int t = i-l.begin();
		cout << t << "\t" << i->x() << "\t" << i->y() << endl;
	      }
	    cout << SwinMean << endl;	   
	    cout << "**** fishy ****" << endl;
	  }
      } while(--N>0 && guess!=oldguess);   
    }
}

void tell(L3_Track &t)
{
  cout << "Rejecting " << ++xcnt << "-th track:";
  cout << t.chisqr() << ' ' <<  t.chisqrCut() << ' ' << t.numHits() << endl;
  vector< L3_SpacePoint> v = t.getPunktList();
  for(vector<L3_SpacePoint>::iterator i=v.begin();i!=v.end();i++)
    {
      int super = i->gesl();
      cout << super << "(";
      vector<L3_Hitb> h = i->HList();
      cout << h.size() << ") ";	      		  
      for(vector< L3_Hitb >::iterator q=h.begin();q!=h.end();q++)
	{
	  int sector=q->sec();
	  int lay=q->lay(); 
	  const L3_Plane& plane = L3_Geometry::getPlane(super-1,sector-1,lay-1);
	  char c = (fabs( plane.cos()) > 0.5 ) ? 'y' : 'x';
	  cout << sector << ',' << lay << ',' << c << ' ';
	}
      cout << endl;
    }
}

int L3_Silicon::SearchNextSL(L3_Track& Track, const int& layer, int depth){
  depth++;      
  int SLtoSearch = layer-depth;
  int Nr_of_Tracks_reconstructed = 0;
  L3_Window SearchWindow;

  for(int sec=0;sec<4;sec++){  //loop over all sectors
    if (depth > 1) {
      Track.fit( L3_Geometry::SLz(SLtoSearch, sec) );
      SearchWindow = Track.window();
    } else {
      SearchWindow = Track.window( L3_Geometry::SLz(SLtoSearch, sec) );
    }
//   cout << " Track: " << Track << endl;
//   cout << " layer: " << layer << endl;
//   cout << " sec: " << sec << endl;
//   cout << " depth: " << depth << endl;
//   cout << " SearchWindow " << SearchWindow << endl;
//   for (vector<L3_SpacePoint>::iterator
// 	 SpIt = (SpacePointList[SLtoSearch][sec]).begin();
//        SpIt != (SpacePointList[SLtoSearch][sec]).end(); ++SpIt) {
//     cout << " Spacepoint: " << *SpIt;
//   }
#ifdef ULLI 
    //const L3_Window &w = L3_Geometry::getSector(SLtoSearch,sec);
    //if( w.z()>-10.0 && !w.overlap(SearchWindow)) continue; //skip this sector
    int from = 0;
    int to = -1;
    findHits(from,to,SpacePointList[SLtoSearch][sec],SearchWindow);
    for(int i=from;i<=to;i++){   //loop over all spacepoints found in window
      L3_SpacePoint *SpIt = &SpacePointList[SLtoSearch][sec][i];
#else
    for (vector<L3_SpacePoint>::iterator
	   SpIt = lower_bound( (SpacePointList[SLtoSearch][sec]).begin(),
			       (SpacePointList[SLtoSearch][sec]).end(),
			       SearchWindow, RPWin_less() ) ;
	 SpIt != lower_bound( (SpacePointList[SLtoSearch][sec]).begin(),
			      (SpacePointList[SLtoSearch][sec]).end(),
			      SearchWindow, RPWin_greater() ) ; 
	 ++SpIt) {
#endif
      if ( (SpIt->y() >= SearchWindow.ymin()) &&
	   (SpIt->y() <= SearchWindow.ymax()) ) {      
	L3_Track newTrack(Track);
	newTrack.add(*SpIt);
	if (SLtoSearch > 0) { // Noch weitere Superlage anzuschauen
	  int Nr = SearchNextSL(newTrack,layer,depth);	
	  if (Nr == 0) { // No Spacepoint in Searchwindow in Next Superlayer
	    if (depth > 1) {
	      newTrack.refit();
	      if( newTrack.chisqr() < newTrack.chisqrCut() ){
		TrackList.push_back(newTrack);
		Nr_of_Tracks_reconstructed++;
	      } else tell(newTrack);
	    }
	  }
	} else { // reached front of silicon or track touches 5 Superlayers
	  newTrack.refit();
	  if( newTrack.chisqr() < newTrack.chisqrCut() ){
	    TrackList.push_back(newTrack);
	    Nr_of_Tracks_reconstructed++;
	  } else tell(newTrack);
	}
      }
    }
  }
  return Nr_of_Tracks_reconstructed;
}

int L3_Silicon::SearchNextSL98(L3_Track& Track, const int& layer, const int& sec,
			       int depth, vector<L3_Track>& tmpTrackList) {
  depth++;
  int SLtoSearch = layer-depth;
#ifdef L3_DEBUG_SILICON
  cout << " SLtoSearch = " << SLtoSearch << endl;
#endif
  int Nr_of_Tracks_reconstructed = 0;
//  cout << " SLtoSearch (z)= " << L3_Geometry::SLz((SLtoSearch),sec) << endl;
  if ( L3_Geometry::SLz(SLtoSearch, sec) == 0. ) {
    return 0;
  }
  L3_Window SwinMean = Track.window( L3_Geometry::SLz(SLtoSearch, sec) );
  vector<L3_SpacePoint>::iterator SPListbegin;
  vector<L3_SpacePoint>::iterator SPListend;
  SPListbegin = (SpacePointList[SLtoSearch][sec]).begin();
  SPListend = (SpacePointList[SLtoSearch][sec]).end();
#ifdef L3_ONLINE_MODE
  if ( SLtoSearch != 2) {
#endif
#ifdef L3_OFFLINE_MODE
  if ( SLtoSearch != 0) {
#endif
    SPListbegin = (SpacePoint2XList[SLtoSearch][sec]).begin();
    SPListend = (SpacePoint2XList[SLtoSearch][sec]).end();
  }
//   if ( SLtoSearch == 2 &&  (SpacePointList[SLtoSearch][sec]).size() == 0 ) {
//     if (  (SpacePoint2List[SLtoSearch][sec]).size() == 0 ) {
//       SPListbegin = (SpacePoint2XList[SLtoSearch][sec]).begin();
//       SPListend = (SpacePoint2XList[SLtoSearch][sec]).end();
//     } else {
//       SPListbegin = (SpacePoint2List[SLtoSearch][sec]).begin();
//       SPListend = (SpacePoint2List[SLtoSearch][sec]).end();
//     }
//   } else {
//     SPListbegin = (SpacePointList[SLtoSearch][sec]).begin();
//     SPListend = (SpacePointList[SLtoSearch][sec]).end();
//   }
//   int forks=0;
//   if ( SLtoSearch == 1 || SLtoSearch == 2 ) {
//     SPListbegin = (SpacePointList[SLtoSearch][sec]).begin();
//     SPListend = (SpacePointList[SLtoSearch][sec]).end();
//     distance( lower_bound( SPListbegin, SPListend, SwinMean, RPWin_less() ),
// 	      lower_bound( SPListbegin, SPListend, SwinMean, RPWin_greater() ),
// 	      forks);
//     if (forks == 0) {
//       SPListbegin = (SpacePoint2List[SLtoSearch][sec]).begin();
//       SPListend = (SpacePoint2List[SLtoSearch][sec]).end();
//       distance( lower_bound( SPListbegin, SPListend, SwinMean, RPWin_less() ),
// 		lower_bound( SPListbegin, SPListend, SwinMean, RPWin_greater() ),
// 		forks);
//       if (forks == 0) {
// 	SPListbegin = (SpacePoint2XList[SLtoSearch][sec]).begin();
// 	SPListend = (SpacePoint2XList[SLtoSearch][sec]).end();
//       }
//     }
//   } else {
//     SPListbegin = (SpacePointList[SLtoSearch][sec]).begin();
//     SPListend = (SpacePointList[SLtoSearch][sec]).end();
//   }
//   cout << "Window: xmin = " << SwinMean.xmin(); 
//   cout << "  xmax = " << SwinMean.xmax(); 
//   cout << "    ymin = " << SwinMean.ymin(); 
//   cout << "  ymax = " << SwinMean.ymax() << endl; 
//   for (vector<L3_SpacePoint>::iterator SpIt = SPListbegin;
//        SpIt != SPListend; ++SpIt) {
//     cout << " Spacpoint: " << *SpIt;
//   }
//   int forks = 0;
//   distance( lower_bound( SPListbegin, SPListend, SwinMean, RPWin_less() ),
// 	    lower_bound( SPListbegin, SPListend, SwinMean, RPWin_greater() ),
// 	    forks);
//   cout << " Number of Spacepoints in Window: " << forks << endl;
  int forks = 0;
  for (vector<L3_SpacePoint>::iterator SpIt 
	 = lower_bound( SPListbegin, SPListend, SwinMean, RPWin_less() );
       SpIt != lower_bound( SPListbegin, SPListend,
			    SwinMean, RPWin_greater() ); ++SpIt) {
    L3_Window Swin;
    if ( SpIt->numHits() < 4 ) {
      Swin = Track.window(SpIt->z());
    } else {
      Swin = SwinMean;
    }
    if ( (SpIt->y() >= Swin.ymin()) &&
	 (SpIt->y() <= Swin.ymax()) ) {
      L3_Track newTrack(Track);
      newTrack.add(*SpIt);
      forks++;
#ifdef L3_DEBUG_SILICON
      cout << "newTrack:" << newTrack << endl;
#endif
      if (SLtoSearch > 0) { // Noch weitere Superlage anzuschauen
	int Nr = 0;
	for (int nsec=0; nsec<4; nsec++) {
	  Nr += SearchNextSL98(newTrack, layer, nsec, depth, tmpTrackList);
	}
	if (Nr == 0) { // No Spacepoint in Searchwindow in Next Superlayer
	  if (depth > 1) {
	    newTrack.refit();
	    tmpTrackList.push_back(newTrack);
	    Nr_of_Tracks_reconstructed++;
	  }
	}
      } else { // reached front of silicon
	newTrack.refit();
	tmpTrackList.push_back(newTrack);
	Nr_of_Tracks_reconstructed++;
      }
    }
  }
  if (forks == 0) {
#ifdef L3_ONLINE_MODE
    if ( SLtoSearch != 2) {
#endif
#ifdef L3_OFFLINE_MODE
    if ( SLtoSearch != 0) {
#endif
      SPListbegin = (SpacePoint2List[SLtoSearch][sec]).begin();
      SPListend = (SpacePoint2List[SLtoSearch][sec]).end();
      for (vector<L3_SpacePoint>::iterator SpIt 
	     = lower_bound( SPListbegin, SPListend, SwinMean, RPWin_less() );
	   SpIt != lower_bound( SPListbegin, SPListend,
				SwinMean, RPWin_greater() ); ++SpIt) {
	L3_Window Swin = Track.window(SpIt->z());
	if ( (SpIt->y() >= Swin.ymin()) &&
	     (SpIt->y() <= Swin.ymax()) ) {
	  L3_Track newTrack(Track);
	  newTrack.add(*SpIt);
	  forks++;
	  if (SLtoSearch > 0) { // Noch weitere Superlage anzuschauen
	    int Nr = 0;
	    for (int nsec=0; nsec<4; nsec++) {
	      Nr += SearchNextSL98(newTrack, layer, nsec, depth, tmpTrackList);
	    }
	    if (Nr == 0) { // No Spacepoint in Searchwindow in next Superlayer
	      if (depth > 1) {
		newTrack.refit();
		tmpTrackList.push_back(newTrack);
		Nr_of_Tracks_reconstructed++;
	      }
	    }
	  } else { //reached front of silicon
	    newTrack.refit();
	    tmpTrackList.push_back(newTrack);
	    Nr_of_Tracks_reconstructed++;
	  }
	}
      }
    }
  }
  if (forks == 0) {
#ifdef L3_ONLINE_MODE
    if ( SLtoSearch != 2) {
#endif
#ifdef L3_OFFLINE_MODE
    if ( SLtoSearch != 0) {
#endif
      SPListbegin = (SpacePoint2XList[SLtoSearch][sec]).begin();
      SPListend = (SpacePoint2XList[SLtoSearch][sec]).end();
      for (vector<L3_SpacePoint>::iterator SpIt 
	     = lower_bound( SPListbegin, SPListend, SwinMean, RPWin_less() );
	   SpIt != lower_bound( SPListbegin, SPListend,
				SwinMean, RPWin_greater() ); ++SpIt) {
	L3_Window Swin = Track.window(SpIt->z());
	if ( (SpIt->y() >= Swin.ymin()) &&
	     (SpIt->y() <= Swin.ymax()) ) {
	  L3_Track newTrack(Track);
	  newTrack.add(*SpIt);
	  forks++;
	  if (SLtoSearch > 0) { // Noch weitere Superlage anzuschauen
	    int Nr = 0;
	    for (int nsec=0; nsec<4; nsec++) {
	      Nr += SearchNextSL98(newTrack, layer, nsec, depth, tmpTrackList);
	    }
	    if (Nr == 0) { // No Spacepoint in Searchwindow in Next Superlayer
	      if (depth > 1) {
		newTrack.refit();
		tmpTrackList.push_back(newTrack);
		Nr_of_Tracks_reconstructed++;
	      }
	    }
	  } else { // reached front of silicon
	    newTrack.refit();
	    tmpTrackList.push_back(newTrack);
	    Nr_of_Tracks_reconstructed++;
	  }
	}
      }
    }
  }
#ifdef L3_DEBUG_SILICON
  cout << " Nr of Tracks reconstr: " <<  Nr_of_Tracks_reconstructed << endl;
#endif
  return Nr_of_Tracks_reconstructed;
}

// TRACKinfo: Utility structure for L3_Silicon::findValidTracks
TRACKinfo::TRACKinfo() {}
TRACKinfo::TRACKinfo(double p, int n): the_probability( p ), the_numHits( n ) {}

// - Find a set of L3_Track's, which do not exclude each other too much -
// Returns number of reasonable tracks
// Arguments:
//   maxCommonHits: maximum number of shared hits of two tracks 5
//   maxProbDiff:   maximal probability loss allowed for additional hits 0.2
//   minProb:       minimal accepted probability 0.0001
void L3_Silicon::findValidTracks() {
  int maxCommonHits = L3_Parameter::maxCommonHits();
  float maxProbDiff = L3_Parameter::maxProbDiff();
  float minProb = L3_Parameter::minProb();
  vector<L3_Track> validTracks;
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::findValidTracks START" << endl;
#endif 
#ifdef L3_EGCS
  map<L3_Track,TRACKinfo> info;
  for (vector<L3_Track>::iterator itrack = TrackList.begin();
       itrack != TrackList.end(); ++itrack ) {    
    float p = itrack->prob();
    if (p > minProb) {
      int maxSoFar = -1;
      L3_Track soFar;
      //find track with highest number of common hits for this candidate
      for ( vector<L3_Track>::iterator ifound = validTracks.begin();
	    ifound != validTracks.end(); ++ifound ) {
	if (fabs(itrack->mx() - ifound->mx()) < 0.02) {
	  if (fabs(itrack->my() - ifound->my()) < 0.02) {
	    int commonHits = itrack->getCommonHits(*ifound);
	    if(commonHits > maxSoFar) {
	      maxSoFar = commonHits;
	      soFar = *ifound;
	    }
	  }
	}
      }
      if (maxSoFar <= maxCommonHits) { //accept as a new track candidate
	info[*itrack] = TRACKinfo(p, itrack->numHits());
	validTracks.push_back(*itrack);
      } else if (info[*itrack].the_numHits > info[soFar].the_numHits) {  //replace 
	if (p + maxProbDiff > info[soFar].the_probability) {
	  *find(validTracks.begin(), validTracks.end(), soFar) = *itrack;
	}
      }
    }
  }
#endif
#ifdef L3_GCC
  vector<TRACKinfo> info;
  for (vector<L3_Track>::iterator itrack = TrackList.begin();
       itrack != TrackList.end(); ++itrack ) {
    int nHits = itrack->numHits();
    float p = itrack->prob();
    if ( p > minProb ) {
      int maxSoFar = -1;
      int soFar =0;
      int count =0;
      for ( vector<L3_Track>::iterator iFound = validTracks.begin();
	    iFound != validTracks.end(); ++iFound ) {
	if (fabs(itrack->mx() - iFound->mx()) < 0.02) {
	  if (fabs(itrack->my() - iFound->my()) < 0.02) {
	    int commonHits = itrack->getCommonHits(*iFound);
	    if (commonHits > maxSoFar) {
	      maxSoFar = commonHits;
	      soFar = count;
	    }
	  }
	}
	count++;
      }
      if ( maxSoFar <= maxCommonHits ) {
	validTracks.push_back(*itrack);
	info.push_back(TRACKinfo(p, nHits));
      } else if ( (p + maxProbDiff) > info[soFar].the_probability
		  && (nHits > info[soFar].the_numHits) ) {
	validTracks[soFar] = *itrack;
      }
    }
  }
#endif
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::findValidTracks STOP: #tracks = " << validTracks.size() << endl;
#endif 
  TrackList = validTracks;
  //assign a number 1....N to each track
  for(vector<L3_Track>::iterator q = TrackList.begin();q!=TrackList.end();q++)
    {
      q->setnr( q - TrackList.begin() + 1 );
    }
}

void L3_Silicon::FillHits() {
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::FillHits" << endl;
#endif
  //When running in ARTE we get our hits from the HITB-Table
#ifdef L3_ARTE_FRAME
// GEPT workaround until silarte is fixed
  if( !ArteTable<MCIN>::booked() ){  //these are real data
    for (ArteTable<GEDE>::iterator PIt=ArteTable<GEDE>::begin(); 
	 PIt != ArteTable<GEDE>::end(); ++PIt ) {
      ArtePointer<GEDE>& Pl = *PIt;
      if (Pl->cmp == 1) {
	Pl->hit1 = ArtePointer<HITB>(0);
	Pl->hitl = ArtePointer<HITB>(-1);
      }
    }
  }
  if (ArteTable<HITB>::booked() && ArtePointer<HITB>::nrow()>0 ) {
    for (ArteTable<HITB>::iterator hitbIt=ArteTable<HITB>::begin(); 
	 hitbIt!=ArteTable<HITB>::end(); ++hitbIt ) {
      ArtePointer<HITB>& hitb = *hitbIt;
      ArtePointer<GEDE> Plane(hitb->gede);
      if (Plane->cmp == 1) {
	L3_Hitb hit(hitb);
	if( !ArteTable<MCIN>::booked() ){ //we are processing real data!
	  if (*(Plane->hit1) == ArtePointer<HITB>(0)) {
	    Plane->hit1 = ArtePointer<HITB>( hit.hit() );
	    Plane->hitl = ArtePointer<HITB>( hit.hit() );
	  }
	  if (*(Plane->hitl) < ArtePointer<HITB>(hit.hit()) ) {
	    Plane->hitl = ArtePointer<HITB>( hit.hit() );
	  }
	  if (*(Plane->hit1) > ArtePointer<HITB>(hit.hit()) ) {
	    Plane->hit1 = ArtePointer<HITB>( hit.hit() );
	  }
	}
	if (hit.z() < 180) {
	  HitbList[hit.gesl() -1][hit.sec() -1][hit.lay() -1].push_back(hit);
	}
      }
    }
  }
#endif  

#ifdef L3_SLP_FRAME
  int nhitb = 0;
  VDS_hitb* hitb = vdsslt_fillHITB(&nhitb);
  for (int i=0; i < nhitb; i++) {
    L3_Hitb hit(i, hitb[i]);
    if (hit.gesl() < 8 && hit.gesl()> -1) {
#ifdef L3_DEBUG_SILICON
      cout << "Hit: "   << hit.hit()  << "  "
	   << " gesl= " << hit.gesl() << " sec= "  << hit.sec()
	   << " lay= "  << hit.lay()  << " gede= " << hit.gede() << endl;
#endif 	  
      HitbList[hit.gesl() -1][hit.sec() -1][hit.lay() -1].push_back(hit);
    }
  }
#endif

//loop over all sectors, delete those with too many hits
#ifdef L3_EGCS
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      for (int k=0; k<4; k++) { //Locate Plane
	if ( (HitbList[i][j][k]).size() > 45 ) {
	  (HitbList[i][j][k]).clear();
	}
      }
    }
  }
#endif
#ifdef L3_GCC
  for (int i=0; i<7; i++) { //Locate Superlayer
    for (int j=0; j<4; j++) { //Locate Quadrant
      for (int k=0; k<4; k++) { //Locate Plane
	if ( (HitbList[i][j][k]).size() > 30 ) {
	  (HitbList[i][j][k]).erase((HitbList[i][j][k]).begin(), (HitbList[i][j][k]).end());;
	}
      }
    }
  }
#endif
#ifdef L3_VERBOSE
  if(L3_Parameter::Trace()) cout << "L3_Silicon::FillHits: found " << nHitbs() << " hits." << endl;
#endif
}

void L3_Silicon::print() const {
#ifdef L3_VERBOSE
  //print(HList());
  //print(Points());
  //print(Tracks());
#endif
}

void L3_Silicon::printRecoSummary() const {
#ifdef L3_VERBOSE
  cout << "HITBs in SiVD            : " << nHitbs()  << endl; 
  cout << "Reconstructed Spacepoints: " << nPoints() << endl;
  cout << "Reconstructed Tracks     : " << nTracks() << endl;
#endif
}

void L3_Silicon::print(const vector<L3_Hitb>& HitbList) const {
#ifdef L3_VERBOSE
  cout << "Silicon Hit List" << endl;
  cout << "==================================================================="
       << endl;
  for (vector<L3_Hitb>::const_iterator HitIt = HitbList.begin();
       HitIt != HitbList.end(); ++HitIt) {
    cout << "Hit x: " << *HitIt << " gede: " << HitIt->gede() << endl;
  }
#endif
}

void L3_Silicon::print(const vector<L3_SpacePoint>& SpacePointList) const {
#ifdef L3_VERBOSE
  cout << "Reconstructed Spacepoints" << endl;
  cout << "==================================================================="
       << endl;
  cout << "Nr.\t#Hits\t Layer1\t\t Layer2\t\t Layer3\t\t Layer4\t\t\t\tL3_SpacePoint" << endl;
  cout << "-------------------------------------------------------------------"
       << endl;
  for (vector<L3_SpacePoint>::const_iterator SpacePointListIt =
	 SpacePointList.begin(); SpacePointListIt != SpacePointList.end();
       ++SpacePointListIt) {
    int SpacePointNumber=0;
    distance(SpacePointList.begin(), SpacePointListIt, SpacePointNumber);
    if (SpacePointNumber <= 300) {
      cout << (SpacePointNumber+1);
      cout << *SpacePointListIt;
    }
  }
  cout << "*******************************************************************"
       << endl;
#endif
}

void L3_Silicon::print(const vector<L3_Track>& TrackList) const {
#ifdef L3_VERBOSE
  cout << "Reconstructed Tracks" << endl;
  cout << "==================================================================="
       << endl;
  cout << "Nr.\tTrack" << endl;
  cout << "-------------------------------------------------------------------"
       << endl;
  for (vector<L3_Track>::const_iterator SpurIt = TrackList.begin();
       SpurIt != TrackList.end(); ++SpurIt) {
    int TrackNumber=0;
    distance(TrackList.begin(), SpurIt, TrackNumber);
    if (TrackNumber <= 100) {
      cout << (TrackNumber+1);
      cout << *SpurIt;
      cout << endl;
    }
  }
  cout << "*******************************************************************"
       << endl;
#endif
}










