#include "L3_Trigger.hh"
#ifdef L3_GCC
#include <string.h>
#include <iostream.h>
#endif
#ifdef L3_EGCS
#include <cstring>
#include <iostream>
#endif
#include "L3_Macros.hh"
#include "L3_SLP.hh"
#include "L3_Geometry.hh"
#include "L3_Parameter.hh"
#include "L3_Silicon.hh"

#include "L3_State.hh"

L3_Trigger::L3_Trigger() {  
}

L3_Trigger::L3_Trigger(const L3_Trigger& a) { *this = a; }

L3_Trigger& L3_Trigger::operator = (const L3_Trigger& a) {
  State = a.State;
  Timer = a.Timer;
#ifndef L3_NO_RHP
  Monitor = a.Monitor;
#endif
#ifdef L3_ARTE_FRAME
  ArteInterface = a.ArteInterface;
#endif
  DTLT_Table = a.DTLT_Table;
  return *this;
}

void L3_Trigger::init(int argc, char** argv) {  
  L3_Parameter::init(argc,argv);
  L3_Geometry::initialize();
  L3_SpacePoint::initFitMatrizen();
  L3_Origin::init();
#ifndef L3_NO_RHP
  Monitor.init();
#endif
  L3_SLP::init();
}

void L3_Trigger::process() {
  //Timer.reset();
  L3_SLP::fill();          
  Timer.start_Trigger();
  Event.release();
  Event.fill();
  Event.reconstruct(Timer);
  trigger();
  State.update(Event);
  DTLT_Table.fill(Event, State);
  Timer.stop_Trigger();
  if(L3_Parameter::Info()) Timer.print();
  State.read(Timer);

#ifdef L3_PRINT_EVENT_SUMMARY
  if(L3_Parameter::Info()){
    printEventSummaryHeader();
    State.print();
    State.printEVHD();
    Event.print();
    Event.printRecoSummary();
    printEventSummaryFooter();
  }
#endif
#ifdef L3_ARTE_FRAME
  ArteInterface.fill(Event);
#endif
#ifndef L3_NO_RHP
  Monitor.put(Event);
  Monitor.put(State);
  Monitor.put(Timer);
#endif  
  L3_SLP::clear();
}

void L3_Trigger::update() {
  L3_SpacePoint::setFitMatrizen();
}

void L3_Trigger::stop() {
#ifndef L3_NO_RHP
  Monitor.stop();
#endif
  if(L3_Parameter::Info()) State.printSummary();
}

L3_Event L3_Trigger::getEvent() const { return Event; }
const L3_Timer& L3_Trigger::getTimer() const { return Timer; }

void L3_Trigger::trigger() {
}

void L3_Trigger::printEventSummaryHeader() const {
  cout << "*******************************************************************"
       << endl;
  cout << "*                 L3_Trigger: EventSummary                        *"
       << endl;
  cout << "*******************************************************************"
       << endl;
}

void L3_Trigger::printEventSummaryFooter() const {
  cout << "*******************************************************************"
       << endl;
  cout << "*                    End of EventSummary                          *"
       << endl;
  cout << "*******************************************************************"
       << endl;
}




