#include "L3_Vertex.hh"


L3_Vertex::L3_Vertex(const L3_Vertex& a) { operator=(a); }

L3_Vertex::L3_Vertex(const L3_Track& track1, const L3_Track& track2) {
  AssociatedTracks.push_back(track1);
  AssociatedTracks.push_back(track2);
  fit2(track1, track2);
}

L3_Vertex::L3_Vertex(const L3_Vtx &a)
{
  AssociatedTracks = a.AssociatedTracks;
  Vertex.setX( a.x() );
  Vertex.setY( a.y() );
  Vertex.setZ( a.z() );
  for(int i=0;i<3;i++) 
    {
      covmatrix[i][i] = a.Cov(i,i);
      for(int k=0;k<i;k++)
	{
	  covmatrix[i][k] = a.Cov(i,k);
	  covmatrix[k][i] = a.Cov(k,i);
	}
    }
  chiSquare = a.chi2();
  Status = a.Status;
}

L3_Vertex& L3_Vertex::operator = (const L3_Vertex& a) {
  AssociatedTracks = a.AssociatedTracks;
  Vertex = a.Vertex;
  for(int i=0;i<3;i++) 
    {
      covmatrix[i][i] = a.covmatrix[i][i];
      for(int k=0;k<i;k++)
	{
	  covmatrix[i][k] = a.covmatrix[i][k];
	  covmatrix[k][i] = a.covmatrix[k][i];
	}
    }
  chiSquare = a.chiSquare;
  Status = a.Status;
  return *this;
}

void L3_Vertex::fit2(const L3_Track& track1, const L3_Track& track2) {
  double tmp,tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8;
  double dxv,dyv,dzv;
  double a,b,c,d,e,f1,f2,f3,D;
  double dx1,dx2,dy1,dy2;
  double e11[2][2],e21[2][2],e22[2][2];
  double sigx1,sigx2,sigy1,sigy2;
  double dsigx1,dsigx2,dsigy1,dsigy2;
  
  a=b=c=d=e=D=0.;
  tmp1 = track1.mx()-track2.mx();
  tmp3 = track1.my()-track2.my();
  tmp = (tmp1*tmp1 + tmp3*tmp3);
  
  int q=11;
  if (tmp > 1.0E-10) {
    tmp2 = track1.x()-track2.x();
    tmp4 = track1.y()-track2.y();
    
    Vertex.setZ(-(tmp1*tmp2 + tmp3*tmp4) / tmp);
    
    tmp5 = track1.x() + track1.mx() * Vertex.z();
    tmp6 = track1.y() + track1.my() * Vertex.z();
    tmp7 = track2.x() + track2.mx() * Vertex.z();
    tmp8 = track2.y() + track2.my() * Vertex.z();
    //cout << "L3_Vertex::fit2  z = "  << Vertex.z() << endl;
    //cout << "L3_Vertex::fit2  x = "  << (tmp5+tmp7) * 0.5 << endl;
    //cout << "L3_Vertex::fit2  y = "  << (tmp6+tmp8) * 0.5 << endl;
    double D2xy = (tmp5-tmp7)*(tmp5-tmp7)+(tmp6-tmp8)*(tmp6-tmp8);
    double CUTD2xy= 100000.;
    if (D2xy<CUTD2xy) { // Jpsi trigger
// first guess on Vertex
      Vertex.setX((tmp5+tmp7)*0.5);
      Vertex.setY((tmp6+tmp8)*0.5);
// Errors from track 1
      e11[0][0] =   track1.Cmx(); //t
      e21[0][0] = - track1.Cmxx();//tx
      e22[0][0] =   track1.Cx(); //x

      e11[0][1] =   track1.Cmx(); //t
      e21[0][1] = - track1.Cmxx();//tx
      e22[0][1] =   track1.Cx(); //x
// Errors from track 2
      e11[1][0] =   track2.Cmx(); //t
      e21[1][0] = - track2.Cmxx();//tx
      e22[1][0] =   track2.Cx(); //x

      e11[1][1] =   track2.Cmx(); //t
      e21[1][1] = - track2.Cmxx();//tx
      e22[1][1] =   track2.Cx(); //x

      q=0;
      dxv = dyv = dzv = 1.;
      while (q<10) { // next Vertex position
        ++q;
// propagate errors from z=0 to z=zv                
        sigx1= e11[0][0]*Vertex.z()*Vertex.z() 
	  + 2. * e21[0][0]*Vertex.z() + e22[0][0];
        sigy1= e11[0][1]*Vertex.z()*Vertex.z() 
	  + 2. * e21[0][1]*Vertex.z() + e22[0][1];
        sigx2= e11[1][0]*Vertex.z()*Vertex.z() 
	  + 2. * e21[1][0]*Vertex.z() + e22[1][0];
        sigy2= e11[1][1]*Vertex.z()*Vertex.z() 
	  + 2. * e21[1][1]*Vertex.z() + e22[1][1];
// Newton-Raphson Method
        dsigx1 =2.*(e11[0][0]*Vertex.z() + e21[0][0]);
        dsigy1 =2.*(e11[0][1]*Vertex.z() + e21[0][1]);
        dsigx2 =2.*(e11[1][0]*Vertex.z() + e21[1][0]);
        dsigy2 =2.*(e11[1][1]*Vertex.z() + e21[1][1]);
                        
        dx1 = track1.x()+track1.mx()*Vertex.z()-Vertex.x();
        dx2 = track2.x()+track2.mx()*Vertex.z()-Vertex.x();
        dy1 = track1.y()+track1.my()*Vertex.z()-Vertex.y();
        dy2 = track2.y()+track2.my()*Vertex.z()-Vertex.y();
              
        f1 = -2. * (dx1/sigx1+dx2/sigx2);
        f2 = -2. * (dy1/sigy1+dy2/sigy2);
        f3 =  2. * (dx1*track1.mx()/sigx1 + dx2*track2.mx()/sigx2
		    + dy1*track1.my()/sigy1 + dy2*track2.my()/sigy2);
        a = 2./sigx1 + 2./sigx2;
        b = -2.*track1.mx()/sigx1 - 2.*track2.mx()/sigx2;
        c = 2./sigy1 + 2./sigy2;
        d = -2.*track1.my()/sigy1 - 2.*track2.my()/sigy2;
        e = 2. * (track1.mx()*track1.mx()/sigx1
		  + track2.mx()*track2.mx()/sigx2
		  + track1.my()*track1.my()/sigy1
		  + track2.my()*track2.my()/sigy2);
        D = a*(c*e-d*d)-b*b*c;
        if (D > 0.) {
          dxv = -(f1*(c*e-d*d)+f2*b*d-f3*c*b)/D;
          dyv = -(f1*b*d+f2*(a*e-b*b)-f3*a*d)/D;
          dzv = -(-f1*b*c-f2*a*d+f3*a*c)/D;
        } else {
          q=11;
        }
	Vertex.setX(Vertex.x()+dxv);
	Vertex.setY(Vertex.y()+dyv);
	Vertex.setZ(Vertex.z()+dzv);
	chiSquare = 0.2 * (dx1*dx1/sigx1 + dx2*dx2/sigx2
			   + dy1*dy1/sigy1 + dy2*dy2/sigy2);
      }
      if (q < 11) {	
        //covmatrix[0][0] = 2.* sqrt( (c*e-d*d)/D );
        //covmatrix[1][1] = 2.* sqrt( (a*e-b*b)/D );
        //covmatrix[2][2] = 2.* sqrt(    a*c   /D );
        //I guess the factor 4 should be 1 here,.....
        covmatrix[0][0] = 4 * ( (c*e-d*d)/D );
        covmatrix[1][1] = 4 * ( (a*e-b*b)/D );
        covmatrix[2][2] = 4 * (    a*c   /D );
	//set remaining elements to 0
	for(int i=0;i<3;i++) for(int k=i+1;k<3;k++) covmatrix[i][k] = covmatrix[k][i] = 0;
      }
    }
  }
  if ( q < 11) {
    Status=0; // Fit OK.
  } else {
    Status=1; // Fit not possible: Numerical problems
    cout << " Vertex Fit Numerical problems!!! " << endl;
  }
}
