#ifndef _L3_HITB_HH_
#define _L3_HITB_HH_
#include "L3_Defines.hh"
#include "L3_Geometry.hh"
#ifdef L3_GCC
#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#endif
#ifdef L3_EGCS
#include <iostream>
#include <iomanip>
#include <cmath>
#endif

#ifdef L3_ARTE_FRAME
#include "arte/arte.hh"
typedef ArtePointer<GEDE> gedePointer;
#endif

#ifdef L3_SLP_FRAME
typedef int gedePointer;
#include "vdsHITB.h"
#endif

class L3_Hitb {
public:
  inline L3_Hitb();
  inline L3_Hitb(const L3_Hitb&);
#ifdef L3_ARTE_FRAME
  inline L3_Hitb(const ArtePointer<HITB>&);
#endif
#ifdef L3_SLP_FRAME
  inline L3_Hitb(const int& nr, const VDS_hitb&);
#endif
  inline L3_Hitb& operator = (const L3_Hitb&);

  inline bool operator < (const L3_Hitb&) const;  
  inline int operator == (const L3_Hitb&) const;
  inline int operator != (const L3_Hitb&) const;
  
  inline ostream& operator << (const L3_Hitb&);

  inline int   hit()  const; // hitb number
  inline float z()    const; // z coordinate
  inline float x()    const; // x coordinate
  inline float y()    const; // y coordinate (for pad chambers only)
  inline float q()    const; // Quantity: amplitude for SVD, MSGC
                      // drift time for outer tracker
                      // 0 for RICH, TRD, MUON
  inline float r()    const; // Resolution of quantity Q
  inline gedePointer gede() const; // detector plane
  inline int   gesl() const; // detector superlayer
  inline int   sec()  const; // detector sector
  inline int   lay()  const; // detector layer
  inline const L3_Plane& getPlane() const; //return corresponding L3_Plane
private:
  int   the_hit; // hitb number
  float the_z;   // z coordinate
  float the_x;   // x coordinate
  float the_y;   // y coordinate (for pad chambers only)
  float the_q;   // Quantity: amplitude for SVD, MSGC
                 // drift time for outer tracker
                 // 0 for RICH, TRD, MUON
  float the_r;   // Resolution of quantity Q
  gedePointer the_gede;// detector plane
  int   the_gesl;// detector superlayer
  int   the_sec; // detector sector
  int   the_lay; // detector layer
};

#include "L3_Hitb.icc"
#endif
