#ifndef _L3_SILICON_HH_
#define _L3_SILICON_HH_
#include "L3_Defines.hh"

#ifdef L3_EGCS
#include <vector>
#endif
#ifdef L3_GCC
#include <vector.h>
#endif

#include "L3_Hitb.hh"
#include "L3_SpacePoint.hh"
#include "L3_Track.hh"
#include "L3_Timer.hh"

// Utility structure for L3_Silicon::findValidTracks
struct TRACKinfo {
  TRACKinfo();
  TRACKinfo(double, int);
  double the_probability;
  int the_numHits;
};

class L3_Silicon {
public:
  inline L3_Silicon();
  L3_Silicon(const L3_Silicon&);
  L3_Silicon& operator = (const L3_Silicon&);
  int nHitbs() const;  //return number of HITBs 
  int nPoints() const; //return number of spacepoints
  inline int nTracks() const; //return number of reconstructed tracks
  vector<L3_Hitb> HList() const;  //return a new vector with all HITBs
  inline vector<L3_Hitb> Hitbs(const int& sl, const int& quad, const int& pl) const;
  vector<L3_SpacePoint> Points() const;  //return a new vector with all spacepoints 
  inline vector<L3_SpacePoint> Points(const int& sl, const int& quad) const;
  inline vector<L3_Track> Tracks() const; //return the vector of reconstructed tracks
  void reconstruct(L3_Timer&);
  void release();      //reset all lists....
  void FillHits();  
  void print() const;
  void printRecoSummary() const;
private:
  vector<L3_Hitb> HitbList[7][4][4];
  vector<L3_SpacePoint> SpacePoint2List[7][4];
  vector<L3_SpacePoint> SpacePoint2XList[7][4];
  vector<L3_Track> TrackList;
  vector<L3_SpacePoint> SpacePointList[7][4];
  void SpacePointReco();
  void SpacePointRecoTest();
  void SpacePointRecoChi2();
  void SpacePairReco();
  void PurgeSpacePoints();  //discard spacepoints which are identical
  void SortSpacePoints();
  void SortSpacePairs();
  void TrackReconstruction();
  void TrackReconstruction98();
  int SearchNextSL(L3_Track&, const int&,int);
  int SearchNextSL98(L3_Track&, const int&, const int&, int, vector<L3_Track>&);
  void findValidTracks();
  void print(const vector<L3_Hitb>&) const;
  void print(const vector<L3_SpacePoint>&) const;
  void print(const vector<L3_Track>&) const;
};

#include "L3_Silicon.icc"

#endif
