#ifndef _L3_SPACEPOINT_HH_
#define _L3_SPACEPOINT_HH_
#include "L3_Defines.hh"
//#define L3_Spacepoint_DEBUG
#ifdef L3_EGCS
#include <vector>
#endif
#ifdef L3_GCC
#include <vector.h>
#endif

#include <CLHEP/Vector/ThreeVector.h>
#include <CLHEP/Matrix/Matrix.h>
#include <CLHEP/Matrix/Vector.h>

#include "L3_Hitb.hh"

//#define HREF

class L3_SpacePoint {
public:
  inline L3_SpacePoint();
  inline L3_SpacePoint(const L3_SpacePoint&);
  inline L3_SpacePoint(const L3_Hitb&, const L3_Hitb&);
  inline L3_SpacePoint(const L3_Hitb&, const L3_Hitb&, const int&);
  inline L3_SpacePoint& operator = (const L3_SpacePoint&);
  inline void add(const L3_Hitb&);
  inline void add(const L3_Hitb&, const L3_Hitb&);
  inline void add(const vector<L3_Hitb>&);

  inline int operator == (const L3_SpacePoint&) const;
  inline int operator != (const L3_SpacePoint&) const;
  inline ostream& operator << (const L3_SpacePoint&);
  inline Hep3Vector Punkt() const;
  inline int numHits() const;
  inline vector<L3_Hitb> HList() const;
  inline void setflag(const int&);
  inline int flag() const;
  static void initFitMatrizen();
  static void setFitMatrizen();

  int gesl() const; //return superlayer number
  int sec() const; //return sector number
  inline int nr() const; //return number
  inline void setnr(int nr);
  inline float x() const;
  inline float y() const;
  inline float z() const;
  inline float chiSquare() const;

  inline int getCopyOfHitbs(vector<L3_Hitb> &list) const;
  inline int getCommonHits(const L3_SpacePoint &another) const;

private:
#ifdef HREF
  typedef const L3_Hitb* Hitb;
#else
  typedef L3_Hitb Hitb;
#endif

  inline void calcPunkt(const L3_Hitb&);
  inline void calcPunkt(const L3_Hitb&, const L3_Hitb&);
  inline void calcPunktX(const L3_Hitb&, const L3_Hitb&);
  Hep3Vector derPunkt;
  float the_chiSquare;
  vector<Hitb> hitList;
  int Flag,Nr;
  static vector<HepMatrix> FitMatrizen;
  static vector<HepMatrix> MessMatrizen;
};

#include "L3_SpacePoint.icc"

#endif
