#ifndef _L3_TARGET_
#define _L3_TARGET_
#include "L3_Defines.hh"
#include "L3_Macros.hh"

#ifdef L3_ARTE_FRAME
#include "arte/arte.hh"
#endif

#ifdef L3_SLP_FRAME
extern "C" {
#include "l2targ.h"
}
#endif

#include <CLHEP/Matrix/Matrix.h>
#include <CLHEP/Matrix/Vector.h>
#include <CLHEP/Vector/ThreeVector.h>

class L3_Target {   // a wire target, same data as in ARTE
public:
  friend ostream& operator<<(ostream&,const L3_Target&);
  inline L3_Target();
  inline L3_Target(const L3_Target& p);
  L3_Target& operator=(const L3_Target&);

#ifdef L3_ARTE_FRAME
  L3_Target(const ArtePointer<GTAR>);
#endif
#ifdef L3_SLP_FRAME
  L3_Target(const L2TARG &t);
#endif

  //Takes a vector in HERA-B-cood-syst.and returns its distance from target center
  //in the target's (possibly rotated) coordinate system. You can use
  //fabs(h[0]),fabs(h[1]) and fabs(h[2]) to cut on the distance
  inline void distance(HepVector &h) const;  

  inline float x() const; //coordinates of target center
  inline float y() const;
  inline float z() const;
  inline int cmp() const;
  inline int wnr() const;
  inline float dx() const;
  inline float dy() const;
  inline float dz() const;
  inline float ms() const;
  inline float x0bm() const;
  inline float y0bm() const;
  inline float sxbm() const;
  inline float sybm() const;
  inline HepMatrix getA() const;
  inline HepVector getcenter() const;
  inline Hep3Vector vector() const;
  inline Hep3Vector get3center() const;
  inline const char* name() const;

private:
  int t_cmp;
  int t_wnr;
  float t_dx;
  float t_dy;
  float t_dz;
  HepVector center;
  HepMatrix A,Ainverse;
  float t_ms;
  float t_x0bm;
  float t_y0bm;
  float t_sxbm;
  float t_sybm;
  char t_name[5];
  char* get_name();
};

ostream& operator<<(ostream&,const L3_Target&);

#include "L3_Target.icc"

#endif
