#ifndef _L3_TRACK_HH_
#define _L3_TRACK_HH_
#include "L3_Defines.hh"

#ifdef L3_GCC
#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include <vector.h>
#include <function.h>
#include <algo.h>
#endif
#ifdef L3_EGCS
#include <iostream>
#include <iomanip>
#include <cmath>
#include <vector>
#include <functional>
#include <algorithm>
#endif

#include <CLHEP/Vector/ThreeVector.h>
#include <CLHEP/Matrix/Matrix.h>
#include <CLHEP/Matrix/Vector.h>

#include "L3_Hitb.hh"
#include "L3_SpacePoint.hh"
#include "L3_Window.hh"
#include "L3_Parameter.hh"
#include "L3_Geometry.hh"
#include "L3_Mathlib.hh"

//#define L3_DEBUG_TRACK

class L3_Track {
public:
  inline L3_Track();
  inline L3_Track(const L3_Track&);
  inline L3_Track(const int&, const L3_SpacePoint&);
  inline L3_Track(const int& nr, const float& x, const float& y, const float& mx, const float& my,
		  const float& refz);
  inline L3_Track& operator = (const L3_Track&);
  inline void add(const L3_SpacePoint&);
  inline void add(const vector<L3_SpacePoint>&);
  inline bool operator < (const L3_Track&) const;
  inline int operator == (const L3_Track&) const;
  inline int operator != (const L3_Track&) const;
  inline ostream& operator << (const L3_Track&);
  inline int nr() const;
  inline void setnr(int val);
  inline float x() const;
  inline float y() const;
  inline float mx() const;
  inline float my() const;
  inline float refz() const;
  inline float chisqr() const;
  inline float chisqrCut() const;
  inline float prob() const;
  inline float Cx() const;
  inline float Cmx() const;
  inline float Cmxx() const;
  inline float Cy() const;
  inline float Cmy() const;
  inline float Cmyy() const;
  inline double Cov(const int&, const int&) const;
  //----------------------------------------
  inline void SetCov(int i,int j,double val);
  inline void Setx(float x);
  inline void Sety(float y);
  inline void Setmx(float mx);
  inline void Setmy(float my);
  inline void Setrefz(float z);
  //----------------------------------------
  inline void fit();
  inline void fit(const float&);
  inline void kalTransport(const double&);
  inline void refit();
  inline int numPunkte() const;
  inline int numPunkteSL(int sl) const;
  inline int numHits() const;
  inline L3_Window window(const float&);
  inline L3_Window window();
  inline vector<L3_SpacePoint> getPunktList() const;
  inline vector<L3_Hitb> HList() const;

  inline int getCommonHits(const L3_Track &a) const;
  inline Hep3Vector MinDistanceToPoint(const Hep3Vector &v) const;
  inline Hep3Vector TrackDistance( const L3_Track &a ) const;
  inline Hep3Vector TrackDistance2( const L3_Track &a ) const;
  inline Hep3Vector MinDistanceToTarget( const L3_Target &t ) const;
  inline Hep3Vector Vector() const;
  inline Hep3Vector UnitVector() const;
  inline float AngleWithTrack( const L3_Track &a ) const;
  inline float AngleWithVector( const Hep3Vector &v ) const;

private:
  typedef const L3_SpacePoint* SpacePoint;

  inline void parTransport(const float&);
  inline void calcSpur(const float&);
  inline void ncalcSpur(const float&);
  inline void mscalcSpur(const float&);
  inline void kalMultScat();
  inline void kalFilter(double&, const double, const double, const double);
  int the_nr;
  float the_x;
  float the_y;
  float the_mx;
  float the_my;
  float chiSquare;
  double cov[4][4];
  float the_refz;
  vector<SpacePoint> RaumPunktList;
};

#include "L3_Track.icc"

#endif
