*CMZ :          07/11/96  16.34.53  by  T. Oest
*-- Author :    T. Oest   29/10/96

      SUBROUTINE V0FITI(INIT, COVAR, TRK1, TRK2, NZ1, ZMUSC1, ANG1,
     +                  NZ2, ZMUSC2, ANG2, PAR, VPAR, WPAR,
     +                  DCHI2, CHI2V0, IRET)

************************************************************************
*
* Description: returns new vertex/track parameters or the
*              covariance matrix
*
* Called from:
*
* External references:
*
* Argument in/out description
* INIT     in     = .TRUE.    do initialization
* COVAR    in     = .FALSE.   calculate new parameters
*                 = .TRUE.    calculate covariance matrix
* TRK1     in     first track
* TRK2     in     second track
* NZ1,2    in     number of multiple scattering planes for track 1,2
* ZMUSC1,2 in     zposition of multiple scattering planes for track 1,2
* ANG1,2   in     (multiple scattering angle)**2 for 1 GeV, tx=ty=0
* PAR      out    vertex, track parameters at START
*                 (ZSTART,x,y,tx1,ty1,1/p1,tx2,ty2,1/p2)
* VPAR     out    covariance matrix
* WPAR     out    (VPAR)**-1
* DCHI2    out    increase in CHI2  (COVAR=.FALSE.)
* CHI2V0   out    CHI2 of fit (COVAR=.TRUE.)
* IRET     out    =  0 OK
*                 = -1 chi2 increasing
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"

      LOGICAL          COVAR, INIT
      INTEGER          TRK1, TRK2, IRET, TRK(2), NZ1, NZ2
      DOUBLE PRECISION PAR(9), VPAR(9,9), WPAR(9,9), DCHI2, CHI2V0
      DOUBLE PRECISION ZMUSC1(*), ZMUSC2(*), ANG1(*), ANG2(*)


      INTEGER          I, J, K, L, M, IERROR, INDEX(5,5), IC1, IC2
      LOGICAL          COVMS, SCATT, REDO, COVTRK(2)
      DOUBLE PRECISION VTRK(2,5,5), WTRK(2,5,5)
      DOUBLE PRECISION A(2,5,3), B(2,5,3), W(2,5,5)
      DOUBLE PRECISION E(2,3,3), EINV(2,3,3), DD(2,5), DQ(2,3)
      DOUBLE PRECISION D0(3,3), D(2,3,3)
      DOUBLE PRECISION DX(3)
      DOUBLE PRECISION MA(6,3), MB(6,3), MVMS(6,6), MS(2,5,5)
      DOUBLE PRECISION CHIOLD, PAROLD(9), FAC
c test
      DOUBLE PRECISION MATEST(6,3), MBTEST(6,3), PTEST(6), POTEST(6)
c end test
      DOUBLE PRECISION PIN(6), POUT(6), ZOUT, DZ, PREC(4)
      DOUBLE PRECISION MW5(5,5), RW5(5), RW3(3), MW3(3,3)
      DOUBLE PRECISION WA(2,5,3), WB(2,5,3), BE(2,5,3), DSUM

      DOUBLE PRECISION C00(3,3), C11(3,3), C22(3,3)
      DOUBLE PRECISION C10(3,3), C20(3,3), C21(3,3)

      SAVE C00, EINV, D, DD, W, VTRK, WTRK
      SAVE CHIOLD, FAC

#include "arte/Arte.ff"
#include "auto/EV.ff"

      DATA INDEX /  1,  2,  4,  7, 11,
     +              2,  3,  5,  8, 12,
     +              4,  5,  6,  9, 13,
     +              7,  8,  9, 10, 14,
     +             11, 12, 13, 14, 15 /

      DATA SCATT /.TRUE./

C-----------------------------------------------------------------
C-----------------------------------------------------------------
C  initialization
C-----------------------------------------------------------------
      IRET   = 0
      TRK(1) = TRK1
      TRK(2) = TRK2


      IF (PRTRA_HITB(TRK1).EQ.0) THEN
       IC1 = 0
      ELSE
       IC1 = 1
      ENDIF
      IF (PRTRA_HITB(TRK2).EQ.0) THEN
       IC2 = 0
      ELSE
       IC2 = 1
      ENDIF

      REDO = .FALSE.
C-----------------------------------------------------------------
C  covariance matrix
C-----------------------------------------------------------------
      IF (INIT) THEN

       CHIOLD = -1.0D0

       DO I=1,2

         DO J=1,5
         DO K=J,5
          MW5(J,K) = DBLE(FRTRA_CVF(INDEX(J,K), TRK(I)))
          MW5(K,J) = MW5(J,K)
          VTRK(I,J,K) = MW5(J,K)
          VTRK(I,K,J) = MW5(K,J)
         ENDDO
        ENDDO

         CALL DINV(5, MW5, 5, RW5, IERROR)
         IF (IERROR.NE.0) THEN
          IRET = -2
          GOTO 9000
         ENDIF

         DO J=1,5
         DO K=1,5
          WTRK(I,J,K) = MW5(J,K)
         ENDDO
         ENDDO

       ENDDO
       GOTO 9000

      ENDIF
C-----------------------------------------------------------------
C  new parameter or covariance matrix ?
C-----------------------------------------------------------------
      IF (COVAR) GOTO 1000
C-----------------------------------------------------------------
C  swim track parameter to measured tracks
C         --> DD, A, B
C-----------------------------------------------------------------

10    CONTINUE


      PIN(1) = PAR(1)
      PIN(2) = PAR(2)
      PIN(3) = PAR(3)

      DO I=1,2

       PIN(4) = PAR(1+I*3)
       PIN(5) = PAR(2+I*3)
       PIN(6) = PAR(3+I*3)
       ZOUT   = DBLE(FRTRA_ZF(TRK(I)))
       DZ     = 100. * ABS(PIN(1) - ZOUT)
       PREC(1)= SQRT(FRTRA_CVF( 1,TRK(I))) / DZ
       PREC(2)= SQRT(FRTRA_CVF( 3,TRK(I))) / DZ
       PREC(3)= SQRT(FRTRA_CVF( 6,TRK(I))) / DZ
       PREC(4)= SQRT(FRTRA_CVF(10,TRK(I))) / DZ

       IF (I.EQ.1) THEN
        CALL V0FEXT(IC1, SCATT, NZ1, ZMUSC1, ANG1, PIN, ZOUT, PREC, 
     +              POUT, MA, MB, COVMS, MVMS, IERROR)
       ELSE
        CALL V0FEXT(IC2, SCATT, NZ2, ZMUSC2, ANG2, PIN, ZOUT, PREC, 
     +              POUT, MA, MB, COVMS, MVMS, IERROR)
       ENDIF
       IF (IERROR.NE.0) THEN
        IRET = -3
        GOTO 9000
       ENDIF

       COVTRK(I) = COVMS



       DO J=1,5
        DO K=1,3
         A(I,J,K)  = MA(J+1,K)
         B(I,J,K)  = MB(J+1,K)
        ENDDO
       ENDDO
       DO J=1,5
       DO K=1,5
         MS(I,J,K) = MVMS(J+1,K+1)
       ENDDO
       ENDDO

       DD(I,1) = FRTRA_XF (TRK(I)) - POUT(2)
       DD(I,2) = FRTRA_YF (TRK(I)) - POUT(3)
       DD(I,3) = FRTRA_TXF(TRK(I)) - POUT(4)
       DD(I,4) = FRTRA_TYF(TRK(I)) - POUT(5)
       DD(I,5) = FRTRA_PF (TRK(I)) - POUT(6)


      ENDDO

C-----------------------------------------------------------------
C  fill covariance matrices
C-----------------------------------------------------------------
C ---   W1, W2

        DO I=1,2

         IF (COVTRK(I)) THEN
          DO J=1,5
          DO K=J,5
           MW5(J,K) = VTRK(I,J,K) + MS(I,J,K)
           MW5(K,J) = MW5(J,K)
          ENDDO
          ENDDO

          CALL DINV(5, MW5, 5, RW5, IERROR)
          IF (IERROR.NE.0) THEN
           IRET = -4
           GOTO 9000
          ENDIF

          DO J=1,5
          DO K=1,5
           W(I,J,K) = MW5(J,K)
          ENDDO
          ENDDO

         ELSE
          DO J=1,5
          DO K=1,5
           W(I,J,K) = WTRK(I,J,K)
          ENDDO
          ENDDO
         ENDIF

        ENDDO

C-----------------------------------------------------------------
C  check old chi2
C   return if chi2 from last step increases
C-----------------------------------------------------------------
      CHI2V0 = 0.0
      DO I=1,5
      DO J=1,5
       CHI2V0 = CHI2V0 + DD(1,I)*W(1,I,J)*DD(1,J)
     +                 + DD(2,I)*W(2,I,J)*DD(2,J)
      ENDDO
      ENDDO

      IF (CHIOLD.GE.0.0 .AND. CHIOLD+0.001.LT.CHI2V0) THEN
       DO I=1,9
        PAR(I) = PAROLD(I)
       ENDDO
       FAC    = 0.5 * FAC

c       PRINT*,' CHI2 DECREASING ',sngl(CHIOLD), sngl(CHI2V0),sngl(fac)
       IF (FAC.LT.0.1) THEN
        IRET = -1
        GOTO 9000
       ELSE
        REDO = .TRUE.
        GOTO 10
       ENDIF

      ELSEIF (.NOT.REDO) THEN
       FAC = 1.0
       CHIOLD = CHI2V0
       DO I=1,9
        PAROLD(I) = PAR(I)
       ENDDO
      ENDIF

c      PRINT*,' CHI2 ',sngl(CHI2V0),SNGL(PAR(1))
C-----------------------------------------------------------------
C  fill matrices
C-----------------------------------------------------------------
        DO J=1,3
        DO K=1,3
         D0(J,K) = 0.0
        ENDDO
        ENDDO


        DO I=1,2

C ---   WA, WB
        DO J=1,5
        DO K=1,3
         WA(I,J,K) = 0.0
         WB(I,J,K) = 0.0
         DO L=1,5
          WA(I,J,K) = WA(I,J,K) + W(I,J,L) * A(I,L,K)
          WB(I,J,K) = WB(I,J,K) + W(I,J,L) * B(I,L,K)
         ENDDO
        ENDDO
        ENDDO

C --->  E, D0

        DO J=1,3
        DO K=1,3
         D(I,J,K) = 0.0
         MW3(J,K) = 0.0
         DO L=1,5
          D0(J,K)  = D0(J,K)  + A(I,L,J) * WA(I,L,K)
          D(I,J,K) = D(I,J,K) + A(I,L,J) * WB(I,L,K)
          MW3(J,K) = MW3(J,K) + B(I,L,J) * WB(I,L,K)
         ENDDO
         E(I,J,K) = MW3(J,K)
        ENDDO
        ENDDO

C --->  EINV

         CALL DINV(3, MW3, 3, RW3, IERROR)
         IF (IERROR.NE.0) THEN
          IRET = -5
          GOTO 9000
         ENDIF

         DO J=1,3
         DO K=1,3
          EINV(I,J,K) = MW3(J,K)
         ENDDO
         ENDDO

C --->  BE
         DO J=1,5
         DO K=1,3
          BE(I,J,K) = 0.0
          DO L=1,3
           BE(I,J,K) = BE(I,J,K) + B(I,J,L) * EINV(I,L,K)
          ENDDO
         ENDDO
         ENDDO


        ENDDO

C --->  C00
        DO I=1,3
        DO J=I,3
         C00(I,J) = D0(I,J)
         DO K=1,3
         DO L=1,3
          C00(I,J) = C00(I,J)
     +               - D(1,I,K)*EINV(1,K,L)*D(1,J,L)
     +               - D(2,I,K)*EINV(2,K,L)*D(2,J,L)
         ENDDO
         ENDDO
         C00(J,I) = C00(I,J)
        ENDDO
        ENDDO

c        print*,' C00 ',sngl(c00(1,1)),sngl(c00(1,2)),sngl(c00(1,3))
c        print*,'     ',sngl(c00(2,1)),sngl(c00(2,2)),sngl(c00(2,3))
c        print*,'     ',sngl(c00(3,1)),sngl(c00(3,2)),sngl(c00(3,3))
        CALL DINV(3, C00, 3, RW3, IERROR)
        IF (IERROR.NE.0) THEN
         IRET = -6
         GOTO 9000
        ENDIF

C-----------------------------------------------------------------
C DX, DQ
C-----------------------------------------------------------------
        DX(1) = 0.0
        DX(2) = 0.0
        DX(3) = 0.0

        DO I=1,2

         DO J=1,5
          DO K=1,5
           MW5(J,K) = 0.0
          ENDDO
          MW5(J,J) = 1.0
         ENDDO

         DO J=1,5
         DO K=1,5
          DO L=1,3
           MW5(J,K) = MW5(J,K) - BE(I,J,L) * WB(I,K,L)
          ENDDO
         ENDDO
         ENDDO

         DO J=1,3
         DO K=1,3
         DO L=1,5
         DO M=1,5
          DX(J) = DX(J) + C00(J,K) * WA(I,L,K) * MW5(L,M) * DD(I,M)
         ENDDO
         ENDDO
         ENDDO
         ENDDO

        ENDDO

        DO I=1,2

         DO J=1,5
          RW5(J) = DD(I,J)
          DO K=1,3
           RW5(J) = RW5(J) - A(I,J,K) * DX(K)
          ENDDO
         ENDDO

         DO J=1,3
          DQ(I,J) = 0.0
          DO K=1,3
          DO L=1,5
           DQ(I,J) = DQ(I,J) + EINV(I,J,K) * WB(I,L,K) * RW5(L)
          ENDDO
          ENDDO
         ENDDO


        ENDDO
C-----------------------------------------------------------------
C new parameters and chi2 decrease
C-----------------------------------------------------------------
        DO I=1,3
         PAR(I)   = PAR(I)   + FAC * DX(I)
         PAR(I+3) = PAR(I+3) + FAC * DQ(1,I)
         PAR(I+6) = PAR(I+6) + FAC * DQ(2,I)
        ENDDO

        DCHI2 = 0.0

        DO I=1,3
         DSUM = 0.0
         DO J=1,3
          DSUM = DSUM + D0(I,J)  * DX(J)
     +             + 2. * D(1,I,J) * DQ(1,J) + 2. * D(2,I,J) * DQ(2,J)
         ENDDO
         DCHI2 = DCHI2 + DX(I) * DSUM
        ENDDO

        DO I=1,2
        DO J=1,3
        DO K=1,3
         DCHI2 = DCHI2 + DQ(I,J) * E(I,J,K) * DQ(I,K)
        ENDDO
        ENDDO
        ENDDO

c        GOTO 9000
      IF (.NOT. COVAR) GOTO 9000
C-----------------------------------------------------------------
C  covariance matrix
C-----------------------------------------------------------------
 1000 CONTINUE

      DO I=1,3
      DO J=1,3
       C10(I,J) = 0.0
       C20(I,J) = 0.0
       DO K=1,3
       DO L=1,3
        C10(I,J) = C10(I,J) - EINV(1,I,K) * D(1,L,K) * C00(L,J)
        C20(I,J) = C20(I,J) - EINV(2,I,K) * D(2,L,K) * C00(L,J)
       ENDDO
       ENDDO
      ENDDO
      ENDDO

      DO I=1,3
      DO J=1,3
       C11(I,J) = EINV(1,I,J)
       C22(I,J) = EINV(2,I,J)
       C21(I,J) = 0.0
       DO K=1,3
       DO L=1,3
        C11(I,J) = C11(I,J) - C10(I,K) * D(1,K,L) * EINV(1,L,J)
        C22(I,J) = C22(I,J) - C20(I,K) * D(2,K,L) * EINV(2,L,J)
        C21(I,J) = C21(I,J) - C20(I,K) * D(1,K,L) * EINV(1,L,J)
       ENDDO
       ENDDO
      ENDDO
      ENDDO

      DO I=1,3
      DO J=1,3
       VPAR(I  ,J  ) = C00(I,J)
       VPAR(I+3,J  ) = C10(I,J)
       VPAR(I+6,J  ) = C20(I,J)
       VPAR(I  ,J+3) = C10(J,I)
       VPAR(I  ,J+6) = C20(J,I)
       VPAR(I+3,J+3) = C11(I,J)
       VPAR(I+6,J+6) = C22(I,J)
       VPAR(I+6,J+3) = C21(I,J)
       VPAR(I+3,J+6) = C21(J,I)

       WPAR(I  ,J  ) = D0(I,J)
       WPAR(I+3,J  ) = D(1,J,I)
       WPAR(I+6,J  ) = D(2,J,I)
       WPAR(I  ,J+3) = D(1,I,J)
       WPAR(I  ,J+6) = D(2,I,J)
       WPAR(I+3,J+3) = E(1,I,J)
       WPAR(I+6,J+6) = E(2,I,J)
       WPAR(I+6,J+3) = 0.0D0
       WPAR(I+3,J+6) = 0.0D0
      ENDDO
      ENDDO

      CHI2V0 = 0.0
      DO I=1,5
      DO J=1,5
       CHI2V0 = CHI2V0 + DD(1,I)*W(1,I,J)*DD(1,J)
     +                 + DD(2,I)*W(2,I,J)*DD(2,J)
      ENDDO
      ENDDO


C-----------------------------------------------------------------
C  the end
C-----------------------------------------------------------------


 9000 END

************************************************************************
* End of V0FITI
************************************************************************
