/**********************************************************
 Routines for access OTrDetector structures.

 W.Hulsbergen, A.Lanyov    October 1999
 **********************************************************/
 
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <iostream>

#include <arte/arte.hh>
#include <arte/artefunc.h>

#include "wax/DecodeManager.hh"
#include "wax/GedeManager.hh"

//#define DEBUG
//#define PRINT

extern "C" {

  int otr_gede_status(int igede_, int mono, int wire){
    /********************************************************************
      Returns status of wire from wire status using OTrDecode structure.

      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (0..3)
      wire  - number of wire (0..31)
      ********************************************************************/
    int status = -1;
    OTR_Decode *OTR;
    OTR = waxDecodeManager::GetMe()->GetDecodeTable();
    //  printf("OTR pointer got\n");
    
    short igede = igede_ - OTR->fMinGedeIndex ;
    if(igede >= 0 && igede <  OTR->fNGedes) {
      OTrGede* decodegede = OTR->fGedes + igede  ;
      unsigned char cell = wire + 32*mono;
      if(cell < decodegede->fNWires ) {
	unsigned char chanid = 
	  decodegede->fWires[cell].fChannelIndex ;
	unsigned short asdid  =
	  decodegede->fWires[cell].fAsdIndex ;
	if(decodegede->fWires[cell].fAsdIndex != MAXASD) {
	  status = OTR->ASD8s[asdid]->wire[chanid].status ;
#ifdef DEBUG
	  printf("igede %d, cell %d ==> asdid %d, chanid %d, status %d \n",
		 igede_,cell,asdid,chanid,status);
#endif
	}
      }
    } else {
      std::cout << "Gede not in decodestructures: " << igede_ << std::endl ;
    }
    
    return status;
  }
  
  int otr_gede_status_(int *igede, int *mono, int *iwire){
    /********************************************************************
      Fortran integer function:
      call otr_gede_status_(igede, mono, iwire)
      
      Returns status of wire from wire status using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (1..4)
      iwire - number of wire (1..32)
      ********************************************************************/
    int status = otr_gede_status(*igede, (*mono)-1, (*iwire)-1);
    return status;
  }
  


  int otr_gede_channel(int igede_, int mono, int wire){
    /********************************************************************
      Returns channel number using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (0..3)
      wire  - number of wire (0..31)
      ********************************************************************/
    int channel = -1;
    OTR_Decode *OTR;
    OTR = waxDecodeManager::GetMe()->GetDecodeTable();
    //  printf("OTR pointer got\n");
    
    short igede = igede_ - OTR->fMinGedeIndex ;
    if(igede >= 0 && igede <  OTR->fNGedes) {
      OTrGede* decodegede = OTR->fGedes + igede  ;
      unsigned char cell = wire + 32*mono;
      if(cell < decodegede->fNWires ) {
	unsigned char chanid = decodegede->fWires[cell].fChannelIndex ;
	unsigned short asdid = decodegede->fWires[cell].fAsdIndex ;
	if(decodegede->fWires[cell].fAsdIndex != MAXASD) {
	  int asd = OTR->ASD8s[asdid]->ftb_ASD;
	  channel = 16*(asd-1) + chanid + 1;
#ifdef DEBUG
	  printf("igede %d, cell %d ==> asdid %d, asd %d, channel %d \n",
		 igede_,cell,asdid,asd,channel);
#endif
	}
      }
    } else {
      std::cout << "Gede not in decodestructures: " << igede_ << std::endl ;
    }
    
    return channel;
  }
  
  
  
  int otr_gede_channel_(int *igede, int *mono, int *iwire){
    /********************************************************************
      Fortran integer function:
      integer otr_gede_channel,igede,mono,iwire
      channel = otr_gede_channel(igede, mono, iwire)
      
      Returns channel of wire using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table,
      mono  - number of monolayer (1..4)
      iwire - number of wire (1..32)
      ********************************************************************/
    int channel = otr_gede_channel(*igede, (*mono)-1, (*iwire)-1);
    return channel;
  }
  
  int otr_gede_channels(int igede_){
    /********************************************************************
      Returns number of channels for given superlayer/layer/sector
      using OTrDecode structure.
      
      If OTrDecode was not initialized for given quadrant, -1 is returned.
      (sometimes still returns something looking as good).
      
      igede - index in GEDE table
      ********************************************************************/
    int channels = -1;
    OTR_Decode *OTR;
    OTR = waxDecodeManager::GetMe()->GetDecodeTable();
    //  printf("OTR pointer got\n");
    
    short igede = igede_ - OTR->fMinGedeIndex ;
    if(igede >= 0 && igede <  OTR->fNGedes) {
      OTrGede* decodegede = OTR->fGedes + igede  ;
      unsigned char cell = 5; //?
      if(cell < decodegede->fNWires ) {
	unsigned short asdid  =
	  decodegede->fWires[cell].fAsdIndex ;
	if(decodegede->fWires[cell].fAsdIndex != MAXASD) {
	  int lay = OTR->ASD8s[asdid]->Layer;
	  int sec = OTR->ASD8s[asdid]->Sector;
	  int SLid = OTR->ASD8s[asdid]->SuperLayer;
	  int asds = OTR->Slayers[SLid].Nasd[lay][sec];
	  channels = 16*asds;
#ifdef DEBUG
	  printf("igede %d ==> SLid %d, sec %d, lay %d, channels %d \n",
		 igede_,SLid,lay,sec,channels);
#endif
	}
      }
    } else {
      std::cout << "Gede not in decodestructures: " << igede_ << std::endl ;
    }
    
    return channels;
  }
  
  int otr_gede_channels_(int *igede){
    /********************************************************************
      Fortran integer function:
      integer nchan,igede
      nchan = otr_gede_channels(igede)
      
      igede - index in GEDE table
      ********************************************************************/
    return otr_gede_channels(*igede);
  }
  

  int OTrDecode_SuperLayerNameToIndex(const char slname[4], unsigned char* slid) 
    {
      /************************************************************************/
      /* Converts superlayername of form "-t1" to OTrDecode superlayer index  */
      /************************************************************************/
      *slid = atoi(slname+2);
      if      (slname[0]=='+') *slid += 16 ;
      if      (slname[1]=='m') *slid += 0;
      else if (slname[1]=='p') *slid += 8;
      else if (slname[1]=='t') *slid += 12;
      else {
	printf("GetSuperLayerIndex: illegeal superlayer name: %s !!!\n",slname) ;
	*slid = 0 ;
	return 1 ;
      }
      return 0;
    }
  
  int OTrDecode_ChannelToWire(OTR_Decode* OTR, unsigned char slid, 
			      unsigned char layid, unsigned char secid, 
			      unsigned char asdid, unsigned char chanid, 
			      unsigned short* gedeid, unsigned char* wireid) 
    {
      /************************************************************************/
      /* Converts channel identified by (superlayer,layer,sector,asd,channel) */
      /* to wire (GEDE, wire). The wireindex is defined by wire = cell + 32*mono. */
      /************************************************************************/
      unsigned short asdId ;
      OTrWire* chanIt ;
      OTrSuperLayer* slIt;
      unsigned char firstasd ;
      
      slIt = OTR->Slayers + slid ;
      firstasd = slIt->FirstASD[layid][secid] ;
      if(firstasd<255) {
	asdId = slIt->Line[slIt->pos[layid][secid] - firstasd + asdid] ;
	chanIt = OTR->ASD8s[asdId]->wire+chanid ;
	*gedeid = chanIt->GEDE ;
	*wireid = chanIt->cell ;
      } else {
	*gedeid = 0;
	*wireid = 0;
	return 1;
      }
      return 0;
    }
  
}
