#ifndef __WAX_DECODE_MANAGER_HH__
#define __WAX_DECODE_MANAGER_HH__

#ifndef __OTR_DECODE_H__
#include "OTrDecode.h"
#endif
#ifndef __WAX_BASICTYPES_HH__
#include "BasicTypes.hh"
#endif
#ifndef __STRING__
#include <string>
#endif

class waxDecodeManager ;

class waxDecodeChannel
{
public:
  waxDecodeChannel(const OTrWire& chan) : fChannel(chan) {} 
  waxDecodeChannel() {}
  ~waxDecodeChannel() {}

  // decoding the wire structure //
  TdcCount_t     GetTdcCount()      const { return fChannel.dtime; }
  TdcCount_t     GetTZeroCount()    const { return fChannel.tzero; }
  TdcCount_t     GetTZeroErrCount() const { return fChannel.tzerr; }
  Status_t       GetStatus()        const { return fChannel.status; }
  GedeIndex_t    GetGedeIndex()     const { return fChannel.GEDE; }
  WireIndex_t    GetWireIndex()     const { return fChannel.cell; }
  AsdIndex_t     GetAsdIndex()      const { return fChannel.ASD; }
  ChannelIndex_t GetChannelIndex()  const { return fChannel.wire; }
  bool IsConnected() const { return fChannel.GEDE > 0 ; }

  // decoding the tdccount
  DriftTime_t GetTZero()      const { return GetTZero(GetTZeroCount()); }
  DriftTime_t GetTZeroErr()   const { return GetTZeroErr(GetTZeroErrCount()) ; }
  DriftTime_t GetDriftTime()  const { return DriftTime_t(GetTdcCount()) - GetTZero() ; }
  void SetTdcCount(TdcCount_t t) { fChannel.dtime=t; }
  void SetStatus(Status_t st) { fChannel.status=st ; }
  void SetTZero(DriftTime_t tz, DriftTime_t tzerr) ;
  void SetTZeroCount(TdcCount_t tzc, TdcCount_t tzec) ;
  bool IsValidHit() const ;

  static DriftTime_t GetTZero(TdcCount_t tzc) { return float(tzc-128)/2 ; }
  static DriftTime_t GetTZeroErr(TdcCount_t tzec) { return float(tzec)/16 ; }
  //  static DriftTime_t GetDriftTime(TdcCount_t tdc,TdcCount_t tzcount) {
  //  tdc - GetTZero(tdccount) ; }

  static const waxDecodeChannel* GetChannel(AsdIndex_t asdid, ChannelIndex_t chid) ;
  static void SetDecodeManager(const waxDecodeManager* c) { fgDecodeManager=c ; }
private:
  OTrWire fChannel ;
  static const waxDecodeManager* fgDecodeManager ;
  friend class waxDecodeManager ;
} ;

class waxDecodeAsd
{
public:
  //typedefs
  typedef waxDecodeChannel* ChannelIterator ;
  typedef const waxDecodeChannel* ConstChannelIterator ;
 
  unsigned char GetNChannels() const { return 16 ; }
  ConstChannelIterator GetChannels() const { 
    return (ConstChannelIterator)(fAsd.wire) ; }
  ConstChannelIterator GetChannelsEnd() const { 
    return (ConstChannelIterator)(fAsd.wire +GetNChannels() ) ; }
  ConstChannelIterator GetChannel(ChannelIndex_t chId) const { 
    return GetChannels() + chId ; }
  ChannelIterator GetChannels()    { 
    return (ChannelIterator)(fAsd.wire) ; }
  ChannelIterator GetChannelsEnd() { 
    return (ChannelIterator)(fAsd.wire +GetNChannels() ) ; }

  std::string GetName() const ;
  int GetLayerIndex() const { return fAsd.Layer ; }
  int GetSuperLayerIndex() const { return fAsd.SuperLayer ; }
  int GetSectorIndex() const { return fAsd.Sector ; }
  int GetSubAsdIndex() const { return fAsd.ftb_ASD ; }
  FedIndex_t GetFedIndex() const { return fAsd.TDC_FED ; }
  int GetTdcConnectorIndex() const { return fAsd.TDC_Connector ; }
private:
  OTrASD8 fAsd ; 
  friend class waxDecodeManager ;
} ;

class waxDecodeTdc
{
public:
  CrateIndex_t GetCrateIndex() const { return fTdc.crate; }
  SlotIndex_t GetSlotIndex() const  { return fTdc.slot; }
  AsdIndex_t GetAsdIndex(int subasdid) const { return fTdc.ASD[subasdid] ; }
private:
  OTrTDC fTdc ; 
  friend class waxDecodeManager ;
} ;

class waxDecodeWire
{
public:
  AsdIndex_t     GetAsdIndex()     const { return fWire.fAsdIndex ; }
  ChannelIndex_t GetChannelIndex() const { return fWire.fChannelIndex ; }
  const waxDecodeChannel* GetChannel() const ;
  bool IsConnected() const { return fWire.fAsdIndex != MAXASD ; }
private:
  OTrGedeWire fWire ;  
  friend class waxDecodeManager ;
} ;

class waxDecodeGede
{
public:
  waxDecodeGede() { fGede.fNWires=0; }
  typedef const waxDecodeWire* WireIterator ; 
  unsigned char GetNWires()  const { return fGede.fNWires ; }
  WireIterator GetWires()    const { return (WireIterator)(fGede.fWires) ; }
  WireIterator GetWiresEnd() const { return GetWires()+GetNWires(); }
  WireIterator GetWire(WireIndex_t wid) const { return GetWires()+wid ; }
private:
  OTrGede fGede ;  
  friend class waxDecodeManager ;
} ;

class waxDecodeManager
{
public:
  // typedefs, enumerates
public:
  typedef const waxDecodeGede* ConstGedeIterator ;
  typedef const waxDecodeAsd*const* ConstAsdIterator ;
  typedef waxDecodeGede* GedeIterator ;
  typedef waxDecodeAsd*const* AsdIterator ;

  // constructors, destructor
  static waxDecodeManager* GetMe()  ;
  static void DeleteMe() ;
  ~waxDecodeManager() ;
  
  // inlines
  AsdIterator GetAsds() ;
  ConstAsdIterator GetAsds() const ;
  ConstGedeIterator GetGedes() const ;
  ConstGedeIterator GetGedesEnd() const ;
  int GetNGedes() const { return fDecodeTable->fNGedes ; }
  const waxDecodeGede* GetGede(GedeIndex_t gedeId) const ;
  const waxDecodeAsd* GetAsd(AsdIndex_t asdId) const ;
  waxDecodeAsd* GetAsd(AsdIndex_t asdId) ;
  GedeIndex_t GetMinGedeIndex() const { return fDecodeTable->fMinGedeIndex ; }
  const waxDecodeTdc* GetTdc(FedIndex_t fedId) const;
  OTR_Decode* GetDecodeTable() { return fDecodeTable ; }
  const OTR_Decode* GetDecodeTable() const { return fDecodeTable ; }

  // static
  static void SetListOfTablesFileName(std::string f) { fgListOfTablesFileName=f ; }
  static void SetTdcFedFileName(std::string f)       { fgTdcFedFileName=f ; }
  static void SetTZeroFileName(std::string f)        { fgTZeroFileName=f ; }
  static void SetMaskFileName(std::string f)         { fgMaskFileName=f ; }
  
  // other methods
  void ReadTZeros() ;
  void WriteTZeros(const std::string&) ;
  void ReadStatus() ;
  int GetNewEvent(int* nhits, waxDecodeChannel*** hitchannels) ;
  int GetNAsds() const ;
  waxDecodeAsd* GetAsd(std::string asdname) ;
  void SwapCables(const std::string& filename) ;

private:
  waxDecodeManager() ;
private:
  GedeIterator GetGedes() ;
  GedeIterator GetGedesEnd() ;
  const GedeIndex_t* GetGedeMap() ;
  const GedeIndex_t* GetGedeMapEnd() ;
  waxDecodeTdc* GetTdc(FedIndex_t fedId) ;

  OTR_Decode* fDecodeTable ;
  static waxDecodeManager* fgMe ;
  static std::string fgListOfTablesFileName ;
  static std::string fgTdcFedFileName ;
  static std::string fgTZeroFileName ;
  static std::string fgMaskFileName ;
  friend class waxDecodeWire ;
  //map<decodeAsd::Id_t, decodeAsd*> fAsdMap ;
} ;

/*** inlines for waxDecodeChannel ****/
inline void waxDecodeChannel::SetTZeroCount(TdcCount_t tzc, TdcCount_t tzec) {
  fChannel.tzero=tzc; fChannel.tzerr=tzec; }

inline const waxDecodeChannel* waxDecodeWire::GetChannel() const { 
  return waxDecodeManager::fgMe->GetAsd(GetAsdIndex())->
    GetChannel(GetChannelIndex()) ; }

/********* inlines for waxDecodeManager *********/

inline waxDecodeManager::AsdIterator 
waxDecodeManager::GetAsds() {
  return (AsdIterator)(fDecodeTable->ASD8s) ; }

inline waxDecodeManager::ConstAsdIterator 
waxDecodeManager::GetAsds() const {
  return (ConstAsdIterator)(fDecodeTable->ASD8s) ; }

inline waxDecodeAsd*
waxDecodeManager::GetAsd(AsdIndex_t asdId) { 
  return GetAsds()[asdId] ; }
//(waxDecodeAsd*)(fDecodeTable->ASD8s + asdId) ; }

inline const waxDecodeAsd*
waxDecodeManager::GetAsd(AsdIndex_t asdId) const { 
  return GetAsds()[asdId] ; }
//  return (const waxDecodeAsd*)(fDecodeTable->ASD8s + asdId) ; }

inline waxDecodeManager::GedeIterator
waxDecodeManager::GetGedes() { 
  return (GedeIterator)(fDecodeTable->fGedes) ; }

inline waxDecodeManager::GedeIterator
waxDecodeManager::GetGedesEnd() { 
  return (GedeIterator)(fDecodeTable->fGedes+fDecodeTable->fNGedes) ; }

inline waxDecodeManager::ConstGedeIterator
waxDecodeManager::GetGedes() const { 
  return (ConstGedeIterator)(fDecodeTable->fGedes) ; }

inline waxDecodeManager::ConstGedeIterator
waxDecodeManager::GetGedesEnd() const { 
  return (ConstGedeIterator)(fDecodeTable->fGedes+fDecodeTable->fNGedes) ; }

inline const GedeIndex_t*
waxDecodeManager::GetGedeMap() { return fDecodeTable->fGedeMap ; }

inline const GedeIndex_t*
waxDecodeManager::GetGedeMapEnd() { return fDecodeTable->fGedeMapEnd ; }
  
inline const waxDecodeGede*
waxDecodeManager::GetGede(GedeIndex_t gedeId) const {
  return (waxDecodeGede*)(fDecodeTable->fGedes +
		       gedeId - fDecodeTable->fMinGedeIndex) ; }

inline const waxDecodeChannel* waxDecodeChannel::GetChannel(AsdIndex_t asdid, ChannelIndex_t chid) {
  return fgDecodeManager->GetAsd(asdid)->GetChannel(chid) ; }

inline const waxDecodeTdc* waxDecodeManager::GetTdc(FedIndex_t fedId) const {
  int tdcId=fDecodeTable->FEDs[fedId];
  if(tdcId==MAXFED) return 0;
  return (const waxDecodeTdc*)fDecodeTable->TDCs[tdcId] ; 
}

inline waxDecodeTdc* waxDecodeManager::GetTdc(FedIndex_t fedId) {
  int tdcId=fDecodeTable->FEDs[fedId];
  if(tdcId==MAXFED) return 0;
  return (waxDecodeTdc*)fDecodeTable->TDCs[tdcId] ; 
}

#endif
