#include <string>
#include <fstream>
#include "wax/RTRelation.hh"
#include "wax/DSRManager.hh"
#include "wax/GedeManager.hh"
#include "wax/Gede.hh"
#include "wax/otr_db.h"
#include "arte/ArteFlag.h"

#ifdef USE_ARTE
#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#endif

waxDSRManager* waxDSRManager::fgMe = 0;

std::string 
waxDSRManager::fgConfigDirName = "" ;

waxDSRManager* waxDSRManager::GetMe()
{
  if(!fgMe) fgMe = new waxDSRManager() ;
  return fgMe ;
}

void waxDSRManager::DeleteMe() 
{
  delete fgMe ;
  fgMe=0;
}

waxDSRManager::~waxDSRManager() 
{
  if(fgMe == this) fgMe = 0;
}

void
waxDSRManager::ReadDB()
{
  std::cout << "waxDSRManager::ReadDB()" << std::endl ;
  // first find max index
  int max_irt(0) ;
  int min_irt(1000) ;
  for(LedaTableIterator<class OTR_r_t> 
	ledartIt = LedaTableIterator<class OTR_r_t>();
      ledartIt.valid(); ledartIt.next()) {
    max_irt=max(max_irt,ledartIt->irt) ;
    min_irt=min(min_irt,ledartIt->irt) ;
  }
  std::cout << min_irt << " " << max_irt << std::endl ;

  // now read rt-relation
  fRTRelations.resize(max_irt-min_irt,0) ;
  fRTIndexMin = min_irt ;
  for(LedaTableIterator<class OTR_r_t> 
	ledartIt = LedaTableIterator<class OTR_r_t>();
      ledartIt.valid(); ledartIt.next()) {
    DSRContainer::iterator rtIt = fRTRelations.begin()
      +(ledartIt->irt-fRTIndexMin) ;
    if(!(*rtIt)) *rtIt = new waxRTRelation(ledartIt) ;
    else (*rtIt)->ReadDB() ;
  }

  // last but not least: set pointer of gede to rtrelation
  for(waxGedeManager::GedeContainer::iterator
	gedeIt=fGedeManager->GetGedes().begin() ;
      gedeIt!=fGedeManager->GetGedes().end() ; gedeIt++)
    if(*gedeIt) {
      LedaPointer<class OTR_nede> ledaNedeIt(OTR_nede->get->key((*gedeIt)->GetHWC())) ;
      LedaPointer<class OTR_r_t>  ledaRtIt(ledaNedeIt->tor_t) ;
      waxRTRelation* rtr = GetRTRelationFromRTIndex(ledaRtIt->irt) ;
      (*gedeIt)->SetRTRelation(rtr) ;
      if(!rtr) std::cout << "Could not find rt relation in db !" << std::endl ;
    }
  
}

void
waxDSRManager::WriteDB()
{
  std::cout << "waxDSRManager::WriteDB()" << std::endl ;
  //OTR_r_t->initialize();
  //std::cout << "OTR_r_t initialized ..." << std::endl ;

  // Write dsrs
  for(DSRContainer::const_iterator dsrIt = fRTRelations.begin();
      dsrIt != fRTRelations.end(); dsrIt++) 
    (*dsrIt)->WriteDB() ;
  // Write relation to nede
//   for(waxGedeManager::GedeContainer::const_iterator
// 	gedeIt=fGedeManager->GetGedes().begin() ;
//       gedeIt!=fGedeManager->GetGedes().end() ; gedeIt++)
//     if(*gedeIt) {
//       RelateRows(OTR_nede,tor_t,  key((*gedeIt)->GetGEDE()->hwc),
// 		 OTR_r_t, tonede, key((*gedeIt)->GetRTRelation()->GetLedaKey()) );
//     }
  std::cout << "End of waxDSRManager::WriteDB()" << std::endl ; 
}

// void
// waxDSRManager::FillOTTS()
// {
// }

waxDSRManager::waxDSRManager() 
  : fRTRelations(4)
{
  fgMe = this ;
  string dirnameUser = fgConfigDirName ;
  char runname[9] ;
  sprintf(runname,"run%5.5d",arteflag_.numrun) ;
  string dirnameGen = "/afs/desy.de/group/hera-b/OTR/db/dsr/default" ;
  string dirnameRun = string("/afs/desy.de/group/hera-b/OTR/db/dsr/") + runname  ;
 
  string dirName = "";
  if(dirnameUser == "") {
    std::ifstream ifs((dirnameRun + "/rtr01.dat").c_str()) ;
    if(ifs) {
      ifs.close() ;
      dirName = dirnameRun ;
    } else {
      ifs.open((dirnameGen + "/rtr01.dat").c_str()) ;
       if(ifs) {
	 ifs.close() ;
	 dirName = dirnameGen ;
       } else std::cout << "Could not find rtr01.dat in any of these dirs: " << std::endl
		   << dirnameRun << std::endl << dirnameGen << std::endl ;
    }
  } else {
    std::ifstream ifs((dirnameUser + "/rtr01.dat").c_str()) ;
    if(ifs) {
      ifs.close() ;
      dirName = dirnameUser ;
    } 
  }
  
  if(dirName != "") {
    std::cout << "Reading rt relation from " << dirName << std::endl ;
    SetRTRelation05mm(new waxRTRelation(1,dirName + "/rtr01.dat")) ;
    SetRTRelation10mm(new waxRTRelation(2,dirName + "/rtr02.dat")) ;
    SetRTRelation05mmMagnet(new waxRTRelation(3,dirName + "/rtr03.dat")) ;
    SetRTRelation10mmMagnet(new waxRTRelation(4,dirName + "/rtr04.dat")) ;
  } else {
    std::cout << "Creating new rt relations. " << std::endl ;
    SetRTRelation05mm(new waxRTRelation(1,waxRTRelation::kPitchFive,
					waxRTRelation::kRofT,8,3)) ;
    SetRTRelation10mm(new waxRTRelation(2,waxRTRelation::kPitchTen,
					waxRTRelation::kRofT,8,3)) ;
    SetRTRelation05mmMagnet(new waxRTRelation(3,waxRTRelation::kPitchFive,
					      waxRTRelation::kRofT,8,3)) ;
    SetRTRelation10mmMagnet(new waxRTRelation(4,waxRTRelation::kPitchTen,
					      waxRTRelation::kRofT,8,3)) ;
  }
  
  fRTIndexMin = 1 ;
  waxGedeManager* gm = waxGedeManager::GetMe() ;
  for(waxGedeManager::GedeContainer::iterator 
	gedeIt = gm->GetGedes().begin() ;
      gedeIt != gm->GetGedes().end() ; gedeIt++) 
    if(*gedeIt) {
      switch((*gedeIt)->GetEPitch())
	{
	case waxGede::kFiveMM:
	  if((*gedeIt)->GetHWC()[1]=='m')
	    (*gedeIt)->SetRTRelation(GetRTRelation05mmMagnet()) ;
	  else 
	    (*gedeIt)->SetRTRelation(GetRTRelation05mm()) ;
	  break ;
	case waxGede::kTenMM:
	  if((*gedeIt)->GetHWC()[1]=='m')
	    (*gedeIt)->SetRTRelation(GetRTRelation10mmMagnet()) ;
	  else
	    (*gedeIt)->SetRTRelation(GetRTRelation10mm()) ;
	  break ;
	}
    }
}
