#include <fstream>
#include <iomanip>
#include <algorithm>
#include "wax/DecodeManager.hh"
#include "wax/GedeManager.hh"
#include "wax/Gede.hh"
#include "wax/GedeManager.hh"
#include "wax/OTrAsdId.h"
#include "wax/OTrRead.h"
//#include "arte/ArteFlag.h"
#include "arte/arte.hh"

const waxDecodeManager*
waxDecodeChannel::fgDecodeManager = 0;

waxDecodeManager*
waxDecodeManager::fgMe = 0;

string waxDecodeManager::fgListOfTablesFileName =
"/afs/desy.de/group/hera-b/OTR/db/tables/list_of_tables_hallw" ;

string waxDecodeManager::fgTdcFedFileName =
"/afs/desy.de/group/hera-b/OTR/db/tables/tdc_fed_hallw" ;

string waxDecodeManager::fgTZeroFileName = "" ; 
string waxDecodeManager::fgMaskFileName = "" ;

void waxDecodeChannel::SetTZero(DriftTime_t tz, DriftTime_t tzerr) {
  // check that tz and tzerr are within range
  int tzc = int(128 + 2*tz    + 0.5) ; 
  int tzec = int(16*tzerr + 0.5) ; 
  if(tzc>255) {
    tzc=255 ;
    tzec=255 ;
  } else if(tzc<0) {
    tzc=0 ;
    tzec=255 ;
  } else if(tzec>255) {
    tzec=255 ;
  } else if(tzec<0) 
    tzec=0 ;
  SetTZeroCount(tzc,tzec) ;
}

waxDecodeManager* waxDecodeManager::GetMe() 
{
  if(!fgMe) fgMe = new waxDecodeManager() ;
  return fgMe ;
}

void
waxDecodeManager::DeleteMe()
{
  delete fgMe ;
  fgMe=0 ;
}
 
bool waxCompareGede(const unsigned short lhsgedeId, const unsigned short rhsgedeId)
{
  static const waxGedeManager* gedemanager = waxGedeManager::GetMe() ;
  static const int mingedeid = waxDecodeManager::GetMe()->GetMinGedeIndex() ;
  const waxGede* lhsGede = gedemanager->GetGede(lhsgedeId+mingedeid) ;
  const waxGede* rhsGede = gedemanager->GetGede(rhsgedeId+mingedeid) ;
  int lhsindex = 
    lhsGede->GetModuleId() + 100*
    (lhsGede->GetSectorId()+ 100*
     (lhsGede->GetLayerId() + 10*lhsGede->GetSuperLayerId())) ;
  int rhsindex = 
    rhsGede->GetModuleId() + 100*
    (rhsGede->GetSectorId()+ 100*
     (rhsGede->GetLayerId() + 10*rhsGede->GetSuperLayerId())) ;
  return lhsindex<rhsindex ;
}

waxDecodeManager::waxDecodeManager()
{
  cout << "waxDecodeManager constructor: initializing decoding." << endl ; 
  if(!fgMe) fgMe = this ;

  int mode=0;
  fDecodeTable = OTr_Decode_init(mode,fgListOfTablesFileName.c_str(),
				 fgTdcFedFileName.c_str()) ;
  if(fDecodeTable!=NULL) {
    waxDecodeChannel::SetDecodeManager(this) ; 
    // sort gedemap in decode table
    sort(fDecodeTable->fGedeMap,fDecodeTable->fGedeMapEnd,waxCompareGede) ;
    for(AsdIterator asdIt = GetAsds() ; *asdIt != 0; asdIt++)
      for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels(); 
	  chanIt !=  (*asdIt)->GetChannelsEnd(); chanIt++) {
	if(ArteTable<MCIN>::booked()) chanIt->SetTZero(0,0) ;
	else chanIt->SetTZero(0,100) ;
	chanIt->SetStatus(0) ;
      }
    ReadStatus() ;
    ReadTZeros() ;
#ifdef __WAX_DEBUG__
    cout << "Wax DEBUGGING mode: " << endl ;

    AsdIterator firstAsd =  GetAsds() ;
    int asdId=0;
    for(AsdIterator asdIt = firstAsd; *asdIt != 0 ; 
	asdIt++,asdId++) { 
      int chanId=0;
      for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels() ;
	  chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++, chanId++) {
// 	for(ConstAsdIterator asdIt2 = asdIt ; *asdIt2 != 0 ; 
// 	    asdIt2++) { 
// 	  for(waxDecodeAsd::ConstChannelIterator chanIt2 = (*asdIt2)->GetChannels();
// 	      chanIt2 != (*asdIt2)->GetChannelsEnd() ; chanIt2++) {
// 	    if(chanIt != chanIt2 && chanIt->GetGedeIndex() != 0 &&
// 	       chanIt->GetGedeIndex() == chanIt2->GetGedeIndex() &&
// 	       chanIt->GetWireIndex() == chanIt2->GetWireIndex()) {
// 	      std::cout << " These channels share gede-wire : " 
// 			<< asdIt - firstAsd << " " 
// 			<< chanIt - (*asdIt)->GetChannels() << " "
// 			<< asdIt2 - firstAsd << " "
// 			<< chanIt2 - (*asdIt2)->GetChannels() << " "
// 			<< ArtePointer<GEDE>(chanIt->GetGedeIndex())->hwc << " "
// 			<< chanIt->GetGedeIndex() << " "
// 			<< int(chanIt->GetWireIndex()) << std::endl ;
// 	    }
// 	  }
// 	}
//       }
//     }
 
	int gedeId = chanIt->GetGedeIndex() ;
	if(gedeId != 0) {
	  int wireId = chanIt->GetWireIndex() ;
	  const waxDecodeGede* gedeIt = GetGede(gedeId) ; 
	  const waxDecodeWire* wireIt = gedeIt->GetWires() + wireId ;
	  if(asdId  != wireIt->GetAsdIndex()  || chanId != wireIt->GetChannelIndex() ) {
	    waxDecodeChannel* altChanIt = 
	      GetAsd(wireIt->GetAsdIndex())->
	      GetChannel( wireIt->GetChannelIndex()) ;
	    altChanIt->SetStatus(2) ;
	    chanIt->SetStatus(2) ;
	    // 	    std::cout << "Shit !!!! "  
	    // 		      << setw(6) << gedeId 
	    // 		      << setw(4) << wireId 
	    // 		      << setw(6) << asdId
	    // 		      << setw(4) << chanId 
	    // 		      << setw(6) << wireIt->GetAsdIndex() 
	    // 		      << setw(4) << int(wireIt->GetChannelIndex())
	    // 		      << setw(6) << altChanIt->GetGedeIndex() 
	    // 		      << setw(4) << int(altChanIt->GetWireIndex()) << std::endl ;
	  }
	}
      }
    }
  
//     int gedeId=GetMinGedeIndex();
//     for(GedeIterator gedeIt = GetGedes() ; gedeIt!= GetGedesEnd(); 
// 	gedeIt++, gedeId++) {
//       int wireId=0;
//       for(waxDecodeGede::WireIterator wireIt = gedeIt->GetWires() ;
// 	  wireIt != gedeIt->GetWiresEnd() ; wireIt++, wireId++) {
//       int asdId = wireIt->GetAsdIndex() ;
//       if(asdId != MAXASD) {
// 	int chanId = wireIt->GetChannelIndex() ;
// 	const waxDecodeAsd* asdIt = GetAsd(asdId) ;
// 	const waxDecodeChannel* chanIt = asdIt->GetChannel(chanId) ;
// 	// 	std::cout << gedeId << " " <<  wireId << " ==> "
// 	// 	     << asdId << " " << chanId << " "
// 	// 	     << chanIt->GetStatus() << std::endl ;
// 	if(gedeId != chanIt->GetGedeIndex() ||
// 	   wireId != chanIt->GetWireIndex()) {
// 	  std::cout << "Help !" << gedeId << " " << wireId << " "
// 		    << chanIt->GetGedeIndex()<< " " << chanIt->GetWireIndex()<< " "
// 		    << asdId << " " << chanIt - asdIt->GetChannels() << std::endl ;
	  
// 	}
//       }
//       }
//     }
#endif
  } else {
    std::cout << "waxDecodeManager::waxDecodeManager(): "
	 << "Could not create OTR_Decode object." << std::endl ;
  }
}

/*
void waxDecode_create_gedes(OTR_Decode *decodeTable)
{
  int i,mingedeid,maxgedeid ;
  OTrASD8** asdIt ;
  OTrWire* chanIt ;
  OTrGede* gedeIt ;
  OTrGedeWire* wireIt ;
  ConstAsdIterator asdIt ;
  waxDecodeAsd::ConstChannelIterator chanIt ;
  
  // determine number of gedes in OTrDecode structures
  fMinGedeIndex = 1000000;
  int maxgedeid=0;
  for(asdIt = GetAsds() ; *asdIt != 0; asdIt++)
    for(chanIt = asdIt->GetChannels() ; 
	chanIt != asdIt->GetChannelsEnd() ; chanIt++) {
      int gedeindex = chanIt->GetGedeIndex() ;
      if(gedeindex>0) {
	if(gedeindex<fMinGedeIndex) fMinGedeIndex=gedeindex ;
	if(gedeindex>maxgedeid) maxgedeid=gedeindex ;
      }
    }
  fGedes.reserve(maxgedeid-fMinGedeIndex+1) ;
  
  // determine number of wires in every gede
  vector<int> nwires(maxgedeid-fMinGedeIndex+1,0) ;
  for(asdIt = GetAsds() ; *asdIt != 0; asdIt++)
    for(chanIt = asdIt->GetChannels() ; 
	chanIt != asdIt->GetChannelsEnd() ; chanIt++)
      if(chanIt->GEDE>0)
	if(nwires[chanIt->GEDE - fMinGedeIndex] < 1+chanIt->cell)
	  nwires[chanIt->GEDE - fMinGedeIndex] = 1+chanIt->cell ;

  // allocate gedes + wires
  for(vector<int>::iterator nwiIt = nwires.begin() ;
      nwiIt != nwires.end(); nwiIt++) 
    fGedes.push_back(waxDecodeGede(*nwiIt)) ;

  // initialize wires
  for(asdIt = GetAsds() ; *asdIt != 0; asdIt++)
    for(chanIt = asdIt->GetChannels() ; 
	chanIt != asdIt->GetChannelsEnd() ; chanIt++) { 
      waxDecodeWire* wireIt = 
	fGedes[chanIt->GEDE-decodeTable->fMinGedeIndex].GetWire(chanIt->cell) ;
      wireIt->SetAsdIndex(asdIt - GetAsds()) ;
      wireIt->SetChannelIndex(chanIt - asdIt->GetChannels()) ;
    }

  // create map for fast access in correct order
  fGedeMap.reserve(fGedes.size()) ;
  for(GedeIterator gedeIt = fGedes.begin() ;
      gedeIt != fGedes.end() ; gedeIt++)
    if(gedeIt->GetNWires()>0) fGedeMap.push_back(gedeIt) ;
  
  sort(fGedeMap.begin(),fGedeMap.end(),waxCompareGede) ;
}

*/

waxDecodeManager::~waxDecodeManager() 
{
  //delete fDecodeTable ;
  if(fgMe==this) fgMe = 0;
}

int
waxDecodeManager::GetNewEvent(int* nhits, waxDecodeChannel*** hitchannels)
{
  return OTr_get_event(fDecodeTable,nhits,(OTrWire***)(hitchannels)) ;
}

int waxDecodeManager::GetNAsds() const {
  int n=0;
  for(ConstAsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++) n++ ; 
  return n;
}

waxDecodeAsd* waxDecodeManager::GetAsd(string asdname) 
{
  waxDecodeAsd* asdIt=0;
  unsigned char slid,layid,secid,ftbid ;
  if(asdid2num(asdname.c_str(),&slid,&layid,&secid,&ftbid)==0 && slid<32) {
    OTrSuperLayer* slIt =  fDecodeTable->Slayers + slid ;
    unsigned char firstasd = slIt->FirstASD[layid][secid] ;
    if(firstasd<255) {
      unsigned short asdId = 
	slIt->Line[slIt->pos[layid][secid] - firstasd + ftbid] ;
      asdIt = GetAsd(asdId) ;
    }
  }
  return asdIt ;
}

string waxDecodeAsd::GetName() const
{
  static char name[12] ;
  num2asdid(GetSuperLayerIndex(),GetLayerIndex(),
	    GetSectorIndex(),GetSubAsdIndex(),name) ;
  return string(name) ;
}

void waxDecodeManager::ReadTZeros() 
{
  std::cout << "waxDecodeManager::ReadTZeros(): " ; 
  const string filenameUser = fgTZeroFileName ;
  if(fgTZeroFileName==string("none")) {
    std::cout << "TZeroFile = none -->Skip reading tzeros." << std::endl ;
    return ;
  }
  char runname[9] ;
  //sprintf(runname,"run%5.5d",arteflag_.numrun) ;
  sprintf(runname,"run%5.5d",ArteTable<EVHD>::begin()->run) ;
  const string filenameRun = "/afs/desy.de/group/hera-b/OTR/db/tzero/" +
    string(runname) + ".tzero" ;
  const string filenameGen = "/afs/desy.de/group/hera-b/OTR/db/tzero/otr.tzero"  ;
  
  std::ifstream ifs(filenameUser.c_str()) ;
  if(ifs) std::cout << "Opened " << filenameUser << std::endl;
  else {
    ifs.open(filenameRun.c_str()) ;
    if(ifs) std::cout << "Opened " << filenameRun << std::endl ;
    else {
      ifs.open(filenameGen.c_str()) ;
      if(ifs) std::cout << "Opened " << filenameGen << std::endl ;
      else
	std::cout << "Could not open file under any of these names: " << std::endl
	     << filenameUser << std::endl << filenameRun << std::endl << filenameGen << std::endl ;
    }
  }
  while(ifs) {
    string asdname ;
    waxDecodeAsd* asdIt ;
    ifs >> asdname ;
    asdIt = GetAsd(asdname) ;
    int tzero,tzeroerr ;
    if(asdIt) {
      for(waxDecodeAsd::ChannelIterator chanIt = asdIt->GetChannels() ;
	  chanIt != asdIt->GetChannelsEnd() ; chanIt++) {
	ifs >> tzero >> tzeroerr ;
	chanIt->SetTZeroCount(tzero,tzeroerr) ;
      }
    } else for(int i=0; i<16; i++) ifs >> tzero >> tzeroerr ;
  }
  ifs.close() ;
}

void waxDecodeManager::WriteTZeros(const string& filename) 
{
  std::ofstream ofs(filename.c_str()) ;
  int asdId=0;
  for(ConstAsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++, asdId++) {
    ofs << (*asdIt)->GetName() << " " ;
    for(waxDecodeAsd::ConstChannelIterator chanIt = (*asdIt)->GetChannels() ;
	chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++) 
      ofs << setw(4) << int(chanIt->GetTZeroCount())
	  << setw(4) << int(chanIt->GetTZeroErrCount()) ;
    ofs << std::endl ;
  }
  ofs.close() ;
}

void waxDecodeManager::ReadStatus() 
{
  std::cout << "waxDecodeManager::ReadStatus(): " ; 
  const string filenameUser = fgMaskFileName ;
  if(fgMaskFileName==string("none")) {
    std::cout << "MaskFile = none -->Skip reading masks." << std::endl ;
    return ;
  }
  char runname[9] ;
  //sprintf(runname,"run%5.5d",arteflag_.numrun) ;
  sprintf(runname,"run%5.5d",ArteTable<EVHD>::begin()->run) ;
  const string filenameRun = "/afs/desy.de/group/hera-b/OTR/db/masks/beam-" +
    string(runname) + "/otr.mask" ;
  const string filenameGen = "/afs/desy.de/group/hera-b/OTR/db/masks/beam/otr.mask"  ;
  
  std::ifstream ifs(filenameUser.c_str()) ;
  if(ifs) std::cout << "Opened " << filenameUser << std::endl;
  else {
    ifs.open(filenameRun.c_str()) ;
    if(ifs) std::cout << "Opened " << filenameRun << std::endl ;
    else {
      ifs.open(filenameGen.c_str()) ;
      if(ifs) std::cout << "Opened " << filenameGen << std::endl ;
      else
	std::cout << "Could not open file under any of these names: " << std::endl
		  << filenameUser << std::endl 
		  << filenameRun << std::endl 
		  << filenameGen << std::endl ;
    }
  }
  while(ifs) {
    char line[400];
    ifs.getline(line,400) ;
    if(line[0] != '#' && strlen(line)>12) {
      char asdname[12] ;
      char statusstr[17] ;
      sscanf(line,"%s %s",asdname,statusstr) ;
      waxDecodeAsd* asdIt = GetAsd(asdname) ;
      if(asdIt) {
	char* statusIt = statusstr ;
	for(waxDecodeAsd::ChannelIterator chanIt = asdIt->GetChannels() ;
	    chanIt != asdIt->GetChannelsEnd() ; chanIt++, statusIt++) {
	  switch(*statusIt) {
	  case 'D':
	  case 'W':
	  case 'H':
	  case 'X':
	  case 'I':
	  case 'V':
	  case 'N':
	  case 'F':
	    chanIt->SetStatus(1) ;
	    break;
	  case '.':
	  case 'C':
	  default:
	    chanIt->SetStatus(0) ;
	  }
      }
      }
    }
  }
  ifs.close() ;
}

void waxDecodeManager::SwapCables(const std::string& filename)
{
  cout << "waxDecodeManager::SwapCables(...): reading from " << filename << endl ; 
  std::ifstream ifs(filename.c_str()) ;
  if(ifs) {
    int format ;
    ifs >> format ;
    if(format==1) {
      int nlines(0) ;
      int nasds ;
      ifs >> nasds ;
      while(ifs) {
	std::vector<waxDecodeAsd*> asdvec(nasds) ;
	string name;
	bool all_ok = true ;
	for(int i=0; i<nasds; i++) {
	  ifs >> name ;
	  asdvec[i] = GetAsd(name) ;
	  if(asdvec[i]==0) {
	    all_ok = false ;
	    cout << "Strange name: " << name << endl ;
	  }
	}	
	if(all_ok) {

	  // let's check first
	  for(int i=0; i<nasds; ++i) {
	    waxDecodeAsd* asdIt = asdvec[i];
	    waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ;
	    if(GetAsd(tdcIt->GetAsdIndex(asdIt->GetTdcConnectorIndex()-1)) !=
	       asdIt) cout << "Shit: already connection error." << endl ;
	  }
	  
	  for(int i=0; i<nasds-1; ++i) {
	    waxDecodeAsd* asdIt = asdvec[i];
	    waxDecodeAsd* nextasdIt = asdvec[i+1];
	    waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ; 
	    waxDecodeTdc* nexttdcIt = GetTdc(nextasdIt->GetFedIndex()) ;
	    // swap info in tdc
	    swap(tdcIt->fTdc.ASD[asdIt->GetTdcConnectorIndex()-1],
		 nexttdcIt->fTdc.ASD[nextasdIt->GetTdcConnectorIndex()-1]) ;  
	    // swap info in asd
	    swap(asdIt->fAsd.TDC_FED,nextasdIt->fAsd.TDC_FED) ;
	    swap(asdIt->fAsd.TDC_Connector,nextasdIt->fAsd.TDC_Connector) ;
	  }
	  
	  // now check:
	  for(int i=0; i<nasds; ++i) {
	    waxDecodeAsd* asdIt = asdvec[i];
	    waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ;
	    if(GetAsd(tdcIt->GetAsdIndex(asdIt->GetTdcConnectorIndex()-1)) !=
	       asdIt) cout << "Shit: connection error inntroduced." << endl ;
	  }
	}
	nlines++;
	ifs >> nasds ;
      }
      cout << "Read " << nlines << " lines." << endl ;
    } else cout << "SwapCables error: wrong format " << endl ;
  } else cout << "Error: No such file." << endl ;
}
