#include <algorithm>
#include "wax/Event.hh"
#include "wax/Hit.hh"
#include "wax/HITB.hh"

#ifdef USE_ARTE
 #include <arte/arte.hh>
 #include <arte/artefunc.h>
#endif

//__________________________________________________________________
// waxEvent
//
// Container for:
// * Hits
// Functionality:
// * Read new hits
// * Append to HITB
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

// static const unsigned char fgMinTdcCount[2] = { 50,20 } ;
// static const unsigned char fgMaxTdcCount[2] = { 255,255 } ;

extern "C" 
{
int otr_gede_status(int, int, int);
}

waxEvent::waxEvent() : fId(-1), fHits() {} ;

waxEvent::~waxEvent() { 
  //for(HitContainer::const_iterator hitp=fHits.begin(); 
  //    hitp != fHits.end(); hitp++) delete (*hitp);
}

void
waxEvent::Next(waxDecodeManager* dm)
{
  //cout << "waxEvent::Next(waxDecodeManager* dm)" << endl ;
  //for(HitContainer::const_iterator hitp=fHits.begin();
  //    hitp != fHits.end(); hitp++) delete (*hitp);
  int id = ArteTable<EVHD>::begin()->evt ;
  if(id == fId) {
    std::cout << "waxEvent::Next(...): Event already processed. Skipping." << std::endl ;
  } else {
    fId = id ;
    fHits.erase( fHits.begin(), fHits.end() );
    
    if( ArteTable<MCIN>::booked()) { // monte carlo
      //cout << "waxEvent::Next(): Processing MC event." << endl ;
      // For the moment: reset the decoding structures, give all channels that have
      // entry in HITB bank a `valid' drifttime.
      for(waxDecodeManager::ConstAsdIterator asdIt = dm->GetAsds() ; *asdIt != 0 ; asdIt++) 
	for(waxDecodeAsd::ConstChannelIterator chanIt = (*asdIt)->GetChannels() ;
	    chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++)
	  (const_cast<waxDecodeChannel*>(chanIt))->SetTdcCount(0) ;
	  
      for(ArteTable<HITB>::iterator hitbIt = ArteTable<HITB>::begin() ;
	  hitbIt != ArteTable<HITB>::end() ; hitbIt++) 
	if(hitbIt->gede->cmp==2) {
	  waxHITB otrhitb = waxHITB(hitbIt) ;
	  (const_cast<waxDecodeChannel*>(otrhitb.GetChannel()))->SetTdcCount(100) ;
	}
      
    } else if( ArteTable<DOTR>::booked() ) { // data
      //cout << "waxEvent::Next(): Processing data event." << endl ;
      // Reset all pointers in gede bank
      waxGedeManager::GetMe()->ResetHITBPointers() ;
      // Copy hits from OTrDecode
      int nhits(0) ;
      waxDecodeChannel **hitchannels(0) ;
      if(dm->GetNewEvent(&nhits,&hitchannels)==1) {
	fHits.reserve(nhits) ;
	for(int i=0; i<nhits; i++) 
	  //     if((hitchannels[i]->GetStatus()&0xFF)=0x00)
	  if((hitchannels[i]->GetStatus()==0) &&
	     (hitchannels[i]->GetWireIndex()!=128) &&
	     (hitchannels[i]->GetTdcCount()>20) ) 
	    fHits.push_back(waxHit(hitchannels[i])) ;
      } //else std::cout << "wax_get_event(...) returns 0 ..." << std::endl ;
      // Append all hits to hitb bank
      AppendHITB() ;

    } // else: `minidst'
  }
}

void 
waxEvent::DumpHits(std::ostream& os) const
{
  os << "waxEvent::DumpHits(...) dumps " << fHits.size() 
     << " hits." << std::endl ;
  for(HitContainer::const_iterator hitIt=fHits.begin();
      hitIt != fHits.end(); hitIt++)
    os << *hitIt << std::endl ;
  return ;
}

inline bool compareHit( const waxHit& lhs, const waxHit& rhs )
{
  return (lhs.GetGedeIndex() < rhs.GetGedeIndex()) ||
    ((lhs.GetGedeIndex()==rhs.GetGedeIndex()) 
     && (lhs.GetWireIndex() < rhs.GetWireIndex()));
}

void
waxEvent::SortHits()
{
  sort( fHits.begin(), fHits.end(), compareHit );
}

#ifdef USE_ARTE
void
waxEvent::AppendHITB()  
{
  // Append all hits in this superlayer to the HITB bank. Before
  // calling this function make sure that the hits are sorted with
  // SortHits() and that the GEDE::hit1 and hitl entries are reset.

  HitContainer::iterator hitIt = GetHits().begin() ;
  if(hitIt != GetHits().end()) {
    //ArtePointer<HITB> hitbIt = (*hitIt).AppendHITB();  //old
    ArtePointer<HITB> hitbIt = 
      ArteTable<HITB>::allocate(GetHits().size()) ;
    (*hitIt).AppendHITB(hitbIt) ; //new 
    ArtePointer<GEDE> currentGEDE = hitbIt->gede ;
    currentGEDE->hit1 = hitbIt ;
    ArtePointer<HITB> prevhitbIt=hitbIt ;
    hitIt++ ;
    hitbIt++ ;
    for(; hitIt != GetHits().end() ; hitIt++,hitbIt++) {
      //hitbIt = (*hitIt).AppendHITB();
      (*hitIt).AppendHITB(hitbIt) ;
      if(hitbIt->gede != currentGEDE) {
	currentGEDE->hitl = prevhitbIt ;
	currentGEDE = hitbIt->gede ;
	currentGEDE->hit1 = hitbIt ;
      }
      prevhitbIt = hitbIt ;
    }
    currentGEDE->hitl = prevhitbIt ;
  }
}
#endif
