#include <iostream>
#include <algorithm>
#include "wax/EventBuilder.hh"
#include "wax/GedeManager.hh"
#include "wax/DSRManager.hh"
#include "wax/Event.hh"
#include "wax/DecodeManager.hh"
#include "wax/Gede.hh"
#include "wax/otr_db.h"

extern "C" int ot_init() ;
extern "C" int otdr_init() ;

//__________________________________________________________________
// waxEventBuilder
//
// Container for: 
// * waxEvent 
// * waxDecode
// * waxGedeManager
//
// Functionality:
// Building of the waxEvent from the arte HITB bank (dst and monte
// carlo), from the DOTR bank or from the DAQ data files.
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

waxEventBuilder* waxEventBuilder::fgMe = 0;

std::string waxEventBuilder::fgConfigDirName = "/afs/desy.de/group/hera-b/OTR/db" ;

waxEventBuilder::waxEventBuilder()
  : fDecodeManager(0), fGedeManager(0), fEvent(0),
    fDataType(0), fNProcessedEvents(0) 
{
  std::cout << "waxEventBuilder::waxEventBuilder()" << std::endl ;
  if(fgMe) {
    std::cout << "Already an waxEventBuilder object!" << std::endl ;
    return ;
  }
  fgMe = this ;
  
  fGedeManager = waxGedeManager::GetMe() ;
  fDSRManager  = waxDSRManager::GetMe() ;
  fDecodeManager = waxDecodeManager::GetMe() ;

  if(fDecodeManager->GetDecodeTable() != NULL) {
    ot_init() ;   // init the otmo tables
    otdr_init() ; // init the dead regions table
    fEvent = new waxEvent() ;
  } else {
    fEvent = 0;
  }
    
//   MizziDatabase(db) ;
//   AddStderrStream(stderr,"stderr");
//   AddStderrStream(fopen("Otr_db.err","w"),"Otr_db.err");
//   AddStdoutStream(stdout,"stdout");
//   AddStdoutStream(fopen("Otr_db.log","w"),"Otr_db.log"); 
//   SetLogLevel(100);
//   /* pre-initialization */
//   OTR_nede->initialize();
//   OTR_nesl->initialize();
//   OTR_cab->initialize();
//   OTR_calib->initialize();
//   OTR_sharc->initialize();
//   OTR_align->initialize();
//   OTR_r_t->initialize();
//    /* Check relation net consistency and complete initialization */
//   LedaCheckDD();
//   db.open("Otr.db",0);
//   std::cout << "Opened database file." << std::endl;
//   db.stamp(10,1);
//   OTR_r_t->book(db);
//   OTR_r_t->fetch_data(DBeq);
//   OTR_r_t->set_keys() ;
//   fDSRManager->ReadDB() ;
//   db.close() ;
  
  std::cout << "End of waxEventBuilder::waxEventBuilder()" << std::endl ;
}

waxEventBuilder::~waxEventBuilder()
{
  delete fDecodeManager ;
  delete fGedeManager ;
  delete fEvent ;
  if(fgMe==this) fgMe = 0;
}

waxEvent*
waxEventBuilder::ProcessOneEvent()
{ 
  //std::cout << "waxEventBuilder::ProcessOneEvent()" << std::endl ;
  if(fEvent) fEvent->Next(fDecodeManager) ;
  fNProcessedEvents++ ;
  return fEvent ;
}
